program mpipaper5analysis_time
  use netcdf
  use mpi
  implicit none

  ! debug output
  logical :: debug_output = .false.

  ! precision
  integer, parameter :: SP=4
  integer, parameter :: DP=8

  ! cloud and ice grids criteria
  real(SP), parameter :: qliq_lower_limit=1.e-6
  real(SP), parameter :: qice_lower_limit=1.e-9

  ! polycrystals exist or not, and ice-axis version
  logical, parameter :: poly_exist=.false.
  logical, parameter :: prevent_unphysical_length=.false.
  integer, parameter :: iceaxis_version=3
  real(SP), parameter :: spx_p=0.25E0

  ! io
  integer :: current_io_number
  integer, parameter :: MAX_IO_NUMBER=30

  ! input file model variables
  integer :: IO_INPUT_CONFIG                ! input file io
  character (len=256) :: INPUT_CONFIG_FNAME ! input file name
  integer :: prc_num_x, prc_num_y           ! model process numbers
  integer :: grd_x, grd_y, grd_z            ! model global grid numbers
  integer :: IMAX, JMAX, KMAX               ! model local grid numbers
  real(DP) :: DX, DY                        ! model horizontal grid spacings
  logical :: FILE_HISTORY_CARTESC_BOUNDARY=.false. ! model boundary output

  ! data file variables
  integer, allocatable :: IO_DATA(:)

  ! netcdf file variables
  !character (len=256), parameter :: INPUT_DIR="/home/OngChiaRui/WORKING/scale-5.4.4/scale-rm/test/tutorial/mpace/data/bulk_surface/2300klein_lw/sim_mpace_3D_nohydro_tlsABLmy1_c09nccn30_fracdust0013/" ! input file directory
  character (len=256), parameter :: INPUT_DIR="/home/OngChiaRui/WORKING/scale-5.4.4/scale-rm/test/tutorial/sheba/sim_3D_den0gamma2_fracdust00004/"
  character (len = 256), allocatable :: FILE_DIR(:)
  character (len = 256), allocatable :: FILE_NAME(:)
  character (len = 6) :: word_zeros
  character (len = 6) :: word
  integer, parameter :: TOTAL_FILE_NUMBER=2

  ! output file
  integer :: IO_OUTPUT_CONFIG ! output file io
  character (len=256) :: OUTPUT_CONFIG_FNAME ! output file name
  !character (len=256), parameter :: OUTPUT_DIR="/home/OngChiaRui/WORKING/scale-5.4.4/scale-rm/test/tutorial/mpace/data/bulk_surface/2300klein_lw/sim_mpace_3D_nohydro_tlsABLmy1_c09nccn30_fracdust0013/paper5_figures/" ! output file directory
  character (len=256), parameter :: OUTPUT_DIR="/home/OngChiaRui/WORKING/scale-5.4.4/scale-rm/test/tutorial/sheba/sim_3D_den0gamma2_fracdust00004/paper5_figures/" ! output file directory

  ! netcdf indices
  integer :: ncid, dimid, varid

  ! MPI communication variables
  logical :: PRC_mpi_alive = .false.
  integer :: isize, irank
  integer :: sendcount_median, count_global_median
  integer, parameter :: master=0
  integer, parameter :: MAX_PRC_X=4
  integer :: TOTAL_PRC
  integer :: JOBS_PER_MPI, JOBS_PER_MPI_X
  integer, allocatable :: jobs_index(:)
  integer :: mpi_x, mpi_y
  integer :: start_3d(4), count_3d(4), start_2d(3), count_2d(3)

  ! history items
  character (len=32) :: NAME
  character (len=32) :: varname

  ! grid
  integer :: is, ie, isg, ieg, js, je, jsg, jeg, ks, ke, ts, te
  integer :: nfx, nfz, nx, ny, nz, nt, nt_partial(TOTAL_FILE_NUMBER)
  real(DP), allocatable :: fx(:), fz(:), cx(:), cy(:), cz(:), ct(:)

  integer,  allocatable :: is_below_cloud(:,:,:)

  ! data
  real(SP), allocatable :: global_var_1d(:), local_var_1d(:) ! local x-y array
  real(SP), allocatable :: global_var_nliq(:), local_var_nliq(:) ! local x-y-z array
  real(SP), allocatable :: global_var_nice(:), local_var_nice(:) ! local x-y-z array
  real(SP), allocatable :: global_var_nice200(:), local_var_nice200(:) ! local x-y-z array
  real(SP), allocatable :: global_var_nice200belowcloud(:), local_var_nice200belowcloud(:) ! local x-y-z array
  
  real(SP), allocatable :: qliq(:,:,:), qice(:,:,:), qhyd(:,:,:), dens(:,:,:) ! i, j, k
  real(SP), allocatable :: temperature(:,:,:)
  real(SP), allocatable :: ice_mass_vapor(:,:,:), ice_mass_eva(:,:,:), ice_mass_rim(:,:,:)
  real(SP), allocatable :: ice_mass_imm(:,:,:), ice_conc_imm(:,:,:), liq_mass_rim(:,:,:)
  real(SP), allocatable :: prec(:,:), LWdn(:,:), SWdn(:,:)
  !real(SP), allocatable :: nc(:,:,:), nr(:,:,:), ni(:,:,:), ns(:,:,:), ng(:,:,:)
  real(SP), allocatable :: bin_liq_con(:,:,:,:), bin_ice_con(:,:,:,:) ! i, j, k, bin
  real(SP), allocatable :: bin_ice_mass(:,:,:,:), bin_ice_mcry(:,:,:,:)
  real(SP), allocatable :: bin_ice_magg(:,:,:,:), bin_ice_mrim(:,:,:,:)
  real(SP), allocatable :: bin_ice_alen(:,:,:,:), bin_ice_clen(:,:,:,:)
  real(SP), allocatable :: bin_ice_dlen(:,:,:,:), bin_ice_aglen(:,:,:,:)
  real(SP), allocatable :: bin_ice_cglen(:,:,:,:), bin_ice_nex(:,:,:,:)
  real(SP), allocatable :: bin_ice_Dmax(:,:,:,:)
  real(SP), allocatable :: LWP(:), IWP(:), MEAN_NLIQ_INCLOUD(:), MEAN_NICE_INICE(:), COUNTER_INCLOUD(:), COUNTER_BELOWCLOUD(:), COUNTER_INICE(:)
  real(SP), allocatable :: MEAN_NICE_INCLOUD(:), MEAN_DLIQ_INCLOUD(:), MEAN_DICE_INICE(:)
  real(SP), allocatable :: MEAN_DICE_INCLOUD(:)
  real(SP), allocatable :: RMRIMLIQ_INCLOUD(:), RMRIM_INCLOUD(:), RMIMM_INCLOUD(:), RMIMMC_INCLOUD(:)
  real(SP), allocatable :: RMVAP_INCLOUD(:), CTOPT(:), RMVAP(:), RMVAP_BELOWCLOUD(:)
  real(SP), allocatable :: MCRY_INCLOUD(:), MAGG_INCLOUD(:), MRIM_INCLOUD(:), MCRY(:)
  real(SP), allocatable :: MAGG(:), MRIM(:), SFC_PREC(:), SFC_LWDN(:), SFC_SWDN(:)
  real(SP), allocatable :: MED_NLIQ_INCLOUD(:), MED_NICE_INCLOUD(:), MED_NICE200_INCLOUD(:), MED_NICE200_BELOWCLOUD(:)
  
  ! liquid bins
  integer, parameter :: max_binr = 40
  real(SP), allocatable :: massbin(:), massbin_bnd(:)
  real(SP), allocatable :: diameterbin(:), diameterbin_bnd(:)
  real(SP), parameter :: c_min = 0.000000000000004188790205, c_max = 0.0000000654498
  real(SP), parameter :: c_maxmass = 0.52359870

  ! ice bins
  integer, parameter :: max_bini = 20
  real(SP), allocatable :: massbini(:), massbini_bnd(:)
  real(SP), allocatable :: diameteribin(:), diameteribin_bnd(:)
  real(SP), allocatable :: massbini_bothbnd(:,:)
  real(SP) :: ci_min, ci_max

  ! local variables 
  integer :: i, j, k, t, t_local, k_file, ibin, counter1, counter2, counter3, counter4
  integer :: median_array_size1, median_array_size2, median_array_size3, median_array_size4
  real(SP), allocatable :: median_array1(:), median_array2(:), median_array3(:), median_array4(:)
  integer :: ierr
  logical :: lstat
  real(SP) :: temp, temp1, temp2, temp3, temp4, temp5, temp6, temp7, temp8, temp9, temp9c, temp10
  real(SP) :: temp11, temp12, temp13, temp14, temp15, temp16, temp17, temp18, temp19
  real(SP) :: temp20, temp21, temp22, temp23, temp24
  real(SP) :: incloud_counter, topcloud_counter, belowcloud_counter, inice_counter
  real(SP) :: dbin, debugtemp, D2, D3
  real(SP) :: global_sum_lwp, global_sum_iwp, sum_lwp, sum_iwp
  real(SP) :: global_sum_meandliqincloud, global_sum_meandiceinice, global_sum_prec
  real(SP) :: global_sum_meandiceincloud
  real(SP) :: global_sum_lwdn, global_sum_swdn, global_sum_rimincloud
  real(SP) :: global_sum_immincloud, global_sum_immcincloud, global_sum_netvaporincloud, global_sum_cloudtopT
  real(SP) :: global_sum_netvapor, global_sum_netvaporbelowcloud, global_sum_mcry
  real(SP) :: global_sum_magg, global_sum_mrim, global_sum_meanniceinice
  real(SP) :: global_sum_meannliqincloud, global_sum_meanniceincloud, global_sum_rimliqincloud
  real(SP) :: global_sum_mcryincloud, global_sum_maggincloud, global_sum_mrimincloud
  real(SP) :: global_sum_incloudcounter, global_sum_topcloudcounter, global_sum_belowcloudcounter, global_sum_inicecounter
  real(SP), parameter :: pi = 3.141592653589793238462643
  integer :: ice_type, my_habit, habit, test_incloud
  real(SP) :: vcs, vic, alpha_ic, alpha_im, den_i, kp, mlmt, mass_agg_cry, mass_agg_cry_rim, local_Dmax, asm, csm, pla_e, ros_r

  
  namelist /PARAM_PRC_CARTESC/ &
       PRC_NUM_X, &
       PRC_NUM_Y

  namelist /PARAM_ATMOS_GRID_CARTESC_INDEX/ &
       KMAX, &
       IMAX, &
       JMAX

  namelist /PARAM_ATMOS_GRID_CARTESC/ &
       DX, &
       DY

  namelist /PARAM_FILE_HISTORY_CARTESC/ &
       FILE_HISTORY_CARTESC_BOUNDARY

  namelist /HISTORY_ITEM/ &
       NAME


  call MPI_INIT( ierr )
  PRC_mpi_alive = .true.
  call MPI_COMM_SIZE( MPI_COMM_WORLD, isize, ierr )
  call MPI_COMM_RANK( MPI_COMM_WORLD, irank, ierr )
  if ( irank == master ) then
     write( *, * ) "+++ MPI COMM: Corrective Initialize"
  endif

  !--- Read from argument
  if ( COMMAND_ARGUMENT_COUNT() /= 1 ) then
     write(*, *) "ERROR: Program needs a config file!"
     call PRC_abort
  else
     call get_command_argument(1,INPUT_CONFIG_FNAME)
  endif

  if (irank == master) then
     write(*,*) "INPUT FILE NAME: ", trim(INPUT_CONFIG_FNAME)
  endif

  ! set initial io number value
  current_io_number = 1

  inquire (file=trim(INPUT_CONFIG_FNAME), exist=lstat)
  if ( .not. lstat ) then
     write (*, *) "ERROR: Not found the configuration file :", trim(INPUT_CONFIG_FNAME)
     call PRC_abort
  endif
  call register_io(IO_INPUT_CONFIG,INPUT_CONFIG_FNAME)

  rewind(IO_INPUT_CONFIG)
  read(IO_INPUT_CONFIG,nml=PARAM_PRC_CARTESC,iostat=ierr)

  if( ierr < 0 ) then !--- missing
     write(*,*) irank, "ERROR. Not found namelist: PARAM_PRC_CARTESC."
     call PRC_abort
  !elseif( ierr > 0 ) then !--- fatal error
  !   write(*,*) irank, "Not appropriate names in namelist PARAM_PRC_CARTESC, Check!"
  !   call PRC_abort
  end if
  if (irank == master) then
     write(*,*) "PRC_NUM_X PRC_NUM_Y: ", prc_num_x, prc_num_y
  endif
  
  rewind(IO_INPUT_CONFIG)
  read(IO_INPUT_CONFIG,nml=PARAM_ATMOS_GRID_CARTESC,iostat=ierr)

  if( ierr < 0 ) then !--- missing
     write(*,*) irank, "ERROR. Not found namelist: PARAM_ATMOS_GRID_CARTESC."
     call PRC_abort
  !elseif( ierr > 0 ) then !--- fatal error
  !   write(*,*) irank, "Not appropriate names in namelist PARAM_ATMOS_GRID_CARTESC, Check!"
  !   call PRC_abort
  end if

  rewind(IO_INPUT_CONFIG)
  read(IO_INPUT_CONFIG,nml=PARAM_ATMOS_GRID_CARTESC_INDEX,iostat=ierr)

  if( ierr < 0 ) then !--- missing
     write(*,*) irank, "ERROR. Not found namelist: PARAM_ATMOS_GRID_CARTESC_INDEX."
     call PRC_abort
  !elseif( ierr > 0 ) then !--- fatal error
  !   write(*,*) irank, "Not appropriate names in namelist PARAM_ATMOS_GRID_CARTESC_INDEX, Check!"
  !   call PRC_abort
  end if
  if (irank == master) then
     write(*,'(A)') "REMEMBER TO CHECK THESE VALUES, VERY IMPORTANT!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!"
     write(*,*) "KMAX IMAX JMAX: ", KMAX, IMAX, JMAX
     write(*,*) "DX DY: ", DX, DY
  endif

  grd_x = IMAX*prc_num_x
  grd_y = JMAX*prc_num_y
  grd_z = KMAX

  ! check if mpi number is a square
  mpi_x = 0
  mpi_x = 0
  do i = 1, MAX_PRC_X
     if (i*i == isize) then
        mpi_x = i
        mpi_y = i
     endif
  enddo
  if (mpi_x == 0 .or. mpi_y == 0) then
     write(*,*) irank, "ERROR, MPI NUMBER MUST BE A SQUARE", isize
     call PRC_abort
  else
     if (irank == master) then
        write(*,*) "MPI_X and MPI_Y: ", mpi_x, mpi_y
     endif
  endif
  if (prc_num_x /= prc_num_y .or. mod(prc_num_x,mpi_x) /= 0 .or. mod(prc_num_y,mpi_y) /= 0 .or. prc_num_x > mpi_x .or. prc_num_y > mpi_y) then
     write(*,*) irank, "ERROR, MODEL PROCESS NUMBER MUST BE A SQUARE AND MODEL PROCESS NUMBER MUST BE DIVIDED BY MPI SIZE", prc_num_x, prc_num_y, mpi_x, mpi_y
     call PRC_abort
  endif

  
  TOTAL_PRC = prc_num_x*prc_num_y
  JOBS_PER_MPI = TOTAL_PRC/isize
  JOBS_PER_MPI_X = prc_num_x/mpi_x

  if (irank == master) then
     write(*,*) "JOBS_PER_MPI JOBS_PER_MPI_X", JOBS_PER_MPI, JOBS_PER_MPI_X
  endif

  allocate(jobs_index(JOBS_PER_MPI))
  allocate(FILE_NAME(JOBS_PER_MPI))

  ! assign range of model process index
  i = JOBS_PER_MPI_X * mod(irank,mpi_x) + prc_num_x*JOBS_PER_MPI_X * int(irank/mpi_x)
  j = 0
  do k = 1, JOBS_PER_MPI
     jobs_index(k) = i ! minus 1 because history id starts from zero
     j = j + 1
     if (mod(j,JOBS_PER_MPI_X) == 0) then
        i = i + (prc_num_x - mpi_x)
        j = 0
     else
        i = i + 1
     endif
     write(*,*) "rank-wise job id: ", irank, jobs_index(k)
  enddo


  ! define liquid bins
  allocate(massbin(max_binr),massbin_bnd(max_binr+1))
  allocate(diameterbin(max_binr),diameterbin_bnd(max_binr+1))
  temp1 = c_min
  dbin = (c_max/c_min)**(0.05) ! 0.1 = 1/10, 0.05 = 1/20
  temp2 = temp1*dbin
  massbin_bnd(1) = temp1
  do i = 1, 20
     massbin_bnd(i+1) = temp2
     massbin(i) = 0.5*(temp1+temp2) ! liquid haze bins
     temp1 = temp2
     temp2 = temp2*dbin
  enddo
  dbin = (c_maxmass/c_max)**(0.05)
  temp2 = temp1*dbin
  do i = 21, max_binr
     massbin_bnd(i+1) = temp2
     massbin(i) = 0.5*(temp1+temp2) ! liquid drop bins
     temp1 = temp2
     temp2 = temp2*dbin
  enddo
    
  diameterbin_bnd(1) = (massbin_bnd(1)/(3.1416/6.0))**(1.0/3.0)*10000.0
  if (irank == master .and. debug_output) then
     print *, "liquid diameter: ",0, "---", diameterbin_bnd(1)
  endif
  do i = 1, max_binr
     diameterbin(i) = (massbin(i)/(3.1416/6.0))**(1.0/3.0)*10000.0 ! unit is \mu m
     diameterbin_bnd(i+1) = (massbin_bnd(i+1)/(3.1416/6.0))**(1.0/3.0)*10000.0
     if (irank == master .and. debug_output) then
        print*, "liquid diameter: ",i, diameterbin_bnd(i+1), diameterbin(i)
     endif
  enddo

  ! define ice bins
  allocate(massbini(max_bini),massbini_bnd(max_bini+1))
  allocate(diameteribin(max_bini),diameteribin_bnd(max_bini+1))
  allocate(massbini_bothbnd(max_bini,2))
  ci_min = 4.18879020478639e-12
  ci_max = 1.0e-6
  temp1 = ci_min
  dbin = (ci_max/ci_min)**(0.25)
  temp2 = temp1*dbin
  massbini_bnd(1) = temp1
  diameteribin_bnd(1) = (temp1/0.916/3.1416*6.0)**(1.0/3.0)*10000.0
  do i = 1, 4
     massbini_bnd(i+1) = temp2
     massbini(i) = 0.5*(temp1+temp2) ! ice deposition bins
     diameteribin_bnd(i+1) = (temp2/0.916/3.1416*6.0)**(1.0/3.0)*10000.0
     diameteribin(i) = (0.5*(temp1+temp2)/0.916/3.1416*6.0)**(1.0/3.0)*10000.0
     temp1 = temp2
     temp2 = temp2*dbin
  enddo
  ci_min = ci_max
  ci_max = 1.0e-2
  dbin = (ci_max/ci_min)**(1.0/10.0)
  temp2 = temp1*dbin
  do i = 5, 14
     massbini_bnd(i+1) = temp2
     massbini(i) = 0.5*(temp1+temp2) ! ice collection bins
     diameteribin_bnd(i+1) = (temp2/0.916/3.1416*6.0)**(1.0/3.0)*10000.0
     diameteribin(i) = (0.5*(temp1+temp2)/0.916/3.1416*6.0)**(1.0/3.0)*10000.0
     temp1 = temp2
     temp2 = temp2*dbin
  enddo
  ci_min = ci_max
  ci_max = 1.0e+1
  dbin = (ci_max/ci_min)**(1.0/6.0)
  temp2 = temp1*dbin
  do i = 15, 20
     massbini_bnd(i+1) = temp2
     massbini(i) = 0.5*(temp1+temp2) ! ice riming bins
     diameteribin_bnd(i+1) = (temp2/0.916/3.1416*6.0)**(1.0/3.0)*10000.0
     diameteribin(i) = (0.5*(temp1+temp2)/0.916/3.1416*6.0)**(1.0/3.0)*10000.0
     temp1 = temp2
     temp2 = temp2*dbin
  enddo

  do i = 1, max_bini
     massbini_bothbnd(i,1) = massbini_bnd(i)
     massbini_bothbnd(i,2) = massbini_bnd(i+1)
  enddo


  
  
  ! assign tag number for message passing
  ! not need here


  ! history file names
  do k = 1, JOBS_PER_MPI
     if (jobs_index(k) < 10) then
        write(word,'(I1)') jobs_index(k)
        word_zeros = "00000"
     else if (jobs_index(k) < 100) then
        write(word,'(I2)') jobs_index(k)
        word_zeros = "0000"
     else if (jobs_index(k) < 1000) then
        write(word,'(I3)') jobs_index(k)
        word_zeros = "000"
     else if (jobs_index(k) < 10000) then
        write(word,'(I4)') jobs_index(k)
        word_zeros = "00"
     else if (jobs_index(k) < 100000) then
        write(word,'(I5)') jobs_index(k)
        word_zeros = "0"
     else
        write(*,*) "IMPOSSIBLE JOBS ID (TOO LARGE): ", jobs_index(k)
        call PRC_abort
     endif
     FILE_NAME(k) = "history.pe"//trim(word_zeros)//trim(word)//".nc"
     write(*,'(I3,A,A)') k, " th FILE_NAME: ", FILE_NAME(k)
  enddo

  if (TOTAL_FILE_NUMBER > 9) then
     write(*,*) "FILE NUMBER SHOULD BE LESS THAN 10", TOTAL_FILE_NUMBER
     call PRC_abort
  endif
  allocate(FILE_DIR(TOTAL_FILE_NUMBER))
  do k_file = 1, TOTAL_FILE_NUMBER
     write(word,'(I1)') k_file
     FILE_DIR(k_file) = trim(INPUT_DIR)//trim(word)//"/" ! netcdf file directory
     if (irank == master) write(*,*) FILE_DIR(k_file)
  enddo

  
  ! read netcdf grid
  do k = 1, 1!JOBS_PER_MPI ! temporary, since x and y information are not needed here
     call check( nf90_open( trim(FILE_DIR(1))//trim(FILE_NAME(k)), nf90_nowrite, ncid ) )

     call check( nf90_inq_dimid ( ncid,  'FZ', varid ) )
     call check( nf90_inquire_dimension( ncid, varid, len=nfz ) )

     call check( nf90_inq_dimid ( ncid,  'FX', varid ) )
     call check( nf90_inquire_dimension( ncid, varid, len=nfx ) )

     call check( nf90_inq_dimid ( ncid,  'CZ', varid ) )
     call check( nf90_inquire_dimension( ncid, varid, len=nz ) )
     call check( nf90_inq_dimid ( ncid,  'CX', varid ) )
     call check( nf90_inquire_dimension( ncid, varid, len=nx ) )
     call check( nf90_inq_dimid ( ncid,  'CY', varid ) )
     call check( nf90_inquire_dimension( ncid, varid, len=ny ) )
   
     allocate(fz(nfz),fx(nfx),cz(nz),cx(nx),cy(ny))

     call check( nf90_inq_varid( ncid, 'FZ', varid ) )
     call check( nf90_get_var( ncid, varid, fz ) )

     call check( nf90_inq_varid( ncid, 'FX', varid ) )
     call check( nf90_get_var( ncid, varid, fx ) )

     call check( nf90_inq_varid( ncid, 'CZ', varid ) )
     call check( nf90_get_var( ncid, varid, cz ) )
     
     call check( nf90_inq_varid(ncid, 'CX', varid) )
     call check( nf90_get_var( ncid, varid, cx ) )
     
     call check( nf90_inq_varid(ncid, 'CY', varid) )
     call check( nf90_get_var( ncid, varid, cy ) )

     call check( nf90_close( ncid ) )
  enddo

  do k_file = 1, TOTAL_FILE_NUMBER
     call check( nf90_open( trim(FILE_DIR(k_file))//trim(FILE_NAME(1)), nf90_nowrite, ncid ) )
     call check( nf90_inq_dimid ( ncid,  'time', varid ) )
     call check( nf90_inquire_dimension( ncid, varid, len=nt_partial(k_file) ) )

     call check( nf90_close( ncid ) )

     if (k_file > 1) nt_partial(k_file) = nt_partial(k_file) - 1
     if (irank == master) write(*,*) k_file, "NT PARTIAL: ", nt_partial(k_file)
  enddo

  nt = sum(nt_partial(:))
  allocate(ct(nt))

  i = 1
  j = 0
  do k_file = 1, TOTAL_FILE_NUMBER
     call check( nf90_open( trim(FILE_DIR(k_file))//trim(FILE_NAME(1)), nf90_nowrite, ncid ) )
     if (k_file > 1) then
        k = 2
     else
        k = 1
     endif
     j = i + nt_partial(k_file) - 1
     call check( nf90_inq_varid(ncid, 'time', varid) )
     call check( nf90_get_var( ncid, varid, ct(i:j), start=(/k/), count=(/nt_partial(k_file)/) ) )
     i = i + nt_partial(k_file)

     call check( nf90_close( ncid ) )
  enddo

  if (TOTAL_FILE_NUMBER > 1) then
     do t =  nt_partial(1)+1, nt
        ct(t) = 2.0E0*ct(t-1) - ct(t-2)
     enddo
  endif

  if (irank == master .and. debug_output) then
     write(*,*) "NFZ NFX NX NY NZ NT: ", nfz, nfx, nx, ny, nz, nt
     do k = 1, 1!JOBS_PER_MPI
        write(*,*) "   -----FZ----- "
        do i = 1, nfz
           write(*,'(I3,E15.6)') i, fz(i)
        enddo
        write(*,*)
        write(*,*) "   -----FX----- "
        do i = 1, nfx
           write(*,'(I3,E15.6)') i, fx(i)
        enddo
        write(*,*)
        write(*,*) "   -----CX----- "
        do i = 1, nx
           write(*,'(I3,E15.6)') i, cx(i)
        enddo
        write(*,*)
        write(*,*) "   -----CY----- "
        do i = 1, ny
           write(*,'(I3,E15.6)') i, cy(i)
        enddo
        write(*,*)
        write(*,*) "   -----CZ----- "
        do i = 1, nz
           write(*,'(I3,E15.6)') i, cz(i)
        enddo
        write(*,*)
        write(*,*) "   -----CT----- "
        do i = 1, nt
           write(*,'(I3,E15.6)') i, ct(i)
        enddo
     enddo
  endif


  isg = 1
  jsg = 1
  ieg = IMAX*JOBS_PER_MPI_X
  jeg = JMAX*JOBS_PER_MPI_X
  !sendcount_median = IMAX*JMAX*JOBS_PER_MPI ! for x-y array
  !count_global_median = IMAX*JMAX*TOTAL_PRC ! for x-y array
  sendcount_median = IMAX*JMAX*KMAX*JOBS_PER_MPI ! for x-y-z array
  count_global_median = IMAX*JMAX*KMAX*TOTAL_PRC ! for x-y-z array
  allocate(is_below_cloud(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX))
  allocate(qliq(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX))
  allocate(qhyd(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX))
  allocate(qice(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX))
  allocate(dens(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX))
  allocate(temperature(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX))
  allocate(ice_mass_vapor(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX))
  allocate(ice_mass_eva(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX))
  allocate(ice_mass_imm(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX))
  allocate(ice_conc_imm(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX))
  allocate(ice_mass_rim(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX))
  allocate(liq_mass_rim(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX))
  allocate(prec(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X))
  allocate(LWdn(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X))
  allocate(SWdn(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X))
  allocate(bin_liq_con(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX,max_binr))
  allocate(bin_ice_con(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX,max_bini))
  allocate(bin_ice_mcry(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX,max_bini))
  allocate(bin_ice_mrim(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX,max_bini))
  allocate(bin_ice_magg(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX,max_bini))
  allocate(bin_ice_mass(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX,max_bini))
  allocate(bin_ice_alen(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX,max_bini))
  allocate(bin_ice_clen(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX,max_bini))
  allocate(bin_ice_dlen(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX,max_bini))
  allocate(bin_ice_Dmax(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX,max_bini))
  if (poly_exist) then
     allocate(bin_ice_aglen(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX,max_bini))
     allocate(bin_ice_cglen(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX,max_bini))
     allocate(bin_ice_nex(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX,max_bini))
  endif
  !allocate(local_var_1d(sendcount_median))
  !allocate(global_var_1d(count_global_median))
  allocate(local_var_nliq(sendcount_median))
  allocate(global_var_nliq(count_global_median))
  allocate(local_var_nice(sendcount_median))
  allocate(global_var_nice(count_global_median))
  allocate(local_var_nice200(sendcount_median))
  allocate(global_var_nice200(count_global_median))
  allocate(local_var_nice200belowcloud(sendcount_median))
  allocate(global_var_nice200belowcloud(count_global_median))

  allocate(LWP(nt),IWP(nt))
  allocate(COUNTER_INCLOUD(nt),COUNTER_BELOWCLOUD(nt),COUNTER_INICE(nt))
  allocate(MEAN_NLIQ_INCLOUD(nt),MEAN_NICE_INICE(nt),MEAN_NICE_INCLOUD(nt))
  allocate(MEAN_DLIQ_INCLOUD(nt),MEAN_DICE_INICE(nt),RMRIMLIQ_INCLOUD(nt))
  allocate(MEAN_DICE_INCLOUD(nt))
  allocate(RMRIM_INCLOUD(nt),RMIMM_INCLOUD(nt),RMIMMC_INCLOUD(nt),RMVAP_INCLOUD(nt))
  allocate(CTOPT(nt),RMVAP(nt),RMVAP_BELOWCLOUD(nt))
  allocate(MCRY_INCLOUD(nt),MAGG_INCLOUD(nt),MRIM_INCLOUD(nt))
  allocate(MCRY(nt),MAGG(nt),MRIM(nt))
  allocate(SFC_PREC(nt),SFC_LWDN(nt),SFC_SWDN(nt))
  allocate(MED_NLIQ_INCLOUD(nt),MED_NICE_INCLOUD(nt),MED_NICE200_INCLOUD(nt),MED_NICE200_BELOWCLOUD(nt))

  LWP = 0.0E0
  IWP = 0.0E0
  COUNTER_INCLOUD = 0.0E0
  COUNTER_INICE = 0.0E0
  COUNTER_BELOWCLOUD = 0.0E0
  MEAN_NLIQ_INCLOUD = 0.0E0
  MEAN_NICE_INICE = 0.0E0
  MEAN_NICE_INCLOUD = 0.0E0
  MEAN_DLIQ_INCLOUD = 0.0E0
  RMRIMLIQ_INCLOUD = 0.0E0
  RMRIM_INCLOUD = 0.0E0
  RMIMM_INCLOUD = 0.0E0
  RMIMMC_INCLOUD = 0.0E0
  RMVAP_INCLOUD = 0.0E0
  CTOPT = 0.0E0
  RMVAP = 0.0E0
  RMVAP_BELOWCLOUD = 0.0E0
  MCRY_INCLOUD = 0.0E0
  MAGG_INCLOUD = 0.0E0
  MRIM_INCLOUD = 0.0E0
  MCRY = 0.0E0
  MAGG = 0.0E0
  MRIM = 0.0E0
  SFC_PREC = 0.0E0
  SFC_LWDN = 0.0E0
  SFC_LWDN = 0.0E0
  MEAN_DICE_INICE = 0.0E0
  MEAN_DICE_INCLOUD = 0.0E0
  MED_NLIQ_INCLOUD = 0.0E0
  MED_NICE_INCLOUD = 0.0E0
  MED_NICE200_INCLOUD = 0.0E0
  MED_NICE200_BELOWCLOUD = 0.0E0

  
  k_file = 1
  t_local = 0
  ts = 0
  do t = 1, nt

     ts = ts + 1
     t_local = t_local + 1
     if (t_local > nt_partial(k_file)) then
        k_file = k_file + 1
        t_local = 1
        if (k_file > 1) then
           ts = 2
        else
           ts = 1
        endif
     endif

     ! read netcdf variables
     do k = 1, JOBS_PER_MPI

        if ( FILE_HISTORY_CARTESC_BOUNDARY ) then
           write(*,*) "still under construction..."
           call PRC_abort
        else
           is = 1 + IMAX*mod(k-1,JOBS_PER_MPI_X)
           js = 1 + JMAX*int((k-1)/JOBS_PER_MPI_X)
           ks = 1
           ie = IMAX + IMAX*mod(k-1,JOBS_PER_MPI_X)
           je = JMAX + JMAX*int((k-1)/JOBS_PER_MPI_X)
           ke = KMAX
        endif

        call check( nf90_open( trim(FILE_DIR(k_file))//trim(FILE_NAME(k)), nf90_nowrite, ncid ) )

        start_3d(1:4) = (/ 1, 1, 1, ts /)
        count_3d(1:4) = (/ IMAX, JMAX, KMAX, 1  /)
        start_2d(1:3) = (/ 1, 1, ts /)
        count_2d(1:3) = (/ IMAX, JMAX, 1  /)
        varname = "QLIQ"
        call check( nf90_inq_varid(ncid, trim(varname), varid) )
        call check( nf90_get_var( ncid, varid, qliq(is:ie,js:je,ks:ke), start=start_3d, count=count_3d ) )

        varname = "QHYD"
        call check( nf90_inq_varid(ncid, trim(varname), varid) )
        call check( nf90_get_var( ncid, varid, qhyd(is:ie,js:je,ks:ke), start=start_3d, count=count_3d ) )
     
        varname = "DENS"
        call check( nf90_inq_varid(ncid, trim(varname), varid) )
        call check( nf90_get_var( ncid, varid, dens(is:ie,js:je,ks:ke), start=start_3d, count=count_3d ) )

        varname = "T"
        call check( nf90_inq_varid(ncid, trim(varname), varid) )
        call check( nf90_get_var( ncid, varid, temperature(is:ie,js:je,ks:ke), start=start_3d, count=count_3d ) )

        varname = "ampsi_vapor"
        call check( nf90_inq_varid(ncid, trim(varname), varid) )
        call check( nf90_get_var( ncid, varid, ice_mass_vapor(is:ie,js:je,ks:ke), start=start_3d, count=count_3d ) )

        varname = "ampsi_eva"
        call check( nf90_inq_varid(ncid, trim(varname), varid) )
        call check( nf90_get_var( ncid, varid, ice_mass_eva(is:ie,js:je,ks:ke), start=start_3d, count=count_3d ) )

        varname = "ampsi_rim"
        call check( nf90_inq_varid(ncid, trim(varname), varid) )
        call check( nf90_get_var( ncid, varid, ice_mass_rim(is:ie,js:je,ks:ke), start=start_3d, count=count_3d ) )

        varname = "ampsl_rim"
        call check( nf90_inq_varid(ncid, trim(varname), varid) )
        call check( nf90_get_var( ncid, varid, liq_mass_rim(is:ie,js:je,ks:ke), start=start_3d, count=count_3d ) )

        varname = "ampsi_imm"
        call check( nf90_inq_varid(ncid, trim(varname), varid) )
        call check( nf90_get_var( ncid, varid, ice_mass_imm(is:ie,js:je,ks:ke), start=start_3d, count=count_3d ) )

        varname = "ampsci_imm"
        call check( nf90_inq_varid(ncid, trim(varname), varid) )
        call check( nf90_get_var( ncid, varid, ice_conc_imm(is:ie,js:je,ks:ke), start=start_3d, count=count_3d ) )

        varname = "PREC"
        call check( nf90_inq_varid(ncid, trim(varname), varid) )
        call check( nf90_get_var( ncid, varid, prec(is:ie,js:je), start=start_2d, count=count_2d ) )

        varname = "SFLX_LW_dn"
        call check( nf90_inq_varid(ncid, trim(varname), varid) )
        call check( nf90_get_var( ncid, varid, LWdn(is:ie,js:je), start=start_2d, count=count_2d ) )

        varname = "SFLX_SW_dn"
        call check( nf90_inq_varid(ncid, trim(varname), varid) )
        call check( nf90_get_var( ncid, varid, SWdn(is:ie,js:je), start=start_2d, count=count_2d ) )

        do i = 1, max_binr
           if (i < 10) then
              write(word,'(I1)') i
           else
              write(word,'(I2)') i
           endif
           varname = "con_lmass"//trim(word)
           call check( nf90_inq_varid(ncid, trim(varname), varid) )
           call check( nf90_get_var( ncid, varid, bin_liq_con(is:ie,js:je,ks:ke,i), start=start_3d, count=count_3d ) )
        enddo

        do i = 1, max_bini
           if (i < 10) then
              write(word,'(I1)') i
           else
              write(word,'(I2)') i
           endif
           varname = "con_imass"//trim(word)
           call check( nf90_inq_varid(ncid, trim(varname), varid) )
           call check( nf90_get_var( ncid, varid, bin_ice_con(is:ie,js:je,ks:ke,i), start=start_3d, count=count_3d ) )

           varname = "ice_imass"//trim(word)
           call check( nf90_inq_varid(ncid, trim(varname), varid) )
           call check( nf90_get_var( ncid, varid, bin_ice_mass(is:ie,js:je,ks:ke,i), start=start_3d, count=count_3d ) )

           varname = "cry_imass"//trim(word)
           call check( nf90_inq_varid(ncid, trim(varname), varid) )
           call check( nf90_get_var( ncid, varid, bin_ice_mcry(is:ie,js:je,ks:ke,i), start=start_3d, count=count_3d ) )

           varname = "rim_imass"//trim(word)
           call check( nf90_inq_varid(ncid, trim(varname), varid) )
           call check( nf90_get_var( ncid, varid, bin_ice_mrim(is:ie,js:je,ks:ke,i), start=start_3d, count=count_3d ) )

           varname = "agg_imass"//trim(word)
           call check( nf90_inq_varid(ncid, trim(varname), varid) )
           call check( nf90_get_var( ncid, varid, bin_ice_magg(is:ie,js:je,ks:ke,i), start=start_3d, count=count_3d ) )

           varname = "vol_imass"//trim(word)
           call check( nf90_inq_varid(ncid, trim(varname), varid) )
           call check( nf90_get_var( ncid, varid, bin_ice_Dmax(is:ie,js:je,ks:ke,i), start=start_3d, count=count_3d ) )

           varname = "a_axis"//trim(word)
           call check( nf90_inq_varid(ncid, trim(varname), varid) )
           call check( nf90_get_var( ncid, varid, bin_ice_alen(is:ie,js:je,ks:ke,i), start=start_3d, count=count_3d ) )

           varname = "c_axis"//trim(word)
           call check( nf90_inq_varid(ncid, trim(varname), varid) )
           call check( nf90_get_var( ncid, varid, bin_ice_clen(is:ie,js:je,ks:ke,i), start=start_3d, count=count_3d ) )

           varname = "d_axis"//trim(word)
           call check( nf90_inq_varid(ncid, trim(varname), varid) )
           call check( nf90_get_var( ncid, varid, bin_ice_dlen(is:ie,js:je,ks:ke,i), start=start_3d, count=count_3d ) )

           if (poly_exist) then
              varname = "ag_axis"//trim(word)
              call check( nf90_inq_varid(ncid, trim(varname), varid) )
              call check( nf90_get_var( ncid, varid, bin_ice_aglen(is:ie,js:je,ks:ke,i), start=start_3d, count=count_3d ) )

              varname = "ag_axis"//trim(word)
              call check( nf90_inq_varid(ncid, trim(varname), varid) )
              call check( nf90_get_var( ncid, varid, bin_ice_cglen(is:ie,js:je,ks:ke,i), start=start_3d, count=count_3d ) )

              varname = "ex_cry"//trim(word)
              call check( nf90_inq_varid(ncid, trim(varname), varid) )
              call check( nf90_get_var( ncid, varid, bin_ice_nex(is:ie,js:je,ks:ke,i), start=start_3d, count=count_3d ) )
           endif

        enddo

        !call check( nf90_inquire_variable(ncid, varid, name=check_NAME, ndims=check_NDIMS, dimids=check_dimids, xtype=check_type) )

        call check( nf90_close( ncid ) )

     enddo


     ! post process

     ! transform all density multiplied quantities back to normal form
     ! calculate qice
     do i = isg, ieg
        do j = jsg, jeg
           do k = ks, ke
              qice(i,j,k) = max(0.0E0,qhyd(i,j,k) - qliq(i,j,k))
              do ibin = 1, max_binr
                 bin_liq_con(i,j,k,ibin) = bin_liq_con(i,j,k,ibin)*dens(i,j,k)
              enddo
              do ibin = 1, max_bini
                 bin_ice_con(i,j,k,ibin) = bin_ice_con(i,j,k,ibin)*dens(i,j,k)
              enddo
              is_below_cloud(i,j,k) = 0
           enddo
        enddo
     enddo


     belowcloud_counter = 0.0E0
     do i = isg, ieg
        do j = jsg, jeg
           do k = ks, ke
              if (qliq(i,j,k) > qliq_lower_limit) then
                 exit
              else
                 is_below_cloud(i,j,k) = 1
                 belowcloud_counter = belowcloud_counter + 1.0E0
              endif
           enddo
        enddo
     enddo


     ! retrieve average and median
     temp1 = 0.0E0 ! lwp
     temp2 = 0.0E0 ! iwp
     temp3 = 0.0E0 ! dliq (liquid grid)
     temp4 = 0.0E0 ! dice (ice grid)
     temp5 = 0.0E0 ! prec
     temp6 = 0.0E0 ! LW
     temp7 = 0.0E0 ! SW
     temp8 = 0.0E0 ! rim
     temp9 = 0.0E0 ! imm
     temp9c = 0.0E0 ! imm conc
     temp10 = 0.0E0 ! vap
     temp11 = 0.0E0 ! ctopT
     temp12 = 0.0E0 ! vapALL
     temp13 = 0.0E0 ! vapBAS
     temp14 = 0.0E0 ! mcry
     temp15 = 0.0E0 ! magg
     temp16 = 0.0E0 ! mrim
     temp17 = 0.0E0 ! mean nice (ice grid)
     temp18 = 0.0E0 ! mean nliq (liquid grid)
     temp19 = 0.0E0 ! mean nice (liquid grid)
     temp20 = 0.0E0 ! RMrimliq
     temp21 = 0.0E0 ! incloud mcry
     temp22 = 0.0E0 ! incloud magg
     temp23 = 0.0E0 ! incloud mrim
     temp24 = 0.0E0 ! dice (cloud grid)
     incloud_counter = 0.0E0
     topcloud_counter = 0.0E0
     inice_counter = 0.0E0
     local_var_nliq(:) = 0.0E0
     local_var_nice(:) = 0.0E0
     local_var_nice200(:) = 0.0E0
     local_var_nice200belowcloud(:) = 0.0E0
     counter1 = 1
     test_incloud = 0
     do k = ks, ke
        sum_lwp = 0.0E0
        sum_iwp = 0.0E0
        do i = isg, ieg
           do j = jsg, jeg
              sum_lwp = sum_lwp + qliq(i,j,k)*1000.0*dens(i,j,k)
              sum_iwp = sum_iwp + qice(i,j,k)*1000.0*dens(i,j,k)
              temp12 = temp12 + (ice_mass_vapor(i,j,k) + ice_mass_eva(i,j,k))*(fz(k+1)-fz(k))*DX*DY
              if (k == ks) then
                 temp5 = temp5 + prec(i,j)*3600.0E0*24.0E0
                 temp6 = temp6 + LWdn(i,j)
                 temp7 = temp7 + SWdn(i,j)
              endif

              if (qice(i,j,k) > qice_lower_limit) then
                 inice_counter = inice_counter + 1.0E0
                 
                 D3 = 0.0
                 D2 = 0.0
                 do ibin = 1, max_bini
                    temp17 = temp17 + bin_ice_con(i,j,k,ibin)*1000.0

                    temp14 = temp14 + bin_ice_mcry(i,j,k,ibin)*dens(i,j,k)*(fz(k+1)-fz(k))*DX*DY
                    temp15 = temp15 + bin_ice_magg(i,j,k,ibin)*dens(i,j,k)*(fz(k+1)-fz(k))*DX*DY
                    temp16 = temp16 + bin_ice_mrim(i,j,k,ibin)*dens(i,j,k)*(fz(k+1)-fz(k))*DX*DY

                    if (bin_ice_con(i,j,k,ibin) > 1.e-11 .and. bin_ice_mcry(i,j,k,ibin) > 0.0E0 .and. bin_ice_alen(i,j,k,ibin) > 0.0E0 .and. bin_ice_clen(i,j,k,ibin) > 0.0E0) then ! 10e-8 L-1 limit
                       ! approximated maximum dimension for ice grids
                       !local_Dmax = (bin_ice_Dmax(i,j,k,ibin)*dens(i,j,k)/bin_ice_con(i,j,k,ibin)*6.0/pi)**(1.0/3.0)*10000.0 ! cm -> um

                       ! rescale axis lengths
                       bin_ice_alen(i,j,k,ibin) = (bin_ice_alen(i,j,k,ibin)*dens(i,j,k)/bin_ice_con(i,j,k,ibin))**(1.0E0/3.0E0)
                       bin_ice_clen(i,j,k,ibin) = (bin_ice_clen(i,j,k,ibin)*dens(i,j,k)/bin_ice_con(i,j,k,ibin))**(1.0E0/3.0E0)
                       bin_ice_dlen(i,j,k,ibin) = min(0.9E0*bin_ice_alen(i,j,k,ibin), (bin_ice_dlen(i,j,k,ibin)*dens(i,j,k)/bin_ice_con(i,j,k,ibin))**(1.0E0/3.0E0))
                       if (poly_exist) then
                          if (iceaxis_version == 3) then
                             bin_ice_aglen(i,j,k,ibin) = (bin_ice_aglen(i,j,k,ibin)*dens(i,j,k)/bin_ice_con(i,j,k,ibin))**(1.0/3.0)*bin_ice_alen(i,j,k,ibin)
                             bin_ice_cglen(i,j,k,ibin) = (bin_ice_cglen(i,j,k,ibin)*dens(i,j,k)/bin_ice_con(i,j,k,ibin))**(1.0/3.0)*bin_ice_clen(i,j,k,ibin)
                             bin_ice_nex(i,j,k,ibin) = bin_ice_nex(i,j,k,ibin)*dens(i,j,k)/bin_ice_con(i,j,k,ibin)
                             !write(*,'(2ES15.6)') bin_ice_nex(i,j,k,ibin), bin_ice_con(i,j,k,ibin)
                          endif
                       endif

                       ! diagnose ice type
                       call Iice_habit(ice_type,bin_ice_mass(i,j,k,ibin),bin_ice_mcry(i,j,k,ibin),bin_ice_magg(i,j,k,ibin),bin_ice_mrim(i,j,k,ibin))

                       ! diagnose ice habit
                       if (poly_exist) then
                          call Iice_shapePoly(my_habit,bin_ice_alen(i,j,k,ibin),bin_ice_clen(i,j,k,ibin),bin_ice_dlen(i,j,k,ibin),bin_ice_aglen(i,j,k,ibin),bin_ice_cglen(i,j,k,ibin),bin_ice_nex(i,j,k,ibin))
                       else
                          call Iice_shape(my_habit,bin_ice_alen(i,j,k,ibin),bin_ice_clen(i,j,k,ibin),bin_ice_dlen(i,j,k,ibin))
                       endif
                       if (my_habit == -1) then
                          print *, "ERROR HABIT", irank, bin_ice_alen(i,j,k,ibin),bin_ice_clen(i,j,k,ibin),bin_ice_dlen(i,j,k,ibin),bin_ice_aglen(i,j,k,ibin),bin_ice_cglen(i,j,k,ibin),bin_ice_nex(i,j,k,ibin)
                          call PRC_abort
                       endif

                       ! calculate elen and rlen
                       call convert_habit(habit,my_habit)
                       if (poly_exist) then
                          call get_elen_rlen(pla_e,ros_r,habit,bin_ice_mcry(i,j,k,ibin)*dens(i,j,k)/bin_ice_con(i,j,k,ibin)/1000.0E0,bin_ice_alen(i,j,k,ibin),bin_ice_clen(i,j,k,ibin),bin_ice_aglen(i,j,k,ibin),bin_ice_cglen(i,j,k,ibin))
                       else
                          pla_e = 0.0E0
                          ros_r = 0.0E0
                       endif

                       den_i = 0.91668E0
                       vcs = bin_ice_Dmax(i,j,k,ibin)*dens(i,j,k)/bin_ice_con(i,j,k,ibin)
                       if (poly_exist) then
                          if (my_habit <= 3) then
                             vic = 4.0E0/3.0E0*pi*(bin_ice_alen(i,j,k,ibin)**2 + bin_ice_clen(i,j,k,ibin)**2)**1.5
                             if (vic == 0.0E0) then
                                write(*,'(A,2I3, 7ES15.6)') "inside volume error: ", my_habit, habit, vcs, vic, pla_e, ros_r, bin_ice_alen(i,j,k,ibin), bin_ice_clen(i,j,k,ibin), pi
                                call PRC_abort
                             endif
                          else
                             if (my_habit == 4) then
                                vic = 4.0E0/3.0E0*pi*pla_e*pla_e*pla_e*(1.0E0 + spx_p**2)**1.5E0
                             else
                                vic = 4.0E0/3.0E0*pi*ros_r*ros_r*ros_r*(1.0E0 + spx_p**2)**1.5E0
                             endif
                          endif
                       else
                          vic = 4.0E0/3.0E0*pi*(bin_ice_alen(i,j,k,ibin)**2 + bin_ice_clen(i,j,k,ibin)**2)**1.5E0
                       endif
                       if (vic < bin_ice_mcry(i,j,k,ibin)*dens(i,j,k)/bin_ice_con(i,j,k,ibin)/1000.0E0/den_i) then
                          vic = bin_ice_mcry(i,j,k,ibin)*dens(i,j,k)/bin_ice_con(i,j,k,ibin)/1000.0E0/den_i
                       endif
                       if (vic == 0.0E0) then
                          write(*,'(A,2I3, 7ES15.6)') "volume error: ", my_habit, habit, vcs, vic, pla_e, ros_r, bin_ice_alen(i,j,k,ibin), bin_ice_clen(i,j,k,ibin), bin_ice_mcry(i,j,k,ibin)
                          call PRC_abort
                       endif

                       if (vcs < vic) then
                          vcs = vic
                       else if (vcs > 4188.79020478639E0) then
                          vcs = 4188.79020478639E0
                       endif
                       ! to prevent unphysical volume and axis lengths, data of gamma1 from 10000
                       if (prevent_unphysical_length == .true.) then
                          if (bin_ice_mrim(i,j,k,ibin) < bin_ice_mcry(i,j,k,ibin) .and. &
                              bin_ice_magg(i,j,k,ibin) < bin_ice_mcry(i,j,k,ibin)) then
                             if (vcs > 10.0E0*vic) then
                                vcs = vic
                                write(*,*) "BUG"
                                call PRC_abort
                                !cycle
                             endif
                             if (vcs > 10.0E0*bin_ice_mass(i,j,k,ibin)*dens(i,j,k)/bin_ice_con(i,j,k,ibin)/1000.0E0/den_i) then
                                vcs = bin_ice_mass(i,j,k,ibin)*dens(i,j,k)/bin_ice_con(i,j,k,ibin)/1000.0E0/den_i
                                vic = vcs
                                bin_ice_alen(i,j,k,ibin) = (vic*3.0E0/4.0E0/pi)**(1.0E0/3.0E0)
                                bin_ice_clen(i,j,k,ibin) = (vic*3.0E0/4.0E0/pi)**(1.0E0/3.0E0)
                                bin_ice_dlen(i,j,k,ibin) = max(bin_ice_dlen(i,j,k,ibin),0.9E0*bin_ice_alen(i,j,k,ibin))
                                write(*,*) "BUG"
                                call PRC_abort
                                !cycle
                             endif
                          else
                             if (vcs > 2.0E0*bin_ice_mrim(i,j,k,ibin)/bin_ice_mcry(i,j,k,ibin)*vic .or. vcs > 2.0E0*bin_ice_mrim(i,j,k,ibin)/bin_ice_mcry(i,j,k,ibin)*bin_ice_mass(i,j,k,ibin)*dens(i,j,k)/bin_ice_con(i,j,k,ibin)/1000.0E0/den_i) then !
                                write(*,*) "BUG"
                                call PRC_abort
                                !cycle
                             endif
                          endif
                       endif

                       ! calculate alpha_ic
                       if (poly_exist) then
                          if (my_habit <= 3) then
                             if (bin_ice_alen(i,j,k,ibin) > 1.e-15 .and. bin_ice_clen(i,j,k,ibin) > 1.e-15) then
                                alpha_ic = bin_ice_clen(i,j,k,ibin)/bin_ice_alen(i,j,k,ibin)
                             else
                                alpha_ic = 1.0E0
                             endif
                          else
                             alpha_ic = spx_p
                          endif
                       else
                          if (bin_ice_alen(i,j,k,ibin) > 1.e-15 .and. bin_ice_clen(i,j,k,ibin) > 1.e-15) then
                             alpha_ic = bin_ice_clen(i,j,k,ibin)/bin_ice_alen(i,j,k,ibin)
                          else
                             alpha_ic = 1.0E0
                          endif
                       endif
                       
                       ! calculate alpha_im
                       if (ice_type == 1) then
                          kp = log10(1.0E0/3.0E0/alpha_ic)
                          mass_agg_cry = bin_ice_mcry(i,j,k,ibin) + bin_ice_magg(i,j,k,ibin)
                          mlmt = 10.0E0*bin_ice_mcry(i,j,k,ibin)
                          if (mass_agg_cry < mlmt) then
                             alpha_im = 1.0E0/3.0E0*(mass_agg_cry/mlmt)**kp
                          else
                             alpha_im = 1.0E0/3.0E0
                          endif
                       else if (ice_type == 3 .or. ice_type == 4) then
                          kp = log10(0.8E0/alpha_ic)
                          mass_agg_cry_rim = bin_ice_mcry(i,j,k,ibin) + bin_ice_magg(i,j,k,ibin) + bin_ice_mrim(i,j,k,ibin)
                          mlmt = 10.0E0*(bin_ice_mcry(i,j,k,ibin) + bin_ice_magg(i,j,k,ibin))
                          if (mass_agg_cry_rim < mlmt) then
                             alpha_im = 0.8E0*(mass_agg_cry_rim/mlmt)**kp
                          else
                             alpha_im = 0.8E0
                          endif
                       else if (ice_type == 5) then
                          if (alpha_ic > 1.0E0) then
                             alpha_im = 1.0E0/sqrt((1.0E0 + 1.0E0/alpha_ic**2)*(vcs/vic)**(2.0E0/3.0E0) - 1.0E0)
                             if (alpha_im < 1.0E0) then
                                alpha_im = 1.0E0
                             endif
                          else
                             alpha_im = sqrt((1.0E0 + alpha_ic**2)*(vcs/vic)**(2.0E0/3.0E0) - 1.0E0)
                             if (alpha_im > 1.0E0) then
                                alpha_im = 1.0E0
                             endif
                          endif
                       else
                          alpha_im = alpha_ic
                       endif

                       ! calculate maximum dimension
                       if (ice_type == 4) then
                          if (alpha_im > 1.0E0) then
                             csm = ((vcs/(4.0E0/3.0E0*pi))**(1.0E0/3.0E0))
                             asm = csm/alpha_im
                             local_Dmax = 2.0E0*csm*10000.0E0
                          else
                             asm = ((vcs/(4.0E0/3.0E0*pi))**(1.0E0/3.0E0))
                             csm = asm*alpha_im
                             local_Dmax = 2.0E0*asm*10000.0E0
                          endif
                       else
                          asm = ((vcs/(4.0E0/3.0E0*pi))**(1.0/3.0))/sqrt(1.0E0 + alpha_im**2)
                          csm = asm*alpha_im
                          local_Dmax = 2.0E0*asm*sqrt(1.0E0 + alpha_im**2)*10000.0E0
                       endif
                       if (abs(asm) > 5.e5 .or. abs(csm) > 5.e5) then
                          print *, "ASM CSM error", irank, asm, csm, alpha_im
                          local_Dmax = 0.0D0
                          !call PRC_abort
                       endif
                       if (.false. .and. local_Dmax > 1500.0E0 .and. ibin < 14) then
                          write(*,'(4I7,6ES15.5,A,3ES15.5)') i, j, k, ibin, &
                               local_Dmax, &
                               bin_ice_mrim(i,j,k,ibin)/bin_ice_mcry(i,j,k,ibin), &
                               bin_ice_magg(i,j,k,ibin)/bin_ice_mcry(i,j,k,ibin), &
                               bin_ice_con(i,j,k,ibin), &
                               vcs, &
                               vic, &
                               "  |  ", &
                               bin_ice_alen(i,j,k,ibin)*10000.0E0, &
                               bin_ice_clen(i,j,k,ibin)*10000.0E0, &
                               bin_ice_mcry(i,j,k,ibin)*dens(i,j,k)/bin_ice_con(i,j,k,ibin)/1000.0E0/den_i
                       endif

                       !bin_ice_aDmax(i,j,k,ibin) = local_Dmax
                       !DI3 = DI3 + bin_ice_con(i,j,k,ibin)*local_Dmax**3
                       !DI2 = DI2 + bin_ice_con(i,j,k,ibin)*local_Dmax**2
                    else
                       local_Dmax = 0.0D0
                    endif
                    D2 = D2 + bin_ice_con(i,j,k,ibin)*local_Dmax**2
                    D3 = D3 + bin_ice_con(i,j,k,ibin)*local_Dmax**3

                    if (is_below_cloud(i,j,k) == 1 .and. local_Dmax >= 200.0E0) then
                       local_var_nice200belowcloud(counter1) = local_var_nice200belowcloud(counter1) + bin_ice_con(i,j,k,ibin)*1000.0E0
                    endif
                 enddo
                 if (D2 /= 0.0D0) temp4 = temp4 + D3/D2
                 if (D2 /= 0.0D0 .and. qliq(i,j,k) > qliq_lower_limit) then
                    temp24 = temp24 + D3/D2
                 endif

              else
                 
                 local_var_nice200belowcloud(counter1) = -10.0E0

              endif

              if (is_below_cloud(i,j,k) == 0) then
                 local_var_nice200belowcloud(counter1) = -10.0E0
              endif

              if (qliq(i,j,k) > qliq_lower_limit) then
                 incloud_counter = incloud_counter + 1.0E0

                 if (test_incloud == 0) then
                    test_incloud = 1
                 endif
                 
                 !local_var_1d(counter1) = ni(i,j,k) + ns(i,j,k) + ng(i,j,k)
                 temp20 = temp20 + liq_mass_rim(i,j,k)*(fz(k+1)-fz(k))*DX*DY
                 temp8 = temp8 + ice_mass_rim(i,j,k)*(fz(k+1)-fz(k))*DX*DY
                 temp9 = temp9 + ice_mass_imm(i,j,k)*(fz(k+1)-fz(k))*DX*DY
                 temp9c = temp9c + ice_conc_imm(i,j,k)*(fz(k+1)-fz(k))*DX*DY
                 temp10 = temp10 + (ice_mass_vapor(i,j,k) + ice_mass_eva(i,j,k))*(fz(k+1)-fz(k))*DX*DY

                 D2 = 0.0
                 D3 = 0.0
                 do ibin = 1, max_binr
                    local_var_nliq(counter1) = local_var_nliq(counter1) + bin_liq_con(i,j,k,ibin)

                    D2 = D2 + bin_liq_con(i,j,k,ibin)*diameterbin(ibin)**2
                    D3 = D3 + bin_liq_con(i,j,k,ibin)*diameterbin(ibin)**3
                    temp18 = temp18 + bin_liq_con(i,j,k,ibin)
                    
                    temp21 = temp21 + bin_ice_mcry(i,j,k,ibin)*dens(i,j,k)*(fz(k+1)-fz(k))*DX*DY
                    temp22 = temp22 + bin_ice_magg(i,j,k,ibin)*dens(i,j,k)*(fz(k+1)-fz(k))*DX*DY
                    temp23 = temp23 + bin_ice_mrim(i,j,k,ibin)*dens(i,j,k)*(fz(k+1)-fz(k))*DX*DY
                 enddo
                 if (D2 /= 0.0) temp3 = temp3 + D3/D2

                 do ibin = 1, max_bini
                    local_var_nice(counter1) = local_var_nice(counter1) + bin_ice_con(i,j,k,ibin)*1000.0
                    temp19 = temp19 + bin_ice_con(i,j,k,ibin)*1000.0
                 enddo

                 do ibin = 1, max_bini
                    if (bin_ice_con(i,j,k,ibin) > 1.e-11 .and. bin_ice_mcry(i,j,k,ibin) > 0.0E0 .and. bin_ice_alen(i,j,k,ibin) > 0.0E0 .and. bin_ice_clen(i,j,k,ibin) > 0.0E0) then ! 10e-8 L-1 limit
                       ! rescale axis lengths
                       bin_ice_alen(i,j,k,ibin) = (bin_ice_alen(i,j,k,ibin)*dens(i,j,k)/bin_ice_con(i,j,k,ibin))**(1.0/3.0)
                       bin_ice_clen(i,j,k,ibin) = (bin_ice_clen(i,j,k,ibin)*dens(i,j,k)/bin_ice_con(i,j,k,ibin))**(1.0/3.0)
                       !bin_ice_dlen(i,j,k,ibin) = (bin_ice_dlen(i,j,k,ibin)*dens(i,j,k)/bin_ice_con(i,j,k,ibin))**(1.0/3.0)
                       bin_ice_dlen(i,j,k,ibin) = min(0.9E0*bin_ice_alen(i,j,k,ibin), (bin_ice_dlen(i,j,k,ibin)*dens(i,j,k)/bin_ice_con(i,j,k,ibin))**(1.0/3.0))
                       if (poly_exist) then
                          if (iceaxis_version == 3) then
                             bin_ice_aglen(i,j,k,ibin) = (bin_ice_aglen(i,j,k,ibin)*dens(i,j,k)/bin_ice_con(i,j,k,ibin))**(1.0/3.0)*bin_ice_alen(i,j,k,ibin)
                             bin_ice_cglen(i,j,k,ibin) = (bin_ice_cglen(i,j,k,ibin)*dens(i,j,k)/bin_ice_con(i,j,k,ibin))**(1.0/3.0)*bin_ice_clen(i,j,k,ibin)
                             bin_ice_nex(i,j,k,ibin) = bin_ice_nex(i,j,k,ibin)*dens(i,j,k)/bin_ice_con(i,j,k,ibin)
                          endif
                       endif

                       ! diagnose ice type
                       call Iice_habit(ice_type,bin_ice_mass(i,j,k,ibin),bin_ice_mcry(i,j,k,ibin),bin_ice_magg(i,j,k,ibin),bin_ice_mrim(i,j,k,ibin))

                       ! diagnose ice habit
                       if (poly_exist) then
                          call Iice_shapePoly(my_habit,bin_ice_alen(i,j,k,ibin),bin_ice_clen(i,j,k,ibin),bin_ice_dlen(i,j,k,ibin),bin_ice_aglen(i,j,k,ibin),bin_ice_cglen(i,j,k,ibin),bin_ice_nex(i,j,k,ibin))
                       else
                          call Iice_shape(my_habit,bin_ice_alen(i,j,k,ibin),bin_ice_clen(i,j,k,ibin),bin_ice_dlen(i,j,k,ibin))
                       endif
                       if (my_habit == -1) then
                          print *, "ERROR HABIT", irank, bin_ice_alen(i,j,k,ibin),bin_ice_clen(i,j,k,ibin),bin_ice_dlen(i,j,k,ibin),bin_ice_aglen(i,j,k,ibin),bin_ice_cglen(i,j,k,ibin),bin_ice_nex(i,j,k,ibin)
                          call PRC_abort
                       endif

                       ! calculate elen and rlen
                       call convert_habit(habit,my_habit)
                       if (poly_exist) then
                          call get_elen_rlen(pla_e,ros_r,habit,bin_ice_mcry(i,j,k,ibin)*dens(i,j,k)/bin_ice_con(i,j,k,ibin)/1000.0E0,bin_ice_alen(i,j,k,ibin),bin_ice_clen(i,j,k,ibin),bin_ice_aglen(i,j,k,ibin),bin_ice_cglen(i,j,k,ibin))
                       else
                          pla_e = 0.0E0
                          ros_r = 0.0E0
                       endif

                          
                       ! calculate vcs and vic
                       den_i = 0.91668E0
                       vcs = bin_ice_Dmax(i,j,k,ibin)*dens(i,j,k)/bin_ice_con(i,j,k,ibin)
                       if (poly_exist) then
                          if (my_habit <= 3) then
                             vic = 4.0E0/3.0E0*pi*(bin_ice_alen(i,j,k,ibin)**2 + bin_ice_clen(i,j,k,ibin)**2)**1.5E0
                          else
                             if (my_habit == 4) then
                                vic = 4.0E0/3.0E0*pi*pla_e*pla_e*pla_e*(1.0E0 + spx_p**2)**1.5E0
                             else
                                vic = 4.0E0/3.0E0*pi*ros_r*ros_r*ros_r*(1.0E0 + spx_p**2)**1.5E0
                             endif
                          endif
                       else
                          vic = 4.0E0/3.0E0*pi*(bin_ice_alen(i,j,k,ibin)**2 + bin_ice_clen(i,j,k,ibin)**2)**1.5E0
                       endif
                       if (vic < bin_ice_mcry(i,j,k,ibin)*dens(i,j,k)/bin_ice_con(i,j,k,ibin)/1000.0E0/den_i) then
                          vic = bin_ice_mcry(i,j,k,ibin)*dens(i,j,k)/bin_ice_con(i,j,k,ibin)/1000.0E0/den_i
                       endif
                       if (vic == 0.0E0) then
                          write(*,'(A,I3, 6ES15.6)') "volume error: ", habit, vcs, vic, pla_e, ros_r, bin_ice_alen(i,j,k,ibin), bin_ice_clen(i,j,k,ibin)
                          call PRC_abort
                       endif

                       if (vcs < vic) then
                          vcs = vic
                       else if (vcs > 4188.79020478639E0) then
                          vcs = 4188.79020478639E0
                       endif

                       ! calculate alpha_ic
                       if (poly_exist) then
                          if (my_habit <= 3) then
                             if (bin_ice_alen(i,j,k,ibin) > 1.e-15 .and. bin_ice_clen(i,j,k,ibin) > 1.e-15) then
                                alpha_ic = bin_ice_clen(i,j,k,ibin)/bin_ice_alen(i,j,k,ibin)
                             else
                                alpha_ic = 1.0E0
                             endif
                          else
                             alpha_ic = spx_p
                          endif
                       else
                          if (bin_ice_alen(i,j,k,ibin) > 1.e-15 .and. bin_ice_clen(i,j,k,ibin) > 1.e-15) then
                             alpha_ic = bin_ice_clen(i,j,k,ibin)/bin_ice_alen(i,j,k,ibin)
                          else
                             alpha_ic = 1.0E0
                          endif
                       endif
                       
                       ! calculate alpha_im
                       if (ice_type == 1) then
                          kp = log10(1.0E0/3.0E0/alpha_ic)
                          mass_agg_cry = bin_ice_mcry(i,j,k,ibin) + bin_ice_magg(i,j,k,ibin)
                          mlmt = 10.0E0*bin_ice_mcry(i,j,k,ibin)
                          if (mass_agg_cry < mlmt) then
                             alpha_im = 1.0E0/3.0E0*(mass_agg_cry/mlmt)**kp
                          else
                             alpha_im = 1.0E0/3.0E0
                          endif
                       else if (ice_type == 3 .or. ice_type == 4) then
                          kp = log10(0.8E0/alpha_ic)
                          mass_agg_cry_rim = bin_ice_mcry(i,j,k,ibin) + bin_ice_magg(i,j,k,ibin) + bin_ice_mrim(i,j,k,ibin)
                          mlmt = 10.0E0*(bin_ice_mcry(i,j,k,ibin) + bin_ice_magg(i,j,k,ibin))
                          if (mass_agg_cry_rim < mlmt) then
                             alpha_im = 0.8E0*(mass_agg_cry_rim/mlmt)**kp
                          else
                             alpha_im = 0.8E0
                          endif
                       else if (ice_type == 5) then
                          if (alpha_ic > 1.0E0) then
                             alpha_im = 1.0E0/sqrt((1.0E0 + 1.0E0/alpha_ic**2)*(vcs/vic)**(2.0E0/3.0E0) - 1.0E0)
                             if (alpha_im < 1.0E0) then
                                alpha_im = 1.0E0
                             endif
                          else
                             alpha_im = sqrt((1.0E0 + alpha_ic**2)*(vcs/vic)**(2.0E0/3.0E0) - 1.0E0)
                             if (alpha_im > 1.0E0) then
                                alpha_im = 1.0E0
                             endif
                          endif
                       else
                          alpha_im = alpha_ic
                       endif

                       ! calculate maximum dimension
                       if (ice_type == 4) then
                          if (alpha_im > 1.0E0) then
                             csm = ((vcs/(4.0E0/3.0E0*pi))**(1.0E0/3.0E0))
                             asm = csm/alpha_im
                             local_Dmax = 2.0E0*csm*10000.0E0
                          else
                             asm = ((vcs/(4.0E0/3.0E0*pi))**(1.0E0/3.0E0))
                             csm = asm*alpha_im
                             local_Dmax = 2.0E0*asm*10000.0E0
                          endif
                       else
                          asm = ((vcs/(4.0E0/3.0E0*pi))**(1.0E0/3.0E0))/sqrt(1.0E0 + alpha_im**2)
                          csm = asm*alpha_im
                          local_Dmax = 2.0E0*asm*sqrt(1.0 + alpha_im**2)*10000.0E0
                       endif
                       if (abs(asm) > 1.e10 .or. abs(csm) > 1.e10) then
                          print *, "ASM CSM error", irank, asm, csm, alpha_im
                          call PRC_abort
                       endif

                       if (local_Dmax >= 200.0E0) then
                          local_var_nice200(counter1) = local_var_nice200(counter1) + bin_ice_con(i,j,k,ibin)*1000.0E0
                       endif
                    endif
                 enddo
              else
                 local_var_nliq(counter1) = -10.0E0
                 local_var_nice(counter1) = -10.0E0
                 local_var_nice200(counter1) = -10.0E0
              endif

              ! cloud base vapor deposition rate
              if (is_below_cloud(i,j,k) == 1) then
                 temp13 = temp13 + (ice_mass_vapor(i,j,k) + ice_mass_eva(i,j,k))*(fz(k+1)-fz(k))*DX*DY
              endif
              
              ! cloud top temperature
              if (test_incloud == 1) then
                 temp11 = temp11 + temperature(i,j,k)
                 topcloud_counter = topcloud_counter + 1.0E0
              endif

              counter1 = counter1 + 1
           enddo
        enddo

        !if (irank == master) then
        !   write(*,'(2E15.6)') cz(k), temp/real(IMAX)/real(JMAX)
        !endif
        temp1 = temp1 + sum_lwp*(fz(k+1)-fz(k))
        temp2 = temp2 + sum_iwp*(fz(k+1)-fz(k))

        !if (k == ks) then
        !   if (irank == master) then
        !      write(*,'(2E15.6)') temp6/real(IMAX)/real(JMAX)/real(JOBS_PER_MPI), &
        !           temp7/real(IMAX)/real(JMAX)/real(JOBS_PER_MPI)
        !   endif
        !endif
        
     enddo
     
     call MPI_REDUCE( temp1, global_sum_lwp, 1, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( temp2, global_sum_iwp, 1, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( temp3, global_sum_meandliqincloud, 1, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( temp4, global_sum_meandiceinice, 1, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( temp5, global_sum_prec, 1, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( temp6, global_sum_lwdn, 1, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( temp7, global_sum_swdn, 1, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( temp8, global_sum_rimincloud, 1, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( temp9, global_sum_immincloud, 1, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( temp9c, global_sum_immcincloud, 1, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( temp10, global_sum_netvaporincloud, 1, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( temp11, global_sum_cloudtopT, 1, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( temp12, global_sum_netvapor, 1, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( temp13, global_sum_netvaporbelowcloud, 1, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( temp14, global_sum_mcry, 1, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( temp15, global_sum_magg, 1, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( temp16, global_sum_mrim, 1, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( temp17, global_sum_meanniceinice, 1, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( temp18, global_sum_meannliqincloud, 1, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( temp19, global_sum_meanniceincloud, 1, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( temp20, global_sum_rimliqincloud, 1, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( temp21, global_sum_mcryincloud, 1, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( temp22, global_sum_maggincloud, 1, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( temp23, global_sum_mrimincloud, 1, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( temp24, global_sum_meandiceincloud, 1, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( incloud_counter, global_sum_incloudcounter, 1, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( topcloud_counter, global_sum_topcloudcounter, 1, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( belowcloud_counter, global_sum_belowcloudcounter, 1, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( inice_counter, global_sum_inicecounter, 1, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )


     !write(*,'(A,I3,E15.6)') "LWP: ", irank, temp1

     call MPI_GATHER(local_var_nliq, sendcount_median, MPI_REAL4, global_var_nliq, sendcount_median, MPI_REAL4, master, MPI_COMM_WORLD, ierr )
     call MPI_GATHER(local_var_nice, sendcount_median, MPI_REAL4, global_var_nice, sendcount_median, MPI_REAL4, master, MPI_COMM_WORLD, ierr )
     call MPI_GATHER(local_var_nice200, sendcount_median, MPI_REAL4, global_var_nice200, sendcount_median, MPI_REAL4, master, MPI_COMM_WORLD, ierr )

     call MPI_GATHER(local_var_nice200belowcloud, sendcount_median, MPI_REAL4, global_var_nice200belowcloud, sendcount_median, MPI_REAL4, master, MPI_COMM_WORLD, ierr )

     if (irank == master) then
        median_array_size1 = 0
        median_array_size2 = 0
        median_array_size3 = 0
        median_array_size4 = 0
        do i = 1, count_global_median
           if (global_var_nliq(i) >= 0.0) then
              !if (t == 1 .and. global_var_nliq(i) > 0.0) then
              !   print *, i, "  global  ", global_var_nliq(i)
              !endif
              median_array_size1 = median_array_size1 + 1
           endif
           if (global_var_nice(i) >= 0.0) then
              median_array_size2 = median_array_size2 + 1
           endif
           if (global_var_nice200(i) >= 0.0) then
              median_array_size3 = median_array_size3 + 1
           endif
           if (global_var_nice200belowcloud(i) >= 0.0) then
              median_array_size4 = median_array_size4 + 1
           endif
        enddo
        
        allocate(median_array1(median_array_size1))
        allocate(median_array2(median_array_size2))
        allocate(median_array3(median_array_size3))
        allocate(median_array4(median_array_size4))

        counter1 = 0
        counter2 = 0
        counter3 = 0
        counter4 = 0
        do i = 1, count_global_median
           if (global_var_nliq(i) >= 0.0) then
              counter1 = counter1 + 1
              median_array1(counter1) = global_var_nliq(i)
           endif
           if (global_var_nice(i) >= 0.0) then
              counter2 = counter2 + 1
              median_array2(counter2) = global_var_nice(i)
           endif
           if (global_var_nice200(i) >= 0.0) then
              counter3 = counter3 + 1
              median_array3(counter3) = global_var_nice200(i)
           endif
           if (global_var_nice200belowcloud(i) >= 0.0) then
              counter4 = counter4 + 1
              median_array4(counter4) = global_var_nice200belowcloud(i)
           endif
        enddo

        write(*,*) "array1: ", median_array_size1
        call quickSorti(median_array1,median_array_size1,1,median_array_size1)
        write(*,*) "array2: ", median_array_size2
        call quickSorti(median_array2,median_array_size2,1,median_array_size2)
        write(*,*) "array3: ", median_array_size3
        call quickSorti(median_array3,median_array_size3,1,median_array_size3)
        write(*,*) "array4: ", median_array_size4
        call quickSorti(median_array4,median_array_size4,1,median_array_size4)


        LWP(t) = global_sum_lwp/real(IMAX)/real(IMAX)/real(TOTAL_PRC)
        IWP(t) = global_sum_iwp/real(IMAX)/real(IMAX)/real(TOTAL_PRC)
        COUNTER_INCLOUD(t) = global_sum_incloudcounter
        COUNTER_INICE(t) = global_sum_inicecounter
        COUNTER_BELOWCLOUD(t) = global_sum_belowcloudcounter
        if (global_sum_incloudcounter /= 0.0E0) then
           MEAN_NLIQ_INCLOUD(t) = global_sum_meannliqincloud/global_sum_incloudcounter
        endif
        if (global_sum_inicecounter /= 0.0E0) then
           MEAN_NICE_INICE(t) = global_sum_meanniceinice/global_sum_inicecounter
        endif
        if (global_sum_incloudcounter /= 0.0E0) then
           MEAN_NICE_INCLOUD(t) = global_sum_meanniceincloud/global_sum_incloudcounter
           MEAN_DLIQ_INCLOUD(t) = global_sum_meandliqincloud/global_sum_incloudcounter
           MEAN_DICE_INCLOUD(t) = global_sum_meandiceincloud/global_sum_incloudcounter
        endif
        if (global_sum_inicecounter /= 0.0E0) then
           MEAN_DICE_INICE(t) = global_sum_meandiceinice/global_sum_inicecounter
        endif
        RMRIMLIQ_INCLOUD(t) = global_sum_rimliqincloud
        RMRIM_INCLOUD(t) = global_sum_rimincloud
        RMIMM_INCLOUD(t) = global_sum_immincloud
        RMIMMC_INCLOUD(t) = global_sum_immcincloud
        RMVAP_INCLOUD(t) = global_sum_netvaporincloud
        if (global_sum_topcloudcounter /= 0.0E0) then
           CTOPT(t) = global_sum_cloudtopT/global_sum_topcloudcounter
        endif
        RMVAP(t) = global_sum_netvapor
        RMVAP_BELOWCLOUD(t) = global_sum_netvaporbelowcloud
        MCRY_INCLOUD(t) = global_sum_mcryincloud
        MAGG_INCLOUD(t) = global_sum_maggincloud
        MRIM_INCLOUD(t) = global_sum_mrimincloud
        MCRY(t) = global_sum_mcry
        MAGG(t) = global_sum_magg
        MRIM(t) = global_sum_mrim
        SFC_PREC(t) = global_sum_prec/real(IMAX)/real(IMAX)/real(TOTAL_PRC)
        SFC_LWDN(t) = global_sum_lwdn/real(IMAX)/real(IMAX)/real(TOTAL_PRC)
        SFC_SWDN(t) = global_sum_swdn/real(IMAX)/real(IMAX)/real(TOTAL_PRC)

        MED_NLIQ_INCLOUD(t) = median_array1(int(median_array_size1/2))
        MED_NICE_INCLOUD(t) = median_array2(int(median_array_size2/2))
        MED_NICE200_INCLOUD(t) = median_array3(int(median_array_size3/2))
        MED_NICE200_BELOWCLOUD(t) = median_array4(int(median_array_size4/2))

        deallocate(median_array1,median_array2,median_array3,median_array4)

        write(*,'(3I3,A,8E15.6)') t, ts, k_file, "  mean LWPIWP, nliq, nice CTT incloud inice: ", LWP(t), IWP(t), MED_NLIQ_INCLOUD(t), MEAN_NLIQ_INCLOUD(t), MED_NICE_INCLOUD(t), MED_NICE200_INCLOUD(t), MED_NICE200_BELOWCLOUD(t), CTOPT(t)

     endif

     !call check ( MPI_BARRIER( MPI_COMM_WORLD, ierr ) )

     !temp = 0.0D0
     !do i = 1, count_global_median
     !   temp = temp + global_var_1d(i)
     !enddo
     !if (irank == master) then
     !   write(*,'(2E15.6)') cz(k), temp/real(count_global_median)
     !endif

  enddo

  
  ! output
  if (irank == master) then
     OUTPUT_CONFIG_FNAME = trim(OUTPUT_DIR)//"data_time_series.dat"
     call register_io(IO_OUTPUT_CONFIG,OUTPUT_CONFIG_FNAME)
     write(IO_OUTPUT_CONFIG,'(A)') "      time         LWP         IWP         Nliq(incloud)         Nice(inice)        medNice(incloud)       Dliq(incloud)         Dice(inice)         PREC          LW           SW           rim(incloud)           imm(incloud)           vap(incloud)           ctopT          vapALL          vapBAS          Nice(incloud)          mcry(incloud)          magg(incloud)          mrim(incloud)          mcry          magg          mrim           medNliq(incloud)          medNice200(incloud)          Dice(incloud)           immc(incloud)          incloud_counter           inice_counter          medNice200(belowcloud)          belowcloud_counter"
     do t = 1, nt
        write(IO_OUTPUT_CONFIG,'(32ES15.5)') &
             ct(t)/3600.0E0, &
             LWP(t), &
             IWP(t), &
             MEAN_NLIQ_INCLOUD(t), &
             MEAN_NICE_INICE(t), &
             MED_NICE_INCLOUD(t), &
             MEAN_DLIQ_INCLOUD(t), &
             MEAN_DICE_INICE(t), &
             SFC_PREC(t), &
             SFC_LWDN(t), &
             SFC_SWDN(t), &
             RMRIM_INCLOUD(t), &
             RMIMM_INCLOUD(t), &
             RMVAP_INCLOUD(t), &
             CTOPT(t), &
             RMVAP(t), &
             RMVAP_BELOWCLOUD(t), &
             MEAN_NICE_INCLOUD(t), &
             MCRY_INCLOUD(t), &
             MAGG_INCLOUD(t), &
             MRIM_INCLOUD(t), &
             MCRY(t), &
             MAGG(t), &
             MRIM(t), &
             MED_NLIQ_INCLOUD(t), &
             MED_NICE200_INCLOUD(t), &
             MEAN_DICE_INCLOUD(t), &
             RMIMMC_INCLOUD(t), &
             COUNTER_INCLOUD(t), &
             COUNTER_INICE(t), &
             MED_NICE200_BELOWCLOUD(t), &
             COUNTER_BELOWCLOUD(t)
     enddo
     call close_io(IO_OUTPUT_CONFIG)

     OUTPUT_CONFIG_FNAME = "data_liq_time_series.dat"
     call register_io(IO_OUTPUT_CONFIG,OUTPUT_CONFIG_FNAME)
     write(IO_OUTPUT_CONFIG,'(A)') "      time         rimliq      "
     do t = 1, nt
        write(IO_OUTPUT_CONFIG,'(2ES15.5)') &
             ct(t), &
             RMRIMLIQ_INCLOUD(t)
     enddo
     call close_io(IO_OUTPUT_CONFIG)
  endif

  deallocate(FILE_NAME,jobs_index)
  deallocate(fz,fx,cx,cy,cz,ct)
  deallocate(global_var_nliq,local_var_nliq)
  deallocate(global_var_nice,local_var_nice)
  deallocate(global_var_nice200,local_var_nice200)
  deallocate(dens,temperature,qliq,qhyd,qice,bin_liq_con,bin_ice_con)
  deallocate(bin_ice_Dmax,bin_ice_mass,bin_ice_magg,bin_ice_mcry,bin_ice_mrim)
  deallocate(bin_ice_alen,bin_ice_clen,bin_ice_dlen)
  deallocate(ice_mass_rim,ice_mass_imm,ice_conc_imm,ice_mass_vapor,ice_mass_eva)
  deallocate(liq_mass_rim,prec,LWdn,SWdn)
  if (poly_exist) then
     deallocate(bin_ice_nex,bin_ice_aglen,bin_ice_cglen)
  endif
  deallocate(LWP,IWP)
  deallocate(MEAN_NLIQ_INCLOUD,MEAN_NICE_INICE,MEAN_NICE_INCLOUD)
  deallocate(MEAN_DLIQ_INCLOUD,MEAN_DICE_INICE,RMRIMLIQ_INCLOUD)
  deallocate(MEAN_DICE_INCLOUD)
  deallocate(RMRIM_INCLOUD,RMIMM_INCLOUD,RMIMMC_INCLOUD,RMVAP_INCLOUD)
  deallocate(CTOPT,RMVAP,RMVAP_BELOWCLOUD)
  deallocate(MCRY_INCLOUD,MAGG_INCLOUD,MRIM_INCLOUD)
  deallocate(MCRY,MAGG,MRIM)
  deallocate(SFC_PREC,SFC_LWDN,SFC_SWDN)
  deallocate(MED_NLIQ_INCLOUD,MED_NICE_INCLOUD,MED_NICE200_INCLOUD)

  call close_io(IO_INPUT_CONFIG)


  call MPI_FINALIZE( ierr )

contains

  subroutine check(status)
    integer, intent ( in) :: status
    
    if(status /= nf90_noerr) then 
      print *, trim(nf90_strerror(status))
      call PRC_abort
      stop "Stopped"
    end if
  end subroutine check  

  subroutine register_io(input,file_name,namelist_file)
    implicit none

    integer, intent(inout) :: input
    character(len=*), intent(in) :: file_name
    logical, optional, intent(in) :: namelist_file
    integer :: i
    logical :: i_opened


    if (current_io_number > MAX_IO_NUMBER) then
       write(*,*) irank, "IO number exceeded max number", current_io_number, MAX_IO_NUMBER
       call PRC_abort
    endif
    do i = 1, MAX_IO_NUMBER
       inquire(i,OPENED=i_opened)
       if (.not. i_opened) then
          input = i
          if (present(namelist_file)) then
             if (namelist_file) then
                open(input,file=trim(file_name),status='old', &
                     form="formatted", delim='apostrophe', iostat=ierr )
             endif
          else
             open(input,file=trim(file_name),status='unknown',iostat=ierr)
          endif
          if ( ierr /= 0 ) then
             write (*, *) "ERROR: Fail to open the configuration file :", trim(INPUT_CONFIG_FNAME)
             call PRC_abort
          endif

          current_io_number = current_io_number + 1
          exit
       endif
    enddo

  end subroutine register_io


  subroutine close_io(input)
    implicit none

    integer, intent(inout) :: input
    integer :: i
    logical :: i_opened


    do i = 1, MAX_IO_NUMBER
       if (i == input) then
          inquire(i,OPENED=i_opened)
          if (i_opened) then
             close(input)
             current_io_number = current_io_number - 1
             return
          endif
       endif
    enddo

    write(*,*) "WARNING IO: ", i, current_io_number
    call PRC_abort

  end subroutine close_io

  subroutine PRC_abort
    implicit none
    if ( PRC_mpi_alive ) then
       ! tentative approach; input "PRC_UNIVERSAL_COMM_WORLD".
       !call MPI_COMM_CALL_ERRHANDLER(MPI_COMM_WORLD,PRC_ABORT_code,ierr)
       write(*,*) "abort operation in rank", irank
       call MPI_FINALIZE( ierr )
    endif

    stop
  end subroutine PRC_abort


  subroutine quickSorti(inputArray,size,start,end)
    implicit none
    integer, intent(in) :: start, end
    integer, intent(in) :: size
    real(SP), intent(inout) :: inputArray(size)

    integer :: temp, temp1, outer_loop, k, kk, kk_pivot, segmentNumber
    integer :: i, pivot_array_size, pivot_index
    integer :: pivots(size), pivot_size, pivot_array(size)
    logical :: loop_hdl

    if (start >= end) then
       return
    endif
    
    loop_hdl = .true.

    temp = end - start + 1
    outer_loop = int(temp/2) + 1
    
    pivot_size = 1
    pivots(pivot_size) = start
    pivot_size = pivot_size + 1
    pivots(pivot_size) = end
    k = 1
    segmentNumber = 1
    do while (k <= outer_loop)
       kk = 1
       temp1 = 0
       kk_pivot = 2
       pivot_array_size = 1
       do while (kk <= segmentNumber)
          call quickPartitioni(inputArray,size,pivot_index,pivots(kk_pivot-1),pivots(kk_pivot))
          if (pivot_index-1 > pivots(kk_pivot-1)) then
             pivot_array(pivot_array_size) = pivots(kk_pivot-1)
             pivot_array_size = pivot_array_size + 1
             pivot_array(pivot_array_size) = pivot_index-1
             pivot_array_size = pivot_array_size + 1
             temp1 = temp1 + 1
          endif
          if (pivot_index+1 < pivots(kk_pivot)) then
             pivot_array(pivot_array_size) = pivot_index+1
             pivot_array_size = pivot_array_size + 1
             pivot_array(pivot_array_size) = pivots(kk_pivot)
             pivot_array_size = pivot_array_size + 1
             temp1 = temp1 + 1
          endif
          kk = kk + 1
          kk_pivot = kk_pivot + 2
       enddo
        
       segmentNumber = temp1
       
       if (temp1 == 0) then
          exit
       endif
       
       i = 1
       do while (i <= pivot_array_size)
          pivots(i) = pivot_array(i)
          i = i + 1
       enddo
       k = k + 1
    enddo
  end subroutine quickSorti
    
  subroutine quickPartitioni(inputArray,size,output,start,end)
    implicit none
    integer, intent(in) :: start, end
    integer, intent(in) :: size
    real(SP), intent(inout) :: inputArray(size)
    integer, intent(out) :: output
    
    real(SP) :: pivot, temp
    integer :: k, i
    
    if (start >= end) then
       output = start
       return
    endif
        
    k = start
    pivot = inputArray(end)
    i = start
    do while (k < end)
       if (inputArray(k) < pivot) then
          temp = inputArray(k)
          inputArray(k) = inputArray(i)
          inputArray(i) = temp
          i = i + 1
       endif
       k = k + 1
    enddo
    inputArray(end) = inputArray(i)
    inputArray(i) = pivot
    output = i
    return

  end subroutine quickPartitioni


  subroutine Iice_habit(ice_type,icemass,crymass,aggmass,rimmass)
    implicit none
    real(SP), intent(in) :: icemass, crymass, aggmass, rimmass
    integer, intent(out) :: ice_type

    ice_type = -1
    if (rimmass > 0.1 * icemass) then
       if (aggmass < 0.1 * icemass) then
          if (rimmass < crymass) then
             ice_type = 5 ! rimed crystals
          else
             ice_type = 4 ! graupel
          endif
       else
          if (rimmass > aggmass) then
             ice_type = 4 ! graupel
          else
             ice_type = 3 ! rimed aggregates
          endif
       endif
    else
       if (aggmass < 0.1 * icemass) then
          ice_type = 2 ! pristine crystals
       else
          ice_type = 1 ! aggregates
       endif
    endif
    return

  end subroutine Iice_habit

  subroutine Iice_shape(habit,a_axis,c_axis,d_axis)
    implicit none
    real(SP), intent(in) :: a_axis, c_axis, d_axis
    integer, intent(out) :: habit

    habit = -1
    if (c_axis > a_axis) then
       habit = 3 ! columnar crystals
    else
       if (d_axis > 2.0/3.0 * a_axis) then
          habit = 2 ! dendrites
       else
          habit = 1 ! hexagonal plates
       endif
    endif
  end subroutine Iice_shape

  subroutine Iice_shapePoly(habit,a_axis,c_axis,d_axis,ag_axis,cg_axis,ex_cry)
    implicit none
    real(SP), intent(in) :: a_axis, c_axis, d_axis, ag_axis, cg_axis, ex_cry
    integer, intent(out) :: habit

    real(SP) :: temp

    habit = -1
    if (ex_cry < 0.5) then
       if (c_axis > a_axis) then
          habit = 3 ! columnar crystals
       else
          if (d_axis > 2.0/3.0 * a_axis) then
             habit = 2 ! dendrites
          else
             habit = 1 ! hexagonal plates
          endif
       endif
    else
       if (a_axis == 0.0 .and. c_axis == 0.0) then
          habit = -1
       else if (a_axis == 0.0) then
          temp = 1.0
       else if (c_axis == 0.0) then
          temp = -1.0
       else
          temp = ag_axis/a_axis - cg_axis/c_axis
       endif
       if (temp >= 0.5) then
          habit = 4 ! planar polycrystals
       else if (temp <= -0.5) then
          habit = 5 ! columnar polycrystals
       else
          habit = 6 ! irregular polycrystals
       endif
    endif

    return
  end subroutine Iice_shapePoly

  function dsign(float_number) result (output)
    implicit none
    real(SP), intent(in) :: float_number
    real(SP) :: output

    if (float_number >= 0.0E0) then
       output = 1.0E0
    else
       output = -1.0E0
    endif
  end function dsign

  subroutine convert_habit(output,habit)
    implicit none
    integer, intent(in) :: habit
    integer, intent(out) :: output

    if (habit == 1) then
       output = 1
    else if (habit == 2) then
       output = 2
    else if (habit == 3) then
       output = 3
    else if (habit == 4) then
       output = 5
    else if (habit == 5) then
       output = 4
    else
       output = 6
    endif
    return
  end subroutine convert_habit

  function get_len_c2a(amass) result (output)
    implicit none
    real(SP), intent(in) :: amass
    real(SP) :: output

    real(SP) :: a_ros, b_ros, N_ros, den_i

    ! mass-dimension relationship of bullet roseete
    ! parameters for mass-dimension relationship from Chiruta and Wang (2003)
    a_ros = 0.3257E0
    b_ros = 0.5206E0
    N_ros = 4.0E0
    den_i = 0.91668E0
    
    output = (amass/(den_i*a_ros*N_ros**b_ros))**(1.0E0/3.0E0)
    
  end function get_len_c2a

  function get_len_s1(amass) result (output)
    implicit none
    real(SP), intent(in) :: amass
    real(SP) :: output

    real(SP) :: a, b, c1

    ! mass-dimension relationship of S1
    ! side plane crystal mass-length relation from Mitchel et al (1990)
    a = 0.0041900509E0
    b = 2.3E0
    ! assume that the shape is cylinder that has the aspect ratio of 0.25 for all sizes, so
    ! when it exceed max density, radiuas has to be increased.
    c1 = 0.885565704066706E0
    output = max(0.5E0*(amass/a)**(1.0E0/b),c1*amass**(1.0E0/3.0E0))

  end function get_len_s1

  function get_len_s3(amass) result (output)
    implicit none
    real(SP), intent(in) :: amass
    real(SP) :: output

    real(SP) :: a, b, c1

    ! mass-dimension relationship of S3
    ! S3 crystal mass-length relation from Mitchel et al (1990)
    a = 0.0027696359E0
    b = 2.1E0
    ! assume that the shape is cylinder that has the aspect ratio of 0.25 for all sizes, so
    ! when it exceed max density, radiuas has to be increased.
    c1 = 0.885565704066706E0
    output = max(0.5E0*(amass/a)**(1.0E0/b),c1*amass**(1.0E0/3.0E0))

  end function get_len_s3

  subroutine get_elen_rlen( elen, rlen, habit, mcry, alen, clen, aglen, cglen )
    implicit none
    real(SP), intent(in) :: mcry, alen, clen, aglen, cglen
    integer, intent(in) :: habit
    real(SP), intent(out) :: elen, rlen

    real(SP) :: xlen_s1, xlen_c2a, xlen_s3, i_ag_ge_cgclen, i_cg_ge_agalen


    xlen_s1 = get_len_s1(mcry)
    xlen_c2a = get_len_c2a(mcry)
    xlen_s3 = get_len_s3(mcry)

    if (habit <= 3 .or. alen < 1.e-15 .or. clen < 1.e-15) then
       elen = 0.0E0
       rlen = 0.0E0
       return
    else
       i_ag_ge_cgclen = 0.5E0*(1.0E0 + dsign(aglen-(cglen/clen+0.5E0)*alen))
       i_cg_ge_agalen = 0.5E0*(1.0E0 + dsign(cglen-(aglen/alen+0.5E0)*clen))
    endif

    ! side planes or radiating assemblage of plates: S1 or P7a
    ! bullet rosette
    ! combination of side planes, bullets, and columns: S3
    elen=&
          i_ag_ge_cgclen*xlen_s1 + &
          (1.0E0 - i_ag_ge_cgclen)*i_cg_ge_agalen*0.0E0 + &
          (1.0E0 - i_ag_ge_cgclen)*(1.0E0 - i_cg_ge_agalen)* &
          xlen_s3

    rlen=&
          i_ag_ge_cgclen*0.0E0 + &
          (1.0E0 - i_ag_ge_cgclen)*i_cg_ge_agalen* &
          xlen_c2a + &
          (1.0E0 - i_ag_ge_cgclen)*(1.0E0 - i_cg_ge_agalen)* &
          xlen_s3

    if (elen <= 0.0E0 .or. rlen < 0.0E0) then
       write(*,'(A,5ES15.6)') "elen error: ", elen, rlen, xlen_s1, xlen_c2a, xlen_s3
       !call PRC_abort
    endif

    return

  end subroutine get_elen_rlen

end program mpipaper5analysis_time 


