program mpipaper5analysis_time
  use netcdf
  use mpi
  implicit none

  ! debug output
  logical :: debug_output = .false.

  ! precision
  integer, parameter :: SP=4
  integer, parameter :: DP=8

  ! cloud and ice grids criteria
  real(SP), parameter :: qliq_lower_limit=1.e-6
  real(SP), parameter :: qice_lower_limit=1.e-9

  ! polycrystals exist or not, and ice-axis version
  logical, parameter :: poly_exist=.false.
  integer, parameter :: iceaxis_version=3
  real(SP), parameter :: spx_p=0.25E0

  ! io
  integer :: current_io_number
  integer, parameter :: MAX_IO_NUMBER=100

  ! input file model variables
  integer :: IO_INPUT_CONFIG               ! input file io
  character (len=256) :: INPUT_CONFIG_FNAME ! input file name
  integer :: prc_num_x, prc_num_y          ! model process numbers
  integer :: grd_x, grd_y, grd_z           ! model global grid numbers
  integer :: IMAX, JMAX, KMAX              ! model local grid numbers
  real(DP) :: DX, DY                       ! model horizontal grid spacings
  logical :: FILE_HISTORY_CARTESC_BOUNDARY=.false. ! model boundary output

  ! data file variables
  integer, allocatable :: IO_DATA(:)

  ! netcdf file variables
  character (len=256), parameter :: INPUT_DIR="/home/OngChiaRui/WORKING/scale-5.4.4/scale-rm/test/tutorial/mpace/data/bulk_surface/2300klein_lw/sim_mpace_3D_nohydro_tlsABLmy1_c09nccn30_crystal_fracdust0013/" ! input file directory
  !character (len=256), parameter :: INPUT_DIR="/home/OngChiaRui/WORKING/scale-5.4.4/scale-rm/test/tutorial/sheba/sim_3D_noicerad_fracdust00004/"
  character (len = 256), allocatable :: FILE_DIR(:)
  character (len = 256), allocatable :: FILE_NAME(:)
  character (len = 6) :: word_zeros
  character (len = 6) :: word
  integer, parameter :: TOTAL_FILE_NUMBER=4

  ! output file
  integer :: IO_OUTPUT_CONFIG ! output file io
  character (len=256) :: OUTPUT_CONFIG_FNAME ! output file name
  character (len=256), parameter :: OUTPUT_DIR="/home/OngChiaRui/WORKING/scale-5.4.4/scale-rm/test/tutorial/mpace/data/bulk_surface/2300klein_lw/sim_mpace_3D_nohydro_tlsABLmy1_c09nccn30_crystal_fracdust0013/paper5_figures/" ! output file directory
  !character (len=256), parameter :: OUTPUT_DIR="/home/OngChiaRui/WORKING/scale-5.4.4/scale-rm/test/tutorial/sheba/sim_3D_noicerad_fracdust00004/paper5_figures/" ! output file directory

  ! netcdf indices
  integer :: ncid, dimid, varid

  ! MPI communication variables
  logical :: PRC_mpi_alive = .false.
  integer :: isize, irank
  integer :: sendcount_median, count_global_median
  integer, parameter :: master=0
  integer, parameter :: MAX_PRC_X=4
  integer :: TOTAL_PRC
  integer :: JOBS_PER_MPI, JOBS_PER_MPI_X
  integer, allocatable :: jobs_index(:)
  integer :: mpi_x, mpi_y
  integer :: start_3d(4), count_3d(4), start_2d(3), count_2d(3)

  ! history items
  character (len=32) :: NAME
  character (len=32) :: varname

  ! grid
  integer :: is, ie, isg, ieg, js, je, jsg, jeg, ks, ke, ts, te
  integer :: nfx, nfz, nx, ny, nz, nt, nt_partial(TOTAL_FILE_NUMBER)
  real(DP), allocatable :: fx(:), fz(:), cx(:), cy(:), cz(:), ct(:)

  ! data
  real(SP), allocatable :: global_var_T(:), global_var_qv(:)
  real(SP), allocatable :: global_var_P(:), global_var_dens(:)
  real(SP), allocatable :: global_var_rh(:), global_var_rhi(:)
  real(SP), allocatable :: global_var_lwc(:), global_var_iwc(:), global_var_iwccryrcr(:)
  real(SP), allocatable :: global_var_iwccry(:), global_var_iwcrcr(:), global_var_iwcgra(:), global_var_iwcagg(:), global_var_iwcrag(:)
  real(SP), allocatable :: global_var_u(:), global_var_tke(:)
  real(SP), allocatable :: global_var_rmrim(:), global_var_rmvap(:)
  real(SP), allocatable :: global_var_rmimm(:), global_var_cry(:), global_var_cryall(:)
  real(SP), allocatable :: global_var_aggall(:), global_var_graall(:)
  real(SP), allocatable :: global_var_rcrall(:), global_var_ragall(:)
  real(SP), allocatable :: global_var_agg(:), global_var_gra(:)
  real(SP), allocatable :: global_var_rcr(:), global_var_rag(:)
  real(SP), allocatable :: global_var_monopl(:), global_var_monocl(:)
  real(SP), allocatable :: global_var_monodd(:), global_var_polypl(:)
  real(SP), allocatable :: global_var_polycl(:), global_var_polyir(:)
  real(SP), allocatable :: global_var_nliq(:), global_var_nice(:), global_var_nice200(:)
  real(SP), allocatable :: global_var_nliqall(:), global_var_niceall(:), global_var_nice200all(:)
  real(SP), allocatable :: global_var_dliq(:), global_var_dice(:)

  real(SP), allocatable :: local_var_T(:), local_var_qv(:)
  real(SP), allocatable :: local_var_P(:), local_var_dens(:)
  real(SP), allocatable :: local_var_rh(:), local_var_rhi(:)
  real(SP), allocatable :: local_var_lwc(:), local_var_iwc(:), local_var_iwccryrcr(:)
  real(SP), allocatable :: local_var_iwccry(:), local_var_iwcrcr(:), local_var_iwcgra(:), local_var_iwcagg(:), local_var_iwcrag(:)
  real(SP), allocatable :: local_var_u(:), local_var_tke(:)
  real(SP), allocatable :: local_var_rmrim(:), local_var_rmvap(:)
  real(SP), allocatable :: local_var_rmimm(:), local_var_cry(:), local_var_cryall(:)
  real(SP), allocatable :: local_var_aggall(:), local_var_graall(:)
  real(SP), allocatable :: local_var_rcrall(:), local_var_ragall(:)
  real(SP), allocatable :: local_var_agg(:), local_var_gra(:)
  real(SP), allocatable :: local_var_rcr(:), local_var_rag(:)
  real(SP), allocatable :: local_var_monopl(:), local_var_monocl(:)
  real(SP), allocatable :: local_var_monodd(:), local_var_polypl(:)
  real(SP), allocatable :: local_var_polycl(:), local_var_polyir(:)
  real(SP), allocatable :: local_var_nliq(:), local_var_nice(:), local_var_nice200(:)
  real(SP), allocatable :: local_var_nliqall(:), local_var_niceall(:), local_var_nice200all(:)
  real(SP), allocatable :: local_var_dliq(:), local_var_dice(:)

  real(SP), allocatable :: qliq(:,:,:), qice(:,:,:), qhyd(:,:,:), dens(:,:,:) ! i, j, k
  real(SP), allocatable :: qv(:,:,:), rh(:,:,:), rhi(:,:,:), temperature(:,:,:)
  real(SP), allocatable :: u(:,:,:), v(:,:,:), w(:,:,:), potential(:,:,:)
  real(SP), allocatable :: lwdn(:,:,:), swdn(:,:,:), sink(:,:,:), rhoh_rd(:,:,:)
  real(SP), allocatable :: rhoh_mp(:,:,:), pressure(:,:,:)
  real(SP), allocatable :: ice_mass_vapor(:,:,:), ice_mass_eva(:,:,:), ice_mass_rim(:,:,:)
  real(SP), allocatable :: ice_mass_imm(:,:,:), liq_mass_rim(:,:,:)
  !real(SP), allocatable :: nc(:,:,:), nr(:,:,:), ni(:,:,:), ns(:,:,:), ng(:,:,:)
  real(SP), allocatable :: bin_liq_con(:,:,:,:), bin_ice_con(:,:,:,:) ! i, j, k, bin
  real(SP), allocatable :: bin_ice_mass(:,:,:,:), bin_ice_mcry(:,:,:,:)
  real(SP), allocatable :: bin_ice_magg(:,:,:,:), bin_ice_mrim(:,:,:,:)
  real(SP), allocatable :: bin_ice_alen(:,:,:,:), bin_ice_clen(:,:,:,:)
  real(SP), allocatable :: bin_ice_dlen(:,:,:,:), bin_ice_aglen(:,:,:,:)
  real(SP), allocatable :: bin_ice_cglen(:,:,:,:), bin_ice_nex(:,:,:,:)
  real(SP), allocatable :: bin_ice_Dmax(:,:,:,:)

  real(SP), allocatable :: ini_T1d(:), ini_P1d(:), ini_qv1d(:), ini_u1d(:)
  real(SP), allocatable :: ini_lwc1d(:), ini_rh1d(:), ini_rhi1d(:), ini_qliq1d(:)
  real(SP), allocatable :: ini_lwdn1d(:), ini_swdn1d(:), ini_dens1d(:)
  real(SP), allocatable :: ini_PT1d(:), ini_nliq1d(:), ini_nliqall1d(:), ini_sink1d(:)
  real(SP), allocatable :: ini_rhoh_rd1d(:), ini_rhoh_mp1d(:)
  
  real(SP), allocatable :: T1d(:,:), T1d1(:,:), T1d9(:,:)
  real(SP), allocatable :: P1d(:,:), P1d1(:,:), P1d9(:,:)
  real(SP), allocatable :: dens1d(:,:), dens1d1(:,:), dens1d9(:,:)
  real(SP), allocatable :: rh1d(:,:), rh1d1(:,:), rh1d9(:,:)
  real(SP), allocatable :: rhi1d(:,:), rhi1d1(:,:), rhi1d9(:,:)
  real(SP), allocatable :: nliq1d(:,:), nliq1d1(:,:), nliq1d9(:,:)
  real(SP), allocatable :: nice1d(:,:), nice1d1(:,:), nice1d9(:,:)
  real(SP), allocatable :: nice2001d(:,:), nice2001d1(:,:), nice2001d9(:,:)
  real(SP), allocatable :: nliqall1d(:,:), nliqall1d1(:,:), nliqall1d9(:,:)
  real(SP), allocatable :: niceall1d(:,:), niceall1d1(:,:), niceall1d9(:,:)
  real(SP), allocatable :: nice200all1d(:,:), nice200all1d1(:,:), nice200all1d9(:,:)
  real(SP), allocatable :: dliq1d(:,:), dliq1d1(:,:), dliq1d9(:,:)
  real(SP), allocatable :: dice1d(:,:), dice1d1(:,:), dice1d9(:,:)
  real(SP), allocatable :: monopl1d(:,:), monopl1d1(:,:), monopl1d9(:,:)
  real(SP), allocatable :: monocl1d(:,:), monocl1d1(:,:), monocl1d9(:,:)
  real(SP), allocatable :: monodd1d(:,:), monodd1d1(:,:), monodd1d9(:,:)
  real(SP), allocatable :: polypl1d(:,:), polypl1d1(:,:), polypl1d9(:,:)
  real(SP), allocatable :: polycl1d(:,:), polycl1d1(:,:), polycl1d9(:,:)
  real(SP), allocatable :: polyir1d(:,:), polyir1d1(:,:), polyir1d9(:,:)
  real(SP), allocatable :: qv1d(:,:), qv1d1(:,:), qv1d9(:,:)
  real(SP), allocatable :: u1d(:,:), u1d1(:,:), u1d9(:,:)
  real(SP), allocatable :: lwc1d(:,:), lwc1d1(:,:), lwc1d9(:,:)
  real(SP), allocatable :: iwc1d(:,:), iwc1d1(:,:), iwc1d9(:,:)
  real(SP), allocatable :: iwccry1d(:,:), iwccry1d1(:,:), iwccry1d9(:,:)
  real(SP), allocatable :: iwcrcr1d(:,:), iwcrcr1d1(:,:), iwcrcr1d9(:,:)
  real(SP), allocatable :: iwcgra1d(:,:), iwcgra1d1(:,:), iwcgra1d9(:,:)
  real(SP), allocatable :: iwccryrcr1d(:,:), iwccryrcr1d1(:,:), iwccryrcr1d9(:,:)
  real(SP), allocatable :: iwcagg1d(:,:), iwcagg1d1(:,:), iwcagg1d9(:,:)
  real(SP), allocatable :: iwcrag1d(:,:), iwcrag1d1(:,:), iwcrag1d9(:,:)
  real(SP), allocatable :: tke1d(:,:), tke1d1(:,:), tke1d9(:,:)
  real(SP), allocatable :: cry1d(:,:), cry1d1(:,:), cry1d9(:,:)
  real(SP), allocatable :: rmrim1d(:,:), rmrim1d1(:,:), rmrim1d9(:,:)
  real(SP), allocatable :: rmimm1d(:,:), rmimm1d1(:,:), rmimm1d9(:,:)
  real(SP), allocatable :: rmvap1d(:,:), rmvap1d1(:,:), rmvap1d9(:,:)
  real(SP), allocatable :: agg1d(:,:), agg1d1(:,:), agg1d9(:,:)
  real(SP), allocatable :: gra1d(:,:), gra1d1(:,:), gra1d9(:,:)
  real(SP), allocatable :: rcr1d(:,:), rcr1d1(:,:), rcr1d9(:,:)
  real(SP), allocatable :: rag1d(:,:), rag1d1(:,:), rag1d9(:,:)
  real(SP), allocatable :: cryall1d(:,:), cryall1d1(:,:), cryall1d9(:,:)
  real(SP), allocatable :: aggall1d(:,:), aggall1d1(:,:), aggall1d9(:,:)
  real(SP), allocatable :: graall1d(:,:), graall1d1(:,:), graall1d9(:,:)
  real(SP), allocatable :: rcrall1d(:,:), rcrall1d1(:,:), rcrall1d9(:,:)
  real(SP), allocatable :: ragall1d(:,:), ragall1d1(:,:), ragall1d9(:,:)
  
  ! liquid bins
  integer, parameter :: max_binr = 40
  real(SP), allocatable :: massbin(:), massbin_bnd(:)
  real(SP), allocatable :: diameterbin(:), diameterbin_bnd(:)
  real(SP), parameter :: c_min = 0.000000000000004188790205, c_max = 0.0000000654498
  real(SP), parameter :: c_maxmass = 0.52359870

  ! ice bins
  integer, parameter :: max_bini = 20
  real(SP), allocatable :: massbini(:), massbini_bnd(:)
  real(SP), allocatable :: diameteribin(:), diameteribin_bnd(:)
  real(SP), allocatable :: massbini_bothbnd(:,:)
  real(SP) :: ci_min, ci_max

  ! local variables 
  integer :: i, j, k, t, t_local, k_file, ibin, counter1
  integer :: median_array_size1, median_array_size2, median_array_size3
  real(SP), allocatable :: median_array1(:), median_array2(:), median_array3(:)
  real(SP), allocatable :: median_array2_200(:)
  real(SP), allocatable :: median_array4(:), median_array5(:), median_array6(:)
  real(SP), allocatable :: median_array7(:), median_array8(:), median_array9(:)
  real(SP), allocatable :: median_array10(:), median_array11(:), median_array12(:)
  real(SP), allocatable :: median_array13(:), median_array14(:), median_array15(:)
  real(SP), allocatable :: median_array16(:), median_array16_200(:)
  real(SP), allocatable :: median_array17(:), median_array18(:)
  real(SP), allocatable :: median_array19(:), median_array20(:), median_array21(:)
  integer :: ierr
  logical :: lstat
  real(SP) :: temp1, temp2
  real(SP) :: temp3, temp4, temp5, temp6, temp7, temp8, temp9, temp10, temp11
  real(SP) :: temp12, temp13, temp14, temp15, temp16, temp17
  real(SP) :: dbin, debugtemp, D2, D3
  real(SP) :: global_sum_T, global_sum_P, global_sum_PT, global_sum_qv, global_sum_qliq
  real(SP) :: global_sum_dens, global_sum_lwc, global_sum_lwdn, global_sum_swdn
  real(SP) :: global_sum_rhoh_rd, global_sum_rhoh_mp, global_sum_sink, global_sum_nliq
  real(SP) :: global_sum_rh, global_sum_rhi, global_sum_u
  real(SP), parameter :: pi = 3.141592653589793238462643
  integer :: ice_type, habit, my_habit
  real(SP) :: vcs, vic, alpha_ic, alpha_im, den_i, kp, mlmt, mass_agg_cry, mass_agg_cry_rim, local_Dmax, asm, csm, pla_e, ros_r
  real(SP) :: lbin_ice_alen, lbin_ice_clen
  real(SP) :: lbin_ice_dlen, lbin_ice_aglen
  real(SP) :: lbin_ice_cglen, lbin_ice_nex


  
  namelist /PARAM_PRC_CARTESC/ &
       PRC_NUM_X, &
       PRC_NUM_Y

  namelist /PARAM_ATMOS_GRID_CARTESC_INDEX/ &
       KMAX, &
       IMAX, &
       JMAX

  namelist /PARAM_ATMOS_GRID_CARTESC/ &
       DX, &
       DY

  namelist /PARAM_FILE_HISTORY_CARTESC/ &
       FILE_HISTORY_CARTESC_BOUNDARY

  namelist /HISTORY_ITEM/ &
       NAME


  call MPI_INIT( ierr )
  PRC_mpi_alive = .true.
  call MPI_COMM_SIZE( MPI_COMM_WORLD, isize, ierr )
  call MPI_COMM_RANK( MPI_COMM_WORLD, irank, ierr )
  if ( irank == master ) then
     write( *, * ) "+++ MPI COMM: Corrective Initialize"
  endif

  !--- Read from argument
  if ( COMMAND_ARGUMENT_COUNT() /= 1 ) then
     write(*, *) "ERROR: Program needs a config file!"
     call PRC_abort
  else
     call get_command_argument(1,INPUT_CONFIG_FNAME)
  endif

  if (irank == master) then
     write(*,*) "INPUT FILE NAME: ", trim(INPUT_CONFIG_FNAME)
  endif

  ! set initial io number value
  current_io_number = 1

  inquire (file=trim(INPUT_CONFIG_FNAME), exist=lstat)
  if ( .not. lstat ) then
     write (*, *) "ERROR: Not found the configuration file :", trim(INPUT_CONFIG_FNAME)
     call PRC_abort
  endif
  call register_io(IO_INPUT_CONFIG,INPUT_CONFIG_FNAME,.true.)

  rewind(IO_INPUT_CONFIG)
  read(IO_INPUT_CONFIG,nml=PARAM_PRC_CARTESC,iostat=ierr)

  if( ierr < 0 ) then !--- missing
     write(*,*) irank, "ERROR. Not found namelist: PARAM_PRC_CARTESC."
     call PRC_abort
  !elseif( ierr > 0 ) then !--- fatal error
  !   write(*,*) irank, "Not appropriate names in namelist PARAM_PRC_CARTESC, Check!"
  !   call PRC_abort
  end if
  if (irank == master) then
     write(*,*) "PRC_NUM_X PRC_NUM_Y: ", prc_num_x, prc_num_y
  endif
  
  rewind(IO_INPUT_CONFIG)
  read(IO_INPUT_CONFIG,nml=PARAM_ATMOS_GRID_CARTESC,iostat=ierr)

  if( ierr < 0 ) then !--- missing
     write(*,*) irank, "ERROR. Not found namelist: PARAM_ATMOS_GRID_CARTESC."
     call PRC_abort
  !elseif( ierr > 0 ) then !--- fatal error
  !   write(*,*) irank, "Not appropriate names in namelist PARAM_ATMOS_GRID_CARTESC, Check!"
  !   call PRC_abort
  end if

  rewind(IO_INPUT_CONFIG)
  read(IO_INPUT_CONFIG,nml=PARAM_ATMOS_GRID_CARTESC_INDEX,iostat=ierr)

  if( ierr < 0 ) then !--- missing
     write(*,*) irank, "ERROR. Not found namelist: PARAM_ATMOS_GRID_CARTESC_INDEX."
     call PRC_abort
  !elseif( ierr > 0 ) then !--- fatal error
  !   write(*,*) irank, "Not appropriate names in namelist PARAM_ATMOS_GRID_CARTESC_INDEX, Check!"
  !   call PRC_abort
  end if
  if (irank == master) then
     write(*,'(A)') "REMEMBER TO CHECK THESE VALUES, VERY IMPORTANT!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!"
     write(*,*) "KMAX IMAX JMAX: ", KMAX, IMAX, JMAX
     write(*,*) "DX DY: ", DX, DY
  endif

  grd_x = IMAX*prc_num_x
  grd_y = JMAX*prc_num_y
  grd_z = KMAX

  ! check if mpi number is a square
  mpi_x = 0
  mpi_x = 0
  do i = 1, MAX_PRC_X
     if (i*i == isize) then
        mpi_x = i
        mpi_y = i
     endif
  enddo
  if (mpi_x == 0 .or. mpi_y == 0) then
     write(*,*) irank, "ERROR, MPI NUMBER MUST BE A SQUARE", isize
     call PRC_abort
  else
     if (irank == master) then
        write(*,*) "MPI_X and MPI_Y: ", mpi_x, mpi_y
     endif
  endif
  if (prc_num_x /= prc_num_y .or. mod(prc_num_x,mpi_x) /= 0 .or. mod(prc_num_y,mpi_y) /= 0 .or. prc_num_x > mpi_x .or. prc_num_y > mpi_y) then
     write(*,*) irank, "ERROR, MODEL PROCESS NUMBER MUST BE A SQUARE AND MODEL PROCESS NUMBER MUST BE DIVIDED BY MPI SIZE", prc_num_x, prc_num_y, mpi_x, mpi_y
     call PRC_abort
  endif

  
  TOTAL_PRC = prc_num_x*prc_num_y
  JOBS_PER_MPI = TOTAL_PRC/isize
  JOBS_PER_MPI_X = prc_num_x/mpi_x

  if (irank == master) then
     write(*,*) "JOBS_PER_MPI JOBS_PER_MPI_X", JOBS_PER_MPI, JOBS_PER_MPI_X
  endif

  allocate(jobs_index(JOBS_PER_MPI))
  allocate(FILE_NAME(JOBS_PER_MPI))

  ! assign range of model process index
  i = JOBS_PER_MPI_X * mod(irank,mpi_x) + prc_num_x*JOBS_PER_MPI_X * int(irank/mpi_x)
  j = 0
  do k = 1, JOBS_PER_MPI
     jobs_index(k) = i ! minus 1 because history id starts from zero
     j = j + 1
     if (mod(j,JOBS_PER_MPI_X) == 0) then
        i = i + (prc_num_x - mpi_x)
        j = 0
     else
        i = i + 1
     endif
     write(*,*) "rank-wise job id: ", irank, jobs_index(k)
  enddo


  ! define liquid bins
  allocate(massbin(max_binr),massbin_bnd(max_binr+1))
  allocate(diameterbin(max_binr),diameterbin_bnd(max_binr+1))
  temp1 = c_min
  dbin = (c_max/c_min)**(0.05) ! 0.1 = 1/10, 0.05 = 1/20
  temp2 = temp1*dbin
  massbin_bnd(1) = temp1
  do i = 1, 20
     massbin_bnd(i+1) = temp2
     massbin(i) = 0.5*(temp1+temp2) ! liquid haze bins
     temp1 = temp2
     temp2 = temp2*dbin
  enddo
  dbin = (c_maxmass/c_max)**(0.05)
  temp2 = temp1*dbin
  do i = 21, max_binr
     massbin_bnd(i+1) = temp2
     massbin(i) = 0.5*(temp1+temp2) ! liquid drop bins
     temp1 = temp2
     temp2 = temp2*dbin
  enddo
    
  diameterbin_bnd(1) = (massbin_bnd(1)/(3.1416/6.0))**(1.0/3.0)*10000.0
  if (irank == master .and. debug_output) then
     print *, "liquid diameter: ",0, "---", diameterbin_bnd(1)
  endif
  do i = 1, max_binr
     diameterbin(i) = (massbin(i)/(3.1416/6.0))**(1.0/3.0)*10000.0 ! unit is \mu m
     diameterbin_bnd(i+1) = (massbin_bnd(i+1)/(3.1416/6.0))**(1.0/3.0)*10000.0
     if (irank == master .and. debug_output) then
        print*, "liquid diameter: ",i, diameterbin_bnd(i+1), diameterbin(i)
     endif
  enddo

  ! define ice bins
  allocate(massbini(max_bini),massbini_bnd(max_bini+1))
  allocate(diameteribin(max_bini),diameteribin_bnd(max_bini+1))
  allocate(massbini_bothbnd(max_bini,2))
  ci_min = 4.18879020478639e-12
  ci_max = 1.0e-6
  temp1 = ci_min
  dbin = (ci_max/ci_min)**(0.25)
  temp2 = temp1*dbin
  massbini_bnd(1) = temp1
  diameteribin_bnd(1) = (temp1/0.916/3.1416*6.0)**(1.0/3.0)*10000.0
  do i = 1, 4
     massbini_bnd(i+1) = temp2
     massbini(i) = 0.5*(temp1+temp2) ! ice deposition bins
     diameteribin_bnd(i+1) = (temp2/0.916/3.1416*6.0)**(1.0/3.0)*10000.0
     diameteribin(i) = (0.5*(temp1+temp2)/0.916/3.1416*6.0)**(1.0/3.0)*10000.0
     temp1 = temp2
     temp2 = temp2*dbin
  enddo
  ci_min = ci_max
  ci_max = 1.0e-2
  dbin = (ci_max/ci_min)**(1.0/10.0)
  temp2 = temp1*dbin
  do i = 5, 14
     massbini_bnd(i+1) = temp2
     massbini(i) = 0.5*(temp1+temp2) ! ice collection bins
     diameteribin_bnd(i+1) = (temp2/0.916/3.1416*6.0)**(1.0/3.0)*10000.0
     diameteribin(i) = (0.5*(temp1+temp2)/0.916/3.1416*6.0)**(1.0/3.0)*10000.0
     temp1 = temp2
     temp2 = temp2*dbin
  enddo
  ci_min = ci_max
  ci_max = 1.0e+1
  dbin = (ci_max/ci_min)**(1.0/6.0)
  temp2 = temp1*dbin
  do i = 15, 20
     massbini_bnd(i+1) = temp2
     massbini(i) = 0.5*(temp1+temp2) ! ice riming bins
     diameteribin_bnd(i+1) = (temp2/0.916/3.1416*6.0)**(1.0/3.0)*10000.0
     diameteribin(i) = (0.5*(temp1+temp2)/0.916/3.1416*6.0)**(1.0/3.0)*10000.0
     temp1 = temp2
     temp2 = temp2*dbin
  enddo

  do i = 1, max_bini
     massbini_bothbnd(i,1) = massbini_bnd(i)
     massbini_bothbnd(i,2) = massbini_bnd(i+1)
  enddo


  
  
  ! assign tag number for message passing
  ! not need here


  ! history file names
  do k = 1, JOBS_PER_MPI
     if (jobs_index(k) < 10) then
        write(word,'(I1)') jobs_index(k)
        word_zeros = "00000"
     else if (jobs_index(k) < 100) then
        write(word,'(I2)') jobs_index(k)
        word_zeros = "0000"
     else if (jobs_index(k) < 1000) then
        write(word,'(I3)') jobs_index(k)
        word_zeros = "000"
     else if (jobs_index(k) < 10000) then
        write(word,'(I4)') jobs_index(k)
        word_zeros = "00"
     else if (jobs_index(k) < 100000) then
        write(word,'(I5)') jobs_index(k)
        word_zeros = "0"
     else
        write(*,*) "IMPOSSIBLE JOBS ID (TOO LARGE): ", jobs_index(k)
        call PRC_abort
     endif
     FILE_NAME(k) = "history.pe"//trim(word_zeros)//trim(word)//".nc"
     write(*,'(I3,A,A)') k, " th FILE_NAME: ", FILE_NAME(k)
  enddo

  if (TOTAL_FILE_NUMBER > 9) then
     write(*,*) "FILE NUMBER SHOULD BE LESS THAN 10", TOTAL_FILE_NUMBER
     call PRC_abort
  endif
  allocate(FILE_DIR(TOTAL_FILE_NUMBER))
  do k_file = 1, TOTAL_FILE_NUMBER
     write(word,'(I1)') k_file
     FILE_DIR(k_file) = trim(INPUT_DIR)//trim(word)//"/" ! netcdf file directory
     if (irank == master) write(*,*) FILE_DIR(k_file)
  enddo

  
  ! read netcdf grid
  do k = 1, 1!JOBS_PER_MPI ! temporary, since x and y information are not needed here
     call check( nf90_open( trim(FILE_DIR(1))//trim(FILE_NAME(k)), nf90_nowrite, ncid ) )

     call check( nf90_inq_dimid ( ncid,  'FZ', varid ) )
     call check( nf90_inquire_dimension( ncid, varid, len=nfz ) )

     call check( nf90_inq_dimid ( ncid,  'FX', varid ) )
     call check( nf90_inquire_dimension( ncid, varid, len=nfx ) )

     call check( nf90_inq_dimid ( ncid,  'CZ', varid ) )
     call check( nf90_inquire_dimension( ncid, varid, len=nz ) )
     call check( nf90_inq_dimid ( ncid,  'CX', varid ) )
     call check( nf90_inquire_dimension( ncid, varid, len=nx ) )
     call check( nf90_inq_dimid ( ncid,  'CY', varid ) )
     call check( nf90_inquire_dimension( ncid, varid, len=ny ) )

     allocate(fz(nfz),fx(nfx),cz(nz),cx(nx),cy(ny))

     call check( nf90_inq_varid( ncid, 'FZ', varid ) )
     call check( nf90_get_var( ncid, varid, fz ) )

     call check( nf90_inq_varid( ncid, 'FX', varid ) )
     call check( nf90_get_var( ncid, varid, fx ) )

     call check( nf90_inq_varid( ncid, 'CZ', varid ) )
     call check( nf90_get_var( ncid, varid, cz ) )
     
     call check( nf90_inq_varid(ncid, 'CX', varid) )
     call check( nf90_get_var( ncid, varid, cx ) )
     
     call check( nf90_inq_varid(ncid, 'CY', varid) )
     call check( nf90_get_var( ncid, varid, cy ) )

     call check( nf90_close( ncid ) )
  enddo

  do k_file = 1, TOTAL_FILE_NUMBER
     call check( nf90_open( trim(FILE_DIR(k_file))//trim(FILE_NAME(1)), nf90_nowrite, ncid ) )
     call check( nf90_inq_dimid ( ncid,  'time', varid ) )
     call check( nf90_inquire_dimension( ncid, varid, len=nt_partial(k_file) ) )

     call check( nf90_close( ncid ) )

     if (k_file > 1) nt_partial(k_file) = nt_partial(k_file) - 1
     if (irank == master) write(*,*) k_file, "NT PARTIAL: ", nt_partial(k_file)
  enddo

  nt = sum(nt_partial(:))
  allocate(ct(nt))

  i = 1
  j = 0
  do k_file = 1, TOTAL_FILE_NUMBER
     call check( nf90_open( trim(FILE_DIR(k_file))//trim(FILE_NAME(1)), nf90_nowrite, ncid ) )
     if (k_file > 1) then
        k = 2
     else
        k = 1
     endif
     j = i + nt_partial(k_file) - 1
     call check( nf90_inq_varid(ncid, 'time', varid) )
     call check( nf90_get_var( ncid, varid, ct(i:j), start=(/k/), count=(/nt_partial(k_file)/) ) )
     i = i + nt_partial(k_file)

     call check( nf90_close( ncid ) )
  enddo

  if (TOTAL_FILE_NUMBER > 1) then
     do t =  nt_partial(1)+1, nt
        ct(t) = 2.0E0*ct(t-1) - ct(t-2)
     enddo
  endif

  if (irank == master) then
     write(*,*) "NFZ NFX NX NY NZ NT: ", nfz, nfx, nx, ny, nz, nt
  endif

  !if (irank == 1 .or. irank == master) then
  !   write(*,*) "   -----CX----- "
  !   do i = 1, nx
  !      write(*,'(2I3,E15.6)') irank, i, cx(i)
  !   enddo
  !   write(*,*)
  !endif
  !call PRC_abort

  if (irank == master .and. debug_output) then
     write(*,*) "NFZ NFX NX NY NZ NT: ", nfz, nfx, nx, ny, nz, nt
     do k = 1, 1!JOBS_PER_MPI
        write(*,*) "   -----FZ----- "
        do i = 1, nfz
           write(*,'(I3,E15.6)') i, fz(i)
        enddo
        write(*,*)
        write(*,*) "   -----FX----- "
        do i = 1, nfx
           write(*,'(I3,E15.6)') i, fx(i)
        enddo
        write(*,*)
        write(*,*) "   -----CX----- "
        do i = 1, nx
           write(*,'(I3,E15.6)') i, cx(i)
        enddo
        write(*,*)
        write(*,*) "   -----CY----- "
        do i = 1, ny
           write(*,'(I3,E15.6)') i, cy(i)
        enddo
        write(*,*)
        write(*,*) "   -----CZ----- "
        do i = 1, nz
           write(*,'(I3,E15.6)') i, cz(i)
        enddo
        write(*,*)
        write(*,*) "   -----CT----- "
        do i = 1, nt
           write(*,'(I3,E15.6)') i, ct(i)
        enddo
     enddo
  endif


  isg = 1
  jsg = 1
  ieg = IMAX*JOBS_PER_MPI_X
  jeg = JMAX*JOBS_PER_MPI_X
  sendcount_median = IMAX*JMAX*JOBS_PER_MPI ! for x-y array
  count_global_median = IMAX*JMAX*TOTAL_PRC ! for x-y array
  !sendcount_median = IMAX*JMAX*KMAX*JOBS_PER_MPI ! for x-y-z array
  !count_global_median = IMAX*JMAX*KMAX*TOTAL_PRC ! for x-y-z array
  allocate(qv(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX))
  allocate(qliq(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX))
  allocate(qhyd(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX))
  allocate(qice(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX))
  allocate(dens(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX))
  allocate(pressure(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX))
  allocate(u(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX))
  allocate(v(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX))
  allocate(w(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX))
  allocate(rh(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX))
  allocate(rhi(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX))
  allocate(sink(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX))
  allocate(lwdn(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX))
  allocate(swdn(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX))
  allocate(rhoh_rd(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX))
  allocate(rhoh_mp(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX))
  allocate(temperature(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX))
  allocate(potential(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX))
  allocate(ice_mass_vapor(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX))
  allocate(ice_mass_eva(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX))
  allocate(ice_mass_imm(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX))
  allocate(ice_mass_rim(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX))
  allocate(bin_liq_con(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX,max_binr))
  allocate(bin_ice_con(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX,max_bini))
  allocate(bin_ice_mcry(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX,max_bini))
  allocate(bin_ice_mrim(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX,max_bini))
  allocate(bin_ice_magg(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX,max_bini))
  allocate(bin_ice_mass(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX,max_bini))
  allocate(bin_ice_alen(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX,max_bini))
  allocate(bin_ice_clen(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX,max_bini))
  allocate(bin_ice_dlen(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX,max_bini))
  allocate(bin_ice_Dmax(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX,max_bini))
  if (poly_exist) then
     allocate(bin_ice_aglen(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX,max_bini))
     allocate(bin_ice_cglen(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX,max_bini))
     allocate(bin_ice_nex(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX,max_bini))
  endif

  ! liquid number concentration
  allocate(nliqall1d(nt,KMAX))
  allocate(nliqall1d1(nt,KMAX))
  allocate(nliqall1d9(nt,KMAX))
  allocate(ini_nliqall1d(KMAX))
  ! ice number concentration
  allocate(niceall1d(nt,KMAX))
  allocate(niceall1d1(nt,KMAX))
  allocate(niceall1d9(nt,KMAX))
  ! ice number concentration > 200 um
  allocate(nice200all1d(nt,KMAX))
  allocate(nice200all1d1(nt,KMAX))
  allocate(nice200all1d9(nt,KMAX))
  ! liquid number concentration
  allocate(nliq1d(nt,KMAX))
  allocate(nliq1d1(nt,KMAX))
  allocate(nliq1d9(nt,KMAX))
  allocate(ini_nliq1d(KMAX))
  ! ice number concentration
  allocate(nice1d(nt,KMAX))
  allocate(nice1d1(nt,KMAX))
  allocate(nice1d9(nt,KMAX))
  ! ice number concentration > 200 um
  allocate(nice2001d(nt,KMAX))
  allocate(nice2001d1(nt,KMAX))
  allocate(nice2001d9(nt,KMAX))
  ! liquid effective diameter
  allocate(dliq1d(nt,KMAX))
  allocate(dliq1d1(nt,KMAX))
  allocate(dliq1d9(nt,KMAX))
  ! ice effective diameter (maximum dimension)
  allocate(dice1d(nt,KMAX))
  allocate(dice1d1(nt,KMAX))
  allocate(dice1d9(nt,KMAX))
  ! pressure profile
  allocate(P1d(nt,KMAX))
  allocate(P1d1(nt,KMAX))
  allocate(P1d9(nt,KMAX))
  allocate(ini_P1d(KMAX))
  ! density profile
  allocate(dens1d(nt,KMAX))
  allocate(dens1d1(nt,KMAX))
  allocate(dens1d9(nt,KMAX))
  allocate(ini_dens1d(KMAX))
  ! temperature profile
  allocate(T1d(nt,KMAX))
  allocate(T1d1(nt,KMAX))
  allocate(T1d9(nt,KMAX))
  allocate(ini_T1d(KMAX))
  ! potential temperature profile
  allocate(ini_PT1d(KMAX))
  ! liquid mixing ratio profile
  allocate(ini_qliq1d(KMAX))
  ! downwelling longwave profile
  allocate(ini_lwdn1d(KMAX))
  ! downwelling shortwave profile
  allocate(ini_swdn1d(KMAX))
  ! subsidence profile
  allocate(ini_sink1d(KMAX))
  ! rhoh rd profile
  allocate(ini_rhoh_rd1d(KMAX))
  ! rhoh mp profile
  allocate(ini_rhoh_mp1d(KMAX))
  ! liquid water content profile
  allocate(lwc1d(nt,KMAX))
  allocate(lwc1d1(nt,KMAX))
  allocate(lwc1d9(nt,KMAX))
  allocate(ini_lwc1d(KMAX))
  ! ice water content profile
  allocate(iwc1d(nt,KMAX))
  allocate(iwc1d1(nt,KMAX))
  allocate(iwc1d9(nt,KMAX))
  allocate(iwccry1d(nt,KMAX))
  allocate(iwccry1d1(nt,KMAX))
  allocate(iwccry1d9(nt,KMAX))
  allocate(iwcrcr1d(nt,KMAX))
  allocate(iwcrcr1d1(nt,KMAX))
  allocate(iwcrcr1d9(nt,KMAX))
  allocate(iwcgra1d(nt,KMAX))
  allocate(iwcgra1d1(nt,KMAX))
  allocate(iwcgra1d9(nt,KMAX))
  allocate(iwccryrcr1d(nt,KMAX))
  allocate(iwccryrcr1d1(nt,KMAX))
  allocate(iwccryrcr1d9(nt,KMAX))
  allocate(iwcagg1d(nt,KMAX))
  allocate(iwcagg1d1(nt,KMAX))
  allocate(iwcagg1d9(nt,KMAX))
  allocate(iwcrag1d(nt,KMAX))
  allocate(iwcrag1d1(nt,KMAX))
  allocate(iwcrag1d9(nt,KMAX))
  ! qv profile
  allocate(qv1d(nt,KMAX))
  allocate(qv1d1(nt,KMAX))
  allocate(qv1d9(nt,KMAX))
  allocate(ini_qv1d(KMAX))
  ! wind speed profile
  allocate(u1d(nt,KMAX))
  allocate(u1d1(nt,KMAX))
  allocate(u1d9(nt,KMAX))
  allocate(ini_u1d(KMAX))
  ! RH profile
  allocate(rh1d(nt,KMAX))
  allocate(rh1d1(nt,KMAX))
  allocate(rh1d9(nt,KMAX))
  allocate(ini_rh1d(KMAX))
  ! RHI profile
  allocate(rhi1d(nt,KMAX))
  allocate(rhi1d1(nt,KMAX))
  allocate(rhi1d9(nt,KMAX))
  allocate(ini_rhi1d(KMAX))
  ! TKE profile
  allocate(tke1d(nt,KMAX))
  allocate(tke1d1(nt,KMAX))
  allocate(tke1d9(nt,KMAX))
  ! ice types
  allocate(agg1d(nt,KMAX))
  allocate(cry1d(nt,KMAX))
  allocate(rag1d(nt,KMAX))
  allocate(rcr1d(nt,KMAX))
  allocate(gra1d(nt,KMAX))
  allocate(agg1d1(nt,KMAX))
  allocate(cry1d1(nt,KMAX))
  allocate(rag1d1(nt,KMAX))
  allocate(rcr1d1(nt,KMAX))
  allocate(gra1d1(nt,KMAX))
  allocate(agg1d9(nt,KMAX))
  allocate(cry1d9(nt,KMAX))
  allocate(rag1d9(nt,KMAX))
  allocate(rcr1d9(nt,KMAX))
  allocate(gra1d9(nt,KMAX))
  allocate(aggall1d(nt,KMAX))
  allocate(cryall1d(nt,KMAX))
  allocate(ragall1d(nt,KMAX))
  allocate(rcrall1d(nt,KMAX))
  allocate(graall1d(nt,KMAX))
  allocate(aggall1d1(nt,KMAX))
  allocate(cryall1d1(nt,KMAX))
  allocate(ragall1d1(nt,KMAX))
  allocate(rcrall1d1(nt,KMAX))
  allocate(graall1d1(nt,KMAX))
  allocate(aggall1d9(nt,KMAX))
  allocate(cryall1d9(nt,KMAX))
  allocate(ragall1d9(nt,KMAX))
  allocate(rcrall1d9(nt,KMAX))
  allocate(graall1d9(nt,KMAX))
  ! ice habits
  allocate(monopl1d(nt,KMAX))
  allocate(monodd1d(nt,KMAX))
  allocate(monocl1d(nt,KMAX))
  allocate(monopl1d1(nt,KMAX))
  allocate(monodd1d1(nt,KMAX))
  allocate(monocl1d1(nt,KMAX))
  allocate(monopl1d9(nt,KMAX))
  allocate(monodd1d9(nt,KMAX))
  allocate(monocl1d9(nt,KMAX))
  allocate(polypl1d(nt,KMAX))
  allocate(polyir1d(nt,KMAX))
  allocate(polycl1d(nt,KMAX))
  allocate(polypl1d1(nt,KMAX))
  allocate(polyir1d1(nt,KMAX))
  allocate(polycl1d1(nt,KMAX))
  allocate(polypl1d9(nt,KMAX))
  allocate(polyir1d9(nt,KMAX))
  allocate(polycl1d9(nt,KMAX))
  ! microphysics process
  allocate(rmrim1d(nt,KMAX))
  allocate(rmrim1d1(nt,KMAX))
  allocate(rmrim1d9(nt,KMAX))
  allocate(rmvap1d(nt,KMAX))
  allocate(rmvap1d1(nt,KMAX))
  allocate(rmvap1d9(nt,KMAX))
  allocate(rmimm1d(nt,KMAX))
  allocate(rmimm1d1(nt,KMAX))
  allocate(rmimm1d9(nt,KMAX))


  T1d(:,:) = 0.0E0
  T1d1(:,:) = 0.0E0
  T1d9(:,:) = 0.0E0
  P1d(:,:) = 0.0E0
  P1d1(:,:) = 0.0E0
  P1d9(:,:) = 0.0E0
  dens1d(:,:) = 0.0E0
  dens1d1(:,:) = 0.0E0
  dens1d9(:,:) = 0.0E0
  rh1d(:,:) = 0.0E0
  rh1d1(:,:) = 0.0E0
  rh1d9(:,:) = 0.0E0
  rhi1d(:,:) = 0.0E0
  rhi1d1(:,:) = 0.0E0
  rhi1d9(:,:) = 0.0E0
  monopl1d(:,:) = 0.0E0
  monopl1d1(:,:) = 0.0E0
  monopl1d9(:,:) = 0.0E0
  monocl1d(:,:) = 0.0E0
  monocl1d1(:,:) = 0.0E0
  monocl1d9(:,:) = 0.0E0
  monodd1d(:,:) = 0.0E0
  monodd1d1(:,:) = 0.0E0
  monodd1d9(:,:) = 0.0E0
  polypl1d(:,:) = 0.0E0
  polypl1d1(:,:) = 0.0E0
  polypl1d9(:,:) = 0.0E0
  polycl1d(:,:) = 0.0E0
  polycl1d1(:,:) = 0.0E0
  polycl1d9(:,:) = 0.0E0
  polyir1d(:,:) = 0.0E0
  polyir1d1(:,:) = 0.0E0
  polyir1d9(:,:) = 0.0E0
  qv1d(:,:) = 0.0E0
  qv1d1(:,:) = 0.0E0
  qv1d9(:,:) = 0.0E0
  u1d(:,:) = 0.0E0
  u1d1(:,:) = 0.0E0
  u1d9(:,:) = 0.0E0
  lwc1d(:,:) = 0.0E0
  lwc1d1(:,:) = 0.0E0
  lwc1d9(:,:) = 0.0E0
  iwc1d(:,:) = 0.0E0
  iwc1d1(:,:) = 0.0E0
  iwc1d9(:,:) = 0.0E0
  iwccry1d(:,:) = 0.0E0
  iwccry1d1(:,:) = 0.0E0
  iwccry1d9(:,:) = 0.0E0
  iwcrcr1d(:,:) = 0.0E0
  iwcrcr1d1(:,:) = 0.0E0
  iwcrcr1d9(:,:) = 0.0E0
  iwcgra1d(:,:) = 0.0E0
  iwcgra1d1(:,:) = 0.0E0
  iwcgra1d9(:,:) = 0.0E0
  iwccryrcr1d(:,:) = 0.0E0
  iwccryrcr1d1(:,:) = 0.0E0
  iwccryrcr1d9(:,:) = 0.0E0
  iwcagg1d(:,:) = 0.0E0
  iwcagg1d1(:,:) = 0.0E0
  iwcagg1d9(:,:) = 0.0E0
  iwcrag1d(:,:) = 0.0E0
  iwcrag1d1(:,:) = 0.0E0
  iwcrag1d9(:,:) = 0.0E0
  tke1d(:,:) = 0.0E0
  tke1d1(:,:) = 0.0E0
  tke1d9(:,:) = 0.0E0
  cry1d(:,:) = 0.0E0
  cry1d1(:,:) = 0.0E0
  cry1d9(:,:) = 0.0E0
  rmrim1d(:,:) = 0.0E0
  rmrim1d1(:,:) = 0.0E0
  rmrim1d9(:,:) = 0.0E0
  rmimm1d(:,:) = 0.0E0
  rmimm1d1(:,:) = 0.0E0
  rmimm1d9(:,:) = 0.0E0
  rmvap1d(:,:) = 0.0E0
  rmvap1d1(:,:) = 0.0E0
  rmvap1d9(:,:) = 0.0E0
  agg1d(:,:) = 0.0E0
  agg1d1(:,:) = 0.0E0
  agg1d9(:,:) = 0.0E0
  gra1d(:,:) = 0.0E0
  gra1d1(:,:) = 0.0E0
  gra1d9(:,:) = 0.0E0
  rcr1d(:,:) = 0.0E0
  rcr1d1(:,:) = 0.0E0
  rcr1d9(:,:) = 0.0E0
  rag1d(:,:) = 0.0E0
  rag1d1(:,:) = 0.0E0
  rag1d9(:,:) = 0.0E0
  cryall1d(:,:) = 0.0E0
  cryall1d1(:,:) = 0.0E0
  cryall1d9(:,:) = 0.0E0
  aggall1d(:,:) = 0.0E0
  aggall1d1(:,:) = 0.0E0
  aggall1d9(:,:) = 0.0E0
  graall1d(:,:) = 0.0E0
  graall1d1(:,:) = 0.0E0
  graall1d9(:,:) = 0.0E0
  rcrall1d(:,:) = 0.0E0
  rcrall1d1(:,:) = 0.0E0
  rcrall1d9(:,:) = 0.0E0
  ragall1d(:,:) = 0.0E0
  ragall1d1(:,:) = 0.0E0
  ragall1d9(:,:) = 0.0E0

  allocate(local_var_T(sendcount_median))
  allocate(local_var_P(sendcount_median))
  allocate(local_var_dens(sendcount_median))
  allocate(local_var_rh(sendcount_median))
  allocate(local_var_rhi(sendcount_median))
  allocate(local_var_qv(sendcount_median))
  allocate(local_var_u(sendcount_median))
  allocate(local_var_tke(sendcount_median))
  allocate(local_var_lwc(sendcount_median))
  allocate(local_var_iwc(sendcount_median))
  allocate(local_var_iwccry(sendcount_median))
  allocate(local_var_iwcrcr(sendcount_median))
  allocate(local_var_iwcgra(sendcount_median))
  allocate(local_var_iwccryrcr(sendcount_median))
  allocate(local_var_iwcagg(sendcount_median))
  allocate(local_var_iwcrag(sendcount_median))
  allocate(local_var_nliqall(sendcount_median))
  allocate(local_var_niceall(sendcount_median))
  allocate(local_var_nice200all(sendcount_median))
  allocate(local_var_nliq(sendcount_median))
  allocate(local_var_nice(sendcount_median))
  allocate(local_var_nice200(sendcount_median))
  allocate(local_var_dliq(sendcount_median))
  allocate(local_var_dice(sendcount_median))
  allocate(local_var_cryall(sendcount_median))
  allocate(local_var_aggall(sendcount_median))
  allocate(local_var_ragall(sendcount_median))
  allocate(local_var_graall(sendcount_median))
  allocate(local_var_rcrall(sendcount_median))
  allocate(local_var_cry(sendcount_median))
  allocate(local_var_agg(sendcount_median))
  allocate(local_var_rag(sendcount_median))
  allocate(local_var_gra(sendcount_median))
  allocate(local_var_rcr(sendcount_median))
  allocate(local_var_monopl(sendcount_median))
  allocate(local_var_monocl(sendcount_median))
  allocate(local_var_monodd(sendcount_median))
  allocate(local_var_polypl(sendcount_median))
  allocate(local_var_polycl(sendcount_median))
  allocate(local_var_polyir(sendcount_median))
  allocate(local_var_rmrim(sendcount_median))
  allocate(local_var_rmimm(sendcount_median))
  allocate(local_var_rmvap(sendcount_median))

  allocate(global_var_T(count_global_median))
  allocate(global_var_P(count_global_median))
  allocate(global_var_dens(count_global_median))
  allocate(global_var_rh(count_global_median))
  allocate(global_var_rhi(count_global_median))
  allocate(global_var_qv(count_global_median))
  allocate(global_var_u(count_global_median))
  allocate(global_var_tke(count_global_median))
  allocate(global_var_lwc(count_global_median))
  allocate(global_var_iwc(count_global_median))
  allocate(global_var_iwccry(count_global_median))
  allocate(global_var_iwcrcr(count_global_median))
  allocate(global_var_iwcgra(count_global_median))
  allocate(global_var_iwccryrcr(count_global_median))
  allocate(global_var_iwcagg(count_global_median))
  allocate(global_var_iwcrag(count_global_median))
  allocate(global_var_nliq(count_global_median))
  allocate(global_var_nice(count_global_median))
  allocate(global_var_nice200(count_global_median))
  allocate(global_var_nliqall(count_global_median))
  allocate(global_var_niceall(count_global_median))
  allocate(global_var_nice200all(count_global_median))
  allocate(global_var_dliq(count_global_median))
  allocate(global_var_dice(count_global_median))
  allocate(global_var_cryall(count_global_median))
  allocate(global_var_aggall(count_global_median))
  allocate(global_var_ragall(count_global_median))
  allocate(global_var_graall(count_global_median))
  allocate(global_var_rcrall(count_global_median))
  allocate(global_var_cry(count_global_median))
  allocate(global_var_agg(count_global_median))
  allocate(global_var_rag(count_global_median))
  allocate(global_var_gra(count_global_median))
  allocate(global_var_rcr(count_global_median))
  allocate(global_var_monopl(count_global_median))
  allocate(global_var_monocl(count_global_median))
  allocate(global_var_monodd(count_global_median))
  allocate(global_var_polypl(count_global_median))
  allocate(global_var_polycl(count_global_median))
  allocate(global_var_polyir(count_global_median))
  allocate(global_var_rmrim(count_global_median))
  allocate(global_var_rmimm(count_global_median))
  allocate(global_var_rmvap(count_global_median))


  k_file = 1
  t_local = 0
  ts = 0
  do t = 1, nt

     ts = ts + 1
     t_local = t_local + 1
     if (t_local > nt_partial(k_file)) then
        k_file = k_file + 1
        t_local = 1
        if (k_file > 1) then
           ts = 2
        else
           ts = 1
        endif
     endif

     if (t == 1) then

        if (irank == master) print *, "current T: ", t, t_local, k_file

        ! read netcdf variables
        do k = 1, JOBS_PER_MPI

           if ( FILE_HISTORY_CARTESC_BOUNDARY ) then
              write(*,*) "still under construction..."
              call PRC_abort
           else
              is = 1 + IMAX*mod(k-1,JOBS_PER_MPI_X)
              js = 1 + JMAX*int((k-1)/JOBS_PER_MPI_X)
              ks = 1
              ie = IMAX + IMAX*mod(k-1,JOBS_PER_MPI_X)
              je = JMAX + JMAX*int((k-1)/JOBS_PER_MPI_X)
              ke = KMAX
           endif

           call check( nf90_open( trim(FILE_DIR(k_file))//trim(FILE_NAME(k)), nf90_nowrite, ncid ) )

           start_3d(1:4) = (/ 1, 1, 1, ts /)
           count_3d(1:4) = (/ IMAX, JMAX, KMAX, 1  /)
           varname = "QLIQ"
           call check( nf90_inq_varid(ncid, trim(varname), varid) )
           call check( nf90_get_var( ncid, varid, qliq(is:ie,js:je,ks:ke), start=start_3d, count=count_3d ) )

           varname = "QHYD"
           call check( nf90_inq_varid(ncid, trim(varname), varid) )
           call check( nf90_get_var( ncid, varid, qhyd(is:ie,js:je,ks:ke), start=start_3d, count=count_3d ) )

           varname = "QV"
           call check( nf90_inq_varid(ncid, trim(varname), varid) )
           call check( nf90_get_var( ncid, varid, qv(is:ie,js:je,ks:ke), start=start_3d, count=count_3d ) )
     
           varname = "DENS"
           call check( nf90_inq_varid(ncid, trim(varname), varid) )
           call check( nf90_get_var( ncid, varid, dens(is:ie,js:je,ks:ke), start=start_3d, count=count_3d ) )

           varname = "PRES"
           call check( nf90_inq_varid(ncid, trim(varname), varid) )
           call check( nf90_get_var( ncid, varid, pressure(is:ie,js:je,ks:ke), start=start_3d, count=count_3d ) )

           varname = "T"
           call check( nf90_inq_varid(ncid, trim(varname), varid) )
           call check( nf90_get_var( ncid, varid, temperature(is:ie,js:je,ks:ke), start=start_3d, count=count_3d ) )

           varname = "PT"
           call check( nf90_inq_varid(ncid, trim(varname), varid) )
           call check( nf90_get_var( ncid, varid, potential(is:ie,js:je,ks:ke), start=start_3d, count=count_3d ) )

           varname = "U"
           call check( nf90_inq_varid(ncid, trim(varname), varid) )
           call check( nf90_get_var( ncid, varid, u(is:ie,js:je,ks:ke), start=start_3d, count=count_3d ) )

           varname = "V"
           call check( nf90_inq_varid(ncid, trim(varname), varid) )
           call check( nf90_get_var( ncid, varid, v(is:ie,js:je,ks:ke), start=start_3d, count=count_3d ) )

           varname = "RH"
           call check( nf90_inq_varid(ncid, trim(varname), varid) )
           call check( nf90_get_var( ncid, varid, rh(is:ie,js:je,ks:ke), start=start_3d, count=count_3d ) )

           varname = "RHI"
           call check( nf90_inq_varid(ncid, trim(varname), varid) )
           call check( nf90_get_var( ncid, varid, rhi(is:ie,js:je,ks:ke), start=start_3d, count=count_3d ) )

           varname = "RADFLUX_LWDN"
           call check( nf90_inq_varid(ncid, trim(varname), varid) )
           call check( nf90_get_var( ncid, varid, lwdn(is:ie,js:je,ks:ke), start=start_3d, count=count_3d ) )

           varname = "RADFLUX_SWDN"
           call check( nf90_inq_varid(ncid, trim(varname), varid) )
           call check( nf90_get_var( ncid, varid, swdn(is:ie,js:je,ks:ke), start=start_3d, count=count_3d ) )

           varname = "subsidence_sink"
           call check( nf90_inq_varid(ncid, trim(varname), varid) )
           call check( nf90_get_var( ncid, varid, sink(is:ie,js:je,ks:ke), start=start_3d, count=count_3d ) )

           varname = "RHOH_RD"
           call check( nf90_inq_varid(ncid, trim(varname), varid) )
           call check( nf90_get_var( ncid, varid, rhoh_rd(is:ie,js:je,ks:ke), start=start_3d, count=count_3d ) )

           varname = "RHOH_MP"
           call check( nf90_inq_varid(ncid, trim(varname), varid) )
           call check( nf90_get_var( ncid, varid, rhoh_mp(is:ie,js:je,ks:ke), start=start_3d, count=count_3d ) )

           do i = 1, max_binr
              if (i < 10) then
                 write(word,'(I1)') i
              else
                 write(word,'(I2)') i
              endif
              varname = "con_lmass"//trim(word)
              call check( nf90_inq_varid(ncid, trim(varname), varid) )
              call check( nf90_get_var( ncid, varid, bin_liq_con(is:ie,js:je,ks:ke,i), start=start_3d, count=count_3d ) )
           enddo

        enddo

        ! post process

        ! transform all density multiplied quantities back to normal form
        ! calculate qice
        do i = isg, ieg
           do j = jsg, jeg
              do k = ks, ke
                 do ibin = 1, max_binr
                    bin_liq_con(i,j,k,ibin) = bin_liq_con(i,j,k,ibin)*dens(i,j,k)
                 enddo
              enddo
           enddo
        enddo


        ! retrieve average and median
        do k = ks, ke

           temp1 = 0.0E0
           temp2 = 0.0E0
           temp3 = 0.0E0
           temp4 = 0.0E0
           temp5 = 0.0E0
           temp6 = 0.0E0
           temp7 = 0.0E0
           temp8 = 0.0E0
           temp9 = 0.0E0
           temp10 = 0.0E0
           temp11 = 0.0E0
           temp12 = 0.0E0
           temp13 = 0.0E0
           temp14 = 0.0E0
           temp15 = 0.0E0
           temp16 = 0.0E0

           do i = isg, ieg
              do j = jsg, jeg
                 temp1 = temp1 + temperature(i,j,k)
                 temp2 = temp2 + sqrt(u(i,j,k)**2 + v(i,j,k)**2)
                 temp3 = temp3 + rh(i,j,k)
                 temp4 = temp4 + rhi(i,j,k)
                 temp5 = temp5 + dens(i,j,k)*qliq(i,j,k)*1000.0
                 temp6 = temp6 + qliq(i,j,k)
                 temp7 = temp7 + qv(i,j,k)
                 temp8 = temp8 + dens(i,j,k)
                 temp9 = temp9 + rhoh_rd(i,j,k)
                 temp10 = temp10 + rhoh_mp(i,j,k)
                 temp11 = temp11 + sink(i,j,k)
                 temp12 = temp12 + potential(i,j,k)
                 temp13 = temp13 + lwdn(i,j,k)
                 temp14 = temp14 + swdn(i,j,k)
                 do ibin = 1, max_binr
                    temp15 = temp15 + bin_liq_con(i,j,k,ibin)
                 enddo
                 temp16 = temp16 + pressure(i,j,k)
              enddo
           enddo

           call MPI_REDUCE( temp1, global_sum_T, 1, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
           call MPI_REDUCE( temp2, global_sum_u, 1, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
           call MPI_REDUCE( temp3, global_sum_rh, 1, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
           call MPI_REDUCE( temp4, global_sum_rhi, 1, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
           call MPI_REDUCE( temp5, global_sum_lwc, 1, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
           call MPI_REDUCE( temp6, global_sum_qliq, 1, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
           call MPI_REDUCE( temp7, global_sum_qv, 1, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
           call MPI_REDUCE( temp8, global_sum_dens, 1, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
           call MPI_REDUCE( temp9, global_sum_rhoh_rd, 1, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
           call MPI_REDUCE( temp10, global_sum_rhoh_mp, 1, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
           call MPI_REDUCE( temp11, global_sum_sink, 1, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
           call MPI_REDUCE( temp12, global_sum_PT, 1, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
           call MPI_REDUCE( temp13, global_sum_lwdn, 1, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
           call MPI_REDUCE( temp14, global_sum_swdn, 1, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
           call MPI_REDUCE( temp15, global_sum_nliq, 1, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
           call MPI_REDUCE( temp16, global_sum_P, 1, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )

           if (irank == master) then
              ini_T1d(k) = global_sum_T/real(IMAX)/real(JMAX)/real(TOTAL_PRC)
              ini_P1d(k) = global_sum_P/real(IMAX)/real(JMAX)/real(TOTAL_PRC)
              ini_u1d(k) = global_sum_u/real(IMAX)/real(JMAX)/real(TOTAL_PRC)
              ini_rh1d(k) = global_sum_rh/real(IMAX)/real(JMAX)/real(TOTAL_PRC)
              ini_rhi1d(k) = global_sum_rhi/real(IMAX)/real(JMAX)/real(TOTAL_PRC)
              ini_lwc1d(k) = global_sum_lwc/real(IMAX)/real(JMAX)/real(TOTAL_PRC)
              ini_qliq1d(k) = global_sum_qliq/real(IMAX)/real(JMAX)/real(TOTAL_PRC)
              ini_dens1d(k) = global_sum_dens/real(IMAX)/real(JMAX)/real(TOTAL_PRC)
              ini_sink1d(k) = global_sum_sink/real(IMAX)/real(JMAX)/real(TOTAL_PRC)
              ini_lwdn1d(k) = global_sum_lwdn/real(IMAX)/real(JMAX)/real(TOTAL_PRC)
              ini_swdn1d(k) = global_sum_swdn/real(IMAX)/real(JMAX)/real(TOTAL_PRC)
              ini_qv1d(k) = global_sum_qv/real(IMAX)/real(JMAX)/real(TOTAL_PRC)
              ini_rhoh_rd1d(k) = global_sum_rhoh_rd/real(IMAX)/real(JMAX)/real(TOTAL_PRC)
              ini_rhoh_mp1d(k) = global_sum_rhoh_mp/real(IMAX)/real(JMAX)/real(TOTAL_PRC)
              ini_PT1d(k) = global_sum_PT/real(IMAX)/real(JMAX)/real(TOTAL_PRC)
              ini_nliq1d(k) = global_sum_nliq/real(IMAX)/real(JMAX)/real(TOTAL_PRC)
           endif

        enddo

     endif

     if (t >= 145) then

        if (irank == master) print *, "current T: ", t, t_local, k_file

        ! read netcdf variables
        do k = 1, JOBS_PER_MPI

           if ( FILE_HISTORY_CARTESC_BOUNDARY ) then
              write(*,*) "still under construction..."
              call PRC_abort
           else
              is = 1 + IMAX*mod(k-1,JOBS_PER_MPI_X)
              js = 1 + JMAX*int((k-1)/JOBS_PER_MPI_X)
              ks = 1
              ie = IMAX + IMAX*mod(k-1,JOBS_PER_MPI_X)
              je = JMAX + JMAX*int((k-1)/JOBS_PER_MPI_X)
              ke = KMAX
           endif

           call check( nf90_open( trim(FILE_DIR(k_file))//trim(FILE_NAME(k)), nf90_nowrite, ncid ) )

           start_3d(1:4) = (/ 1, 1, 1, ts /)
           count_3d(1:4) = (/ IMAX, JMAX, KMAX, 1  /)
           varname = "QLIQ"
           call check( nf90_inq_varid(ncid, trim(varname), varid) )
           call check( nf90_get_var( ncid, varid, qliq(is:ie,js:je,ks:ke), start=start_3d, count=count_3d ) )

           varname = "QHYD"
           call check( nf90_inq_varid(ncid, trim(varname), varid) )
           call check( nf90_get_var( ncid, varid, qhyd(is:ie,js:je,ks:ke), start=start_3d, count=count_3d ) )

           varname = "QV"
           call check( nf90_inq_varid(ncid, trim(varname), varid) )
           call check( nf90_get_var( ncid, varid, qv(is:ie,js:je,ks:ke), start=start_3d, count=count_3d ) )
     
           varname = "DENS"
           call check( nf90_inq_varid(ncid, trim(varname), varid) )
           call check( nf90_get_var( ncid, varid, dens(is:ie,js:je,ks:ke), start=start_3d, count=count_3d ) )

           varname = "T"
           call check( nf90_inq_varid(ncid, trim(varname), varid) )
           call check( nf90_get_var( ncid, varid, temperature(is:ie,js:je,ks:ke), start=start_3d, count=count_3d ) )

           varname = "U"
           call check( nf90_inq_varid(ncid, trim(varname), varid) )
           call check( nf90_get_var( ncid, varid, u(is:ie,js:je,ks:ke), start=start_3d, count=count_3d ) )

           varname = "V"
           call check( nf90_inq_varid(ncid, trim(varname), varid) )
           call check( nf90_get_var( ncid, varid, v(is:ie,js:je,ks:ke), start=start_3d, count=count_3d ) )

           varname = "W"
           call check( nf90_inq_varid(ncid, trim(varname), varid) )
           call check( nf90_get_var( ncid, varid, w(is:ie,js:je,ks:ke), start=start_3d, count=count_3d ) )

           varname = "RH"
           call check( nf90_inq_varid(ncid, trim(varname), varid) )
           call check( nf90_get_var( ncid, varid, rh(is:ie,js:je,ks:ke), start=start_3d, count=count_3d ) )

           varname = "RHI"
           call check( nf90_inq_varid(ncid, trim(varname), varid) )
           call check( nf90_get_var( ncid, varid, rhi(is:ie,js:je,ks:ke), start=start_3d, count=count_3d ) )

           varname = "ampsi_vapor"
           call check( nf90_inq_varid(ncid, trim(varname), varid) )
           call check( nf90_get_var( ncid, varid, ice_mass_vapor(is:ie,js:je,ks:ke), start=start_3d, count=count_3d ) )

           varname = "ampsi_eva"
           call check( nf90_inq_varid(ncid, trim(varname), varid) )
           call check( nf90_get_var( ncid, varid, ice_mass_eva(is:ie,js:je,ks:ke), start=start_3d, count=count_3d ) )

           varname = "ampsi_rim"
           call check( nf90_inq_varid(ncid, trim(varname), varid) )
           call check( nf90_get_var( ncid, varid, ice_mass_rim(is:ie,js:je,ks:ke), start=start_3d, count=count_3d ) )

           varname = "ampsi_imm"
           call check( nf90_inq_varid(ncid, trim(varname), varid) )
           call check( nf90_get_var( ncid, varid, ice_mass_imm(is:ie,js:je,ks:ke), start=start_3d, count=count_3d ) )

           do i = 1, max_binr
              if (i < 10) then
                 write(word,'(I1)') i
              else
                 write(word,'(I2)') i
              endif
              varname = "con_lmass"//trim(word)
              call check( nf90_inq_varid(ncid, trim(varname), varid) )
              call check( nf90_get_var( ncid, varid, bin_liq_con(is:ie,js:je,ks:ke,i), start=start_3d, count=count_3d ) )
           enddo

           do i = 1, max_bini
              if (i < 10) then
                 write(word,'(I1)') i
              else
                 write(word,'(I2)') i
              endif
              varname = "con_imass"//trim(word)
              call check( nf90_inq_varid(ncid, trim(varname), varid) )
              call check( nf90_get_var( ncid, varid, bin_ice_con(is:ie,js:je,ks:ke,i), start=start_3d, count=count_3d ) )

              varname = "ice_imass"//trim(word)
              call check( nf90_inq_varid(ncid, trim(varname), varid) )
              call check( nf90_get_var( ncid, varid, bin_ice_mass(is:ie,js:je,ks:ke,i), start=start_3d, count=count_3d ) )

              varname = "cry_imass"//trim(word)
              call check( nf90_inq_varid(ncid, trim(varname), varid) )
              call check( nf90_get_var( ncid, varid, bin_ice_mcry(is:ie,js:je,ks:ke,i), start=start_3d, count=count_3d ) )

              varname = "rim_imass"//trim(word)
              call check( nf90_inq_varid(ncid, trim(varname), varid) )
              call check( nf90_get_var( ncid, varid, bin_ice_mrim(is:ie,js:je,ks:ke,i), start=start_3d, count=count_3d ) )

              varname = "agg_imass"//trim(word)
              call check( nf90_inq_varid(ncid, trim(varname), varid) )
              call check( nf90_get_var( ncid, varid, bin_ice_magg(is:ie,js:je,ks:ke,i), start=start_3d, count=count_3d ) )

              varname = "vol_imass"//trim(word)
              call check( nf90_inq_varid(ncid, trim(varname), varid) )
              call check( nf90_get_var( ncid, varid, bin_ice_Dmax(is:ie,js:je,ks:ke,i), start=start_3d, count=count_3d ) )

              varname = "a_axis"//trim(word)
              call check( nf90_inq_varid(ncid, trim(varname), varid) )
              call check( nf90_get_var( ncid, varid, bin_ice_alen(is:ie,js:je,ks:ke,i), start=start_3d, count=count_3d ) )

              varname = "c_axis"//trim(word)
              call check( nf90_inq_varid(ncid, trim(varname), varid) )
              call check( nf90_get_var( ncid, varid, bin_ice_clen(is:ie,js:je,ks:ke,i), start=start_3d, count=count_3d ) )

              varname = "d_axis"//trim(word)
              call check( nf90_inq_varid(ncid, trim(varname), varid) )
              call check( nf90_get_var( ncid, varid, bin_ice_dlen(is:ie,js:je,ks:ke,i), start=start_3d, count=count_3d ) )

              if (poly_exist) then
                 varname = "ag_axis"//trim(word)
                 call check( nf90_inq_varid(ncid, trim(varname), varid) )
                 call check( nf90_get_var( ncid, varid, bin_ice_aglen(is:ie,js:je,ks:ke,i), start=start_3d, count=count_3d ) )

                 varname = "ag_axis"//trim(word)
                 call check( nf90_inq_varid(ncid, trim(varname), varid) )
                 call check( nf90_get_var( ncid, varid, bin_ice_cglen(is:ie,js:je,ks:ke,i), start=start_3d, count=count_3d ) )

                 varname = "ex_cry"//trim(word)
                 call check( nf90_inq_varid(ncid, trim(varname), varid) )
                 call check( nf90_get_var( ncid, varid, bin_ice_nex(is:ie,js:je,ks:ke,i), start=start_3d, count=count_3d ) )
              endif

           enddo

           !call check( nf90_inquire_variable(ncid, varid, name=check_NAME, ndims=check_NDIMS, dimids=check_dimids, xtype=check_type) )

           call check( nf90_close( ncid ) )

        enddo


        ! post process

        ! transform all density multiplied quantities back to normal form
        ! calculate qice
        do i = isg, ieg
           do j = jsg, jeg
              do k = ks, ke
                 qice(i,j,k) = max(0.0E0,qhyd(i,j,k) - qliq(i,j,k))
                 do ibin = 1, max_binr
                    bin_liq_con(i,j,k,ibin) = bin_liq_con(i,j,k,ibin)*dens(i,j,k)
                 enddo
                 do ibin = 1, max_bini
                    bin_ice_con(i,j,k,ibin) = bin_ice_con(i,j,k,ibin)*dens(i,j,k)
                 enddo
              enddo
           enddo
        enddo
     
        ! retrieve average and median
        do k = ks, ke

           counter1 = 1
           local_var_T(:) = 0.0E0
           local_var_P(:) = 0.0E0
           local_var_dens(:) = 0.0E0
           local_var_u(:) = 0.0E0
           local_var_qv(:) = 0.0E0
           local_var_rh(:) = 0.0E0
           local_var_rhi(:) = 0.0E0
           local_var_lwc(:) = 0.0E0
           local_var_iwc(:) = 0.0E0
           local_var_iwccry(:) = 0.0E0
           local_var_iwcrcr(:) = 0.0E0
           local_var_iwcgra(:) = 0.0E0
           local_var_iwcagg(:) = 0.0E0
           local_var_iwcrag(:) = 0.0E0
           local_var_iwccryrcr(:) = 0.0E0
           local_var_tke(:) = 0.0E0
           local_var_cryall(:) = 0.0E0
           local_var_aggall(:) = 0.0E0
           local_var_ragall(:) = 0.0E0
           local_var_graall(:) = 0.0E0
           local_var_rcrall(:) = 0.0E0
           local_var_cry(:) = 0.0E0
           local_var_agg(:) = 0.0E0
           local_var_rag(:) = 0.0E0
           local_var_gra(:) = 0.0E0
           local_var_rcr(:) = 0.0E0
           local_var_monopl(:) = 0.0E0
           local_var_monocl(:) = 0.0E0
           local_var_monodd(:) = 0.0E0
           local_var_polyir(:) = 0.0E0
           local_var_polypl(:) = 0.0E0
           local_var_polycl(:) = 0.0E0
           local_var_rmrim(:) = 0.0E0
           local_var_rmvap(:) = 0.0E0
           local_var_rmimm(:) = 0.0E0
           local_var_niceall(:) = 0.0E0
           local_var_nice200all(:) = 0.0E0
           local_var_nliqall(:) = 0.0E0
           local_var_nice(:) = 0.0E0
           local_var_nice200(:) = 0.0E0
           local_var_nliq(:) = 0.0E0
           local_var_dliq(:) = 0.0E0
           local_var_dice(:) = 0.0E0

           do i = isg, ieg
              do j = jsg, jeg
                 local_var_T(counter1) = temperature(i,j,k)
                 local_var_P(counter1) = pressure(i,j,k)
                 local_var_dens(counter1) = dens(i,j,k)
                 local_var_u(counter1) = sqrt(u(i,j,k)**2 + v(i,j,k)**2)
                 local_var_rh(counter1) = rh(i,j,k)
                 local_var_rhi(counter1) = rhi(i,j,k)
                 local_var_tke(counter1) = dens(i,j,k)*w(i,j,k)**2
                 local_var_lwc(counter1) = dens(i,j,k)*qliq(i,j,k)*1000.0
                 local_var_iwc(counter1) = dens(i,j,k)*qice(i,j,k)*1000.0
                 local_var_qv(counter1) = qv(i,j,k)
                 local_var_rmrim(counter1) = ice_mass_rim(i,j,k)*1000000.0/11.0*DX*DY*(fz(k+1) - fz(k))
                 local_var_rmimm(counter1) = ice_mass_imm(i,j,k)*1000000.0/11.0*DX*DY*(fz(k+1) - fz(k))
                 local_var_rmvap(counter1) = (ice_mass_vapor(i,j,k) + ice_mass_eva(i,j,k))*1000000.0/10.0*DX*DY*(fz(k+1) - fz(k))

                 if (qice(i,j,k) > 0.0E0) then !qice_lower_limit

                    do ibin = 1, max_bini
                       local_var_niceall(counter1) = local_var_niceall(counter1) + bin_ice_con(i,j,k,ibin)*1000.0E0

                       if (bin_ice_con(i,j,k,ibin) > 1.e-11 .and. bin_ice_mcry(i,j,k,ibin) > 0.0E0 .and. bin_ice_alen(i,j,k,ibin) > 0.0E0 .and. bin_ice_clen(i,j,k,ibin) > 0.0E0) then ! 10e-8 L-1 limit
                          ! rescale axis lengths
                          lbin_ice_alen = (bin_ice_alen(i,j,k,ibin)*dens(i,j,k)/bin_ice_con(i,j,k,ibin))**(1.0/3.0)
                          lbin_ice_clen = (bin_ice_clen(i,j,k,ibin)*dens(i,j,k)/bin_ice_con(i,j,k,ibin))**(1.0/3.0)
                          lbin_ice_dlen = min(0.9E0*lbin_ice_alen, (bin_ice_dlen(i,j,k,ibin)*dens(i,j,k)/bin_ice_con(i,j,k,ibin))**(1.0/3.0))
                          if (poly_exist) then
                             if (iceaxis_version == 3) then
                                lbin_ice_aglen = (bin_ice_aglen(i,j,k,ibin)*dens(i,j,k)/bin_ice_con(i,j,k,ibin))**(1.0/3.0)*lbin_ice_alen
                                lbin_ice_cglen = (bin_ice_cglen(i,j,k,ibin)*dens(i,j,k)/bin_ice_con(i,j,k,ibin))**(1.0/3.0)*lbin_ice_clen
                                lbin_ice_nex = bin_ice_nex(i,j,k,ibin)*dens(i,j,k)/bin_ice_con(i,j,k,ibin)
                             endif
                          endif

                          ! diagnose ice type
                          call Iice_habit(ice_type,bin_ice_mass(i,j,k,ibin),bin_ice_mcry(i,j,k,ibin),bin_ice_magg(i,j,k,ibin),bin_ice_mrim(i,j,k,ibin))

                          ! diagnose ice habit
                          if (poly_exist) then
                             call Iice_shapePoly(my_habit,lbin_ice_alen,lbin_ice_clen,lbin_ice_dlen,lbin_ice_aglen,lbin_ice_cglen,lbin_ice_nex)
                          else
                             call Iice_shape(my_habit,lbin_ice_alen,lbin_ice_clen,lbin_ice_dlen)
                          endif
                          if (my_habit == -1) then
                             print *, "ERROR HABIT", irank, lbin_ice_alen,lbin_ice_clen,lbin_ice_dlen,lbin_ice_aglen,lbin_ice_cglen,lbin_ice_nex
                             call PRC_abort
                          endif
                          if (poly_exist) then
                             call get_elen_rlen(pla_e,ros_r,habit,bin_ice_mcry(i,j,k,ibin)*dens(i,j,k)/bin_ice_con(i,j,k,ibin)/1000.0E0,lbin_ice_alen,lbin_ice_clen,lbin_ice_aglen,lbin_ice_cglen)
                          else
                             pla_e = 0.0E0
                             ros_r = 0.0E0
                          endif
                          
                          ! calculate vcs and vic
                          den_i = 0.91668
                          vcs = bin_ice_Dmax(i,j,k,ibin)*dens(i,j,k)/bin_ice_con(i,j,k,ibin)
                          if (poly_exist) then
                             if (my_habit <= 3) then
                                vic = 4.0E0/3.0E0*pi*(lbin_ice_alen**2 + lbin_ice_clen**2)**1.5E0
                             else
                                if (my_habit == 4) then
                                   vic = 4.0E0/3.0E0*pi*pla_e*pla_e*pla_e*(1.0E0 + spx_p**2)**1.5E0
                                else
                                   vic = 4.0E0/3.0E0*pi*ros_r*ros_r*ros_r*(1.0E0 + spx_p**2)**1.5E0
                                endif
                             endif
                          else
                             vic = 4.0/3.0*pi*(lbin_ice_alen**2.0 + lbin_ice_clen**2.0)**1.5
                          endif
                          if (vic < bin_ice_mcry(i,j,k,ibin)*dens(i,j,k)/bin_ice_con(i,j,k,ibin)/1000.0E0/den_i) then
                             vic = bin_ice_mcry(i,j,k,ibin)*dens(i,j,k)/bin_ice_con(i,j,k,ibin)/1000.0E0/den_i
                          endif
                          if (vic == 0.0E0) then
                             write(*,'(A,I3, 6ES15.6)') "volume error: ", habit, vcs, vic, pla_e, ros_r, bin_ice_alen(i,j,k,ibin), bin_ice_clen(i,j,k,ibin)
                             call PRC_abort
                          endif
                          
                          if (vcs < vic) then
                             vcs = vic
                          else if (vcs > 4188.79020478639) then
                             vcs = 4188.79020478639
                          endif

                          ! calculate alpha_ic
                          if (poly_exist) then
                             if (my_habit <= 3) then
                                if (lbin_ice_alen > 1.e-15 .and. lbin_ice_clen > 1.e-15) then
                                   alpha_ic = lbin_ice_clen/lbin_ice_alen
                                else
                                   alpha_ic = 1.0
                                endif
                             else
                                alpha_ic = spx_p
                             endif
                          else
                             if (lbin_ice_alen > 1.e-15 .and. lbin_ice_clen > 1.e-15) then
                                alpha_ic = lbin_ice_clen/lbin_ice_alen
                             else
                                alpha_ic = 1.0
                             endif
                          endif
                       
                          ! calculate alpha_im
                          if (ice_type == 1) then
                             kp = log10(1.0/3.0/alpha_ic)
                             mass_agg_cry = bin_ice_mcry(i,j,k,ibin) + bin_ice_magg(i,j,k,ibin)
                             mlmt = 10.0*bin_ice_mcry(i,j,k,ibin)
                             if (mass_agg_cry < mlmt) then
                                alpha_im = 1.0/3.0*(mass_agg_cry/mlmt)**kp
                             else
                                alpha_im = 1.0/3.0
                             endif
                          else if (ice_type == 3 .or. ice_type == 4) then
                             kp = log10(0.8/alpha_ic)
                             mass_agg_cry_rim = bin_ice_mcry(i,j,k,ibin) + bin_ice_magg(i,j,k,ibin) + bin_ice_mrim(i,j,k,ibin)
                             mlmt = 10.0*(bin_ice_mcry(i,j,k,ibin) + bin_ice_magg(i,j,k,ibin))
                             if (mass_agg_cry_rim < mlmt) then
                                alpha_im = 0.8*(mass_agg_cry_rim/mlmt)**kp
                             else
                                alpha_im = 0.8
                             endif
                          else if (ice_type == 5) then
                             if (alpha_ic > 1.0) then
                                alpha_im = 1.0/sqrt((1.0 + 1.0/alpha_ic**2.0)*(vcs/vic)**(2.0/3.0) - 1.0)
                                if (alpha_im < 1.0E0) then
                                   alpha_im = 1.0E0
                                endif
                             else
                                alpha_im = sqrt((1.0 + alpha_ic**2.0)*(vcs/vic)**(2.0/3.0) - 1.0)
                                if (alpha_im > 1.0E0) then
                                   alpha_im = 1.0E0
                                endif
                             endif
                          else
                             alpha_im = alpha_ic
                          endif

                          ! calculate maximum dimension
                          if (ice_type == 4) then
                             if (alpha_im > 1.0) then
                                csm = ((vcs/(4.0/3.0*pi))**(1.0/3.0))
                                asm = csm/alpha_im
                                local_Dmax = 2.0*csm*10000.0
                             else
                                asm = ((vcs/(4.0/3.0*pi))**(1.0/3.0))
                                csm = asm*alpha_im
                                local_Dmax = 2.0*asm*10000.0
                             endif
                          else
                             asm = ((vcs/(4.0/3.0*pi))**(1.0/3.0))/sqrt(1.0 + alpha_im**2.0)
                             csm = asm*alpha_im
                             local_Dmax = 2.0*asm*sqrt(1.0 + alpha_im**2.0)*10000.0
                          endif
                          if (abs(asm) > 1.e10 .or. abs(csm) > 1.e10) then
                             print *, "ASM CSM error", irank, asm, csm, alpha_im
                             call PRC_abort
                          endif

                          if (local_Dmax > 200.0D0) then
                             local_var_nice200all(counter1) = local_var_nice200all(counter1) + bin_ice_con(i,j,k,ibin)*1000.0E0
                          endif

                          if (ice_type == 1) then
                             local_var_aggall(counter1) = local_var_aggall(counter1) + bin_ice_con(i,j,k,ibin)
                             local_var_iwcagg(counter1) = local_var_iwcagg(counter1) + dens(i,j,k)*bin_ice_mass(i,j,k,ibin)*1000.0
                          else if (ice_type == 2) then
                             local_var_cryall(counter1) = local_var_cryall(counter1) + bin_ice_con(i,j,k,ibin)
                             local_var_iwccry(counter1) = local_var_iwccry(counter1) + dens(i,j,k)*bin_ice_mass(i,j,k,ibin)*1000.0
                          else if (ice_type == 3) then
                             local_var_ragall(counter1) = local_var_ragall(counter1) + bin_ice_con(i,j,k,ibin)
                             local_var_iwcrag(counter1) = local_var_iwcrag(counter1) + dens(i,j,k)*bin_ice_mass(i,j,k,ibin)*1000.0
                          else if (ice_type == 4) then
                             local_var_graall(counter1) = local_var_graall(counter1) + bin_ice_con(i,j,k,ibin)
                             local_var_iwcgra(counter1) = local_var_iwcgra(counter1) + dens(i,j,k)*bin_ice_mass(i,j,k,ibin)*1000.0
                          else if (ice_type == 5) then
                             local_var_rcrall(counter1) = local_var_rcrall(counter1) + bin_ice_con(i,j,k,ibin)
                             local_var_iwcrcr(counter1) = local_var_iwcrcr(counter1) + dens(i,j,k)*bin_ice_mass(i,j,k,ibin)*1000.0
                          endif
                          if (ice_type == 2 .or. ice_type == 5) then
                             local_var_iwccryrcr(counter1) = local_var_iwccryrcr(counter1) + dens(i,j,k)*bin_ice_mass(i,j,k,ibin)*1000.0
                          endif

                       endif

                    enddo

                 else

                    local_var_niceall(counter1) = -10.0E0
                    local_var_nice200all(counter1) = -10.0E0
                    local_var_aggall(counter1) = -10.0E0
                    local_var_cryall(counter1) = -10.0E0
                    local_var_rcrall(counter1) = -10.0E0
                    local_var_graall(counter1) = -10.0E0
                    local_var_ragall(counter1) = -10.0E0
                    
                 endif

                 if (qliq(i,j,k) > qliq_lower_limit) then
                    !incloud_counter = incloud_counter + 1.0E0

                    D2 = 0.0
                    D3 = 0.0
                    do ibin = 1, max_binr
                       local_var_nliq(counter1) = local_var_nliq(counter1) + bin_liq_con(i,j,k,ibin)
                       D2 = D2 + bin_liq_con(i,j,k,ibin)*diameterbin(ibin)**2
                       D3 = D3 + bin_liq_con(i,j,k,ibin)*diameterbin(ibin)**3
                       
                    enddo
                    if (D2 /= 0.0) then
                       local_var_dliq(counter1) = D3/D2
                    endif

                    D2 = 0.0
                    D3 = 0.0
                    do ibin = 1, max_bini
                       local_var_nice(counter1) = local_var_nice(counter1) + bin_ice_con(i,j,k,ibin)*1000.0E0


                       if (bin_ice_con(i,j,k,ibin) > 1.e-11 .and. bin_ice_mcry(i,j,k,ibin) > 0.0E0 .and. bin_ice_alen(i,j,k,ibin) > 0.0E0 .and. bin_ice_clen(i,j,k,ibin) > 0.0E0) then ! 10e-8 L-1 limit
                          ! rescale axis lengths
                          bin_ice_alen(i,j,k,ibin) = (bin_ice_alen(i,j,k,ibin)*dens(i,j,k)/bin_ice_con(i,j,k,ibin))**(1.0/3.0)
                          bin_ice_clen(i,j,k,ibin) = (bin_ice_clen(i,j,k,ibin)*dens(i,j,k)/bin_ice_con(i,j,k,ibin))**(1.0/3.0)
                          !bin_ice_dlen(i,j,k,ibin) = (bin_ice_dlen(i,j,k,ibin)*dens(i,j,k)/bin_ice_con(i,j,k,ibin))**(1.0/3.0)
                          bin_ice_dlen(i,j,k,ibin) = min(0.9E0*bin_ice_alen(i,j,k,ibin), (bin_ice_dlen(i,j,k,ibin)*dens(i,j,k)/bin_ice_con(i,j,k,ibin))**(1.0/3.0))
                          if (poly_exist) then
                             if (iceaxis_version == 3) then
                                bin_ice_aglen(i,j,k,ibin) = (bin_ice_aglen(i,j,k,ibin)*dens(i,j,k)/bin_ice_con(i,j,k,ibin))**(1.0/3.0)*bin_ice_alen(i,j,k,ibin)
                                bin_ice_cglen(i,j,k,ibin) = (bin_ice_cglen(i,j,k,ibin)*dens(i,j,k)/bin_ice_con(i,j,k,ibin))**(1.0/3.0)*bin_ice_clen(i,j,k,ibin)
                                bin_ice_nex(i,j,k,ibin) = bin_ice_nex(i,j,k,ibin)*dens(i,j,k)/bin_ice_con(i,j,k,ibin)
                             endif
                          endif

                          ! diagnose ice type
                          call Iice_habit(ice_type,bin_ice_mass(i,j,k,ibin),bin_ice_mcry(i,j,k,ibin),bin_ice_magg(i,j,k,ibin),bin_ice_mrim(i,j,k,ibin))

                          ! diagnose ice habit
                          if (poly_exist) then
                             call Iice_shapePoly(my_habit,bin_ice_alen(i,j,k,ibin),bin_ice_clen(i,j,k,ibin),bin_ice_dlen(i,j,k,ibin),bin_ice_aglen(i,j,k,ibin),bin_ice_cglen(i,j,k,ibin),bin_ice_nex(i,j,k,ibin))
                          else
                             call Iice_shape(my_habit,bin_ice_alen(i,j,k,ibin),bin_ice_clen(i,j,k,ibin),bin_ice_dlen(i,j,k,ibin))
                          endif
                          if (my_habit == -1) then
                             print *, "ERROR HABIT", irank, bin_ice_alen(i,j,k,ibin),bin_ice_clen(i,j,k,ibin),bin_ice_dlen(i,j,k,ibin),bin_ice_aglen(i,j,k,ibin),bin_ice_cglen(i,j,k,ibin),bin_ice_nex(i,j,k,ibin)
                             call PRC_abort
                          endif
                          if (poly_exist) then
                             call get_elen_rlen(pla_e,ros_r,habit,bin_ice_mcry(i,j,k,ibin)*dens(i,j,k)/bin_ice_con(i,j,k,ibin)/1000.0E0,bin_ice_alen(i,j,k,ibin),bin_ice_clen(i,j,k,ibin),bin_ice_aglen(i,j,k,ibin),bin_ice_cglen(i,j,k,ibin))
                          else
                             pla_e = 0.0E0
                             ros_r = 0.0E0
                          endif
                          
                          ! calculate vcs and vic
                          den_i = 0.91668
                          vcs = bin_ice_Dmax(i,j,k,ibin)*dens(i,j,k)/bin_ice_con(i,j,k,ibin)
                          if (poly_exist) then
                             if (my_habit <= 3) then
                                vic = 4.0E0/3.0E0*pi*(bin_ice_alen(i,j,k,ibin)**2 + bin_ice_clen(i,j,k,ibin)**2)**1.5E0
                             else
                                if (my_habit == 4) then
                                   vic = 4.0E0/3.0E0*pi*pla_e*pla_e*pla_e*(1.0E0 + spx_p**2)**1.5E0
                                else
                                   vic = 4.0E0/3.0E0*pi*ros_r*ros_r*ros_r*(1.0E0 + spx_p**2)**1.5E0
                                endif
                             endif
                          else
                             vic = 4.0/3.0*pi*(bin_ice_alen(i,j,k,ibin)**2.0 + bin_ice_clen(i,j,k,ibin)**2.0)**1.5
                          endif
                          if (vic < bin_ice_mcry(i,j,k,ibin)*dens(i,j,k)/bin_ice_con(i,j,k,ibin)/1000.0E0/den_i) then
                             vic = bin_ice_mcry(i,j,k,ibin)*dens(i,j,k)/bin_ice_con(i,j,k,ibin)/1000.0E0/den_i
                          endif
                          if (vic == 0.0E0) then
                             write(*,'(A,I3, 6ES15.6)') "volume error: ", habit, vcs, vic, pla_e, ros_r, bin_ice_alen(i,j,k,ibin), bin_ice_clen(i,j,k,ibin)
                             call PRC_abort
                          endif
                          
                          if (vcs < vic) then
                             vcs = vic
                          else if (vcs > 4188.79020478639) then
                             vcs = 4188.79020478639
                          endif

                          ! calculate alpha_ic
                          if (poly_exist) then
                             if (my_habit <= 3) then
                                if (bin_ice_alen(i,j,k,ibin) > 1.e-15 .and. bin_ice_clen(i,j,k,ibin) > 1.e-15) then
                                   alpha_ic = bin_ice_clen(i,j,k,ibin)/bin_ice_alen(i,j,k,ibin)
                                else
                                   alpha_ic = 1.0
                                endif
                             else
                                alpha_ic = spx_p
                             endif
                          else
                             if (bin_ice_alen(i,j,k,ibin) > 1.e-15 .and. bin_ice_clen(i,j,k,ibin) > 1.e-15) then
                                alpha_ic = bin_ice_clen(i,j,k,ibin)/bin_ice_alen(i,j,k,ibin)
                             else
                                alpha_ic = 1.0
                             endif
                          endif
                       
                          ! calculate alpha_im
                          if (ice_type == 1) then
                             kp = log10(1.0/3.0/alpha_ic)
                             mass_agg_cry = bin_ice_mcry(i,j,k,ibin) + bin_ice_magg(i,j,k,ibin)
                             mlmt = 10.0*bin_ice_mcry(i,j,k,ibin)
                             if (mass_agg_cry < mlmt) then
                                alpha_im = 1.0/3.0*(mass_agg_cry/mlmt)**kp
                             else
                                alpha_im = 1.0/3.0
                             endif
                          else if (ice_type == 3 .or. ice_type == 4) then
                             kp = log10(0.8/alpha_ic)
                             mass_agg_cry_rim = bin_ice_mcry(i,j,k,ibin) + bin_ice_magg(i,j,k,ibin) + bin_ice_mrim(i,j,k,ibin)
                             mlmt = 10.0*(bin_ice_mcry(i,j,k,ibin) + bin_ice_magg(i,j,k,ibin))
                             if (mass_agg_cry_rim < mlmt) then
                                alpha_im = 0.8*(mass_agg_cry_rim/mlmt)**kp
                             else
                                alpha_im = 0.8
                             endif
                          else if (ice_type == 5) then
                             if (alpha_ic > 1.0) then
                                alpha_im = 1.0/sqrt((1.0 + 1.0/alpha_ic**2.0)*(vcs/vic)**(2.0/3.0) - 1.0)
                                if (alpha_im < 1.0E0) then
                                   alpha_im = 1.0E0
                                endif
                             else
                                alpha_im = sqrt((1.0 + alpha_ic**2.0)*(vcs/vic)**(2.0/3.0) - 1.0)
                                if (alpha_im > 1.0E0) then
                                   alpha_im = 1.0E0
                                endif
                             endif
                          else
                             alpha_im = alpha_ic
                          endif

                          ! calculate maximum dimension
                          if (ice_type == 4) then
                             if (alpha_im > 1.0) then
                                csm = ((vcs/(4.0/3.0*pi))**(1.0/3.0))
                                asm = csm/alpha_im
                                local_Dmax = 2.0*csm*10000.0
                             else
                                asm = ((vcs/(4.0/3.0*pi))**(1.0/3.0))
                                csm = asm*alpha_im
                                local_Dmax = 2.0*asm*10000.0
                             endif
                          else
                             asm = ((vcs/(4.0/3.0*pi))**(1.0/3.0))/sqrt(1.0 + alpha_im**2.0)
                             csm = asm*alpha_im
                             local_Dmax = 2.0*asm*sqrt(1.0 + alpha_im**2.0)*10000.0
                          endif
                          if (abs(asm) > 1.e10 .or. abs(csm) > 1.e10) then
                             print *, "ASM CSM error", irank, asm, csm, alpha_im
                             call PRC_abort
                          endif

                          D2 = D2 + bin_ice_con(i,j,k,ibin)*local_Dmax**2
                          D3 = D3 + bin_ice_con(i,j,k,ibin)*local_Dmax**3

                          if (local_Dmax > 200.0D0) then
                             local_var_nice200(counter1) = local_var_nice200(counter1) + bin_ice_con(i,j,k,ibin)*1000.0E0
                          endif

                          if (ice_type == 1) then
                             local_var_agg(counter1) = local_var_agg(counter1) + bin_ice_con(i,j,k,ibin)
                          else if (ice_type == 2) then
                             local_var_cry(counter1) = local_var_cry(counter1) + bin_ice_con(i,j,k,ibin)
                          else if (ice_type == 3) then
                             local_var_rag(counter1) = local_var_rag(counter1) + bin_ice_con(i,j,k,ibin)
                          else if (ice_type == 4) then
                             local_var_gra(counter1) = local_var_gra(counter1) + bin_ice_con(i,j,k,ibin)
                          else if (ice_type == 5) then
                             local_var_rcr(counter1) = local_var_rcr(counter1) + bin_ice_con(i,j,k,ibin)
                          endif
                          
                          if (my_habit == 1) then
                             local_var_monopl(counter1) = local_var_monopl(counter1) + bin_ice_con(i,j,k,ibin)
                          else if (my_habit == 2) then
                             local_var_monodd(counter1) = local_var_monodd(counter1) + bin_ice_con(i,j,k,ibin)
                          else if (my_habit == 3) then
                             local_var_monocl(counter1) = local_var_monocl(counter1) + bin_ice_con(i,j,k,ibin)
                          else if (my_habit == 4) then
                             local_var_polypl(counter1) = local_var_polypl(counter1) + bin_ice_con(i,j,k,ibin)
                          else if (my_habit == 5) then
                             local_var_polycl(counter1) = local_var_polycl(counter1) + bin_ice_con(i,j,k,ibin)
                          else if (my_habit == 6) then
                             local_var_polyir(counter1) = local_var_polyir(counter1) + bin_ice_con(i,j,k,ibin)
                          endif

                       endif

                    enddo
                    if (D2 /= 0.0) then
                       local_var_dice(counter1) = D3/D2
                    endif
                    
                 else

                    local_var_nliq(counter1) = -10.0E0
                    local_var_dliq(counter1) = -10.0E0
                    local_var_nice(counter1) = -10.0E0
                    local_var_nice200(counter1) = -10.0E0
                    local_var_dice(counter1) = -10.0E0
                    local_var_agg(counter1) = -10.0E0
                    local_var_cry(counter1) = -10.0E0
                    local_var_rcr(counter1) = -10.0E0
                    local_var_gra(counter1) = -10.0E0
                    local_var_rag(counter1) = -10.0E0
                    local_var_monopl(counter1) = -10.0E0
                    local_var_monocl(counter1) = -10.0E0
                    local_var_monodd(counter1) = -10.0E0
                    local_var_polypl(counter1) = -10.0E0
                    local_var_polycl(counter1) = -10.0E0
                    local_var_polyir(counter1) = -10.0E0
                    local_var_polypl(counter1) = -10.0E0

                 endif
                 
                 counter1 = counter1 + 1
              enddo
           enddo


           !call MPI_REDUCE( incloud_counter, global_sum_incloudcounter, 1, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
           
           call MPI_GATHER(local_var_nliq, sendcount_median, MPI_REAL4, global_var_nliq, sendcount_median, MPI_REAL4, master, MPI_COMM_WORLD, ierr )
           call MPI_GATHER(local_var_dliq, sendcount_median, MPI_REAL4, global_var_dliq, sendcount_median, MPI_REAL4, master, MPI_COMM_WORLD, ierr )
           call MPI_GATHER(local_var_niceall, sendcount_median, MPI_REAL4, global_var_niceall, sendcount_median, MPI_REAL4, master, MPI_COMM_WORLD, ierr )
           call MPI_GATHER(local_var_nice200all, sendcount_median, MPI_REAL4, global_var_nice200all, sendcount_median, MPI_REAL4, master, MPI_COMM_WORLD, ierr )
           call MPI_GATHER(local_var_nice, sendcount_median, MPI_REAL4, global_var_nice, sendcount_median, MPI_REAL4, master, MPI_COMM_WORLD, ierr )
           call MPI_GATHER(local_var_nice200, sendcount_median, MPI_REAL4, global_var_nice200, sendcount_median, MPI_REAL4, master, MPI_COMM_WORLD, ierr )
           call MPI_GATHER(local_var_dice, sendcount_median, MPI_REAL4, global_var_dice, sendcount_median, MPI_REAL4, master, MPI_COMM_WORLD, ierr )
           call MPI_GATHER(local_var_aggall, sendcount_median, MPI_REAL4, global_var_aggall, sendcount_median, MPI_REAL4, master, MPI_COMM_WORLD, ierr )
           call MPI_GATHER(local_var_graall, sendcount_median, MPI_REAL4, global_var_graall, sendcount_median, MPI_REAL4, master, MPI_COMM_WORLD, ierr )
           call MPI_GATHER(local_var_rcrall, sendcount_median, MPI_REAL4, global_var_rcrall, sendcount_median, MPI_REAL4, master, MPI_COMM_WORLD, ierr )
           call MPI_GATHER(local_var_cryall, sendcount_median, MPI_REAL4, global_var_cryall, sendcount_median, MPI_REAL4, master, MPI_COMM_WORLD, ierr )
           call MPI_GATHER(local_var_ragall, sendcount_median, MPI_REAL4, global_var_ragall, sendcount_median, MPI_REAL4, master, MPI_COMM_WORLD, ierr )
           call MPI_GATHER(local_var_agg, sendcount_median, MPI_REAL4, global_var_agg, sendcount_median, MPI_REAL4, master, MPI_COMM_WORLD, ierr )
           call MPI_GATHER(local_var_gra, sendcount_median, MPI_REAL4, global_var_gra, sendcount_median, MPI_REAL4, master, MPI_COMM_WORLD, ierr )
           call MPI_GATHER(local_var_rcr, sendcount_median, MPI_REAL4, global_var_rcr, sendcount_median, MPI_REAL4, master, MPI_COMM_WORLD, ierr )
           call MPI_GATHER(local_var_cry, sendcount_median, MPI_REAL4, global_var_cry, sendcount_median, MPI_REAL4, master, MPI_COMM_WORLD, ierr )
           call MPI_GATHER(local_var_rag, sendcount_median, MPI_REAL4, global_var_rag, sendcount_median, MPI_REAL4, master, MPI_COMM_WORLD, ierr )
           call MPI_GATHER(local_var_monopl, sendcount_median, MPI_REAL4, global_var_monopl, sendcount_median, MPI_REAL4, master, MPI_COMM_WORLD, ierr )
           call MPI_GATHER(local_var_monodd, sendcount_median, MPI_REAL4, global_var_monodd, sendcount_median, MPI_REAL4, master, MPI_COMM_WORLD, ierr )
           call MPI_GATHER(local_var_monocl, sendcount_median, MPI_REAL4, global_var_monocl, sendcount_median, MPI_REAL4, master, MPI_COMM_WORLD, ierr )
           call MPI_GATHER(local_var_polyir, sendcount_median, MPI_REAL4, global_var_polyir, sendcount_median, MPI_REAL4, master, MPI_COMM_WORLD, ierr )
           call MPI_GATHER(local_var_polypl, sendcount_median, MPI_REAL4, global_var_polypl, sendcount_median, MPI_REAL4, master, MPI_COMM_WORLD, ierr )
           call MPI_GATHER(local_var_polycl, sendcount_median, MPI_REAL4, global_var_polycl, sendcount_median, MPI_REAL4, master, MPI_COMM_WORLD, ierr )
           call MPI_GATHER(local_var_T, sendcount_median, MPI_REAL4, global_var_T, sendcount_median, MPI_REAL4, master, MPI_COMM_WORLD, ierr )
           call MPI_GATHER(local_var_P, sendcount_median, MPI_REAL4, global_var_P, sendcount_median, MPI_REAL4, master, MPI_COMM_WORLD, ierr )
           call MPI_GATHER(local_var_dens, sendcount_median, MPI_REAL4, global_var_dens, sendcount_median, MPI_REAL4, master, MPI_COMM_WORLD, ierr )
           call MPI_GATHER(local_var_rh, sendcount_median, MPI_REAL4, global_var_rh, sendcount_median, MPI_REAL4, master, MPI_COMM_WORLD, ierr )
           call MPI_GATHER(local_var_rhi, sendcount_median, MPI_REAL4, global_var_rhi, sendcount_median, MPI_REAL4, master, MPI_COMM_WORLD, ierr )
           call MPI_GATHER(local_var_qv, sendcount_median, MPI_REAL4, global_var_qv, sendcount_median, MPI_REAL4, master, MPI_COMM_WORLD, ierr )
           call MPI_GATHER(local_var_u, sendcount_median, MPI_REAL4, global_var_u, sendcount_median, MPI_REAL4, master, MPI_COMM_WORLD, ierr )
           call MPI_GATHER(local_var_tke, sendcount_median, MPI_REAL4, global_var_tke, sendcount_median, MPI_REAL4, master, MPI_COMM_WORLD, ierr )
           call MPI_GATHER(local_var_lwc, sendcount_median, MPI_REAL4, global_var_lwc, sendcount_median, MPI_REAL4, master, MPI_COMM_WORLD, ierr )
           call MPI_GATHER(local_var_iwc, sendcount_median, MPI_REAL4, global_var_iwc, sendcount_median, MPI_REAL4, master, MPI_COMM_WORLD, ierr )
           call MPI_GATHER(local_var_iwccry, sendcount_median, MPI_REAL4, global_var_iwccry, sendcount_median, MPI_REAL4, master, MPI_COMM_WORLD, ierr )
           call MPI_GATHER(local_var_iwcgra, sendcount_median, MPI_REAL4, global_var_iwcgra, sendcount_median, MPI_REAL4, master, MPI_COMM_WORLD, ierr )
           call MPI_GATHER(local_var_iwcrcr, sendcount_median, MPI_REAL4, global_var_iwcrcr, sendcount_median, MPI_REAL4, master, MPI_COMM_WORLD, ierr )
           call MPI_GATHER(local_var_iwccryrcr, sendcount_median, MPI_REAL4, global_var_iwccryrcr, sendcount_median, MPI_REAL4, master, MPI_COMM_WORLD, ierr )
           call MPI_GATHER(local_var_iwcagg, sendcount_median, MPI_REAL4, global_var_iwcagg, sendcount_median, MPI_REAL4, master, MPI_COMM_WORLD, ierr )
           call MPI_GATHER(local_var_iwcrag, sendcount_median, MPI_REAL4, global_var_iwcrag, sendcount_median, MPI_REAL4, master, MPI_COMM_WORLD, ierr )
           call MPI_GATHER(local_var_rmrim, sendcount_median, MPI_REAL4, global_var_rmrim, sendcount_median, MPI_REAL4, master, MPI_COMM_WORLD, ierr )
           call MPI_GATHER(local_var_rmimm, sendcount_median, MPI_REAL4, global_var_rmimm, sendcount_median, MPI_REAL4, master, MPI_COMM_WORLD, ierr )
           call MPI_GATHER(local_var_rmvap, sendcount_median, MPI_REAL4, global_var_rmvap, sendcount_median, MPI_REAL4, master, MPI_COMM_WORLD, ierr )

           if (irank == master) then
              median_array_size1 = 0
              median_array_size3 = 0
              do i = 1, count_global_median
                 if (global_var_nliq(i) >= 0.0) then
                    median_array_size1 = median_array_size1 + 1
                 endif
                 if (global_var_niceall(i) >= 0.0) then
                    median_array_size3 = median_array_size3 + 1
                 endif
              enddo

              median_array_size2 = count_global_median
              !write(*,*) "array1: ", median_array_size1, k

              if (median_array_size3 >= 1) then
                 allocate(median_array16(median_array_size3))
                 allocate(median_array16_200(median_array_size3))
                 allocate(median_array17(median_array_size3))
                 allocate(median_array18(median_array_size3))
                 allocate(median_array19(median_array_size3))
                 allocate(median_array20(median_array_size3))
                 allocate(median_array21(median_array_size3))
                 
                 counter1 = 0
                 do i = 1, count_global_median
                    if (global_var_niceall(i) >= 0.0) then
                       counter1 = counter1 + 1
                       median_array16(counter1) = global_var_niceall(i)
                       median_array16_200(counter1) = global_var_nice200all(i)
                       median_array17(counter1) = global_var_cryall(i)
                       median_array18(counter1) = global_var_aggall(i)
                       median_array19(counter1) = global_var_graall(i)
                       median_array20(counter1) = global_var_rcrall(i)
                       median_array21(counter1) = global_var_ragall(i)
                    endif
                 enddo
              endif

              if (median_array_size3 >= 2) then

                 call quickSorti(median_array16,median_array_size3,1,median_array_size3)
                 call quickSorti(median_array16_200,median_array_size3,1,median_array_size3)

                 call quickSorti(median_array17,median_array_size3,1,median_array_size3)
                 call quickSorti(median_array18,median_array_size3,1,median_array_size3)
                 call quickSorti(median_array19,median_array_size3,1,median_array_size3)
                 call quickSorti(median_array20,median_array_size3,1,median_array_size3)
                 call quickSorti(median_array21,median_array_size3,1,median_array_size3)

              endif

              if (median_array_size3 >= 1) then

                 niceall1d(t,k) = median_array16(int(median_array_size3/2))
                 niceall1d1(t,k) = median_array16(int(median_array_size3*0.05))
                 niceall1d9(t,k) = median_array16(int(median_array_size3*0.95))

                 nice200all1d(t,k) = median_array16_200(int(median_array_size3/2))
                 nice200all1d1(t,k) = median_array16_200(int(median_array_size3*0.05))
                 nice200all1d9(t,k) = median_array16_200(int(median_array_size3*0.95))

                 cryall1d(t,k) = median_array17(int(median_array_size3/2))
                 cryall1d1(t,k) = median_array17(int(median_array_size3*0.05))
                 cryall1d9(t,k) = median_array17(int(median_array_size3*0.95))
                 
                 aggall1d(t,k) = median_array18(int(median_array_size3/2))
                 aggall1d1(t,k) = median_array18(int(median_array_size3*0.05))
                 aggall1d9(t,k) = median_array18(int(median_array_size3*0.95))

                 graall1d(t,k) = median_array19(int(median_array_size3/2))
                 graall1d1(t,k) = median_array19(int(median_array_size3*0.05))
                 graall1d9(t,k) = median_array19(int(median_array_size3*0.95))

                 rcrall1d(t,k) = median_array20(int(median_array_size3/2))
                 rcrall1d1(t,k) = median_array20(int(median_array_size3*0.05))
                 rcrall1d9(t,k) = median_array20(int(median_array_size3*0.95))

                 ragall1d(t,k) = median_array21(int(median_array_size3/2))
                 ragall1d1(t,k) = median_array21(int(median_array_size3*0.05))
                 ragall1d9(t,k) = median_array21(int(median_array_size3*0.95))
                 
              endif

              if (median_array_size3 >= 1) then
                 deallocate(median_array16,median_array16_200)
                 deallocate(median_array17,median_array18,median_array19,median_array20,median_array21)
              endif


              
              if (median_array_size1 >= 1) then
                 allocate(median_array1(median_array_size1))
                 allocate(median_array2(median_array_size1))
                 allocate(median_array2_200(median_array_size1))
                 allocate(median_array3(median_array_size1))
                 allocate(median_array4(median_array_size1))
                 allocate(median_array5(median_array_size1))
                 allocate(median_array6(median_array_size1))
                 allocate(median_array7(median_array_size1))
                 allocate(median_array8(median_array_size1))
                 allocate(median_array9(median_array_size1))
                 allocate(median_array10(median_array_size1))
                 allocate(median_array11(median_array_size1))
                 allocate(median_array12(median_array_size1))
                 allocate(median_array13(median_array_size1))
                 allocate(median_array14(median_array_size1))
                 allocate(median_array15(median_array_size1))
              
                 counter1 = 0
                 do i = 1, count_global_median
                    if (global_var_nliq(i) >= 0.0) then
                       counter1 = counter1 + 1
                       median_array1(counter1) = global_var_nliq(i)
                       median_array2(counter1) = global_var_nice(i)
                       median_array2_200(counter1) = global_var_nice200(i)
                       median_array3(counter1) = global_var_dliq(i)
                       median_array4(counter1) = global_var_dice(i)
                       median_array5(counter1) = global_var_cry(i)
                       median_array6(counter1) = global_var_agg(i)
                       median_array7(counter1) = global_var_gra(i)
                       median_array8(counter1) = global_var_rcr(i)
                       median_array9(counter1) = global_var_rag(i)
                       median_array10(counter1) = global_var_monopl(i)
                       median_array11(counter1) = global_var_monodd(i)
                       median_array12(counter1) = global_var_monocl(i)
                       median_array13(counter1) = global_var_polypl(i)
                       median_array14(counter1) = global_var_polycl(i)
                       median_array15(counter1) = global_var_polyir(i)
                    endif
                 enddo

              endif

              if (median_array_size1 >= 2) then
                 call quickSorti(median_array1,median_array_size1,1,median_array_size1)
                 !write(*,*) "array2: ", median_array_size2
                 call quickSorti(median_array2,median_array_size1,1,median_array_size1)
                 call quickSorti(median_array2_200,median_array_size1,1,median_array_size1)
                 !write(*,*) "array3: ", median_array_size3
                 call quickSorti(median_array3,median_array_size1,1,median_array_size1)
                 call quickSorti(median_array4,median_array_size1,1,median_array_size1)
                 call quickSorti(median_array5,median_array_size1,1,median_array_size1)
                 call quickSorti(median_array6,median_array_size1,1,median_array_size1)
                 call quickSorti(median_array7,median_array_size1,1,median_array_size1)
                 call quickSorti(median_array8,median_array_size1,1,median_array_size1)
                 call quickSorti(median_array9,median_array_size1,1,median_array_size1)
                 call quickSorti(median_array10,median_array_size1,1,median_array_size1)
                 call quickSorti(median_array11,median_array_size1,1,median_array_size1)
                 call quickSorti(median_array12,median_array_size1,1,median_array_size1)
                 call quickSorti(median_array13,median_array_size1,1,median_array_size1)
                 call quickSorti(median_array14,median_array_size1,1,median_array_size1)
                 call quickSorti(median_array15,median_array_size1,1,median_array_size1)
              endif

              call quickSorti(global_var_T,median_array_size2,1,median_array_size2)
              call quickSorti(global_var_P,median_array_size2,1,median_array_size2)
              call quickSorti(global_var_dens,median_array_size2,1,median_array_size2)
              call quickSorti(global_var_rh,median_array_size2,1,median_array_size2)
              call quickSorti(global_var_rhi,median_array_size2,1,median_array_size2)
              call quickSorti(global_var_qv,median_array_size2,1,median_array_size2)
              call quickSorti(global_var_u,median_array_size2,1,median_array_size2)
              call quickSorti(global_var_tke,median_array_size2,1,median_array_size2)
              call quickSorti(global_var_lwc,median_array_size2,1,median_array_size2)
              call quickSorti(global_var_iwc,median_array_size2,1,median_array_size2)
              call quickSorti(global_var_iwccry,median_array_size2,1,median_array_size2)
              call quickSorti(global_var_iwcgra,median_array_size2,1,median_array_size2)
              call quickSorti(global_var_iwcrcr,median_array_size2,1,median_array_size2)
              call quickSorti(global_var_iwccryrcr,median_array_size2,1,median_array_size2)
              call quickSorti(global_var_iwcagg,median_array_size2,1,median_array_size2)
              call quickSorti(global_var_iwcrag,median_array_size2,1,median_array_size2)
              call quickSorti(global_var_rmrim,median_array_size2,1,median_array_size2)
              call quickSorti(global_var_rmimm,median_array_size2,1,median_array_size2)
              call quickSorti(global_var_rmvap,median_array_size2,1,median_array_size2)

              T1d(t,k) = global_var_T(int(median_array_size2/2))
              T1d1(t,k) = global_var_T(int(median_array_size2*0.05))
              T1d9(t,k) = global_var_T(int(median_array_size2*0.95))

              P1d(t,k) = global_var_P(int(median_array_size2/2))
              P1d1(t,k) = global_var_P(int(median_array_size2*0.05))
              P1d9(t,k) = global_var_P(int(median_array_size2*0.95))

              dens1d(t,k) = global_var_dens(int(median_array_size2/2))
              dens1d1(t,k) = global_var_dens(int(median_array_size2*0.05))
              dens1d9(t,k) = global_var_dens(int(median_array_size2*0.95))

              rh1d(t,k) = global_var_rh(int(median_array_size2/2))
              rh1d1(t,k) = global_var_rh(int(median_array_size2*0.05))
              rh1d9(t,k) = global_var_rh(int(median_array_size2*0.95))

              rhi1d(t,k) = global_var_rhi(int(median_array_size2/2))
              rhi1d1(t,k) = global_var_rhi(int(median_array_size2*0.05))
              rhi1d9(t,k) = global_var_rhi(int(median_array_size2*0.95))

              qv1d(t,k) = global_var_qv(int(median_array_size2/2))
              qv1d1(t,k) = global_var_qv(int(median_array_size2*0.05))
              qv1d9(t,k) = global_var_qv(int(median_array_size2*0.95))

              u1d(t,k) = global_var_u(int(median_array_size2/2))
              u1d1(t,k) = global_var_u(int(median_array_size2*0.05))
              u1d9(t,k) = global_var_u(int(median_array_size2*0.95))

              tke1d(t,k) = global_var_tke(int(median_array_size2/2))
              tke1d1(t,k) = global_var_tke(int(median_array_size2*0.05))
              tke1d9(t,k) = global_var_tke(int(median_array_size2*0.95))

              lwc1d(t,k) = global_var_lwc(int(median_array_size2/2))
              lwc1d1(t,k) = global_var_lwc(int(median_array_size2*0.05))
              lwc1d9(t,k) = global_var_lwc(int(median_array_size2*0.95))

              iwc1d(t,k) = global_var_iwc(int(median_array_size2/2))
              iwc1d1(t,k) = global_var_iwc(int(median_array_size2*0.05))
              iwc1d9(t,k) = global_var_iwc(int(median_array_size2*0.95))

              iwccry1d(t,k) = global_var_iwccry(int(median_array_size2/2))
              iwccry1d1(t,k) = global_var_iwccry(int(median_array_size2*0.05))
              iwccry1d9(t,k) = global_var_iwccry(int(median_array_size2*0.95))

              iwcgra1d(t,k) = global_var_iwcgra(int(median_array_size2/2))
              iwcgra1d1(t,k) = global_var_iwcgra(int(median_array_size2*0.05))
              iwcgra1d9(t,k) = global_var_iwcgra(int(median_array_size2*0.95))

              iwcrcr1d(t,k) = global_var_iwcrcr(int(median_array_size2/2))
              iwcrcr1d1(t,k) = global_var_iwcrcr(int(median_array_size2*0.05))
              iwcrcr1d9(t,k) = global_var_iwcrcr(int(median_array_size2*0.95))

              iwccryrcr1d(t,k) = global_var_iwccryrcr(int(median_array_size2/2))
              iwccryrcr1d1(t,k) = global_var_iwccryrcr(int(median_array_size2*0.05))
              iwccryrcr1d9(t,k) = global_var_iwccryrcr(int(median_array_size2*0.95))

              iwcagg1d(t,k) = global_var_iwcagg(int(median_array_size2/2))
              iwcagg1d1(t,k) = global_var_iwcagg(int(median_array_size2*0.05))
              iwcagg1d9(t,k) = global_var_iwcagg(int(median_array_size2*0.95))

              iwcrag1d(t,k) = global_var_iwcrag(int(median_array_size2/2))
              iwcrag1d1(t,k) = global_var_iwcrag(int(median_array_size2*0.05))
              iwcrag1d9(t,k) = global_var_iwcrag(int(median_array_size2*0.95))

              rmrim1d(t,k) = global_var_rmrim(int(median_array_size2/2))
              rmrim1d1(t,k) = global_var_rmrim(int(median_array_size2*0.05))
              rmrim1d9(t,k) = global_var_rmrim(int(median_array_size2*0.95))

              rmimm1d(t,k) = global_var_rmimm(int(median_array_size2/2))
              rmimm1d1(t,k) = global_var_rmimm(int(median_array_size2*0.05))
              rmimm1d9(t,k) = global_var_rmimm(int(median_array_size2*0.95))

              rmvap1d(t,k) = global_var_rmvap(int(median_array_size2/2))
              rmvap1d1(t,k) = global_var_rmvap(int(median_array_size2*0.05))
              rmvap1d9(t,k) = global_var_rmvap(int(median_array_size2*0.95))

              if (median_array_size1 >= 1) then
              
                 nliq1d(t,k) = median_array1(int(median_array_size1/2))
                 nliq1d1(t,k) = median_array1(int(median_array_size1*0.05))
                 nliq1d9(t,k) = median_array1(int(median_array_size1*0.95))

                 nice1d(t,k) = median_array2(int(median_array_size1/2))
                 nice1d1(t,k) = median_array2(int(median_array_size1*0.05))
                 nice1d9(t,k) = median_array2(int(median_array_size1*0.95))

                 nice2001d(t,k) = median_array2_200(int(median_array_size1/2))
                 nice2001d1(t,k) = median_array2_200(int(median_array_size1*0.05))
                 nice2001d9(t,k) = median_array2_200(int(median_array_size1*0.95))

                 dliq1d(t,k) = median_array3(int(median_array_size1/2))
                 dliq1d1(t,k) = median_array3(int(median_array_size1*0.05))
                 dliq1d9(t,k) = median_array3(int(median_array_size1*0.95))

                 dice1d(t,k) = median_array4(int(median_array_size1/2))
                 dice1d1(t,k) = median_array4(int(median_array_size1*0.05))
                 dice1d9(t,k) = median_array4(int(median_array_size1*0.95))

                 cry1d(t,k) = median_array5(int(median_array_size1/2))
                 cry1d1(t,k) = median_array5(int(median_array_size1*0.05))
                 cry1d9(t,k) = median_array5(int(median_array_size1*0.95))

                 agg1d(t,k) = median_array6(int(median_array_size1/2))
                 agg1d1(t,k) = median_array6(int(median_array_size1*0.05))
                 agg1d9(t,k) = median_array6(int(median_array_size1*0.95))

                 gra1d(t,k) = median_array7(int(median_array_size1/2))
                 gra1d1(t,k) = median_array7(int(median_array_size1*0.05))
                 gra1d9(t,k) = median_array7(int(median_array_size1*0.95))

                 rcr1d(t,k) = median_array8(int(median_array_size1/2))
                 rcr1d1(t,k) = median_array8(int(median_array_size1*0.05))
                 rcr1d9(t,k) = median_array8(int(median_array_size1*0.95))

                 rag1d(t,k) = median_array9(int(median_array_size1/2))
                 rag1d1(t,k) = median_array9(int(median_array_size1*0.05))
                 rag1d9(t,k) = median_array9(int(median_array_size1*0.95))

                 monopl1d(t,k) = median_array10(int(median_array_size1/2))
                 monopl1d1(t,k) = median_array10(int(median_array_size1*0.05))
                 monopl1d9(t,k) = median_array10(int(median_array_size1*0.95))

                 monodd1d(t,k) = median_array11(int(median_array_size1/2))
                 monodd1d1(t,k) = median_array11(int(median_array_size1*0.05))
                 monodd1d9(t,k) = median_array11(int(median_array_size1*0.95))

                 monocl1d(t,k) = median_array12(int(median_array_size1/2))
                 monocl1d1(t,k) = median_array12(int(median_array_size1*0.05))
                 monocl1d9(t,k) = median_array12(int(median_array_size1*0.95))

                 polypl1d(t,k) = median_array13(int(median_array_size1/2))
                 polypl1d1(t,k) = median_array13(int(median_array_size1*0.05))
                 polypl1d9(t,k) = median_array13(int(median_array_size1*0.95))

                 polycl1d(t,k) = median_array14(int(median_array_size1/2))
                 polycl1d1(t,k) = median_array14(int(median_array_size1*0.05))
                 polycl1d9(t,k) = median_array14(int(median_array_size1*0.95))

                 polyir1d(t,k) = median_array15(int(median_array_size1/2))
                 polyir1d1(t,k) = median_array15(int(median_array_size1*0.05))
                 polyir1d9(t,k) = median_array15(int(median_array_size1*0.95))

              endif

              if (median_array_size1 >= 1) then
                 deallocate(median_array1,median_array2,median_array3)
                 deallocate(median_array2_200)
                 deallocate(median_array4,median_array5,median_array6)
                 deallocate(median_array7,median_array8,median_array9)
                 deallocate(median_array10,median_array11,median_array12)
                 deallocate(median_array13,median_array14,median_array15)
              endif

           endif

        enddo

     endif

  enddo


  
  ! output
  if (irank == master) then
     OUTPUT_CONFIG_FNAME = trim(OUTPUT_DIR)//"data_init_T.dat"
     call register_io(IO_OUTPUT_CONFIG,OUTPUT_CONFIG_FNAME)
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(2ES15.5)') cz(k+2)/1000.0, ini_T1d(k)
     enddo
     call close_io(IO_OUTPUT_CONFIG)

     OUTPUT_CONFIG_FNAME = trim(OUTPUT_DIR)//"data_init_PT.dat"
     call register_io(IO_OUTPUT_CONFIG,OUTPUT_CONFIG_FNAME)
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(2ES15.5)') cz(k+2)/1000.0, ini_PT1d(k)
     enddo
     call close_io(IO_OUTPUT_CONFIG)

     OUTPUT_CONFIG_FNAME = trim(OUTPUT_DIR)//"data_init_P.dat"
     call register_io(IO_OUTPUT_CONFIG,OUTPUT_CONFIG_FNAME)
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(2ES15.5)') cz(k+2)/1000.0, ini_P1d(k)
     enddo
     call close_io(IO_OUTPUT_CONFIG)

     OUTPUT_CONFIG_FNAME = trim(OUTPUT_DIR)//"data_init_rh.dat"
     call register_io(IO_OUTPUT_CONFIG,OUTPUT_CONFIG_FNAME)
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(2ES15.5)') cz(k+2)/1000.0, ini_rh1d(k)
     enddo
     call close_io(IO_OUTPUT_CONFIG)

     OUTPUT_CONFIG_FNAME = trim(OUTPUT_DIR)//"data_init_rhi.dat"
     call register_io(IO_OUTPUT_CONFIG,OUTPUT_CONFIG_FNAME)
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(2ES15.5)') cz(k+2)/1000.0, ini_rhi1d(k)
     enddo
     call close_io(IO_OUTPUT_CONFIG)

     OUTPUT_CONFIG_FNAME = trim(OUTPUT_DIR)//"data_init_lwc.dat"
     call register_io(IO_OUTPUT_CONFIG,OUTPUT_CONFIG_FNAME)
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(2ES15.5)') cz(k+2)/1000.0, ini_lwc1d(k)
     enddo
     call close_io(IO_OUTPUT_CONFIG)

     OUTPUT_CONFIG_FNAME = trim(OUTPUT_DIR)//"data_init_nliq.dat"
     call register_io(IO_OUTPUT_CONFIG,OUTPUT_CONFIG_FNAME)
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(2ES15.5)') cz(k+2)/1000.0, ini_nliq1d(k)
     enddo
     call close_io(IO_OUTPUT_CONFIG)

     OUTPUT_CONFIG_FNAME = trim(OUTPUT_DIR)//"data_init_u.dat"
     call register_io(IO_OUTPUT_CONFIG,OUTPUT_CONFIG_FNAME)
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(2ES15.5)') cz(k+2)/1000.0, ini_u1d(k)
     enddo
     call close_io(IO_OUTPUT_CONFIG)

     OUTPUT_CONFIG_FNAME = trim(OUTPUT_DIR)//"data_init_dens.dat"
     call register_io(IO_OUTPUT_CONFIG,OUTPUT_CONFIG_FNAME)
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(2ES15.5)') cz(k+2)/1000.0, ini_dens1d(k)
     enddo
     call close_io(IO_OUTPUT_CONFIG)

     OUTPUT_CONFIG_FNAME = trim(OUTPUT_DIR)//"data_init_qv.dat"
     call register_io(IO_OUTPUT_CONFIG,OUTPUT_CONFIG_FNAME)
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(2ES15.5)') cz(k+2)/1000.0, ini_qv1d(k)
     enddo
     call close_io(IO_OUTPUT_CONFIG)

     OUTPUT_CONFIG_FNAME = trim(OUTPUT_DIR)//"data_init_qliq.dat"
     call register_io(IO_OUTPUT_CONFIG,OUTPUT_CONFIG_FNAME)
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(2ES15.5)') cz(k+2)/1000.0, ini_qliq1d(k)
     enddo
     call close_io(IO_OUTPUT_CONFIG)

     OUTPUT_CONFIG_FNAME = trim(OUTPUT_DIR)//"data_init_lwdn.dat"
     call register_io(IO_OUTPUT_CONFIG,OUTPUT_CONFIG_FNAME)
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(2ES15.5)') cz(k+2)/1000.0, ini_lwdn1d(k)
     enddo
     call close_io(IO_OUTPUT_CONFIG)

     OUTPUT_CONFIG_FNAME = trim(OUTPUT_DIR)//"data_init_swdn.dat"
     call register_io(IO_OUTPUT_CONFIG,OUTPUT_CONFIG_FNAME)
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(2ES15.5)') cz(k+2)/1000.0, ini_swdn1d(k)
     enddo
     call close_io(IO_OUTPUT_CONFIG)

     OUTPUT_CONFIG_FNAME = trim(OUTPUT_DIR)//"data_init_sink.dat"
     call register_io(IO_OUTPUT_CONFIG,OUTPUT_CONFIG_FNAME)
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(2ES15.5)') cz(k+2)/1000.0, ini_sink1d(k)
     enddo
     call close_io(IO_OUTPUT_CONFIG)

     OUTPUT_CONFIG_FNAME = trim(OUTPUT_DIR)//"data_init_rhoh_rd.dat"
     call register_io(IO_OUTPUT_CONFIG,OUTPUT_CONFIG_FNAME)
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(2ES15.5)') cz(k+2)/1000.0, ini_rhoh_rd1d(k)
     enddo
     call close_io(IO_OUTPUT_CONFIG)

     OUTPUT_CONFIG_FNAME = trim(OUTPUT_DIR)//"data_init_rhoh_mp.dat"
     call register_io(IO_OUTPUT_CONFIG,OUTPUT_CONFIG_FNAME)
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(2ES15.5)') cz(k+2)/1000.0, ini_rhoh_mp1d(k)
     enddo
     call close_io(IO_OUTPUT_CONFIG)


     OUTPUT_CONFIG_FNAME = trim(OUTPUT_DIR)//"data_median_T.dat"
     call register_io(IO_OUTPUT_CONFIG,OUTPUT_CONFIG_FNAME)
     write(IO_OUTPUT_CONFIG,'(289ES15.5)') ct(:)/3600.0
     write(IO_OUTPUT_CONFIG,'(100ES15.5)') cz(3:nz-2)/1000.0
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') T1d(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') T1d1(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') T1d9(:,k)
     enddo     
     call close_io(IO_OUTPUT_CONFIG)

     OUTPUT_CONFIG_FNAME = trim(OUTPUT_DIR)//"data_median_P.dat"
     call register_io(IO_OUTPUT_CONFIG,OUTPUT_CONFIG_FNAME)
     write(IO_OUTPUT_CONFIG,'(289ES15.5)') ct(:)/3600.0
     write(IO_OUTPUT_CONFIG,'(100ES15.5)') cz(3:nz-2)/1000.0
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') P1d(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') P1d1(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') P1d9(:,k)
     enddo     
     call close_io(IO_OUTPUT_CONFIG)

     OUTPUT_CONFIG_FNAME = trim(OUTPUT_DIR)//"data_median_dens.dat"
     call register_io(IO_OUTPUT_CONFIG,OUTPUT_CONFIG_FNAME)
     write(IO_OUTPUT_CONFIG,'(289ES15.5)') ct(:)/3600.0
     write(IO_OUTPUT_CONFIG,'(100ES15.5)') cz(3:nz-2)/1000.0
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') dens1d(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') dens1d1(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') dens1d9(:,k)
     enddo     
     call close_io(IO_OUTPUT_CONFIG)

     OUTPUT_CONFIG_FNAME = trim(OUTPUT_DIR)//"data_median_u.dat"
     call register_io(IO_OUTPUT_CONFIG,OUTPUT_CONFIG_FNAME)
     write(IO_OUTPUT_CONFIG,'(289ES15.5)') ct(:)/3600.0
     write(IO_OUTPUT_CONFIG,'(100ES15.5)') cz(3:nz-2)/1000.0
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') u1d(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') u1d1(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') u1d9(:,k)
     enddo     
     call close_io(IO_OUTPUT_CONFIG)

     OUTPUT_CONFIG_FNAME = trim(OUTPUT_DIR)//"data_median_qv.dat"
     call register_io(IO_OUTPUT_CONFIG,OUTPUT_CONFIG_FNAME)
     write(IO_OUTPUT_CONFIG,'(289ES15.5)') ct(:)/3600.0
     write(IO_OUTPUT_CONFIG,'(100ES15.5)') cz(3:nz-2)/1000.0
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') qv1d(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') qv1d1(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') qv1d9(:,k)
     enddo     
     call close_io(IO_OUTPUT_CONFIG)

     OUTPUT_CONFIG_FNAME = trim(OUTPUT_DIR)//"data_median_TKE.dat"
     call register_io(IO_OUTPUT_CONFIG,OUTPUT_CONFIG_FNAME)
     write(IO_OUTPUT_CONFIG,'(289ES15.5)') ct(:)/3600.0
     write(IO_OUTPUT_CONFIG,'(100ES15.5)') cz(3:nz-2)/1000.0
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') TKE1d(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') TKE1d1(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') TKE1d9(:,k)
     enddo     
     call close_io(IO_OUTPUT_CONFIG)

     OUTPUT_CONFIG_FNAME = trim(OUTPUT_DIR)//"data_median_rh.dat"
     call register_io(IO_OUTPUT_CONFIG,OUTPUT_CONFIG_FNAME)
     write(IO_OUTPUT_CONFIG,'(289ES15.5)') ct(:)/3600.0
     write(IO_OUTPUT_CONFIG,'(100ES15.5)') cz(3:nz-2)/1000.0
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') rh1d(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') rh1d1(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') rh1d9(:,k)
     enddo     
     call close_io(IO_OUTPUT_CONFIG)

     OUTPUT_CONFIG_FNAME = trim(OUTPUT_DIR)//"data_median_rhi.dat"
     call register_io(IO_OUTPUT_CONFIG,OUTPUT_CONFIG_FNAME)
     write(IO_OUTPUT_CONFIG,'(289ES15.5)') ct(:)/3600.0
     write(IO_OUTPUT_CONFIG,'(100ES15.5)') cz(3:nz-2)/1000.0
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') rhi1d(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') rhi1d1(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') rhi1d9(:,k)
     enddo     
     call close_io(IO_OUTPUT_CONFIG)

     OUTPUT_CONFIG_FNAME = trim(OUTPUT_DIR)//"data_median_lwc.dat"
     call register_io(IO_OUTPUT_CONFIG,OUTPUT_CONFIG_FNAME)
     write(IO_OUTPUT_CONFIG,'(289ES15.5)') ct(:)/3600.0
     write(IO_OUTPUT_CONFIG,'(100ES15.5)') cz(3:nz-2)/1000.0
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') lwc1d(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') lwc1d1(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') lwc1d9(:,k)
     enddo     
     call close_io(IO_OUTPUT_CONFIG)

     OUTPUT_CONFIG_FNAME = trim(OUTPUT_DIR)//"data_median_iwc.dat"
     call register_io(IO_OUTPUT_CONFIG,OUTPUT_CONFIG_FNAME)
     write(IO_OUTPUT_CONFIG,'(289ES15.5)') ct(:)/3600.0
     write(IO_OUTPUT_CONFIG,'(100ES15.5)') cz(3:nz-2)/1000.0
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') iwc1d(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') iwc1d1(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') iwc1d9(:,k)
     enddo     
     call close_io(IO_OUTPUT_CONFIG)

     OUTPUT_CONFIG_FNAME = trim(OUTPUT_DIR)//"data_median_iwccry.dat"
     call register_io(IO_OUTPUT_CONFIG,OUTPUT_CONFIG_FNAME)
     write(IO_OUTPUT_CONFIG,'(289ES15.5)') ct(:)/3600.0
     write(IO_OUTPUT_CONFIG,'(100ES15.5)') cz(3:nz-2)/1000.0
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') iwccry1d(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') iwccry1d1(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') iwccry1d9(:,k)
     enddo     
     call close_io(IO_OUTPUT_CONFIG)

     OUTPUT_CONFIG_FNAME = trim(OUTPUT_DIR)//"data_median_iwcgra.dat"
     call register_io(IO_OUTPUT_CONFIG,OUTPUT_CONFIG_FNAME)
     write(IO_OUTPUT_CONFIG,'(289ES15.5)') ct(:)/3600.0
     write(IO_OUTPUT_CONFIG,'(100ES15.5)') cz(3:nz-2)/1000.0
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') iwcgra1d(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') iwcgra1d1(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') iwcgra1d9(:,k)
     enddo     
     call close_io(IO_OUTPUT_CONFIG)

     OUTPUT_CONFIG_FNAME = trim(OUTPUT_DIR)//"data_median_iwcrcr.dat"
     call register_io(IO_OUTPUT_CONFIG,OUTPUT_CONFIG_FNAME)
     write(IO_OUTPUT_CONFIG,'(289ES15.5)') ct(:)/3600.0
     write(IO_OUTPUT_CONFIG,'(100ES15.5)') cz(3:nz-2)/1000.0
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') iwcrcr1d(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') iwcrcr1d1(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') iwcrcr1d9(:,k)
     enddo     
     call close_io(IO_OUTPUT_CONFIG)

     OUTPUT_CONFIG_FNAME = trim(OUTPUT_DIR)//"data_median_iwccryrcr.dat"
     call register_io(IO_OUTPUT_CONFIG,OUTPUT_CONFIG_FNAME)
     write(IO_OUTPUT_CONFIG,'(289ES15.5)') ct(:)/3600.0
     write(IO_OUTPUT_CONFIG,'(100ES15.5)') cz(3:nz-2)/1000.0
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') iwccryrcr1d(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') iwccryrcr1d1(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') iwccryrcr1d9(:,k)
     enddo     
     call close_io(IO_OUTPUT_CONFIG)

     OUTPUT_CONFIG_FNAME = trim(OUTPUT_DIR)//"data_median_iwcagg.dat"
     call register_io(IO_OUTPUT_CONFIG,OUTPUT_CONFIG_FNAME)
     write(IO_OUTPUT_CONFIG,'(289ES15.5)') ct(:)/3600.0
     write(IO_OUTPUT_CONFIG,'(100ES15.5)') cz(3:nz-2)/1000.0
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') iwcagg1d(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') iwcagg1d1(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') iwcagg1d9(:,k)
     enddo     
     call close_io(IO_OUTPUT_CONFIG)

     OUTPUT_CONFIG_FNAME = trim(OUTPUT_DIR)//"data_median_iwcrag.dat"
     call register_io(IO_OUTPUT_CONFIG,OUTPUT_CONFIG_FNAME)
     write(IO_OUTPUT_CONFIG,'(289ES15.5)') ct(:)/3600.0
     write(IO_OUTPUT_CONFIG,'(100ES15.5)') cz(3:nz-2)/1000.0
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') iwcrag1d(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') iwcrag1d1(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') iwcrag1d9(:,k)
     enddo     
     call close_io(IO_OUTPUT_CONFIG)

     OUTPUT_CONFIG_FNAME = trim(OUTPUT_DIR)//"data_median_nliq.dat"
     call register_io(IO_OUTPUT_CONFIG,OUTPUT_CONFIG_FNAME)
     write(IO_OUTPUT_CONFIG,'(289ES15.5)') ct(:)/3600.0
     write(IO_OUTPUT_CONFIG,'(100ES15.5)') cz(3:nz-2)/1000.0
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') nliq1d(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') nliq1d1(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') nliq1d9(:,k)
     enddo     
     call close_io(IO_OUTPUT_CONFIG)

     OUTPUT_CONFIG_FNAME = trim(OUTPUT_DIR)//"data_median_nice.dat"
     call register_io(IO_OUTPUT_CONFIG,OUTPUT_CONFIG_FNAME)
     write(IO_OUTPUT_CONFIG,'(289ES15.5)') ct(:)/3600.0
     write(IO_OUTPUT_CONFIG,'(100ES15.5)') cz(3:nz-2)/1000.0
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') nice1d(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') nice1d1(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') nice1d9(:,k)
     enddo     
     call close_io(IO_OUTPUT_CONFIG)

     OUTPUT_CONFIG_FNAME = trim(OUTPUT_DIR)//"data_median_nice200.dat"
     call register_io(IO_OUTPUT_CONFIG,OUTPUT_CONFIG_FNAME)
     write(IO_OUTPUT_CONFIG,'(289ES15.5)') ct(:)/3600.0
     write(IO_OUTPUT_CONFIG,'(100ES15.5)') cz(3:nz-2)/1000.0
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') nice2001d(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') nice2001d1(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') nice2001d9(:,k)
     enddo     
     call close_io(IO_OUTPUT_CONFIG)

     OUTPUT_CONFIG_FNAME = trim(OUTPUT_DIR)//"data_median_niceall.dat"
     call register_io(IO_OUTPUT_CONFIG,OUTPUT_CONFIG_FNAME)
     write(IO_OUTPUT_CONFIG,'(289ES15.5)') ct(:)/3600.0
     write(IO_OUTPUT_CONFIG,'(100ES15.5)') cz(3:nz-2)/1000.0
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') niceall1d(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') niceall1d1(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') niceall1d9(:,k)
     enddo     
     call close_io(IO_OUTPUT_CONFIG)

     OUTPUT_CONFIG_FNAME = trim(OUTPUT_DIR)//"data_median_nice200all.dat"
     call register_io(IO_OUTPUT_CONFIG,OUTPUT_CONFIG_FNAME)
     write(IO_OUTPUT_CONFIG,'(289ES15.5)') ct(:)/3600.0
     write(IO_OUTPUT_CONFIG,'(100ES15.5)') cz(3:nz-2)/1000.0
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') nice200all1d(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') nice200all1d1(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') nice200all1d9(:,k)
     enddo     
     call close_io(IO_OUTPUT_CONFIG)

     OUTPUT_CONFIG_FNAME = trim(OUTPUT_DIR)//"data_median_Dliq.dat"
     call register_io(IO_OUTPUT_CONFIG,OUTPUT_CONFIG_FNAME)
     write(IO_OUTPUT_CONFIG,'(289ES15.5)') ct(:)/3600.0
     write(IO_OUTPUT_CONFIG,'(100ES15.5)') cz(3:nz-2)/1000.0
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') dliq1d(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') dliq1d1(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') dliq1d9(:,k)
     enddo     
     call close_io(IO_OUTPUT_CONFIG)

     OUTPUT_CONFIG_FNAME = trim(OUTPUT_DIR)//"data_median_Dice.dat"
     call register_io(IO_OUTPUT_CONFIG,OUTPUT_CONFIG_FNAME)
     write(IO_OUTPUT_CONFIG,'(289ES15.5)') ct(:)/3600.0
     write(IO_OUTPUT_CONFIG,'(100ES15.5)') cz(3:nz-2)/1000.0
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') dice1d(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') dice1d1(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') dice1d9(:,k)
     enddo     
     call close_io(IO_OUTPUT_CONFIG)

     OUTPUT_CONFIG_FNAME = trim(OUTPUT_DIR)//"data_median_habit_aggall.dat"
     call register_io(IO_OUTPUT_CONFIG,OUTPUT_CONFIG_FNAME)
     write(IO_OUTPUT_CONFIG,'(289ES15.5)') ct(:)/3600.0
     write(IO_OUTPUT_CONFIG,'(100ES15.5)') cz(3:nz-2)/1000.0
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') aggall1d(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') aggall1d1(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') aggall1d9(:,k)
     enddo     
     call close_io(IO_OUTPUT_CONFIG)

     OUTPUT_CONFIG_FNAME = trim(OUTPUT_DIR)//"data_median_habit_cryall.dat"
     call register_io(IO_OUTPUT_CONFIG,OUTPUT_CONFIG_FNAME)
     write(IO_OUTPUT_CONFIG,'(289ES15.5)') ct(:)/3600.0
     write(IO_OUTPUT_CONFIG,'(100ES15.5)') cz(3:nz-2)/1000.0
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') cryall1d(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') cryall1d1(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') cryall1d9(:,k)
     enddo     
     call close_io(IO_OUTPUT_CONFIG)

     OUTPUT_CONFIG_FNAME = trim(OUTPUT_DIR)//"data_median_habit_rcrall.dat"
     call register_io(IO_OUTPUT_CONFIG,OUTPUT_CONFIG_FNAME)
     write(IO_OUTPUT_CONFIG,'(289ES15.5)') ct(:)/3600.0
     write(IO_OUTPUT_CONFIG,'(100ES15.5)') cz(3:nz-2)/1000.0
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') rcrall1d(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') rcrall1d1(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') rcrall1d9(:,k)
     enddo     
     call close_io(IO_OUTPUT_CONFIG)

     OUTPUT_CONFIG_FNAME = trim(OUTPUT_DIR)//"data_median_habit_graall.dat"
     call register_io(IO_OUTPUT_CONFIG,OUTPUT_CONFIG_FNAME)
     write(IO_OUTPUT_CONFIG,'(289ES15.5)') ct(:)/3600.0
     write(IO_OUTPUT_CONFIG,'(100ES15.5)') cz(3:nz-2)/1000.0
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') graall1d(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') graall1d1(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') graall1d9(:,k)
     enddo     
     call close_io(IO_OUTPUT_CONFIG)

     OUTPUT_CONFIG_FNAME = trim(OUTPUT_DIR)//"data_median_habit_ragall.dat"
     call register_io(IO_OUTPUT_CONFIG,OUTPUT_CONFIG_FNAME)
     write(IO_OUTPUT_CONFIG,'(289ES15.5)') ct(:)/3600.0
     write(IO_OUTPUT_CONFIG,'(100ES15.5)') cz(3:nz-2)/1000.0
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') ragall1d(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') ragall1d1(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') ragall1d9(:,k)
     enddo     
     call close_io(IO_OUTPUT_CONFIG)

     OUTPUT_CONFIG_FNAME = trim(OUTPUT_DIR)//"data_median_habit_agg.dat"
     call register_io(IO_OUTPUT_CONFIG,OUTPUT_CONFIG_FNAME)
     write(IO_OUTPUT_CONFIG,'(289ES15.5)') ct(:)/3600.0
     write(IO_OUTPUT_CONFIG,'(100ES15.5)') cz(3:nz-2)/1000.0
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') agg1d(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') agg1d1(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') agg1d9(:,k)
     enddo     
     call close_io(IO_OUTPUT_CONFIG)

     OUTPUT_CONFIG_FNAME = trim(OUTPUT_DIR)//"data_median_habit_cry.dat"
     call register_io(IO_OUTPUT_CONFIG,OUTPUT_CONFIG_FNAME)
     write(IO_OUTPUT_CONFIG,'(289ES15.5)') ct(:)/3600.0
     write(IO_OUTPUT_CONFIG,'(100ES15.5)') cz(3:nz-2)/1000.0
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') cry1d(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') cry1d1(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') cry1d9(:,k)
     enddo     
     call close_io(IO_OUTPUT_CONFIG)

     OUTPUT_CONFIG_FNAME = trim(OUTPUT_DIR)//"data_median_habit_rcr.dat"
     call register_io(IO_OUTPUT_CONFIG,OUTPUT_CONFIG_FNAME)
     write(IO_OUTPUT_CONFIG,'(289ES15.5)') ct(:)/3600.0
     write(IO_OUTPUT_CONFIG,'(100ES15.5)') cz(3:nz-2)/1000.0
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') rcr1d(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') rcr1d1(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') rcr1d9(:,k)
     enddo     
     call close_io(IO_OUTPUT_CONFIG)

     OUTPUT_CONFIG_FNAME = trim(OUTPUT_DIR)//"data_median_habit_gra.dat"
     call register_io(IO_OUTPUT_CONFIG,OUTPUT_CONFIG_FNAME)
     write(IO_OUTPUT_CONFIG,'(289ES15.5)') ct(:)/3600.0
     write(IO_OUTPUT_CONFIG,'(100ES15.5)') cz(3:nz-2)/1000.0
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') gra1d(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') gra1d1(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') gra1d9(:,k)
     enddo     
     call close_io(IO_OUTPUT_CONFIG)

     OUTPUT_CONFIG_FNAME = trim(OUTPUT_DIR)//"data_median_habit_rag.dat"
     call register_io(IO_OUTPUT_CONFIG,OUTPUT_CONFIG_FNAME)
     write(IO_OUTPUT_CONFIG,'(289ES15.5)') ct(:)/3600.0
     write(IO_OUTPUT_CONFIG,'(100ES15.5)') cz(3:nz-2)/1000.0
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') rag1d(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') rag1d1(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') rag1d9(:,k)
     enddo     
     call close_io(IO_OUTPUT_CONFIG)

     OUTPUT_CONFIG_FNAME = trim(OUTPUT_DIR)//"data_median_habit_monocl.dat"
     call register_io(IO_OUTPUT_CONFIG,OUTPUT_CONFIG_FNAME)
     write(IO_OUTPUT_CONFIG,'(289ES15.5)') ct(:)/3600.0
     write(IO_OUTPUT_CONFIG,'(100ES15.5)') cz(3:nz-2)/1000.0
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') monocl1d(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') monocl1d1(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') monocl1d9(:,k)
     enddo     
     call close_io(IO_OUTPUT_CONFIG)

     OUTPUT_CONFIG_FNAME = trim(OUTPUT_DIR)//"data_median_habit_monopl.dat"
     call register_io(IO_OUTPUT_CONFIG,OUTPUT_CONFIG_FNAME)
     write(IO_OUTPUT_CONFIG,'(289ES15.5)') ct(:)/3600.0
     write(IO_OUTPUT_CONFIG,'(100ES15.5)') cz(3:nz-2)/1000.0
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') monopl1d(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') monopl1d1(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') monopl1d9(:,k)
     enddo     
     call close_io(IO_OUTPUT_CONFIG)

     OUTPUT_CONFIG_FNAME = trim(OUTPUT_DIR)//"data_median_habit_monodd.dat"
     call register_io(IO_OUTPUT_CONFIG,OUTPUT_CONFIG_FNAME)
     write(IO_OUTPUT_CONFIG,'(289ES15.5)') ct(:)/3600.0
     write(IO_OUTPUT_CONFIG,'(100ES15.5)') cz(3:nz-2)/1000.0
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') monodd1d(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') monodd1d1(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') monodd1d9(:,k)
     enddo     
     call close_io(IO_OUTPUT_CONFIG)

     OUTPUT_CONFIG_FNAME = trim(OUTPUT_DIR)//"data_median_habit_polycl.dat"
     call register_io(IO_OUTPUT_CONFIG,OUTPUT_CONFIG_FNAME)
     write(IO_OUTPUT_CONFIG,'(289ES15.5)') ct(:)/3600.0
     write(IO_OUTPUT_CONFIG,'(100ES15.5)') cz(3:nz-2)/1000.0
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') polycl1d(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') polycl1d1(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') polycl1d9(:,k)
     enddo     
     call close_io(IO_OUTPUT_CONFIG)

     OUTPUT_CONFIG_FNAME = trim(OUTPUT_DIR)//"data_median_habit_polypl.dat"
     call register_io(IO_OUTPUT_CONFIG,OUTPUT_CONFIG_FNAME)
     write(IO_OUTPUT_CONFIG,'(289ES15.5)') ct(:)/3600.0
     write(IO_OUTPUT_CONFIG,'(100ES15.5)') cz(3:nz-2)/1000.0
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') polypl1d(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') polypl1d1(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') polypl1d9(:,k)
     enddo     
     call close_io(IO_OUTPUT_CONFIG)

     OUTPUT_CONFIG_FNAME = trim(OUTPUT_DIR)//"data_median_habit_polyir.dat"
     call register_io(IO_OUTPUT_CONFIG,OUTPUT_CONFIG_FNAME)
     write(IO_OUTPUT_CONFIG,'(289ES15.5)') ct(:)/3600.0
     write(IO_OUTPUT_CONFIG,'(100ES15.5)') cz(3:nz-2)/1000.0
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') polyir1d(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') polyir1d1(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') polyir1d9(:,k)
     enddo     
     call close_io(IO_OUTPUT_CONFIG)

     OUTPUT_CONFIG_FNAME = trim(OUTPUT_DIR)//"data_median_riming.dat"
     call register_io(IO_OUTPUT_CONFIG,OUTPUT_CONFIG_FNAME)
     write(IO_OUTPUT_CONFIG,'(289ES15.5)') ct(:)/3600.0
     write(IO_OUTPUT_CONFIG,'(100ES15.5)') cz(3:nz-2)/1000.0
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') rmrim1d(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') rmrim1d1(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') rmrim1d9(:,k)
     enddo     
     call close_io(IO_OUTPUT_CONFIG)

     OUTPUT_CONFIG_FNAME = trim(OUTPUT_DIR)//"data_median_imm.dat"
     call register_io(IO_OUTPUT_CONFIG,OUTPUT_CONFIG_FNAME)
     write(IO_OUTPUT_CONFIG,'(289ES15.5)') ct(:)/3600.0
     write(IO_OUTPUT_CONFIG,'(100ES15.5)') cz(3:nz-2)/1000.0
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') rmimm1d(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') rmimm1d1(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') rmimm1d9(:,k)
     enddo     
     call close_io(IO_OUTPUT_CONFIG)

     OUTPUT_CONFIG_FNAME = trim(OUTPUT_DIR)//"data_median_vapor.dat"
     call register_io(IO_OUTPUT_CONFIG,OUTPUT_CONFIG_FNAME)
     write(IO_OUTPUT_CONFIG,'(289ES15.5)') ct(:)/3600.0
     write(IO_OUTPUT_CONFIG,'(100ES15.5)') cz(3:nz-2)/1000.0
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') rmvap1d(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') rmvap1d1(:,k)
     enddo
     write(IO_OUTPUT_CONFIG,'(A)') "---"
     do k = 1, KMAX
        write(IO_OUTPUT_CONFIG,'(289ES15.5)') rmvap1d9(:,k)
     enddo     
     call close_io(IO_OUTPUT_CONFIG)

  endif


  deallocate(fz,fx,cx,cy,cz,ct)
  deallocate(dens,temperature,potential,u,v,w,qv,qliq,qhyd,qice,bin_liq_con,bin_ice_con)
  deallocate(bin_ice_Dmax,bin_ice_mass,bin_ice_magg,bin_ice_mcry,bin_ice_mrim)
  deallocate(bin_ice_alen,bin_ice_clen,bin_ice_dlen)
  deallocate(ice_mass_rim,ice_mass_imm,ice_mass_vapor,ice_mass_eva)
  deallocate(sink,lwdn,swdn,rhoh_rd,rhoh_mp)
  if (poly_exist) then
     deallocate(bin_ice_nex,bin_ice_aglen,bin_ice_cglen)
  endif

  deallocate(nliq1d)
  deallocate(nliq1d1)
  deallocate(nliq1d9)
  deallocate(ini_nliq1d)
  ! ice number concentration
  deallocate(nice1d)
  deallocate(nice1d1)
  deallocate(nice1d9)
  ! liquid effective diameter
  deallocate(dliq1d)
  deallocate(dliq1d1)
  deallocate(dliq1d9)
  ! ice effective diameter (maximum dimension)
  deallocate(dice1d)
  deallocate(dice1d1)
  deallocate(dice1d9)
  ! density profile
  deallocate(dens1d)
  deallocate(dens1d1)
  deallocate(dens1d9)
  deallocate(ini_dens1d)
  ! pressure profile
  deallocate(P1d)
  deallocate(P1d1)
  deallocate(P1d9)
  deallocate(ini_P1d)
  ! temperature profile
  deallocate(T1d)
  deallocate(T1d1)
  deallocate(T1d9)
  deallocate(ini_T1d)
  ! potential temperature profile
  deallocate(ini_PT1d)
  ! liquid mixing ratio profile
  deallocate(ini_qliq1d)
  ! downwelling longwave profile
  deallocate(ini_lwdn1d)
  ! downwelling shortwave profile
  deallocate(ini_swdn1d)
  ! subsidence profile
  deallocate(ini_sink1d)
  ! rhoh rd profile
  deallocate(ini_rhoh_rd1d)
  ! rhoh mp profile
  deallocate(ini_rhoh_mp1d)
  ! liquid water content profile
  deallocate(lwc1d)
  deallocate(lwc1d1)
  deallocate(lwc1d9)
  deallocate(ini_lwc1d)
  ! ice water content profile
  deallocate(iwc1d)
  deallocate(iwc1d1)
  deallocate(iwc1d9)
  deallocate(iwccry1d)
  deallocate(iwccry1d1)
  deallocate(iwccry1d9)
  deallocate(iwcrcr1d)
  deallocate(iwcrcr1d1)
  deallocate(iwcrcr1d9)
  deallocate(iwcgra1d)
  deallocate(iwcgra1d1)
  deallocate(iwcgra1d9)
  deallocate(iwccryrcr1d)
  deallocate(iwccryrcr1d1)
  deallocate(iwccryrcr1d9)
  deallocate(iwcagg1d)
  deallocate(iwcagg1d1)
  deallocate(iwcagg1d9)
  deallocate(iwcrag1d)
  deallocate(iwcrag1d1)
  deallocate(iwcrag1d9)
  ! qv profile
  deallocate(qv1d)
  deallocate(qv1d1)
  deallocate(qv1d9)
  deallocate(ini_qv1d)
  ! wind speed profile
  deallocate(u1d)
  deallocate(u1d1)
  deallocate(u1d9)
  deallocate(ini_u1d)
  ! RH profile
  deallocate(rh1d)
  deallocate(rh1d1)
  deallocate(rh1d9)
  deallocate(ini_rh1d)
  ! RHI profile
  deallocate(rhi1d)
  deallocate(rhi1d1)
  deallocate(rhi1d9)
  deallocate(ini_rhi1d)
  ! TKE profile
  deallocate(tke1d)
  deallocate(tke1d1)
  deallocate(tke1d9)
  ! ice types
  deallocate(agg1d)
  deallocate(cry1d)
  deallocate(rag1d)
  deallocate(rcr1d)
  deallocate(gra1d)
  deallocate(agg1d1)
  deallocate(cry1d1)
  deallocate(rag1d1)
  deallocate(rcr1d1)
  deallocate(gra1d1)
  deallocate(agg1d9)
  deallocate(cry1d9)
  deallocate(rag1d9)
  deallocate(rcr1d9)
  deallocate(gra1d9)
  ! ice habits
  deallocate(monopl1d)
  deallocate(monodd1d)
  deallocate(monocl1d)
  deallocate(monopl1d1)
  deallocate(monodd1d1)
  deallocate(monocl1d1)
  deallocate(monopl1d9)
  deallocate(monodd1d9)
  deallocate(monocl1d9)
  deallocate(polypl1d)
  deallocate(polyir1d)
  deallocate(polycl1d)
  deallocate(polypl1d1)
  deallocate(polyir1d1)
  deallocate(polycl1d1)
  deallocate(polypl1d9)
  deallocate(polyir1d9)
  deallocate(polycl1d9)
  ! microphysics process
  deallocate(rmrim1d)
  deallocate(rmrim1d1)
  deallocate(rmrim1d9)
  deallocate(rmvap1d)
  deallocate(rmvap1d1)
  deallocate(rmvap1d9)
  deallocate(rmimm1d)
  deallocate(rmimm1d1)
  deallocate(rmimm1d9)

  deallocate(local_var_T)
  deallocate(local_var_rh)
  deallocate(local_var_rhi)
  deallocate(local_var_qv)
  deallocate(local_var_u)
  deallocate(local_var_tke)
  deallocate(local_var_lwc)
  deallocate(local_var_iwc)
  deallocate(local_var_nliq)
  deallocate(local_var_nice)
  deallocate(local_var_nice200)
  deallocate(local_var_dliq)
  deallocate(local_var_dice)
  deallocate(local_var_cry)
  deallocate(local_var_agg)
  deallocate(local_var_rag)
  deallocate(local_var_gra)
  deallocate(local_var_rcr)
  deallocate(local_var_monopl)
  deallocate(local_var_monocl)
  deallocate(local_var_monodd)
  deallocate(local_var_polypl)
  deallocate(local_var_polycl)
  deallocate(local_var_polyir)
  deallocate(local_var_rmrim)
  deallocate(local_var_rmimm)
  deallocate(local_var_rmvap)

  deallocate(global_var_T)
  deallocate(global_var_rh)
  deallocate(global_var_rhi)
  deallocate(global_var_qv)
  deallocate(global_var_u)
  deallocate(global_var_tke)
  deallocate(global_var_lwc)
  deallocate(global_var_iwc)
  deallocate(global_var_nliq)
  deallocate(global_var_nice)
  deallocate(global_var_nice200)
  deallocate(global_var_dliq)
  deallocate(global_var_dice)
  deallocate(global_var_cry)
  deallocate(global_var_agg)
  deallocate(global_var_rag)
  deallocate(global_var_gra)
  deallocate(global_var_rcr)
  deallocate(global_var_monopl)
  deallocate(global_var_monocl)
  deallocate(global_var_monodd)
  deallocate(global_var_polypl)
  deallocate(global_var_polycl)
  deallocate(global_var_polyir)
  deallocate(global_var_rmrim)
  deallocate(global_var_rmimm)
  deallocate(global_var_rmvap)


  call close_io(IO_INPUT_CONFIG)

  call MPI_FINALIZE( ierr )

contains

  subroutine check(status)
    integer, intent ( in) :: status
    
    if(status /= nf90_noerr) then 
      print *, trim(nf90_strerror(status))
      call PRC_abort
      stop "Stopped"
    end if
  end subroutine check  

  subroutine register_io(input,file_name,namelist_file)
    implicit none

    integer, intent(inout) :: input
    character(len=*), intent(in) :: file_name
    logical, optional, intent(in) :: namelist_file
    integer :: i
    logical :: i_opened


    if (current_io_number > MAX_IO_NUMBER) then
       write(*,*) irank, "IO number exceeded max number", current_io_number, MAX_IO_NUMBER
       call PRC_abort
    endif
    do i = 1, MAX_IO_NUMBER
       inquire(i,OPENED=i_opened)
       if (.not. i_opened) then
          input = i
          if (present(namelist_file)) then
             if (namelist_file) then
                open(input,file=trim(file_name),status='old', &
                     form="formatted", delim='apostrophe', iostat=ierr )
             endif
          else
             open(input,file=trim(file_name),status='unknown',iostat=ierr)
          endif
          if ( ierr /= 0 ) then
             write (*, *) "ERROR: Fail to open the configuration file :", trim(INPUT_CONFIG_FNAME)
             call PRC_abort
          endif

          current_io_number = current_io_number + 1
          exit
       endif
    enddo

  end subroutine register_io


  subroutine close_io(input)
    implicit none

    integer, intent(inout) :: input
    integer :: i
    logical :: i_opened


    do i = 1, MAX_IO_NUMBER
       if (i == input) then
          inquire(i,OPENED=i_opened)
          if (i_opened) then
             close(input)
             current_io_number = current_io_number - 1
             return
          endif
       endif
    enddo

    write(*,*) "WARNING IO: ", i, current_io_number
    call PRC_abort

  end subroutine close_io


  subroutine PRC_abort
    implicit none
    if ( PRC_mpi_alive ) then
       ! tentative approach; input "PRC_UNIVERSAL_COMM_WORLD".
       !call MPI_COMM_CALL_ERRHANDLER(MPI_COMM_WORLD,PRC_ABORT_code,ierr)
       write(*,*) "abort operation in rank", irank
       call MPI_FINALIZE( ierr )
    endif

    stop
  end subroutine PRC_abort


  subroutine quickSorti(inputArray,size,start,end)
    implicit none
    integer, intent(in) :: start, end
    integer, intent(in) :: size
    real(SP), intent(inout) :: inputArray(size)

    integer :: temp, temp1, outer_loop, k, kk, kk_pivot, segmentNumber
    integer :: i, pivot_array_size, pivot_index
    integer :: pivots(size), pivot_size, pivot_array(size)
    logical :: loop_hdl

    if (start >= end) then
       return
    endif
    
    loop_hdl = .true.

    temp = end - start + 1
    outer_loop = int(temp/2) + 1
    
    pivot_size = 1
    pivots(pivot_size) = start
    pivot_size = pivot_size + 1
    pivots(pivot_size) = end
    k = 1
    segmentNumber = 1
    do while (k <= outer_loop)
       kk = 1
       temp1 = 0
       kk_pivot = 2
       pivot_array_size = 1
       do while (kk <= segmentNumber)
          call quickPartitioni(inputArray,size,pivot_index,pivots(kk_pivot-1),pivots(kk_pivot))
          if (pivot_index-1 > pivots(kk_pivot-1)) then
             pivot_array(pivot_array_size) = pivots(kk_pivot-1)
             pivot_array_size = pivot_array_size + 1
             pivot_array(pivot_array_size) = pivot_index-1
             pivot_array_size = pivot_array_size + 1
             temp1 = temp1 + 1
          endif
          if (pivot_index+1 < pivots(kk_pivot)) then
             pivot_array(pivot_array_size) = pivot_index+1
             pivot_array_size = pivot_array_size + 1
             pivot_array(pivot_array_size) = pivots(kk_pivot)
             pivot_array_size = pivot_array_size + 1
             temp1 = temp1 + 1
          endif
          kk = kk + 1
          kk_pivot = kk_pivot + 2
       enddo
        
       segmentNumber = temp1
       
       if (temp1 == 0) then
          exit
       endif
       
       i = 1
       do while (i <= pivot_array_size)
          pivots(i) = pivot_array(i)
          i = i + 1
       enddo
       k = k + 1
    enddo
  end subroutine quickSorti
    
  subroutine quickPartitioni(inputArray,size,output,start,end)
    implicit none
    integer, intent(in) :: start, end
    integer, intent(in) :: size
    real(SP), intent(inout) :: inputArray(size)
    integer, intent(out) :: output
    
    real(SP) :: pivot, temp
    integer :: k, i
    
    if (start >= end) then
       output = start
       return
    endif
        
    k = start
    pivot = inputArray(end)
    i = start
    do while (k < end)
       if (inputArray(k) < pivot) then
          temp = inputArray(k)
          inputArray(k) = inputArray(i)
          inputArray(i) = temp
          i = i + 1
       endif
       k = k + 1
    enddo
    inputArray(end) = inputArray(i)
    inputArray(i) = pivot
    output = i
    return

  end subroutine quickPartitioni


  subroutine Iice_habit(ice_type,icemass,crymass,aggmass,rimmass)
    implicit none
    real(SP), intent(in) :: icemass, crymass, aggmass, rimmass
    integer, intent(out) :: ice_type

    ice_type = -1
    if (rimmass > 0.1 * icemass) then
       if (aggmass < 0.1 * icemass) then
          if (rimmass < crymass) then
             ice_type = 5 ! rimed crystals
          else
             ice_type = 4 ! graupel
          endif
       else
          if (rimmass > aggmass) then
             ice_type = 4 ! graupel
          else
             ice_type = 3 ! rimed aggregates
          endif
       endif
    else
       if (aggmass < 0.1 * icemass) then
          ice_type = 2 ! pristine crystals
       else
          ice_type = 1 ! aggregates
       endif
    endif
    return

  end subroutine Iice_habit

  subroutine Iice_shape(habit,a_axis,c_axis,d_axis)
    implicit none
    real(SP), intent(in) :: a_axis, c_axis, d_axis
    integer, intent(out) :: habit

    habit = -1
    if (c_axis > a_axis) then
       habit = 3 ! columnar crystals
    else
       if (d_axis > 2.0/3.0 * a_axis) then
          habit = 2 ! dendrites
       else
          habit = 1 ! hexagonal plates
       endif
    endif
  end subroutine Iice_shape

  subroutine Iice_shapePoly(habit,a_axis,c_axis,d_axis,ag_axis,cg_axis,ex_cry)
    implicit none
    real(SP), intent(in) :: a_axis, c_axis, d_axis, ag_axis, cg_axis, ex_cry
    integer, intent(out) :: habit

    real(SP) :: temp

    habit = -1
    if (ex_cry < 0.5) then
       if (c_axis > a_axis) then
          habit = 3 ! columnar crystals
       else
          if (d_axis > 2.0/3.0 * a_axis) then
             habit = 2 ! dendrites
          else
             habit = 1 ! hexagonal plates
          endif
       endif
    else
       if (a_axis == 0.0 .and. c_axis == 0.0) then
          habit = -1
       else if (a_axis == 0.0) then
          temp = 1.0
       else if (c_axis == 0.0) then
          temp = -1.0
       else
          temp = ag_axis/a_axis - cg_axis/c_axis
       endif
       if (temp >= 0.5) then
          habit = 4 ! planar polycrystals
       else if (temp <= -0.5) then
          habit = 5 ! columnar polycrystals
       else
          habit = 6 ! irregular polycrystals
       endif
    endif

    return
  end subroutine Iice_shapePoly

  function dsign(float_number) result (output)
    implicit none
    real(SP), intent(in) :: float_number
    real(SP) :: output

    if (float_number >= 0.0E0) then
       output = 1.0E0
    else
       output = -1.0E0
    endif
  end function dsign

  subroutine convert_habit(output,habit)
    implicit none
    integer, intent(in) :: habit
    integer, intent(out) :: output

    if (habit == 1) then
       output = 1
    else if (habit == 2) then
       output = 2
    else if (habit == 3) then
       output = 3
    else if (habit == 4) then
       output = 5
    else if (habit == 5) then
       output = 4
    else
       output = 6
    endif
    return
  end subroutine convert_habit

  function get_len_c2a(amass) result (output)
    implicit none
    real(SP), intent(in) :: amass
    real(SP) :: output

    real(SP) :: a_ros, b_ros, N_ros, den_i

    ! mass-dimension relationship of bullet roseete
    ! parameters for mass-dimension relationship from Chiruta and Wang (2003)
    a_ros = 0.3257E0
    b_ros = 0.5206E0
    N_ros = 4.0E0
    den_i = 0.91668E0
    
    output = (amass/(den_i*a_ros*N_ros**b_ros))**(1.0E0/3.0E0)
    
  end function get_len_c2a

  function get_len_s1(amass) result (output)
    implicit none
    real(SP), intent(in) :: amass
    real(SP) :: output

    real(SP) :: a, b, c1

    ! mass-dimension relationship of S1
    ! side plane crystal mass-length relation from Mitchel et al (1990)
    a = 0.0041900509E0
    b = 2.3E0
    ! assume that the shape is cylinder that has the aspect ratio of 0.25 for all sizes, so
    ! when it exceed max density, radiuas has to be increased.
    c1 = 0.885565704066706E0
    output = max(0.5E0*(amass/a)**(1.0E0/b),c1*amass**(1.0E0/3.0E0))

  end function get_len_s1

  function get_len_s3(amass) result (output)
    implicit none
    real(SP), intent(in) :: amass
    real(SP) :: output

    real(SP) :: a, b, c1

    ! mass-dimension relationship of S3
    ! S3 crystal mass-length relation from Mitchel et al (1990)
    a = 0.0027696359E0
    b = 2.1E0
    ! assume that the shape is cylinder that has the aspect ratio of 0.25 for all sizes, so
    ! when it exceed max density, radiuas has to be increased.
    c1 = 0.885565704066706E0
    output = max(0.5E0*(amass/a)**(1.0E0/b),c1*amass**(1.0E0/3.0E0))

  end function get_len_s3

  subroutine get_elen_rlen( elen, rlen, habit, mcry, alen, clen, aglen, cglen )
    implicit none
    real(SP), intent(in) :: mcry, alen, clen, aglen, cglen
    integer, intent(in) :: habit
    real(SP), intent(out) :: elen, rlen

    real(SP) :: xlen_s1, xlen_c2a, xlen_s3, i_ag_ge_cgclen, i_cg_ge_agalen


    xlen_s1 = get_len_s1(mcry)
    xlen_c2a = get_len_c2a(mcry)
    xlen_s3 = get_len_s3(mcry)

    if (habit <= 3 .or. alen < 1.e-15 .or. clen < 1.e-15) then
       elen = 0.0E0
       rlen = 0.0E0
       return
    else
       i_ag_ge_cgclen = 0.5E0*(1.0E0 + dsign(aglen-(cglen/clen+0.5E0)*alen))
       i_cg_ge_agalen = 0.5E0*(1.0E0 + dsign(cglen-(aglen/alen+0.5E0)*clen))
    endif

    ! side planes or radiating assemblage of plates: S1 or P7a
    ! bullet rosette
    ! combination of side planes, bullets, and columns: S3
    elen=&
          i_ag_ge_cgclen*xlen_s1 + &
          (1.0E0 - i_ag_ge_cgclen)*i_cg_ge_agalen*0.0E0 + &
          (1.0E0 - i_ag_ge_cgclen)*(1.0E0 - i_cg_ge_agalen)* &
          xlen_s3

    rlen=&
          i_ag_ge_cgclen*0.0E0 + &
          (1.0E0 - i_ag_ge_cgclen)*i_cg_ge_agalen* &
          xlen_c2a + &
          (1.0E0 - i_ag_ge_cgclen)*(1.0E0 - i_cg_ge_agalen)* &
          xlen_s3 

    return

  end subroutine get_elen_rlen

end program mpipaper5analysis_time 


