program mpipaper5analysis_time
  use netcdf
  use mpi
  use ieee_arithmetic
  implicit none

  ! debug output
  logical :: debug_output = .false.

  ! precision
  integer, parameter :: SP=4
  integer, parameter :: DP=8

  ! cloud and ice grids criteria
  real(SP), parameter :: qliq_lower_limit=1.e-6
  real(SP), parameter :: qice_lower_limit=1.e-9

  ! polycrystals exist or not, and ice-axis version
  logical, parameter :: poly_exist=.false.
  logical, parameter :: prevent_unphysical_length=.true.
  logical, parameter :: force_downwardlook=.true.
  integer, parameter :: iceaxis_version=3
  real(SP), parameter :: spx_p=0.25E0
  logical, parameter :: micro_bin=.true.

  ! plot particle size distribution
  logical, parameter :: plot_liquid_psd=.false.
  logical, parameter :: plot_ice_psd=.false.

  ! io
  integer :: current_io_number
  integer, parameter :: MAX_IO_NUMBER=60

  ! input file model variables
  integer :: IO_INPUT_CONFIG                ! input file io
  character (len=256) :: INPUT_CONFIG_FNAME ! input file name
  integer :: prc_num_x, prc_num_y           ! model process numbers
  integer :: grd_x, grd_y, grd_z            ! model global grid numbers
  integer :: IMAX, JMAX, KMAX               ! model local grid numbers
  real(DP) :: DX, DY                        ! model horizontal grid spacings
  logical :: FILE_HISTORY_CARTESC_BOUNDARY=.false. ! model boundary output

  ! data file variables
  integer, allocatable :: IO_DATA(:)

  ! netcdf file variables
  character (len=256), parameter :: INPUT_DIR="/home/OngChiaRui/WORKING/scale-5.4.4/scale-rm/test/tutorial/mpace/data/bulk_surface/2300klein_lw/sim_mpace_3D_nohydro_tlsABLmy1_c09nccn30_den0gamma1_fracdust0013/" ! input file directory
  !character (len=256), parameter :: INPUT_DIR="/home/OngChiaRui/WORKING/scale-5.4.4/scale-rm/test/tutorial/sheba/sim_3D_noicerad_fracdust00004/" !sim_3D_fracdust00004 sim_3D_den0gamma027_fracdust00004
  character (len = 256), allocatable :: FILE_DIR(:)
  character (len = 256), allocatable :: FILE_NAME(:)
  character (len = 6) :: word_zeros
  character (len = 6) :: word
  integer, parameter :: TOTAL_FILE_NUMBER=4

  ! output file
  integer :: IO_OUTPUT_CONFIG ! output file io
  character (len=256) :: OUTPUT_CONFIG_FNAME ! output file name
  character (len=256), parameter :: OUTPUT_DIR="/home/OngChiaRui/WORKING/scale-5.4.4/scale-rm/test/tutorial/mpace/data/bulk_surface/2300klein_lw/sim_mpace_3D_nohydro_tlsABLmy1_c09nccn30_den0gamma1_fracdust0013/paper5_figures/" ! output file directory
  !character (len=256), parameter :: OUTPUT_DIR="/home/OngChiaRui/WORKING/scale-5.4.4/scale-rm/test/tutorial/sheba/sim_3D_noicerad_fracdust00004/paper5_figures/" ! output file directory

  ! netcdf indices
  integer :: ncid, dimid, varid

  ! MPI communication variables
  logical :: PRC_mpi_alive = .false.
  integer :: isize, irank
  integer :: sendcount_median, count_global_median
  integer, parameter :: master=0
  integer, parameter :: MAX_PRC_X=4
  integer :: TOTAL_PRC
  integer :: JOBS_PER_MPI, JOBS_PER_MPI_X
  integer, allocatable :: jobs_index(:)
  integer :: mpi_x, mpi_y
  integer :: start_3d(4), count_3d(4), start_2d(3), count_2d(3)

  ! history items
  character (len=32) :: NAME
  character (len=32) :: varname

  ! grid
  integer :: is, ie, isg, ieg, js, je, jsg, jeg, ks, ke, ts, te
  integer :: nfx, nfz, nx, ny, nz, nt, nt_partial(TOTAL_FILE_NUMBER)
  real(DP), allocatable :: fx(:), fz(:), cx(:), cy(:), cz(:), ct(:)

  ! data
  real(SP), allocatable :: global_var(:)

  real(SP), allocatable :: local_dNdlogD(:,:), dNdlogD(:,:)
  real(SP), allocatable :: local_dNidlogD(:,:), dNidlogD(:,:)
  real(SP), allocatable :: local_NP(:), local_N(:)
  real(SP), allocatable :: mean_dNdlogD(:), median_dNdlogD(:)
  real(SP), allocatable :: mean_dNidlogD(:), median_dNidlogD(:)

  real(SP), allocatable :: local_pdf_temp(:), local_pdf_temp_rim(:), local_pdf_temp_imm(:)
  real(SP), allocatable :: local_pdf_temp_rimNORMliq(:), local_pdf_temp_vap(:)
  real(SP), allocatable :: local_pdf_temp_aggcon(:), local_pdf_temp_rcrcon(:)
  real(SP), allocatable :: local_pdf_temp_ragcon(:), local_pdf_temp_gracon(:)
  real(SP), allocatable :: local_pdf_temp_crycon(:), local_pdf_rad(:), local_pdf_nliq(:)
  real(SP), allocatable :: local_pdf_lwc(:), local_pdf_wvel(:), local_pdf_temp_sweptVc(:)
  real(SP), allocatable :: local_pdf_temp_sweptVcb(:), local_pdf_temp_sweptVcb_real(:)
  real(SP), allocatable :: local_pdf_cbtv_liq(:)

  real(SP), allocatable :: pdf_temp(:), pdf_temp_rim(:)
  real(SP), allocatable :: pdf_temp_rimNORMliq(:), pdf_temp_vap(:), pdf_temp_imm(:)
  real(SP), allocatable :: pdf_temp_aggcon(:), pdf_temp_rcrcon(:)
  real(SP), allocatable :: pdf_temp_ragcon(:), pdf_temp_gracon(:)
  real(SP), allocatable :: pdf_temp_crycon(:), pdf_rad(:), pdf_nliq(:)
  real(SP), allocatable :: pdf_lwc(:), pdf_wvel(:), pdf_temp_sweptVc(:)
  real(SP), allocatable :: pdf_temp_sweptVcb(:), pdf_temp_sweptVcb_real(:)
  real(SP), allocatable :: pdf_cbtv_liq(:)

  real(SP), allocatable :: X_ice_psd(:), Q_ice_psd(:), den_ice_psd(:), char_len_ice_psd(:)
  real(SP), allocatable :: alpha_ice_psd(:), mean_mass_ice_psd(:), con_ice_psd(:)
  real(SP), allocatable :: mass_ice_psd(:), vtm_ice_psd(:)

  real(SP), allocatable :: qliq(:,:,:), qice(:,:,:), qhyd(:,:,:), dens(:,:,:) ! i, j, k
  real(SP), allocatable :: temperature(:,:,:), pressure(:,:,:), rh(:,:,:), rhi(:,:,:)
  real(SP), allocatable :: w(:,:,:)
  real(SP), allocatable :: ice_mass_vap(:,:,:), ice_mass_eva(:,:,:), ice_mass_rim(:,:,:)
  real(SP), allocatable :: ice_mass_imm(:,:,:)
  real(SP), allocatable :: liq_mass_vap(:,:,:), liq_mass_eva(:,:,:), liq_mass_rim(:,:,:)
  real(SP), allocatable :: liq_mass_imm(:,:,:)
  real(SP), allocatable :: bin_ice_mass_vap(:,:,:,:), bin_ice_mass_eva(:,:,:,:), bin_ice_mass_rim(:,:,:,:)
  real(SP), allocatable :: bin_liq_mass_vap(:,:,:,:), bin_liq_mass_eva(:,:,:,:), bin_liq_mass_rim(:,:,:,:)
  real(SP), allocatable :: bin_liq_mass(:,:,:,:)
  real(SP), allocatable :: bin_liq_ctv(:,:,:,:), bin_liq_mtv(:,:,:,:)
  real(SP), allocatable :: bin_liq_con(:,:,:,:), bin_ice_con(:,:,:,:) ! i, j, k, bin
  real(SP), allocatable :: bin_ice_mass(:,:,:,:), bin_ice_mcry(:,:,:,:)
  real(SP), allocatable :: bin_ice_magg(:,:,:,:), bin_ice_mrim(:,:,:,:)
  real(SP), allocatable :: bin_ice_alen(:,:,:,:), bin_ice_clen(:,:,:,:)
  real(SP), allocatable :: bin_ice_dlen(:,:,:,:), bin_ice_aglen(:,:,:,:)
  real(SP), allocatable :: bin_ice_cglen(:,:,:,:), bin_ice_nex(:,:,:,:)
  real(SP), allocatable :: bin_ice_Dmax(:,:,:,:), bin_ice_aDmax(:,:,:,:)
  real(SP), allocatable :: bin_ice_ctv(:,:,:,:), bin_ice_mtv(:,:,:,:)
  real(SP), allocatable :: bin_ice_mfrz(:,:,:,:)

  real(SP), allocatable :: local_pdf_rhi(:), local_pdf_T(:), local_pdf_conc_liq(:), local_pdf_mass_liq(:)
  real(SP), allocatable :: local_pdf_1riming_ice(:), local_pdf_riming_liq(:), local_pdf_riming_ice(:)
  real(SP), allocatable :: local_pdf_1vapor_ice(:), local_pdf_allvapor_liq(:), local_pdf_vapor_liq(:), local_pdf_vapor_ice(:)
  real(SP), allocatable :: local_pdf_1eva_ice(:), local_pdf_alleva_liq(:), local_pdf_eva_liq(:), local_pdf_eva_ice(:)
  real(SP), allocatable :: local_pdf_crymass(:), local_pdf_aggmass(:), local_pdf_rimmass(:)
  real(SP), allocatable :: local_pdf_cry(:), local_pdf_agg(:), local_pdf_rcr(:), local_pdf_gra(:), local_pdf_rag(:)
  real(SP), allocatable :: local_pdf_crymd(:), local_pdf_gramd(:), local_pdf_rcrmd(:), local_pdf_cryrcrmd(:)
  real(SP), allocatable :: local_pdf_den(:), local_pdf_pla(:), local_pdf_col(:), local_pdf_ppl(:), local_pdf_pcl(:)
  real(SP), allocatable :: local_pdf_pir(:), local_pdf_mtv(:), local_pdf_ctv(:), local_pdf_gctv(:), local_pdf_wctv(:)
  real(SP), allocatable :: local_pdf_RE(:), local_pdf_cbtv(:), local_pdf_citv(:), local_pdf_acratio(:), local_pdf_imacratio(:), local_pdf_arearatio(:), local_pdf_arearatio_sidelook(:), local_pdf_arearatio_total(:), local_pdf_adratio(:)
  real(SP), allocatable :: local_pdf_gimacratio(:), local_pdf_sweptVc(:), local_pdf_sweptVm(:), local_pdf_sweptVcb(:), local_pdf_sweptVcb_real(:)
  real(SP), allocatable :: local_pdf_gsweptVc(:), local_pdf_gsweptVm(:), local_pdf_mass(:), local_pdf_bulk(:), local_pdf_split_cap(:)
  real(SP), allocatable :: local_pdf_mfrac_cry(:), local_pdf_mfrac_rim(:)
  real(SP), allocatable :: local_pdf_mass_pla(:), local_pdf_mass_den(:), local_pdf_mass_col(:), local_pdf_mass_ppl(:), local_pdf_mass_pcl(:), local_pdf_mass_pir(:)
  real(SP), allocatable :: local_pdf_mass_cry(:), local_pdf_mass_rcr(:), local_pdf_mass_agg(:), local_pdf_mass_rag(:), local_pdf_mass_gra(:)
  real(SP), allocatable :: local_pdf_split_vent(:), local_pdf_split_kinetic(:), local_pdf_split_area(:), local_pdf_split_areaiceonly(:), local_pdf_split_deltavcb(:), local_pdf_split_areacry(:), local_pdf_split_areaiceonlycry(:), local_pdf_split_areacryrcr(:), local_pdf_split_areaiceonlycryrcr(:)
  real(SP), allocatable :: local_pdf_split_Nwmass(:), local_pdf_conc(:), local_pdf_gconc(:), local_pdf_count(:)
  real(SP), allocatable :: local_mpdf_sweptVcb(:,:), local_mpdf_vapor_ice(:,:), local_mpdf_riming_ice(:,:)
  real(SP), allocatable :: global_mpdf_sweptVcb(:,:), global_mpdf_vapor_ice(:,:), global_mpdf_riming_ice(:,:)
  real(SP), allocatable :: mpdf_sweptVcb(:,:), mpdf_vapor_ice(:,:), mpdf_riming_ice(:,:)

  real(SP), allocatable :: pdf_rhi(:), pdf_T(:), pdf_conc_liq(:), pdf_mass_liq(:)
  real(SP), allocatable :: pdf_1riming_ice(:), pdf_riming_liq(:), pdf_riming_ice(:)
  real(SP), allocatable :: pdf_1vapor_ice(:), pdf_allvapor_liq(:), pdf_vapor_liq(:), pdf_vapor_ice(:)
  real(SP), allocatable :: pdf_1eva_ice(:), pdf_alleva_liq(:), pdf_eva_liq(:), pdf_eva_ice(:)
  real(SP), allocatable :: pdf_crymass(:), pdf_aggmass(:), pdf_rimmass(:)
  real(SP), allocatable :: pdf_cry(:), pdf_agg(:), pdf_rcr(:), pdf_gra(:), pdf_rag(:)
  real(SP), allocatable :: pdf_crymd(:), pdf_cryrcrmd(:), pdf_rcrmd(:), pdf_gramd(:)
  real(SP), allocatable :: pdf_den(:), pdf_pla(:), pdf_col(:), pdf_ppl(:), pdf_pcl(:)
  real(SP), allocatable :: pdf_pir(:), pdf_mtv(:), pdf_ctv(:), pdf_gctv(:), pdf_wctv(:)
  real(SP), allocatable :: pdf_RE(:), pdf_cbtv(:), pdf_citv(:), pdf_acratio(:), pdf_imacratio(:), pdf_arearatio(:), pdf_arearatio_sidelook(:), pdf_arearatio_total(:), pdf_adratio(:)
  real(SP), allocatable :: pdf_gimacratio(:), pdf_sweptVc(:), pdf_sweptVm(:), pdf_sweptVcb(:), pdf_sweptVcb_real(:)
  real(SP), allocatable :: pdf_gsweptVc(:), pdf_gsweptVm(:), pdf_mass(:), pdf_bulk(:), pdf_split_cap(:)
  real(SP), allocatable :: pdf_mfrac_cry(:), pdf_mfrac_rim(:)
  real(SP), allocatable :: pdf_mass_pla(:), pdf_mass_den(:), pdf_mass_col(:), pdf_mass_ppl(:), pdf_mass_pcl(:), pdf_mass_pir(:)
  real(SP), allocatable :: pdf_mass_cry(:), pdf_mass_rcr(:), pdf_mass_agg(:), pdf_mass_rag(:), pdf_mass_gra(:)
  real(SP), allocatable :: pdf_split_vent(:), pdf_split_kinetic(:), pdf_split_area(:), pdf_split_areaiceonly(:), pdf_split_deltavcb(:), pdf_split_areacry(:), pdf_split_areaiceonlycry(:), pdf_split_areacryrcr(:), pdf_split_areaiceonlycryrcr(:)
  real(SP), allocatable :: pdf_split_Nwmass(:), pdf_conc(:), pdf_gconc(:), pdf_count(:)

  ! liquid bins
  integer, parameter :: max_binr = 40
  real(SP), allocatable :: massbin(:), massbin_bnd(:)
  real(SP), allocatable :: diameterbin(:), diameterbin_bnd(:)
  real(SP), parameter :: c_min = 0.000000000000004188790205, c_max = 0.0000000654498
  real(SP), parameter :: c_maxmass = 0.52359870

  ! ice bins
  integer, parameter :: max_bini = 20
  real(SP), allocatable :: massbini(:), massbini_bnd(:)
  real(SP), allocatable :: diameteribin(:), diameteribin_bnd(:)
  real(SP), allocatable :: massbini_bothbnd(:,:), massbini_leftbnd(:), massbini_rightbnd(:)
  real(SP) :: ci_min, ci_max

  ! local variables 
  integer :: i, j, k, t, t_local, k_file, ibin, jj, counter1
  integer :: median_array_size1
  real(SP), allocatable :: median_array1(:)
  integer :: ierr
  logical :: lstat
  real(SP) :: temp, temp1, temp2, temp3
  real(SP) :: dbin, D2, D3, DI2, DI3
  real(SP) :: global_sum1, global_sum2, global_sum3

  real(SP), parameter :: gg = 980.0e+0
  real(SP), parameter :: pi = 3.141592653589793238462643E0
  real(SP), parameter :: sq_three=sqrt(3.0E0)
  ! Mitchell formulation, Re = aX**b
  real(SP), dimension(5), parameter :: Best_interval = (/0.0E0, 2.167592E+01,1.027213E+03,3.934172E+04, 1.0e+30/)
  real(SP), dimension(4), parameter :: Best_a = (/4.742587e-02,9.447403e-02,2.549053e-01,7.880099e-01/)
  real(SP), dimension(4), parameter :: Best_b = (/9.910232e-01,7.722376e-01,6.299768e-01,5.236878e-01/)

  integer :: ice_type, ships_ice_type, my_habit, habit, is_mod, mode
  real(SP) :: vcs, vic, alpha_ic, alpha_im, alpha, psi_ic, den_i, kp, mlmt, mass_agg_cry, mass_agg_cry_rim, local_Dmax, asm, csm, pla_e, ros_r
  real(SP) :: liquid_mass, liquid_number, liquid_eff_rad, liquid_eff_rad_cm, liquid_cbtv, liquid_Nre
  real(SP) :: d_vis, U_S, lambda_a, X, Y, rad, sig_wa, NBO, NP, ap_den, mean_mass, D_v, fkn, Nre, vtm, vti
  real(SP) :: OMEGA, A_e, A_e_ic, A_c, A_c_ic, P, char_len, q_cs, vip, q, LEN, k_w, QQ, gamma, C_DP, C_DO, beta, N_re_0, X_0, Nrei, arearatio, arearatio_sidelook, A_e_sidelook, A_e_ic_sidelook, arearatio_total, A_e_total, A_e_ic_total, A_e_sph, A_e_ic_sph
  real(SP) :: total_sweptVc, total_sweptVcb, total_sweptVcb_real
  real(SP) :: aggcon, ragcon, rcrcon, gracon, crycon
  real(SP) :: CAP_ros, hex_cap, cry_alen, cry_clen, d, eps, capacitance, N_sc, Xv, fv
  real(SP), allocatable :: a2d(:,:), WVT(:,:)
  integer, allocatable :: error_number(:)
  

  
  namelist /PARAM_PRC_CARTESC/ &
       PRC_NUM_X, &
       PRC_NUM_Y

  namelist /PARAM_ATMOS_GRID_CARTESC_INDEX/ &
       KMAX, &
       IMAX, &
       JMAX

  namelist /PARAM_ATMOS_GRID_CARTESC/ &
       DX, &
       DY

  namelist /PARAM_FILE_HISTORY_CARTESC/ &
       FILE_HISTORY_CARTESC_BOUNDARY

  namelist /HISTORY_ITEM/ &
       NAME


  call MPI_INIT( ierr )
  PRC_mpi_alive = .true.
  call MPI_COMM_SIZE( MPI_COMM_WORLD, isize, ierr )
  call MPI_COMM_RANK( MPI_COMM_WORLD, irank, ierr )
  if ( irank == master ) then
     write( *, * ) "+++ MPI COMM: Corrective Initialize"
     write( *, * ) "prevent_unphysical_length: ", prevent_unphysical_length
  endif

  !--- Read from argument
  if ( COMMAND_ARGUMENT_COUNT() /= 1 ) then
     write(*, *) "ERROR: Program needs a config file!"
     call PRC_abort
  else
     call get_command_argument(1,INPUT_CONFIG_FNAME)
  endif

  if (irank == master) then
     write(*,*) "INPUT FILE NAME: ", trim(INPUT_CONFIG_FNAME)
  endif

  ! set initial io number value
  current_io_number = 1

  inquire (file=trim(INPUT_CONFIG_FNAME), exist=lstat)
  if ( .not. lstat ) then
     write (*, *) "ERROR: Not found the configuration file :", trim(INPUT_CONFIG_FNAME)
     call PRC_abort
  endif
  call register_io(IO_INPUT_CONFIG,INPUT_CONFIG_FNAME,.true.)

  rewind(IO_INPUT_CONFIG)
  read(IO_INPUT_CONFIG,nml=PARAM_PRC_CARTESC,iostat=ierr)

  if( ierr < 0 ) then !--- missing
     write(*,*) irank, "ERROR. Not found namelist: PARAM_PRC_CARTESC."
     call PRC_abort
  !elseif( ierr > 0 ) then !--- fatal error
  !   write(*,*) irank, "Not appropriate names in namelist PARAM_PRC_CARTESC, Check!"
  !   call PRC_abort
  end if
  if (irank == master) then
     write(*,*) "PRC_NUM_X PRC_NUM_Y: ", prc_num_x, prc_num_y
  endif
  
  rewind(IO_INPUT_CONFIG)
  read(IO_INPUT_CONFIG,nml=PARAM_ATMOS_GRID_CARTESC,iostat=ierr)

  if( ierr < 0 ) then !--- missing
     write(*,*) irank, "ERROR. Not found namelist: PARAM_ATMOS_GRID_CARTESC."
     call PRC_abort
  !elseif( ierr > 0 ) then !--- fatal error
  !   write(*,*) irank, "Not appropriate names in namelist PARAM_ATMOS_GRID_CARTESC, Check!"
  !   call PRC_abort
  end if

  rewind(IO_INPUT_CONFIG)
  read(IO_INPUT_CONFIG,nml=PARAM_ATMOS_GRID_CARTESC_INDEX,iostat=ierr)

  if( ierr < 0 ) then !--- missing
     write(*,*) irank, "ERROR. Not found namelist: PARAM_ATMOS_GRID_CARTESC_INDEX."
     call PRC_abort
  !elseif( ierr > 0 ) then !--- fatal error
  !   write(*,*) irank, "Not appropriate names in namelist PARAM_ATMOS_GRID_CARTESC_INDEX, Check!"
  !   call PRC_abort
  end if

  call close_io(IO_INPUT_CONFIG)


  if (irank == master) then
     write(*,'(A)') "REMEMBER TO CHECK THESE VALUES, VERY IMPORTANT!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!"
     write(*,*) "KMAX IMAX JMAX: ", KMAX, IMAX, JMAX
     write(*,*) "DX DY: ", DX, DY
  endif

  grd_x = IMAX*prc_num_x
  grd_y = JMAX*prc_num_y
  grd_z = KMAX

  ! check if mpi number is a square
  mpi_x = 0
  mpi_x = 0
  do i = 1, MAX_PRC_X
     if (i*i == isize) then
        mpi_x = i
        mpi_y = i
     endif
  enddo
  if (mpi_x == 0 .or. mpi_y == 0) then
     write(*,*) irank, "ERROR, MPI NUMBER MUST BE A SQUARE", isize
     call PRC_abort
  else
     if (irank == master) then
        write(*,*) "MPI_X and MPI_Y: ", mpi_x, mpi_y
     endif
  endif
  if (prc_num_x /= prc_num_y .or. mod(prc_num_x,mpi_x) /= 0 .or. mod(prc_num_y,mpi_y) /= 0 .or. prc_num_x > mpi_x .or. prc_num_y > mpi_y) then
     write(*,*) irank, "ERROR, MODEL PROCESS NUMBER MUST BE A SQUARE AND MODEL PROCESS NUMBER MUST BE DIVIDED BY MPI SIZE", prc_num_x, prc_num_y, mpi_x, mpi_y
     call PRC_abort
  endif

  
  TOTAL_PRC = prc_num_x*prc_num_y
  JOBS_PER_MPI = TOTAL_PRC/isize
  JOBS_PER_MPI_X = prc_num_x/mpi_x

  if (irank == master) then
     write(*,*) "JOBS_PER_MPI JOBS_PER_MPI_X", JOBS_PER_MPI, JOBS_PER_MPI_X
  endif

  allocate(jobs_index(JOBS_PER_MPI))
  allocate(FILE_NAME(JOBS_PER_MPI))

  ! assign range of model process index
  i = JOBS_PER_MPI_X * mod(irank,mpi_x) + prc_num_x*JOBS_PER_MPI_X * int(irank/mpi_x)
  j = 0
  do k = 1, JOBS_PER_MPI
     jobs_index(k) = i ! minus 1 because history id starts from zero
     j = j + 1
     if (mod(j,JOBS_PER_MPI_X) == 0) then
        i = i + (prc_num_x - mpi_x)
        j = 0
     else
        i = i + 1
     endif
     write(*,*) "rank-wise job id: ", irank, jobs_index(k)
  enddo


  ! define liquid bins
  allocate(massbin(max_binr),massbin_bnd(max_binr+1))
  allocate(diameterbin(max_binr),diameterbin_bnd(max_binr+1))
  temp1 = c_min
  dbin = (c_max/c_min)**(0.05) ! 0.1 = 1/10, 0.05 = 1/20
  temp2 = temp1*dbin
  massbin_bnd(1) = temp1
  do i = 1, 20
     massbin_bnd(i+1) = temp2
     massbin(i) = 0.5*(temp1+temp2) ! liquid haze bins
     temp1 = temp2
     temp2 = temp2*dbin
  enddo
  dbin = (c_maxmass/c_max)**(0.05)
  temp2 = temp1*dbin
  do i = 21, max_binr
     massbin_bnd(i+1) = temp2
     massbin(i) = 0.5*(temp1+temp2) ! liquid drop bins
     temp1 = temp2
     temp2 = temp2*dbin
  enddo
    
  diameterbin_bnd(1) = (massbin_bnd(1)/(3.1416/6.0))**(1.0/3.0)*10000.0
  if (irank == master .and. debug_output) then
     print *, "liquid diameter: ",0, "---", diameterbin_bnd(1)
  endif
  do i = 1, max_binr
     diameterbin(i) = (massbin(i)/(3.1416/6.0))**(1.0/3.0)*10000.0 ! unit is \mu m
     diameterbin_bnd(i+1) = (massbin_bnd(i+1)/(3.1416/6.0))**(1.0/3.0)*10000.0
     if (irank == master .and. debug_output) then
        print*, "liquid diameter: ",i, diameterbin_bnd(i+1), diameterbin(i)
     endif
  enddo

  ! define ice bins
  allocate(massbini(max_bini),massbini_bnd(max_bini+1))
  allocate(diameteribin(max_bini),diameteribin_bnd(max_bini+1))
  allocate(massbini_bothbnd(max_bini,2))
  allocate(massbini_leftbnd(max_bini))
  allocate(massbini_rightbnd(max_bini))
  ci_min = 4.18879020478639e-12
  ci_max = 1.0e-6
  temp1 = ci_min
  dbin = (ci_max/ci_min)**(0.25)
  temp2 = temp1*dbin
  massbini_bnd(1) = temp1
  diameteribin_bnd(1) = (temp1/0.916/3.1416*6.0)**(1.0/3.0)*10000.0
  do i = 1, 4
     massbini_bnd(i+1) = temp2
     massbini(i) = 0.5*(temp1+temp2) ! ice deposition bins
     diameteribin_bnd(i+1) = (temp2/0.916/3.1416*6.0)**(1.0/3.0)*10000.0
     diameteribin(i) = (0.5*(temp1+temp2)/0.916/3.1416*6.0)**(1.0/3.0)*10000.0
     temp1 = temp2
     temp2 = temp2*dbin
  enddo
  ci_min = ci_max
  ci_max = 1.0e-2
  dbin = (ci_max/ci_min)**(1.0/10.0)
  temp2 = temp1*dbin
  do i = 5, 14
     massbini_bnd(i+1) = temp2
     massbini(i) = 0.5*(temp1+temp2) ! ice collection bins
     diameteribin_bnd(i+1) = (temp2/0.916/3.1416*6.0)**(1.0/3.0)*10000.0
     diameteribin(i) = (0.5*(temp1+temp2)/0.916/3.1416*6.0)**(1.0/3.0)*10000.0
     temp1 = temp2
     temp2 = temp2*dbin
  enddo
  ci_min = ci_max
  ci_max = 1.0e+1
  dbin = (ci_max/ci_min)**(1.0/6.0)
  temp2 = temp1*dbin
  do i = 15, 20
     massbini_bnd(i+1) = temp2
     massbini(i) = 0.5*(temp1+temp2) ! ice riming bins
     diameteribin_bnd(i+1) = (temp2/0.916/3.1416*6.0)**(1.0/3.0)*10000.0
     diameteribin(i) = (0.5*(temp1+temp2)/0.916/3.1416*6.0)**(1.0/3.0)*10000.0
     temp1 = temp2
     temp2 = temp2*dbin
  enddo

  do i = 1, max_bini
     massbini_bothbnd(i,1) = massbini_bnd(i)
     massbini_bothbnd(i,2) = massbini_bnd(i+1)
  enddo


  
  
  ! assign tag number for message passing
  ! not need here


  ! history file names
  do k = 1, JOBS_PER_MPI
     if (jobs_index(k) < 10) then
        write(word,'(I1)') jobs_index(k)
        word_zeros = "00000"
     else if (jobs_index(k) < 100) then
        write(word,'(I2)') jobs_index(k)
        word_zeros = "0000"
     else if (jobs_index(k) < 1000) then
        write(word,'(I3)') jobs_index(k)
        word_zeros = "000"
     else if (jobs_index(k) < 10000) then
        write(word,'(I4)') jobs_index(k)
        word_zeros = "00"
     else if (jobs_index(k) < 100000) then
        write(word,'(I5)') jobs_index(k)
        word_zeros = "0"
     else
        write(*,*) "IMPOSSIBLE JOBS ID (TOO LARGE): ", jobs_index(k)
        call PRC_abort
     endif
     FILE_NAME(k) = "history.pe"//trim(word_zeros)//trim(word)//".nc"
     write(*,'(I3,A,A)') k, " th FILE_NAME: ", FILE_NAME(k)
  enddo

  if (TOTAL_FILE_NUMBER > 9) then
     write(*,*) "FILE NUMBER SHOULD BE LESS THAN 10", TOTAL_FILE_NUMBER
     call PRC_abort
  endif
  allocate(FILE_DIR(TOTAL_FILE_NUMBER))
  do k_file = 1, TOTAL_FILE_NUMBER
     write(word,'(I1)') k_file
     FILE_DIR(k_file) = trim(INPUT_DIR)//trim(word)//"/" ! netcdf file directory
     if (irank == master) write(*,*) FILE_DIR(k_file)
  enddo


  ! read netcdf grid
  do k = 1, 1!JOBS_PER_MPI ! temporary, since x and y information are not needed here
     call check( nf90_open( trim(FILE_DIR(1))//trim(FILE_NAME(k)), nf90_nowrite, ncid ) )

     call check( nf90_inq_dimid ( ncid,  'FZ', varid ) )
     call check( nf90_inquire_dimension( ncid, varid, len=nfz ) )

     call check( nf90_inq_dimid ( ncid,  'FX', varid ) )
     call check( nf90_inquire_dimension( ncid, varid, len=nfx ) )

     call check( nf90_inq_dimid ( ncid,  'CZ', varid ) )
     call check( nf90_inquire_dimension( ncid, varid, len=nz ) )
     call check( nf90_inq_dimid ( ncid,  'CX', varid ) )
     call check( nf90_inquire_dimension( ncid, varid, len=nx ) )
     call check( nf90_inq_dimid ( ncid,  'CY', varid ) )
     call check( nf90_inquire_dimension( ncid, varid, len=ny ) )

     allocate(fz(nfz),fx(nfx),cz(nz),cx(nx),cy(ny))

     call check( nf90_inq_varid( ncid, 'FZ', varid ) )
     call check( nf90_get_var( ncid, varid, fz ) )

     call check( nf90_inq_varid( ncid, 'FX', varid ) )
     call check( nf90_get_var( ncid, varid, fx ) )

     call check( nf90_inq_varid( ncid, 'CZ', varid ) )
     call check( nf90_get_var( ncid, varid, cz ) )
     
     call check( nf90_inq_varid(ncid, 'CX', varid) )
     call check( nf90_get_var( ncid, varid, cx ) )
     
     call check( nf90_inq_varid(ncid, 'CY', varid) )
     call check( nf90_get_var( ncid, varid, cy ) )

     call check( nf90_close( ncid ) )
  enddo

  do k_file = 1, TOTAL_FILE_NUMBER
     call check( nf90_open( trim(FILE_DIR(k_file))//trim(FILE_NAME(1)), nf90_nowrite, ncid ) )
     call check( nf90_inq_dimid ( ncid,  'time', varid ) )
     call check( nf90_inquire_dimension( ncid, varid, len=nt_partial(k_file) ) )

     call check( nf90_close( ncid ) )

     if (k_file > 1) nt_partial(k_file) = nt_partial(k_file) - 1
     if (irank == master) write(*,*) k_file, "NT PARTIAL: ", nt_partial(k_file)
  enddo

  nt = sum(nt_partial(:))
  allocate(ct(nt))

  i = 1
  j = 0
  do k_file = 1, TOTAL_FILE_NUMBER
     call check( nf90_open( trim(FILE_DIR(k_file))//trim(FILE_NAME(1)), nf90_nowrite, ncid ) )
     if (k_file > 1) then
        k = 2
     else
        k = 1
     endif
     j = i + nt_partial(k_file) - 1
     call check( nf90_inq_varid(ncid, 'time', varid) )
     call check( nf90_get_var( ncid, varid, ct(i:j), start=(/k/), count=(/nt_partial(k_file)/) ) )
     i = i + nt_partial(k_file)

     call check( nf90_close( ncid ) )
  enddo

  if (TOTAL_FILE_NUMBER > 1) then
     do t =  nt_partial(1)+1, nt
        ct(t) = 2.0E0*ct(t-1) - ct(t-2)
     enddo
  endif

  if (irank == master .and. debug_output) then
     write(*,*) "NFZ NFX NX NY NZ NT: ", nfz, nfx, nx, ny, nz, nt
     do k = 1, 1!JOBS_PER_MPI
        write(*,*) "   -----FZ----- "
        do i = 1, nfz
           write(*,'(I3,E15.6)') i, fz(i)
        enddo
        write(*,*)
        write(*,*) "   -----FX----- "
        do i = 1, nfx
           write(*,'(I3,E15.6)') i, fx(i)
        enddo
        write(*,*)
        write(*,*) "   -----CX----- "
        do i = 1, nx
           write(*,'(I3,E15.6)') i, cx(i)
        enddo
        write(*,*)
        write(*,*) "   -----CY----- "
        do i = 1, ny
           write(*,'(I3,E15.6)') i, cy(i)
        enddo
        write(*,*)
        write(*,*) "   -----CZ----- "
        do i = 1, nz
           write(*,'(I3,E15.6)') i, cz(i)
        enddo
        write(*,*)
        write(*,*) "   -----CT----- "
        do i = 1, nt
           write(*,'(I3,E15.6)') i, ct(i)
        enddo
     enddo
  endif


  isg = 1
  jsg = 1
  ieg = IMAX*JOBS_PER_MPI_X
  jeg = JMAX*JOBS_PER_MPI_X
  sendcount_median = IMAX*JMAX*KMAX*JOBS_PER_MPI ! for x-y-z array
  count_global_median = IMAX*JMAX*KMAX*TOTAL_PRC ! for x-y-z array
  allocate(qliq(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX))
  allocate(qhyd(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX))
  allocate(qice(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX))
  allocate(dens(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX))
  allocate(temperature(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX))
  allocate(pressure(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX))
  allocate(rh(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX))
  allocate(rhi(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX))
  allocate(w(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX))
  allocate(ice_mass_vap(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX))
  allocate(ice_mass_eva(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX))
  allocate(ice_mass_imm(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX))
  allocate(ice_mass_rim(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX))
  allocate(liq_mass_imm(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX))
  allocate(liq_mass_vap(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX))
  allocate(liq_mass_eva(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX))
  allocate(liq_mass_rim(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX))

  allocate(bin_liq_mass_vap(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX,max_binr))
  allocate(bin_liq_mass_eva(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX,max_binr))
  allocate(bin_liq_mass_rim(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX,max_binr))
  allocate(bin_ice_mass_vap(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX,max_bini))
  allocate(bin_ice_mass_eva(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX,max_bini))
  allocate(bin_ice_mass_rim(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX,max_bini))


  allocate(bin_liq_mass(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX,max_binr))
  allocate(bin_liq_ctv(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX,max_bini))
  allocate(bin_liq_mtv(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX,max_bini))
  allocate(bin_liq_con(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX,max_binr))
  allocate(bin_ice_con(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX,max_bini))
  allocate(bin_ice_mcry(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX,max_bini))
  allocate(bin_ice_mrim(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX,max_bini))
  allocate(bin_ice_magg(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX,max_bini))
  allocate(bin_ice_mass(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX,max_bini))
  allocate(bin_ice_alen(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX,max_bini))
  allocate(bin_ice_clen(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX,max_bini))
  allocate(bin_ice_dlen(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX,max_bini))
  allocate(bin_ice_Dmax(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX,max_bini))
  allocate(bin_ice_mfrz(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX,max_bini))
  if (poly_exist) then
     allocate(bin_ice_aglen(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX,max_bini))
     allocate(bin_ice_cglen(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX,max_bini))
     allocate(bin_ice_nex(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX,max_bini))
  endif
  allocate(bin_ice_ctv(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX,max_bini))
  allocate(bin_ice_mtv(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX,max_bini))
  allocate(bin_ice_aDmax(IMAX*JOBS_PER_MPI_X,JMAX*JOBS_PER_MPI_X,KMAX,max_bini))
  !allocate(local_var_1d(sendcount_median))
  !allocate(global_var_1d(count_global_median))
  allocate(local_pdf_temp(sendcount_median))
  allocate(pdf_temp(count_global_median))
  allocate(local_pdf_temp_rim(sendcount_median))
  allocate(pdf_temp_rim(count_global_median))
  allocate(local_pdf_temp_rimNORMliq(sendcount_median))
  allocate(pdf_temp_rimNORMliq(count_global_median))
  allocate(local_pdf_temp_vap(sendcount_median))
  allocate(pdf_temp_vap(count_global_median))
  allocate(local_pdf_temp_imm(sendcount_median))
  allocate(pdf_temp_imm(count_global_median))
  allocate(local_pdf_temp_aggcon(sendcount_median))
  allocate(pdf_temp_aggcon(count_global_median))
  allocate(local_pdf_temp_ragcon(sendcount_median))
  allocate(pdf_temp_ragcon(count_global_median))
  allocate(local_pdf_temp_crycon(sendcount_median))
  allocate(pdf_temp_crycon(count_global_median))
  allocate(local_pdf_temp_rcrcon(sendcount_median))
  allocate(pdf_temp_rcrcon(count_global_median))
  allocate(local_pdf_temp_gracon(sendcount_median))
  allocate(pdf_temp_gracon(count_global_median))
  allocate(local_pdf_rad(sendcount_median))
  allocate(pdf_rad(count_global_median))
  allocate(local_pdf_lwc(sendcount_median))
  allocate(pdf_lwc(count_global_median))
  allocate(local_pdf_nliq(sendcount_median))
  allocate(pdf_nliq(count_global_median))
  allocate(local_pdf_wvel(sendcount_median))
  allocate(pdf_wvel(count_global_median))
  allocate(local_pdf_temp_sweptVc(sendcount_median))
  allocate(pdf_temp_sweptVc(count_global_median))
  allocate(local_pdf_cbtv_liq(sendcount_median))
  allocate(pdf_cbtv_liq(count_global_median))
  allocate(local_pdf_temp_sweptVcb(sendcount_median))
  allocate(pdf_temp_sweptVcb(count_global_median))
  allocate(local_pdf_temp_sweptVcb_real(sendcount_median))
  allocate(pdf_temp_sweptVcb_real(count_global_median))
  allocate(local_dNdlogD(max_binr,sendcount_median))
  allocate(dNdlogD(max_binr,count_global_median))
  allocate(local_dNidlogD(max_bini,sendcount_median))
  allocate(dNidlogD(max_bini,count_global_median))
  allocate(local_NP(max_bini))
  allocate(local_N(max_bini))

  allocate(mean_dNdlogD(max_binr))
  allocate(median_dNdlogD(max_binr))
  allocate(mean_dNidlogD(max_bini))
  allocate(median_dNidlogD(max_bini))

  !!! INITIALIZE GLOBAL VARIABLES
  ! rhi
  allocate(local_pdf_rhi(max_bini))
  ! temperature
  allocate(local_pdf_T(max_bini))
  ! liquid con and mass
  allocate(local_pdf_conc_liq(max_binr))
  allocate(local_pdf_mass_liq(max_binr))
  ! bin-wise riming mass rate
  allocate(local_pdf_riming_liq(max_binr))
  allocate(local_pdf_riming_ice(max_bini))
  allocate(local_pdf_1riming_ice(max_bini))
  allocate(local_mpdf_riming_ice(max_bini,sendcount_median))
  !local_pdf_riming_nwmass(max_bini))
  ! bin-wise vapor deposition/condensation mass rate
  allocate(local_pdf_vapor_liq(max_binr))
  allocate(local_pdf_allvapor_liq(max_binr))
  allocate(local_pdf_vapor_ice(max_bini))
  allocate(local_pdf_1vapor_ice(max_bini))
  allocate(local_mpdf_vapor_ice(max_bini,sendcount_median))
  ! bin-wise vapor evaporation mass rate
  allocate(local_pdf_eva_liq(max_binr))
  allocate(local_pdf_alleva_liq(max_binr))
  allocate(local_pdf_eva_ice(max_bini))
  allocate(local_pdf_1eva_ice(max_bini))
  ! crystal mass
  allocate(local_pdf_crymass(max_bini))
  ! rimed mass
  allocate(local_pdf_rimmass(max_bini))
  ! aggregate mass
  allocate(local_pdf_aggmass(max_bini))
  ! crystal
  allocate(local_pdf_cry(max_bini))
  allocate(local_pdf_crymd(max_bini))
  allocate(local_pdf_cryrcrmd(max_bini))
  ! rimed crystal
  allocate(local_pdf_rcr(max_bini))
  allocate(local_pdf_rcrmd(max_bini))
  ! aggregate
  allocate(local_pdf_agg(max_bini))
  ! graupel
  allocate(local_pdf_gra(max_bini))
  allocate(local_pdf_gramd(max_bini))
  ! rimed graupel
  allocate(local_pdf_rag(max_bini))
  ! plate
  allocate(local_pdf_pla(max_bini))
  ! dendrite
  allocate(local_pdf_den(max_bini))
  ! column
  allocate(local_pdf_col(max_bini))
  if (poly_exist) then
     ! plate
     allocate(local_pdf_ppl(max_bini))
     ! dendrite
     allocate(local_pdf_pcl(max_bini))
     ! column
     allocate(local_pdf_pir(max_bini))
  endif
  ! mass terminal V
  allocate(local_pdf_mtv(max_bini))
  ! conc terminal V
  allocate(local_pdf_ctv(max_bini))
  allocate(local_pdf_gctv(max_bini))
  ! integrated conc terminal V
  allocate(local_pdf_wctv(max_bini))
  ! conc bohm terminal V
  allocate(local_pdf_cbtv(max_bini))
  ! conc Mitchell terminal V
  allocate(local_pdf_citv(max_bini))
  ! Reynolds number
  allocate(local_pdf_RE(max_bini))
  ! axis ratio
  allocate(local_pdf_acratio(max_bini))
  allocate(local_pdf_adratio(max_bini))
  ! axis ratio (ice model)
  allocate(local_pdf_imacratio(max_bini))
  allocate(local_pdf_gimacratio(max_bini))
  ! area ratio
  allocate(local_pdf_arearatio(max_bini))
  allocate(local_pdf_arearatio_sidelook(max_bini))
  allocate(local_pdf_arearatio_total(max_bini))
  ! swept volume
  allocate(local_pdf_sweptVc(max_bini))
  allocate(local_pdf_sweptVm(max_bini))
  allocate(local_pdf_sweptVcb(max_bini))
  allocate(local_pdf_sweptVcb_real(max_bini))
  allocate(local_pdf_gsweptVc(max_bini))
  allocate(local_pdf_gsweptVm(max_bini))
  allocate(local_mpdf_sweptVcb(max_bini,sendcount_median))
  ! mass fraction
  allocate(local_pdf_mfrac_cry(max_bini))
  allocate(local_pdf_mfrac_rim(max_bini))
  ! total mass
  allocate(local_pdf_mass(max_bini))
  ! total mass by habit
  allocate(local_pdf_mass_pla(max_bini))
  allocate(local_pdf_mass_den(max_bini))
  allocate(local_pdf_mass_col(max_bini))
  allocate(local_pdf_mass_ppl(max_bini))
  allocate(local_pdf_mass_pcl(max_bini))
  allocate(local_pdf_mass_pir(max_bini))
  ! total mass by type
  allocate(local_pdf_mass_cry(max_bini))
  allocate(local_pdf_mass_rcr(max_bini))
  allocate(local_pdf_mass_agg(max_bini))
  allocate(local_pdf_mass_rag(max_bini))
  allocate(local_pdf_mass_gra(max_bini))
  ! bulk density
  allocate(local_pdf_bulk(max_bini))
  ! split capacitance, ventilation, kinetic correction
  allocate(local_pdf_split_cap(max_bini))
  allocate(local_pdf_split_vent(max_bini))
  allocate(local_pdf_split_kinetic(max_bini))
  ! split area, delta Vcb, Nwmw
  allocate(local_pdf_split_area(max_bini))
  allocate(local_pdf_split_areaiceonly(max_bini))
  allocate(local_pdf_split_areacry(max_bini))
  allocate(local_pdf_split_areaiceonlycry(max_bini))
  allocate(local_pdf_split_areacryrcr(max_bini))
  allocate(local_pdf_split_areaiceonlycryrcr(max_bini))
  allocate(local_pdf_split_deltaVcb(max_bini))
  allocate(local_pdf_split_Nwmass(max_bini))
  ! total conc
  allocate(local_pdf_conc(max_bini))
  allocate(local_pdf_gconc(max_bini))
  allocate(local_pdf_count(max_bini))

  ! rhi
  allocate(pdf_rhi(max_bini))
  ! temperature
  allocate(pdf_T(max_bini))
  ! liquid con and mass
  allocate(pdf_conc_liq(max_binr))
  allocate(pdf_mass_liq(max_binr))
  ! bin-wise riming mass rate
  allocate(pdf_riming_liq(max_binr))
  allocate(pdf_riming_ice(max_bini))
  allocate(pdf_1riming_ice(max_bini))
  allocate(global_mpdf_riming_ice(max_bini,count_global_median))
  allocate(mpdf_riming_ice(max_bini,3))
  !pdf_riming_nwmass(max_bini))
  ! bin-wise vapor deposition/condensation mass rate
  allocate(pdf_vapor_liq(max_binr))
  allocate(pdf_allvapor_liq(max_binr))
  allocate(pdf_vapor_ice(max_bini))
  allocate(pdf_1vapor_ice(max_bini))
  allocate(global_mpdf_vapor_ice(max_bini,count_global_median))
  allocate(mpdf_vapor_ice(max_bini,3))
  ! bin-wise vapor evaporation mass rate
  allocate(pdf_eva_liq(max_binr))
  allocate(pdf_alleva_liq(max_binr))
  allocate(pdf_eva_ice(max_bini))
  allocate(pdf_1eva_ice(max_bini))
  ! crystal mass
  allocate(pdf_crymass(max_bini))
  ! rimed mass
  allocate(pdf_rimmass(max_bini))
  ! aggregate mass
  allocate(pdf_aggmass(max_bini))
  ! crystal
  allocate(pdf_cry(max_bini))
  allocate(pdf_crymd(max_bini))
  allocate(pdf_cryrcrmd(max_bini))
  ! rimed crystal
  allocate(pdf_rcr(max_bini))
  allocate(pdf_rcrmd(max_bini))
  ! aggregate
  allocate(pdf_agg(max_bini))
  ! graupel
  allocate(pdf_gra(max_bini))
  allocate(pdf_gramd(max_bini))
  ! rimed graupel
  allocate(pdf_rag(max_bini))
  ! plate
  allocate(pdf_pla(max_bini))
  ! dendrite
  allocate(pdf_den(max_bini))
  ! column
  allocate(pdf_col(max_bini))
  if (poly_exist) then
     ! plate
     allocate(pdf_ppl(max_bini))
     ! dendrite
     allocate(pdf_pcl(max_bini))
     ! column
     allocate(pdf_pir(max_bini))
  endif
  ! mass terminal V
  allocate(pdf_mtv(max_bini))
  ! conc terminal V
  allocate(pdf_ctv(max_bini))
  allocate(pdf_gctv(max_bini))
  ! integrated conc terminal V
  allocate(pdf_wctv(max_bini))
  ! conc bohm terminal V
  allocate(pdf_cbtv(max_bini))
  ! conc Mitchell terminal V
  allocate(pdf_citv(max_bini))
  ! Reynolds number
  allocate(pdf_RE(max_bini))
  ! axis ratio
  allocate(pdf_acratio(max_bini))
  allocate(pdf_adratio(max_bini))
  ! axis ratio (ice model)
  allocate(pdf_imacratio(max_bini))
  allocate(pdf_gimacratio(max_bini))
  ! area ratio
  allocate(pdf_arearatio(max_bini))
  allocate(pdf_arearatio_sidelook(max_bini))
  allocate(pdf_arearatio_total(max_bini))
  ! swept volume
  allocate(pdf_sweptVc(max_bini))
  allocate(pdf_sweptVm(max_bini))
  allocate(pdf_sweptVcb(max_bini))
  allocate(pdf_sweptVcb_real(max_bini))
  allocate(pdf_gsweptVc(max_bini))
  allocate(pdf_gsweptVm(max_bini))
  allocate(global_mpdf_sweptVcb(max_bini,count_global_median))
  allocate(mpdf_sweptVcb(max_bini,3))
  ! mass fraction
  allocate(pdf_mfrac_cry(max_bini))
  allocate(pdf_mfrac_rim(max_bini))
  ! total mass
  allocate(pdf_mass(max_bini))
  ! total mass by habit
  allocate(pdf_mass_pla(max_bini))
  allocate(pdf_mass_den(max_bini))
  allocate(pdf_mass_col(max_bini))
  allocate(pdf_mass_ppl(max_bini))
  allocate(pdf_mass_pcl(max_bini))
  allocate(pdf_mass_pir(max_bini))
  ! total mass by type
  allocate(pdf_mass_cry(max_bini))
  allocate(pdf_mass_rcr(max_bini))
  allocate(pdf_mass_agg(max_bini))
  allocate(pdf_mass_rag(max_bini))
  allocate(pdf_mass_gra(max_bini))
  ! bulk density
  allocate(pdf_bulk(max_bini))
  ! split capacitance, ventilation, kinetic correction
  allocate(pdf_split_cap(max_bini))
  allocate(pdf_split_vent(max_bini))
  allocate(pdf_split_kinetic(max_bini))
  ! split area, delta Vcb, Nwmw
  allocate(pdf_split_area(max_bini))
  allocate(pdf_split_areaiceonly(max_bini))
  allocate(pdf_split_areacry(max_bini))
  allocate(pdf_split_areaiceonlycry(max_bini))
  allocate(pdf_split_areacryrcr(max_bini))
  allocate(pdf_split_areaiceonlycryrcr(max_bini))
  allocate(pdf_split_deltaVcb(max_bini))
  allocate(pdf_split_Nwmass(max_bini))
  ! total conc
  allocate(pdf_conc(max_bini))
  allocate(pdf_gconc(max_bini))
  allocate(pdf_count(max_bini))

  allocate(X_ice_psd(max_bini))
  allocate(Q_ice_psd(max_bini))
  allocate(den_ice_psd(max_bini))
  allocate(char_len_ice_psd(max_bini))
  allocate(alpha_ice_psd(max_bini))
  allocate(mean_mass_ice_psd(max_bini))
  allocate(con_ice_psd(max_bini))
  allocate(mass_ice_psd(max_bini))
  allocate(vtm_ice_psd(max_bini))

  allocate(a2d(max_bini,4))
  allocate(error_number(max_bini))
  allocate(WVT(max_bini,2))


  mpdf_vapor_ice(:,:) = 0.0E0
  mpdf_riming_ice(:,:) = 0.0E0
  mpdf_sweptVcb(:,:) = 0.0E0


  k_file = 1
  t_local = 0
  ts = 0
  do t = 1, nt!145, nt

     ts = ts + 1
     t_local = t_local + 1
     if (t_local > nt_partial(k_file)) then
        k_file = k_file + 1
        t_local = 1
        if (k_file > 1) then
           ts = 2
        else
           ts = 1
        endif
     endif

     if (t < 144) cycle ! 144 201

     if (irank == master) print *, "current T: ", t, t_local, k_file

     ! read netcdf variables
     do k = 1, JOBS_PER_MPI

        if ( FILE_HISTORY_CARTESC_BOUNDARY ) then
           write(*,*) "still under construction..."
           call PRC_abort
        else
           is = 1 + IMAX*mod(k-1,JOBS_PER_MPI_X)
           js = 1 + JMAX*int((k-1)/JOBS_PER_MPI_X)
           ks = 1
           ie = IMAX + IMAX*mod(k-1,JOBS_PER_MPI_X)
           je = JMAX + JMAX*int((k-1)/JOBS_PER_MPI_X)
           ke = KMAX
        endif

        call check( nf90_open( trim(FILE_DIR(k_file))//trim(FILE_NAME(k)), nf90_nowrite, ncid ) )

        start_3d(1:4) = (/ 1, 1, 1, ts /)
        count_3d(1:4) = (/ IMAX, JMAX, KMAX, 1  /)
        start_2d(1:3) = (/ 1, 1, ts /)
        count_2d(1:3) = (/ IMAX, JMAX, 1  /)
        varname = "QLIQ"
        call check( nf90_inq_varid(ncid, trim(varname), varid) )
        call check( nf90_get_var( ncid, varid, qliq(is:ie,js:je,ks:ke), start=start_3d, count=count_3d ) )

        varname = "QHYD"
        call check( nf90_inq_varid(ncid, trim(varname), varid) )
        call check( nf90_get_var( ncid, varid, qhyd(is:ie,js:je,ks:ke), start=start_3d, count=count_3d ) )
     
        varname = "DENS"
        call check( nf90_inq_varid(ncid, trim(varname), varid) )
        call check( nf90_get_var( ncid, varid, dens(is:ie,js:je,ks:ke), start=start_3d, count=count_3d ) )

        varname = "T"
        call check( nf90_inq_varid(ncid, trim(varname), varid) )
        call check( nf90_get_var( ncid, varid, temperature(is:ie,js:je,ks:ke), start=start_3d, count=count_3d ) )

        varname = "PRES"
        call check( nf90_inq_varid(ncid, trim(varname), varid) )
        call check( nf90_get_var( ncid, varid, pressure(is:ie,js:je,ks:ke), start=start_3d, count=count_3d ) )

        varname = "RH"
        call check( nf90_inq_varid(ncid, trim(varname), varid) )
        call check( nf90_get_var( ncid, varid, rh(is:ie,js:je,ks:ke), start=start_3d, count=count_3d ) )

        varname = "RHI"
        call check( nf90_inq_varid(ncid, trim(varname), varid) )
        call check( nf90_get_var( ncid, varid, rhi(is:ie,js:je,ks:ke), start=start_3d, count=count_3d ) )

        varname = "W"
        call check( nf90_inq_varid(ncid, trim(varname), varid) )
        call check( nf90_get_var( ncid, varid, w(is:ie,js:je,ks:ke), start=start_3d, count=count_3d ) )

        varname = "ampsl_vapor"
        call check( nf90_inq_varid(ncid, trim(varname), varid) )
        call check( nf90_get_var( ncid, varid, liq_mass_vap(is:ie,js:je,ks:ke), start=start_3d, count=count_3d ) )

        varname = "ampsl_eva"
        call check( nf90_inq_varid(ncid, trim(varname), varid) )
        call check( nf90_get_var( ncid, varid, liq_mass_eva(is:ie,js:je,ks:ke), start=start_3d, count=count_3d ) )

        varname = "ampsl_rim"
        call check( nf90_inq_varid(ncid, trim(varname), varid) )
        call check( nf90_get_var( ncid, varid, liq_mass_rim(is:ie,js:je,ks:ke), start=start_3d, count=count_3d ) )

        varname = "ampsl_imm"
        call check( nf90_inq_varid(ncid, trim(varname), varid) )
        call check( nf90_get_var( ncid, varid, liq_mass_imm(is:ie,js:je,ks:ke), start=start_3d, count=count_3d ) )

        varname = "ampsi_vapor"
        call check( nf90_inq_varid(ncid, trim(varname), varid) )
        call check( nf90_get_var( ncid, varid, ice_mass_vap(is:ie,js:je,ks:ke), start=start_3d, count=count_3d ) )

        varname = "ampsi_eva"
        call check( nf90_inq_varid(ncid, trim(varname), varid) )
        call check( nf90_get_var( ncid, varid, ice_mass_eva(is:ie,js:je,ks:ke), start=start_3d, count=count_3d ) )

        varname = "ampsi_rim"
        call check( nf90_inq_varid(ncid, trim(varname), varid) )
        call check( nf90_get_var( ncid, varid, ice_mass_rim(is:ie,js:je,ks:ke), start=start_3d, count=count_3d ) )

        varname = "ampsci_imm"
        call check( nf90_inq_varid(ncid, trim(varname), varid) )
        call check( nf90_get_var( ncid, varid, ice_mass_imm(is:ie,js:je,ks:ke), start=start_3d, count=count_3d ) )

        do i = 1, max_binr
           if (i < 10) then
              write(word,'(I1)') i
           else
              write(word,'(I2)') i
           endif
           
           if (micro_bin) then
              varname = "vapor_bin"//trim(word)
              call check( nf90_inq_varid(ncid, trim(varname), varid) )
              call check( nf90_get_var( ncid, varid, bin_liq_mass_vap(is:ie,js:je,ks:ke,i), start=start_3d, count=count_3d ) )

              varname = "eva_bin"//trim(word)
              call check( nf90_inq_varid(ncid, trim(varname), varid) )
              call check( nf90_get_var( ncid, varid, bin_liq_mass_eva(is:ie,js:je,ks:ke,i), start=start_3d, count=count_3d ) )

              varname = "riming_bin"//trim(word)
              call check( nf90_inq_varid(ncid, trim(varname), varid) )
              call check( nf90_get_var( ncid, varid, bin_liq_mass_rim(is:ie,js:je,ks:ke,i), start=start_3d, count=count_3d ) )
           else
              bin_liq_mass_vap(is:ie,js:je,ks:ke,i) = 0.0E0
              bin_liq_mass_eva(is:ie,js:je,ks:ke,i) = 0.0E0
              bin_liq_mass_rim(is:ie,js:je,ks:ke,i) = 0.0E0
           endif

           !varname = "mass_terminalV_liq_"//trim(word)
           !call check( nf90_inq_varid(ncid, trim(varname), varid) )
           !call check( nf90_get_var( ncid, varid, bin_liq_mtv(is:ie,js:je,ks:ke,i), start=start_3d, count=count_3d ) )

           !varname = "conc_terminalV_liq_"//trim(word)
           !call check( nf90_inq_varid(ncid, trim(varname), varid) )
           !call check( nf90_get_var( ncid, varid, bin_liq_ctv(is:ie,js:je,ks:ke,i), start=start_3d, count=count_3d ) )

           varname = "liq_lmass"//trim(word)
           call check( nf90_inq_varid(ncid, trim(varname), varid) )
           call check( nf90_get_var( ncid, varid, bin_liq_mass(is:ie,js:je,ks:ke,i), start=start_3d, count=count_3d ) )

           varname = "con_lmass"//trim(word)
           call check( nf90_inq_varid(ncid, trim(varname), varid) )
           call check( nf90_get_var( ncid, varid, bin_liq_con(is:ie,js:je,ks:ke,i), start=start_3d, count=count_3d ) )
        enddo

        do i = 1, max_bini
           if (i+40 < 100) then
              write(word,'(I2)') i+40
           else
              write(word,'(I3)') i+40
           endif
           !print *, "debug", irank, i
           if (micro_bin) then
              varname = "vapor_bin"//trim(word)
              call check( nf90_inq_varid(ncid, trim(varname), varid) )
              call check( nf90_get_var( ncid, varid, bin_ice_mass_vap(is:ie,js:je,ks:ke,i), start=start_3d, count=count_3d ) )

              varname = "eva_bin"//trim(word)
              call check( nf90_inq_varid(ncid, trim(varname), varid) )
              call check( nf90_get_var( ncid, varid, bin_ice_mass_eva(is:ie,js:je,ks:ke,i), start=start_3d, count=count_3d ) )

              varname = "riming_bin"//trim(word)
              call check( nf90_inq_varid(ncid, trim(varname), varid) )
              call check( nf90_get_var( ncid, varid, bin_ice_mass_rim(is:ie,js:je,ks:ke,i), start=start_3d, count=count_3d ) )
           else
              bin_ice_mass_vap(is:ie,js:je,ks:ke,i) = 0.0E0
              bin_ice_mass_eva(is:ie,js:je,ks:ke,i) = 0.0E0
              bin_ice_mass_rim(is:ie,js:je,ks:ke,i) = 0.0E0
           endif

           varname = "mass_terminalV_ice_"//trim(word)
           call check( nf90_inq_varid(ncid, trim(varname), varid) )
           call check( nf90_get_var( ncid, varid, bin_ice_mtv(is:ie,js:je,ks:ke,i), start=start_3d, count=count_3d ) )

           varname = "conc_terminalV_ice_"//trim(word)
           call check( nf90_inq_varid(ncid, trim(varname), varid) )
           call check( nf90_get_var( ncid, varid, bin_ice_ctv(is:ie,js:je,ks:ke,i), start=start_3d, count=count_3d ) )

           if (i < 10) then
              write(word,'(I1)') i
           else
              write(word,'(I2)') i
           endif
           varname = "con_imass"//trim(word)
           call check( nf90_inq_varid(ncid, trim(varname), varid) )
           call check( nf90_get_var( ncid, varid, bin_ice_con(is:ie,js:je,ks:ke,i), start=start_3d, count=count_3d ) )

           varname = "ice_imass"//trim(word)
           call check( nf90_inq_varid(ncid, trim(varname), varid) )
           call check( nf90_get_var( ncid, varid, bin_ice_mass(is:ie,js:je,ks:ke,i), start=start_3d, count=count_3d ) )

           varname = "cry_imass"//trim(word)
           call check( nf90_inq_varid(ncid, trim(varname), varid) )
           call check( nf90_get_var( ncid, varid, bin_ice_mcry(is:ie,js:je,ks:ke,i), start=start_3d, count=count_3d ) )

           varname = "rim_imass"//trim(word)
           call check( nf90_inq_varid(ncid, trim(varname), varid) )
           call check( nf90_get_var( ncid, varid, bin_ice_mrim(is:ie,js:je,ks:ke,i), start=start_3d, count=count_3d ) )

           varname = "agg_imass"//trim(word)
           call check( nf90_inq_varid(ncid, trim(varname), varid) )
           call check( nf90_get_var( ncid, varid, bin_ice_magg(is:ie,js:je,ks:ke,i), start=start_3d, count=count_3d ) )

           varname = "frw_imass"//trim(word)
           call check( nf90_inq_varid(ncid, trim(varname), varid) )
           call check( nf90_get_var( ncid, varid, bin_ice_mfrz(is:ie,js:je,ks:ke,i), start=start_3d, count=count_3d ) )

           varname = "vol_imass"//trim(word)
           call check( nf90_inq_varid(ncid, trim(varname), varid) )
           call check( nf90_get_var( ncid, varid, bin_ice_Dmax(is:ie,js:je,ks:ke,i), start=start_3d, count=count_3d ) )

           varname = "a_axis"//trim(word)
           call check( nf90_inq_varid(ncid, trim(varname), varid) )
           call check( nf90_get_var( ncid, varid, bin_ice_alen(is:ie,js:je,ks:ke,i), start=start_3d, count=count_3d ) )

           varname = "c_axis"//trim(word)
           call check( nf90_inq_varid(ncid, trim(varname), varid) )
           call check( nf90_get_var( ncid, varid, bin_ice_clen(is:ie,js:je,ks:ke,i), start=start_3d, count=count_3d ) )

           varname = "d_axis"//trim(word)
           call check( nf90_inq_varid(ncid, trim(varname), varid) )
           call check( nf90_get_var( ncid, varid, bin_ice_dlen(is:ie,js:je,ks:ke,i), start=start_3d, count=count_3d ) )

           if (poly_exist) then
              varname = "ag_axis"//trim(word)
              call check( nf90_inq_varid(ncid, trim(varname), varid) )
              call check( nf90_get_var( ncid, varid, bin_ice_aglen(is:ie,js:je,ks:ke,i), start=start_3d, count=count_3d ) )

              varname = "ag_axis"//trim(word)
              call check( nf90_inq_varid(ncid, trim(varname), varid) )
              call check( nf90_get_var( ncid, varid, bin_ice_cglen(is:ie,js:je,ks:ke,i), start=start_3d, count=count_3d ) )

              varname = "ex_cry"//trim(word)
              call check( nf90_inq_varid(ncid, trim(varname), varid) )
              call check( nf90_get_var( ncid, varid, bin_ice_nex(is:ie,js:je,ks:ke,i), start=start_3d, count=count_3d ) )
           endif

        enddo

        !call check( nf90_inquire_variable(ncid, varid, name=check_NAME, ndims=check_NDIMS, dimids=check_dimids, xtype=check_type) )

        call check( nf90_close( ncid ) )

     enddo


     ! post process

     local_pdf_rhi = 0.0E0
     local_pdf_T = 0.0E0
     local_pdf_conc_liq = 0.0E0
     local_pdf_mass_liq = 0.0E0
     local_pdf_riming_liq = 0.0E0
     local_pdf_riming_ice = 0.0E0
     local_mpdf_riming_ice = 0.0E0
     local_pdf_1riming_ice = 0.0E0
     local_pdf_vapor_liq = 0.0E0
     local_pdf_vapor_ice = 0.0E0
     local_mpdf_vapor_ice = 0.0E0
     local_pdf_allvapor_liq = 0.0E0
     local_pdf_1vapor_ice = 0.0E0
     local_pdf_eva_liq = 0.0E0
     local_pdf_eva_ice = 0.0E0
     local_pdf_alleva_liq = 0.0E0
     local_pdf_1eva_ice = 0.0E0
     local_pdf_crymass = 0.0E0
     local_pdf_rimmass = 0.0E0
     local_pdf_aggmass = 0.0E0
     local_pdf_crymd = 0.0E0
     local_pdf_rcrmd = 0.0E0
     local_pdf_cryrcrmd = 0.0E0
     local_pdf_gramd = 0.0E0
     local_pdf_cry = 0.0E0
     local_pdf_rcr = 0.0E0
     local_pdf_agg = 0.0E0
     local_pdf_gra = 0.0E0
     local_pdf_rag = 0.0E0
     local_pdf_pla = 0.0E0
     local_pdf_col = 0.0E0
     local_pdf_den = 0.0E0
     local_pdf_ppl = 0.0E0
     local_pdf_pcl = 0.0E0
     local_pdf_pir = 0.0E0
     local_pdf_mtv = 0.0E0
     local_pdf_ctv = 0.0E0
     local_pdf_gctv = 0.0E0
     local_pdf_wctv = 0.0E0
     local_pdf_cbtv = 0.0E0
     local_pdf_citv = 0.0E0
     local_pdf_RE = 0.0E0
     local_pdf_acratio = 0.0E0
     local_pdf_adratio = 0.0E0
     local_pdf_imacratio = 0.0E0
     local_pdf_gimacratio = 0.0E0
     local_pdf_arearatio = 0.0E0
     local_pdf_arearatio_sidelook = 0.0E0
     local_pdf_arearatio_total = 0.0E0
     local_pdf_sweptVc = 0.0E0
     local_pdf_sweptVm = 0.0E0
     local_pdf_sweptVcb = 0.0E0
     local_mpdf_sweptVcb = 0.0E0
     local_pdf_sweptVcb_real= 0.0E0
     local_pdf_gsweptVc = 0.0E0
     local_pdf_gsweptVm = 0.0E0
     local_pdf_mfrac_cry = 0.0E0
     local_pdf_mfrac_rim = 0.0E0
     local_pdf_mass = 0.0E0
     local_pdf_mass_pla = 0.0E0
     local_pdf_mass_den = 0.0E0
     local_pdf_mass_col = 0.0E0
     local_pdf_mass_ppl = 0.0E0
     local_pdf_mass_pcl = 0.0E0
     local_pdf_mass_pir = 0.0E0
     local_pdf_mass_cry = 0.0E0
     local_pdf_mass_rcr = 0.0E0
     local_pdf_mass_agg = 0.0E0
     local_pdf_mass_rag = 0.0E0
     local_pdf_mass_gra = 0.0E0
     local_pdf_bulk = 0.0E0
     local_pdf_split_cap = 0.0E0
     local_pdf_split_vent = 0.0E0
     local_pdf_split_kinetic = 0.0E0
     local_pdf_split_area = 0.0E0
     local_pdf_split_areaiceonly = 0.0E0
     local_pdf_split_areacry = 0.0E0
     local_pdf_split_areaiceonlycry = 0.0E0
     local_pdf_split_areacryrcr = 0.0E0
     local_pdf_split_areaiceonlycryrcr = 0.0E0
     local_pdf_split_deltaVcb = 0.0E0
     local_pdf_split_Nwmass = 0.0E0
     local_pdf_conc = 0.0E0
     local_pdf_gconc = 0.0E0
     local_pdf_count = 0.0E0

     local_pdf_temp(:) = 0.0E0
     local_pdf_temp_rim(:) = 0.0E0
     local_pdf_temp_rimNORMliq(:) = 0.0E0
     local_pdf_temp_vap(:) = 0.0E0
     local_pdf_temp_imm(:) = 0.0E0
     local_pdf_temp_aggcon(:) = 0.0E0
     local_pdf_temp_rcrcon(:) = 0.0E0
     local_pdf_temp_ragcon(:) = 0.0E0
     local_pdf_temp_gracon(:) = 0.0E0
     local_pdf_temp_crycon(:) = 0.0E0
     local_pdf_rad(:) = 0.0E0
     local_pdf_nliq(:) = 0.0E0
     local_pdf_lwc(:) = 0.0E0
     local_pdf_wvel(:) = 0.0E0
     local_pdf_temp_sweptVc(:) = 0.0E0
     local_pdf_temp_sweptVcb(:) = 0.0E0
     local_pdf_temp_sweptVcb_real(:) = 0.0E0
     local_pdf_cbtv_liq(:) = 0.0E0

     ! transform all density multiplied quantities back to normal form
     ! calculate qice
     do i = isg, ieg
        do j = jsg, jeg
           do k = ks, ke
              qice(i,j,k) = max(0.0E0,qhyd(i,j,k) - qliq(i,j,k))
              do ibin = 1, max_binr
                 bin_liq_con(i,j,k,ibin) = bin_liq_con(i,j,k,ibin)*dens(i,j,k)
              enddo
              do ibin = 1, max_bini
                 bin_ice_con(i,j,k,ibin) = bin_ice_con(i,j,k,ibin)*dens(i,j,k)
              enddo
              !if (irank == master) then
              !   if (k > 30 .and. k < 50) then
              !      do ibin = 1, max_bini
              !         if (bin_ice_con(i,j,k,ibin) /= 0.0E0) then
              !            write(*,*) qliq(i,j,k)*1000.0E0, bin_ice_nex(i,j,k,ibin)*dens(i,j,k)/bin_ice_con(i,j,k,ibin)
              !         endif
              !      enddo
              !   endif
              !endif
           enddo
        enddo
     enddo

     temp1 = 0.0
     temp2 = 0.0
     temp3 = 0.0
     do i = isg, ieg
        do j = jsg, jeg
           do k = ks, ke
              temp1 = temp1 + liq_mass_imm(i,j,k)/11.0
              temp2 = temp2 + liq_mass_vap(i,j,k)/10.0
              temp3 = temp3 + liq_mass_eva(i,j,k)/10.0
           enddo
        enddo
     enddo
     
     call MPI_REDUCE( temp1, global_sum1, 1, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( temp2, global_sum2, 1, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( temp3, global_sum3, 1, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     if (irank == master) print *, "total liquid mass rates: ", global_sum1 ,global_sum2 ,global_sum3

     temp1 = 0.0
     temp2 = 0.0
     temp3 = 0.0
     do i = isg, ieg
        do j = jsg, jeg
           do k = ks, ke
              if (qliq(i,j,k) > qliq_lower_limit) then
                 temp1 = temp1 + liq_mass_imm(i,j,k)/11.0
                 temp2 = temp2 + liq_mass_vap(i,j,k)/10.0
                 temp3 = temp3 + liq_mass_eva(i,j,k)/10.0
              endif
           enddo
        enddo
     enddo
     call MPI_REDUCE( temp1, global_sum1, 1, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( temp2, global_sum2, 1, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( temp3, global_sum3, 1, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     if (irank == master) print *, "incloud total liquid mass rates: ", global_sum1 ,global_sum2 ,global_sum3

     
     ! retrieve average and median
     counter1 = 1
     local_dNdlogD(:,:) = 0.0E0
     local_dNidlogD(:,:) = 0.0E0
     
     do k = ks, ke
        do i = isg, ieg
           do j = jsg, jeg
              do ibin = 1, max_binr
                 local_pdf_allvapor_liq(ibin) = local_pdf_allvapor_liq(ibin) + bin_liq_mass_vap(i,j,k,ibin)*(fz(k+1)-fz(k))*DX*DY
                 local_pdf_alleva_liq(ibin) = local_pdf_alleva_liq(ibin) + bin_liq_mass_eva(i,j,k,ibin)*(fz(k+1)-fz(k))*DX*DY
              enddo

              liquid_mass = 0.0E0
              liquid_number = 0.0E0
              liquid_eff_rad = 0.0E0
              D2 = 0.0E0
              D3 = 0.0E0
              DI2 = 0.0E0
              DI3 = 0.0E0
              temp1 = 0.0E0
              temp2 = 0.0E0
              if (qliq(i,j,k) > qliq_lower_limit) then ! if in clouds
                 do ibin = 1, max_binr
                    temp1 = temp1 + bin_liq_con(i,j,k,ibin)
                    if (jj == 1) then
                       local_dNdlogD(ibin,counter1) = (temp1 - temp2)/(log10(diameterbin(ibin+1)) - log10(diameterbin(ibin)))
                    else if (jj == max_binr) then
                       local_dNdlogD(ibin,counter1) = (temp1 - temp2)/(log10(diameterbin(ibin)) - log10(diameterbin(ibin-1)))
                    else
                       local_dNdlogD(ibin,counter1) = (temp1 - temp2)/(log10(diameterbin(ibin+1)) - log10(diameterbin(ibin-1)))
                    endif
                    
                    temp2 = temp1
                    liquid_number = liquid_number + bin_liq_con(i,j,k,ibin)
                    D3 = D3 + bin_liq_con(i,j,k,ibin)*diameterbin(ibin)**3
                    D2 = D2 + bin_liq_con(i,j,k,ibin)*diameterbin(ibin)**2
                    local_pdf_conc_liq(ibin) = local_pdf_conc_liq(ibin) + bin_liq_con(i,j,k,ibin)
                    local_pdf_mass_liq(ibin) = local_pdf_mass_liq(ibin) + bin_liq_mass(i,j,k,ibin)
                    local_pdf_riming_liq(ibin) = local_pdf_riming_liq(ibin) + bin_liq_mass_rim(i,j,k,ibin)*(fz(k+1)-fz(k))*DX*DY
                    local_pdf_vapor_liq(ibin) = local_pdf_vapor_liq(ibin) + bin_liq_mass_vap(i,j,k,ibin)*(fz(k+1)-fz(k))*DX*DY
                    local_pdf_eva_liq(ibin) = local_pdf_eva_liq(ibin) + bin_liq_mass_eva(i,j,k,ibin)*(fz(k+1)-fz(k))*DX*DY
                 enddo

                 if (D2 /= 0.0E0) then
                    liquid_eff_rad = D3/D2
                 endif
                 
                 d_vis = get_dynamic_viscosity(temperature(i,j,k))
                 liquid_eff_rad_cm = liquid_eff_rad/10000.0E0/2.0E0
                 liquid_mass = 4.0E0/3.0E0*pi*(liquid_eff_rad_cm)**3

                 ! calculate terminal speed of drops (using effective diameter)
                 if (liquid_eff_rad_cm < 0.5e-4) then
                    liquid_cbtv = 0.0
                 else if (0.5e-4 <= liquid_eff_rad_cm .and. liquid_eff_rad_cm < 10.0e-4) then
                    ! calculate Stokes terminal velocity
                    U_S = liquid_eff_rad_cm**2.0E0*gg*(1.0E0 - dens(i,j,k)/1000.0E0)/4.5E0/d_vis
                    lambda_a = 6.6e-6*(d_vis/1.818e-4)*(1013250.0E0/pressure(i,j,k)/10.0E0)*(temperature(i,j,k)/293.15E0)
                    liquid_cbtv = (1.0E0 + 1.26E0*lambda_a/liquid_eff_rad_cm)*U_S/100.0E0

                 else if(liquid_eff_rad_cm < 535.0e-4) then
                    X = log(32.0E0*liquid_eff_rad_cm**3.0E0*(1.0E0-dens(i,j,k)/1000.0E0)*dens(i,j,k)/1000.0E0*gg/3.0/d_vis**2.0E0)
                 
                    Y = -0.318657e+1 + 0.992696E0*X - 0.153193e-2*X*X - 0.987059e-3*X*X*X &
                         -0.578878e-3*X*X*X*X + 0.855176e-4*X*X*X*X*X &
                         -0.327815e-5*X*X*X*X*X*X
                    liquid_Nre = exp(Y)
                    ! +++ calculate the terminal velocity +++
                    liquid_cbtv = liquid_Nre*d_vis/(2.0E0*liquid_eff_rad_cm*dens(i,j,k)/1000.0E0)/100.0E0

                 else
                    rad = min(liquid_eff_rad_cm,3500.0e-4)
                    
                    sig_wa = get_sfc_tension(temperature(i,j,k))
                    NBO = gg*(1.0E0-dens(i,j,k)/1000.0E0)*liquid_eff_rad_cm**2.0E0/sig_wa
                    NP = (sig_wa**3)*((dens(i,j,k)/1000.0E0)**2)/(d_vis**4)/gg
                    
                    X = log(NBO*(NP**(1.0E0/6.0E0))*16.0E0/3.0E0)
                    Y = -0.500015e+1 + 0.523778e+1*X - 0.204914e+1*X*X + 0.475294E0*X*X*X &
                         -0.542819e-1*X*X*X*X + 0.238449e-2*X*X*X*X*X
                    
                    liquid_Nre = (NP**(1.0E0/6.0E0))*exp(Y)
                    ! +++ calculate the terminal velocity +++
                    liquid_cbtv = liquid_Nre*d_vis/(2.0E0*liquid_eff_rad_cm*dens(i,j,k)/1000.0E0)/100.0E0
                 endif



                 X_ice_psd = 0.0E0
                 Q_ice_psd = 0.0E0
                 den_ice_psd = 0.0E0
                 char_len_ice_psd = 0.0E0
                 alpha_ice_psd = 0.0E0
                 mean_mass_ice_psd = 0.0E0
                 con_ice_psd = 0.0E0
                 mass_ice_psd = 0.0E0
                 vtm_ice_psd = 0.0E0
                 gracon = 0.0E0
                 aggcon = 0.0E0
                 ragcon = 0.0E0
                 crycon = 0.0E0
                 rcrcon = 0.0E0
                 total_sweptVc = 0.0E0
                 total_sweptVcb = 0.0E0
                 total_sweptVcb_real = 0.0E0
                 local_NP(:) = 0.0E0
                 do ibin = 1, max_bini
                    if (bin_ice_con(i,j,k,ibin) > 1.e-11 .and. bin_ice_mcry(i,j,k,ibin) > 0.0E0 .and. bin_ice_alen(i,j,k,ibin) > 0.0E0 .and. bin_ice_clen(i,j,k,ibin) > 0.0E0) then ! 10e-8 L-1 limit
                       ! rescale axis lengths
                       bin_ice_alen(i,j,k,ibin) = (bin_ice_alen(i,j,k,ibin)*dens(i,j,k)/bin_ice_con(i,j,k,ibin))**(1.0E0/3.0E0)
                       bin_ice_clen(i,j,k,ibin) = (bin_ice_clen(i,j,k,ibin)*dens(i,j,k)/bin_ice_con(i,j,k,ibin))**(1.0E0/3.0E0)
                       bin_ice_dlen(i,j,k,ibin) = min(0.9E0*bin_ice_alen(i,j,k,ibin), (bin_ice_dlen(i,j,k,ibin)*dens(i,j,k)/bin_ice_con(i,j,k,ibin))**(1.0E0/3.0E0))
                       if (poly_exist) then
                          if (iceaxis_version == 3) then
                             bin_ice_aglen(i,j,k,ibin) = (bin_ice_aglen(i,j,k,ibin)*dens(i,j,k)/bin_ice_con(i,j,k,ibin))**(1.0/3.0)*bin_ice_alen(i,j,k,ibin)
                             bin_ice_cglen(i,j,k,ibin) = (bin_ice_cglen(i,j,k,ibin)*dens(i,j,k)/bin_ice_con(i,j,k,ibin))**(1.0/3.0)*bin_ice_clen(i,j,k,ibin)
                             bin_ice_nex(i,j,k,ibin) = bin_ice_nex(i,j,k,ibin)*dens(i,j,k)/bin_ice_con(i,j,k,ibin)
                             !write(*,'(2ES15.6)') bin_ice_nex(i,j,k,ibin), bin_ice_con(i,j,k,ibin)
                          endif
                       endif
                       
                       ! diagnose ice type
                       call Iice_habit(ice_type,bin_ice_mass(i,j,k,ibin),bin_ice_mcry(i,j,k,ibin),bin_ice_magg(i,j,k,ibin),bin_ice_mrim(i,j,k,ibin))

                       ! diagnose ice habit
                       if (poly_exist) then
                          call Iice_shapePoly(my_habit,bin_ice_alen(i,j,k,ibin),bin_ice_clen(i,j,k,ibin),bin_ice_dlen(i,j,k,ibin),bin_ice_aglen(i,j,k,ibin),bin_ice_cglen(i,j,k,ibin),bin_ice_nex(i,j,k,ibin))
                       else
                          call Iice_shape(my_habit,bin_ice_alen(i,j,k,ibin),bin_ice_clen(i,j,k,ibin),bin_ice_dlen(i,j,k,ibin))
                       endif
                       if (my_habit == -1) then
                          print *, "ERROR HABIT", irank, bin_ice_alen(i,j,k,ibin),bin_ice_clen(i,j,k,ibin),bin_ice_dlen(i,j,k,ibin),bin_ice_aglen(i,j,k,ibin),bin_ice_cglen(i,j,k,ibin),bin_ice_nex(i,j,k,ibin)
                          call PRC_abort
                       endif

                       ! calculate elen and rlen
                       call convert_habit(habit,my_habit)
                       if (poly_exist) then
                          call get_elen_rlen(pla_e,ros_r,habit,bin_ice_mcry(i,j,k,ibin)*dens(i,j,k)/bin_ice_con(i,j,k,ibin)/1000.0E0,bin_ice_alen(i,j,k,ibin),bin_ice_clen(i,j,k,ibin),bin_ice_aglen(i,j,k,ibin),bin_ice_cglen(i,j,k,ibin))
                       else
                          pla_e = 0.0E0
                          ros_r = 0.0E0
                       endif

                       den_i = 0.91668E0
                       vcs = bin_ice_Dmax(i,j,k,ibin)*dens(i,j,k)/bin_ice_con(i,j,k,ibin)
                       if (poly_exist) then
                          if (my_habit <= 3) then
                             vic = 4.0E0/3.0E0*pi*(bin_ice_alen(i,j,k,ibin)**2 + bin_ice_clen(i,j,k,ibin)**2)**1.5
                             if (vic == 0.0E0) then
                                write(*,'(A,2I3, 7ES15.6)') "inside volume error: ", my_habit, habit, vcs, vic, pla_e, ros_r, bin_ice_alen(i,j,k,ibin), bin_ice_clen(i,j,k,ibin), pi
                                call PRC_abort
                             endif
                          else
                             if (my_habit == 4) then
                                vic = 4.0E0/3.0E0*pi*pla_e*pla_e*pla_e*(1.0E0 + spx_p**2)**1.5E0
                             else
                                vic = 4.0E0/3.0E0*pi*ros_r*ros_r*ros_r*(1.0E0 + spx_p**2)**1.5E0
                             endif
                          endif
                       else
                          vic = 4.0E0/3.0E0*pi*(bin_ice_alen(i,j,k,ibin)**2 + bin_ice_clen(i,j,k,ibin)**2)**1.5E0
                       endif
                       if (vic < bin_ice_mcry(i,j,k,ibin)*dens(i,j,k)/bin_ice_con(i,j,k,ibin)/1000.0E0/den_i) then
                          vic = bin_ice_mcry(i,j,k,ibin)*dens(i,j,k)/bin_ice_con(i,j,k,ibin)/1000.0E0/den_i
                       endif
                       if (vic == 0.0E0) then
                          write(*,'(A,2I3, 7ES15.6)') "volume error: ", my_habit, habit, vcs, vic, pla_e, ros_r, bin_ice_alen(i,j,k,ibin), bin_ice_clen(i,j,k,ibin), bin_ice_mcry(i,j,k,ibin)
                          call PRC_abort
                       endif

                       if (vcs < vic) then
                          vcs = vic
                       else if (vcs > 4188.79020478639E0) then
                          vcs = 4188.79020478639E0
                       endif
                       ! to prevent unphysical volume and axis lengths, data of gamma1 from 10000
                       if (prevent_unphysical_length == .true.) then
                          if (bin_ice_mrim(i,j,k,ibin) < bin_ice_mcry(i,j,k,ibin) .and. &
                              bin_ice_magg(i,j,k,ibin) < bin_ice_mcry(i,j,k,ibin)) then
                             if (vcs > 10.0E0*vic) then
                                vcs = vic
                                cycle
                             endif
                             if (vcs > 10.0E0*bin_ice_mass(i,j,k,ibin)*dens(i,j,k)/bin_ice_con(i,j,k,ibin)/1000.0E0/den_i) then
                                vcs = bin_ice_mass(i,j,k,ibin)*dens(i,j,k)/bin_ice_con(i,j,k,ibin)/1000.0E0/den_i
                                vic = vcs
                                bin_ice_alen(i,j,k,ibin) = (vic*3.0E0/4.0E0/pi)**(1.0E0/3.0E0)
                                bin_ice_clen(i,j,k,ibin) = (vic*3.0E0/4.0E0/pi)**(1.0E0/3.0E0)
                                bin_ice_dlen(i,j,k,ibin) = max(bin_ice_dlen(i,j,k,ibin),0.9E0*bin_ice_alen(i,j,k,ibin))
                                cycle
                             endif
                          else
                             if (vcs > 2.0E0*bin_ice_mrim(i,j,k,ibin)/bin_ice_mcry(i,j,k,ibin)*vic .or. vcs > 2.0E0*bin_ice_mrim(i,j,k,ibin)/bin_ice_mcry(i,j,k,ibin)*bin_ice_mass(i,j,k,ibin)*dens(i,j,k)/bin_ice_con(i,j,k,ibin)/1000.0E0/den_i) then !
                                cycle
                             endif
                          endif
                       endif

                       ! calculate alpha_ic
                       if (poly_exist) then
                          if (my_habit <= 3) then
                             if (bin_ice_alen(i,j,k,ibin) > 1.e-15 .and. bin_ice_clen(i,j,k,ibin) > 1.e-15) then
                                alpha_ic = bin_ice_clen(i,j,k,ibin)/bin_ice_alen(i,j,k,ibin)
                             else
                                alpha_ic = 1.0E0
                             endif
                          else
                             alpha_ic = spx_p
                          endif
                       else
                          if (bin_ice_alen(i,j,k,ibin) > 1.e-15 .and. bin_ice_clen(i,j,k,ibin) > 1.e-15) then
                             alpha_ic = bin_ice_clen(i,j,k,ibin)/bin_ice_alen(i,j,k,ibin)
                          else
                             alpha_ic = 1.0E0
                          endif
                       endif
                       
                       ! calculate alpha_im
                       if (ice_type == 1) then
                          kp = log10(1.0E0/3.0E0/alpha_ic)
                          mass_agg_cry = bin_ice_mcry(i,j,k,ibin) + bin_ice_magg(i,j,k,ibin)
                          mlmt = 10.0E0*bin_ice_mcry(i,j,k,ibin)
                          
                          if (mass_agg_cry < mlmt) then
                             alpha_im = 1.0E0/3.0E0*(mass_agg_cry/mlmt)**kp
                          else
                             alpha_im = 1.0E0/3.0E0
                          endif
                       else if (ice_type == 3 .or. ice_type == 4) then
                          kp = log10(0.8E0/alpha_ic)
                          mass_agg_cry_rim = bin_ice_mcry(i,j,k,ibin) + bin_ice_magg(i,j,k,ibin) + bin_ice_mrim(i,j,k,ibin)
                          mlmt = 10.0E0*(bin_ice_mcry(i,j,k,ibin) + bin_ice_magg(i,j,k,ibin))
                          
                          if (mass_agg_cry_rim < mlmt) then
                             alpha_im = 0.8E0*(mass_agg_cry_rim/mlmt)**kp
                          else
                             alpha_im = 0.8E0
                          endif
                       else if (ice_type == 5) then
                          if (alpha_ic > 1.0E0) then
                             alpha_im = 1.0E0/sqrt((1.0E0 + 1.0E0/alpha_ic**2)*(vcs/vic)**(2.0E0/3.0E0) - 1.0E0)
                             if (alpha_im < 1.0E0) then
                                alpha_im = 1.0E0
                             endif
                          else
                             alpha_im = sqrt((1.0E0 + alpha_ic**2)*(vcs/vic)**(2.0E0/3.0E0) - 1.0E0)
                             if (alpha_im > 1.0E0) then
                                alpha_im = 1.0E0
                             endif
                          endif
                       else
                          alpha_im = alpha_ic
                       endif

                       ! calculate maximum dimension
                       if (ice_type == 4) then
                          if (alpha_im > 1.0E0) then
                             csm = ((vcs/(4.0E0/3.0E0*pi))**(1.0E0/3.0E0))
                             asm = csm/alpha_im
                             local_Dmax = 2.0E0*csm*10000.0E0
                          else
                             asm = ((vcs/(4.0E0/3.0E0*pi))**(1.0E0/3.0E0))
                             csm = asm*alpha_im
                             local_Dmax = 2.0E0*asm*10000.0E0
                          endif
                       else
                          asm = ((vcs/(4.0E0/3.0E0*pi))**(1.0/3.0))/sqrt(1.0E0 + alpha_im**2)
                          csm = asm*alpha_im
                          local_Dmax = 2.0E0*asm*sqrt(1.0E0 + alpha_im**2)*10000.0E0
                       endif
                       if (abs(asm) > 1.e10 .or. abs(csm) > 1.e10) then
                          print *, "ASM CSM error", irank, asm, csm, alpha_im
                          call PRC_abort
                       endif
                       if (.false. .and. local_Dmax > 1500.0E0 .and. ibin < 14) then
                          write(*,'(4I7,6ES15.5,A,3ES15.5)') i, j, k, ibin, &
                               local_Dmax, &
                               bin_ice_mrim(i,j,k,ibin)/bin_ice_mcry(i,j,k,ibin), &
                               bin_ice_magg(i,j,k,ibin)/bin_ice_mcry(i,j,k,ibin), &
                               bin_ice_con(i,j,k,ibin), &
                               vcs, &
                               vic, &
                               "  |  ", &
                               bin_ice_alen(i,j,k,ibin)*10000.0E0, &
                               bin_ice_clen(i,j,k,ibin)*10000.0E0, &
                               bin_ice_mcry(i,j,k,ibin)*dens(i,j,k)/bin_ice_con(i,j,k,ibin)/1000.0E0/den_i
                       endif

                       bin_ice_aDmax(i,j,k,ibin) = local_Dmax
                       DI3 = DI3 + bin_ice_con(i,j,k,ibin)*local_Dmax**3
                       DI2 = DI2 + bin_ice_con(i,j,k,ibin)*local_Dmax**2

                       ! calculate apprarent bulk density
                       ap_den = bin_ice_mass(i,j,k,ibin)*dens(i,j,k)/1000.0E0/bin_ice_con(i,j,k,ibin)

                       ! calculate Reynolds number and terminal velocity
                       mean_mass = bin_ice_mass(i,j,k,ibin)*dens(i,j,k)/1000.0E0/bin_ice_con(i,j,k,ibin)
                       d_vis = get_dynamic_viscosity(temperature(i,j,k))
                       D_v = get_diffusivity(temperature(i,j,k),pressure(i,j,k))
                       fkn = get_fkn(temperature(i,j,k),D_v,2,sqrt(asm*asm+csm*csm))
                       call convert_shtype(ships_ice_type,ice_type)
                       alpha = alpha_im
                       if (ships_ice_type == 5) then
                          ! graupel
                          is_mod = 2
                       else
                          is_mod = 1
                       endif
                       if (8.0E0*bin_ice_mfrz(i,j,k,ibin)/bin_ice_mcry(i,j,k,ibin) > 1.0E0 .or. bin_ice_mfrz(i,j,k,ibin)*dens(i,j,k)/1000.0E0/bin_ice_con(i,j,k,ibin) > 3.83978e-6) then
                          is_mod = 2
                       endif
                       if (bin_ice_alen(i,j,k,ibin) > 1.e-15) then
                          psi_ic = bin_ice_dlen(i,j,k,ibin)/bin_ice_alen(i,j,k,ibin)
                       else
                          psi_ic = 0.0E0
                       endif
                       ! SHIPS seems to use mode 1 to calculated weighted TV
                       mode = 2
                       
                       if (mean_mass == 0.0E0 .or. bin_ice_mcry(i,j,k,ibin) < 1.e-15 .or. bin_ice_con(i,j,k,ibin) < 1.e-8 .or. bin_ice_alen(i,j,k,ibin) < 1.e-15 .or. bin_ice_clen(i,j,k,ibin) < 1.e-15 .or. alpha < 1.e-7 .or. alpha > 100.0E0) then
                          arearatio = 0.0E0
                          arearatio_sidelook = 0.0E0
                          arearatio_total = 0.0E0
                          Nre = 0.0E0
                          vtm = 0.0E0
                          vti = 0.0E0
                       else
                          ! +++ calculate the total surface area +++
                          call get_total_sfc_area( OMEGA, ships_ice_type, habit, is_mod, asm, csm, psi_ic, ros_r, mode )
                          call get_total_sfc_area( A_e_total, ships_ice_type, habit, is_mod, asm, csm, psi_ic, ros_r, mode )
                          
                          ! +++ calculate the area ratio +++
                          ! ++++++ calculate the effective cross section ++++++
                          call get_effect_area( A_e, ships_ice_type, habit, is_mod, asm, csm, psi_ic, ros_r, mode )
                          call get_effect_area( A_e_ic, 1, habit, is_mod, bin_ice_alen(i,j,k,ibin), bin_ice_clen(i,j,k,ibin), psi_ic, ros_r, mode )

                          if (force_downwardlook == .true.) then
                             call get_effect_area_sph( A_e_sph, ships_ice_type, habit, is_mod, asm, csm, psi_ic, ros_r, mode )
                             call get_effect_area_sph( A_e_ic_sph, 1, habit, is_mod, bin_ice_alen(i,j,k,ibin), bin_ice_clen(i,j,k,ibin), psi_ic, ros_r, mode )
                          endif

                          ! ++++++ calculate the effective cross section side looking ++++++
                          call get_effect_area_sidelook( A_e_sidelook, ships_ice_type, habit, is_mod, asm, csm, psi_ic, ros_r, mode )
                          call get_effect_area_sidelook( A_e_ic_sidelook, 1, habit, is_mod, bin_ice_alen(i,j,k,ibin), bin_ice_clen(i,j,k,ibin), psi_ic, ros_r, mode )
                          
                          ! ++++++ calculate the circumscribed cross-sectional area ++++++
                          call get_circum_area( A_c, ships_ice_type, habit, is_mod, asm, csm, psi_ic, ros_r, mode )
                          call get_circum_area( A_c_ic, 1, habit, is_mod, bin_ice_alen(i,j,k,ibin), bin_ice_clen(i,j,k,ibin), psi_ic, ros_r, mode )
                          
                          ! +++ calculate the characteristic length based on
                          !     perimeter and total surface area +++
                          call get_perimeter( P, ships_ice_type, habit, is_mod, asm, csm, bin_ice_alen(i,j,k,ibin), bin_ice_dlen(i,j,k,ibin), ros_r, mode )
                          
                          ! +++ calculate characteristic length +++
                          char_len = 2.0E0*sqrt(A_c/pi)
                          
                          ! +++ calculate area ratio +++
                          call get_vip(vip,is_mod,alpha,asm)
                          q_cs=(bin_ice_mcry(i,j,k,ibin)+bin_ice_magg(i,j,k,ibin)+bin_ice_mrim(i,j,k,ibin))*dens(i,j,k)/1000.0E0/bin_ice_con(i,j,k,ibin)/den_i/vip
                           
                          if (A_c == 0.0E0 .or. A_e == 0.0E0 .or. P == 0.0E0 .or. A_c_ic == 0.0E0) then
                             write(*,'(A,2I5)') "MTV error: ", ships_ice_type, habit
                             write(*,'(A,16ES15.6)') "MTV error: ", bin_ice_con(i,j,k,ibin), mean_mass, alpha, A_c, A_c_ic, A_e, A_e_ic, bin_ice_alen(i,j,k,ibin), bin_ice_clen(i,j,k,ibin), bin_ice_dlen(i,j,k,ibin), asm, csm, pla_e, ros_r, vcs, vic
                             stop
                             call PRC_abort
                          endif

                          call get_area_ratio_sol( q, ships_ice_type, habit, is_mod, ap_den, mode, A_e, A_c, A_e_ic, A_c_ic, q_cs, bin_ice_mcry(i,j,k,ibin)*dens(i,j,k)/1000.0E0, bin_ice_magg(i,j,k,ibin)*dens(i,j,k)/1000.0E0, bin_ice_mrim(i,j,k,ibin)*dens(i,j,k)/1000.0E0, bin_ice_con(i,j,k,ibin), vic )

                          if (force_downwardlook == .true.) then
                             call get_arearatio( arearatio, arearatio_sidelook, arearatio_total, ships_ice_type, habit, is_mod, ap_den, mode, A_e_sph, A_c, A_e_ic_sph, A_c_ic, A_e_sidelook, A_e_ic_sidelook, A_e_total, q_cs, bin_ice_mcry(i,j,k,ibin)*dens(i,j,k)/1000.0E0, bin_ice_magg(i,j,k,ibin)*dens(i,j,k)/1000.0E0, bin_ice_mrim(i,j,k,ibin)*dens(i,j,k)/1000.0E0, bin_ice_con(i,j,k,ibin), vic, vcs, bin_ice_alen(i,j,k,ibin), bin_ice_clen(i,j,k,ibin), bin_ice_dlen(i,j,k,ibin), asm, csm )
                          else
                             call get_arearatio( arearatio, arearatio_sidelook, arearatio_total, ships_ice_type, habit, is_mod, ap_den, mode, A_e, A_c, A_e_ic, A_c_ic, A_e_sidelook, A_e_ic_sidelook, A_e_total, q_cs, bin_ice_mcry(i,j,k,ibin)*dens(i,j,k)/1000.0E0, bin_ice_magg(i,j,k,ibin)*dens(i,j,k)/1000.0E0, bin_ice_mrim(i,j,k,ibin)*dens(i,j,k)/1000.0E0, bin_ice_con(i,j,k,ibin), vic, vcs, bin_ice_alen(i,j,k,ibin), bin_ice_clen(i,j,k,ibin), bin_ice_dlen(i,j,k,ibin), asm, csm )
                          endif

                          LEN = OMEGA/P
                        
                          ! +++ calculate the shape factor +++
                          if( alpha <= 0.16E0 ) then
                             k_w = 0.85E0
                          else if ( 0.16E0 < alpha .and. alpha <= 1.0E0 ) then
                             k_w = 0.82E0 + 0.18E0*alpha
                          else if ( 1.0E0 < alpha .and. alpha <= 3.0E0 ) then
                             k_w = 0.37E0 + 0.63E0/alpha
                          else
                             k_w = 1.33E0/( log(alpha) + 1.19E0)
                          endif
                           
                          ! +++ calculate the Davies or Best number +++
                          if ( q <= 1.0E0 ) then
                             X = 8.0E0*mean_mass*gg*dens(i,j,k)/1000.0E0/ &
                                  (pi*(d_vis**2)*max(alpha, 1.0E0)*(q**0.25E0))
                             QQ = q**0.25E0
                          else
                             X = 8.0E0*mean_mass*gg*dens(i,j,k)/1000.0E0/ &
                                   (pi*(d_vis**2)*max(alpha, 1.0E0)*q)
                             QQ = q
                          endif

                          X_ice_psd(ibin) = X/mean_mass
                          Q_ice_psd(ibin) = QQ
                          den_ice_psd(ibin) = ap_den
                          mean_mass_ice_psd(ibin) = mean_mass
                          con_ice_psd(ibin) = bin_ice_con(i,j,k,ibin)
                          mass_ice_psd(ibin) = bin_ice_mass(i,j,k,ibin)*dens(i,j,k)/1000.0E0
                          char_len_ice_psd(ibin) = char_len
                          alpha_ice_psd(ibin) = alpha
                          
                          ! +++ calculate the pressure drag C_DP +++
                          if ( alpha <= 0.3E0 ) then
                             gamma = 1.98E0
                             C_DP = 0.292E0*k_w*gamma
                          else if ( 0.3E0 < alpha .and. alpha < 1.0E0 ) then
                             gamma = 3.76E0 - 8.41E0*alpha + 9.18E0*alpha**2 - 3.53E0*alpha**3
                             C_DP = 0.292E0*k_w*gamma
                          else
                             if ( 1.0E0 <= q ) then
                                gamma = 1.0E0
                                C_DP = (1.46E0*q - 0.46E0)*(0.492E0 - 0.200E0/sqrt(alpha))
                             else
                                gamma = 1.0E0
                                C_DP = 0.292E0*k_w*gamma
                             endif
                          endif
                          
                          ! +++ calculate the Oseen-type drag coefficient, C_DO +++
                          C_DO = 4.5E0*k_w*k_w*max(alpha, 1.0E0)
       
                          ! +++ calculate the Reynolds number +++
                          beta = sqrt(1.0E0 + (C_DP/(6.0E0*k_w))*sqrt(X/C_DP)) - 1.0E0
                          N_re_0 = 6.0E0*k_w*beta*beta/C_DP
       
                          ! +++ correction for transition to turbulent flow +++
                          if( 1000.0E0 <= N_re_0 ) then

                             X_0 = 2.8e+06
                             C_DP = C_DP*(1.0E0 + 1.6E0*(X/X_0)**2)/(1.0E0 + (X/X_0)**2)

                             beta = sqrt(1.0E0 + (C_DP/(6.0E0*k_w))*sqrt(X/C_DP)) - 1.0E0
                             N_re_0 = 6.0E0*k_w*beta*beta/C_DP
                          endif
       
                          ! +++ matching theory +++
                          gamma = (C_DO - C_DP)/(4.0E0*C_DP)
                          Nre = N_re_0*(1.0E0 + 2.0E0*beta*exp(-beta*gamma)/ &
                               ((2.0E0 + beta)*(1.0E0 + beta)))

                          if( Nre < 0.0E0 ) then
                             print *, "error RE", Nre
                          endif
       
                          ! +++ calculate the terminal velocity +++
                          vtm = Nre*d_vis/(char_len*dens(i,j,k)/1000.0E0)/100.0E0
                          vtm_ice_psd(ibin) = vtm

                          !if (A_e < 0.0E0 .or. A_c < 0.0E0) then
                          !   write(*,'(4I4,12ES15.5)') irank, i, j, k, q, arearatio, vcs, vtm, A_e, A_c, A_e_ic, A_c_ic, psi_ic, bin_ice_alen(i,j,k,ibin), bin_ice_clen(i,j,k,ibin), bin_ice_dlen(i,j,k,ibin)
                          !endif

                          Nrei = 0.0E0
                          do jj = 1, 4
                             if (X >= Best_interval(jj) .and. X < Best_interval(jj+1)) then
                                Nrei = Best_a(jj)*X**Best_b(jj)
                                exit
                             endif
                          enddo
                          vti = Nrei*d_vis/(char_len*dens(i,j,k)/1000.0E0)/100.0E0

                       endif

                       do jj = 1, max_bini
                          if (local_Dmax >= diameteribin_bnd(jj) .and. local_Dmax < diameteribin_bnd(jj+1)) then
                             local_NP(jj) = local_NP(jj) + bin_ice_con(i,j,k,ibin)

                             local_pdf_riming_ice(jj) = local_pdf_riming_ice(jj) + bin_ice_mass_rim(i,j,k,ibin)*(fz(k+1) - fz(k))*DX*DY
                             local_mpdf_riming_ice(jj,counter1) = local_mpdf_riming_ice(jj,counter1) + bin_ice_mass_rim(i,j,k,ibin)*(fz(k+1) - fz(k))*DX*DY
                             local_pdf_vapor_ice(jj) = local_pdf_vapor_ice(jj) + bin_ice_mass_vap(i,j,k,ibin)*(fz(k+1) - fz(k))*DX*DY + bin_ice_mass_eva(i,j,k,ibin)*(fz(k+1) - fz(k))*DX*DY
                             local_mpdf_vapor_ice(jj,counter1) = local_mpdf_vapor_ice(jj,counter1) + bin_ice_mass_vap(i,j,k,ibin)*(fz(k+1) - fz(k))*DX*DY
                             local_pdf_eva_ice(jj) = local_pdf_eva_ice(jj) + bin_ice_mass_eva(i,j,k,ibin)*(fz(k+1) - fz(k))*DX*DY
                             local_pdf_1riming_ice(jj) = local_pdf_1riming_ice(jj) + bin_ice_mass_rim(i,j,k,ibin)
                             local_pdf_1vapor_ice(jj) = local_pdf_1vapor_ice(jj) + bin_ice_mass_vap(i,j,k,ibin)
                             local_pdf_1eva_ice(jj) = local_pdf_1eva_ice(jj) + bin_ice_mass_eva(i,j,k,ibin)
                             
                             local_pdf_aggmass(jj) = local_pdf_aggmass(jj) + bin_ice_magg(i,j,k,ibin)*dens(i,j,k)*1000.0E0 ! gm-3
                             local_pdf_crymass(jj) = local_pdf_crymass(jj) + bin_ice_mcry(i,j,k,ibin)*dens(i,j,k)*1000.0E0 ! gm-3
                             local_pdf_rimmass(jj) = local_pdf_rimmass(jj) + bin_ice_mrim(i,j,k,ibin)*dens(i,j,k)*1000.0E0 ! gm-3
                             
                             if (bin_ice_alen(i,j,k,ibin) /= 0.0) then
                                local_pdf_acratio(jj) = local_pdf_acratio(jj) + bin_ice_clen(i,j,k,ibin)/bin_ice_alen(i,j,k,ibin)*bin_ice_con(i,j,k,ibin)*1000.0E0
                                local_pdf_adratio(jj) = local_pdf_adratio(jj) + bin_ice_dlen(i,j,k,ibin)/bin_ice_alen(i,j,k,ibin)*bin_ice_con(i,j,k,ibin)*1000.0E0
                             endif
                             local_pdf_imacratio(jj) = local_pdf_imacratio(jj) + alpha_im*bin_ice_con(i,j,k,ibin)*1000.0E0

                             local_pdf_arearatio(jj) = local_pdf_arearatio(jj) + arearatio*bin_ice_con(i,j,k,ibin)*1000.0E0
                             local_pdf_arearatio_sidelook(jj) = local_pdf_arearatio_sidelook(jj) + arearatio_sidelook*bin_ice_con(i,j,k,ibin)*1000.0E0
                             local_pdf_arearatio_total(jj) = local_pdf_arearatio_total(jj) + arearatio_total*bin_ice_con(i,j,k,ibin)*1000.0E0

                             if (ice_type == 1) then
                                local_pdf_agg(jj) = local_pdf_agg(jj) + bin_ice_con(i,j,k,ibin)*1000.0E0
                                aggcon = aggcon + bin_ice_con(i,j,k,ibin)*1000.0E0
                             else if (ice_type == 2) then
                                local_pdf_cry(jj) = local_pdf_cry(jj) + bin_ice_con(i,j,k,ibin)*1000.0E0
                                local_pdf_crymd(jj) = local_pdf_crymd(jj) + bin_ice_mass(i,j,k,ibin)*1000.0E0
                                crycon = crycon + bin_ice_con(i,j,k,ibin)*1000.0E0
                             else if (ice_type == 3) then
                                local_pdf_rag(jj) = local_pdf_rag(jj) + bin_ice_con(i,j,k,ibin)*1000.0E0
                                ragcon = ragcon + bin_ice_con(i,j,k,ibin)*1000.0E0
                             else if (ice_type == 4) then
                                local_pdf_gra(jj) = local_pdf_gra(jj) + bin_ice_con(i,j,k,ibin)*1000.0E0
                                local_pdf_gramd(jj) = local_pdf_gramd(jj) + bin_ice_mass(i,j,k,ibin)*1000.0E0
                                gracon = gracon + bin_ice_con(i,j,k,ibin)*1000.0E0
                             else if (ice_type == 5) then
                                local_pdf_rcr(jj) = local_pdf_rcr(jj) + bin_ice_con(i,j,k,ibin)*1000.0E0
                                local_pdf_rcrmd(jj) = local_pdf_rcrmd(jj) + bin_ice_mass(i,j,k,ibin)*1000.0E0
                                rcrcon = rcrcon + bin_ice_con(i,j,k,ibin)*1000.0E0
                             endif
                             if (ice_type == 2 .or. ice_type == 5) then
                                local_pdf_cryrcrmd(jj) = local_pdf_cryrcrmd(jj) + bin_ice_mass(i,j,k,ibin)*1000.0E0
                             endif

                             if (ice_type == 4) then
                                local_pdf_gimacratio(jj) = local_pdf_gimacratio(jj) + alpha_im*bin_ice_con(i,j,k,ibin)*1000.0E0
                                local_pdf_gctv(jj) = local_pdf_gctv(jj) + bin_ice_ctv(i,j,k,ibin)*bin_ice_con(i,j,k,ibin)*1000.0E0
                                local_pdf_gconc(jj) = local_pdf_gconc(jj) + bin_ice_con(i,j,k,ibin)*1000.0E0 !L-1
                                if (asm > csm) then
                                   local_pdf_gsweptVc(jj) = local_pdf_gsweptVc(jj) + bin_ice_ctv(i,j,k,ibin)*100.0E0*pi*asm*asm*bin_ice_con(i,j,k,ibin)
                                   local_pdf_gsweptVm(jj) = local_pdf_gsweptVm(jj) + bin_ice_mtv(i,j,k,ibin)*100.0E0*pi*asm*asm*bin_ice_con(i,j,k,ibin)!*bin_ice_mass(i,j,k,ibin)*dens(i,j,k)
                                else
                                   local_pdf_gsweptVc(jj) = local_pdf_gsweptVc(jj) + bin_ice_ctv(i,j,k,ibin)*100.0E0*4.0E0*asm*csm*bin_ice_con(i,j,k,ibin)
                                   local_pdf_gsweptVm(jj) = local_pdf_gsweptVm(jj) + bin_ice_mtv(i,j,k,ibin)*100.0E0*4.0E0*asm*csm*bin_ice_con(i,j,k,ibin)!*bin_ice_mass(i,j,k,ibin)*dens(i,j,k)
                                endif
                             endif

                             if (poly_exist) then
                                call Iice_shapePoly_mod(habit,bin_ice_alen(i,j,k,ibin),bin_ice_clen(i,j,k,ibin),bin_ice_dlen(i,j,k,ibin),bin_ice_aglen(i,j,k,ibin),bin_ice_cglen(i,j,k,ibin),bin_ice_nex(i,j,k,ibin))
                                if (habit == 1) then
                                   local_pdf_pla(jj) = local_pdf_pla(jj) + bin_ice_con(i,j,k,ibin)*1000.0E0
                                else if (habit == 2) then
                                   local_pdf_den(jj) = local_pdf_den(jj) + bin_ice_con(i,j,k,ibin)*1000.0E0
                                else if (habit == 3) then
                                   local_pdf_col(jj) = local_pdf_col(jj) + bin_ice_con(i,j,k,ibin)*1000.0E0
                                else if (habit == 4) then
                                   local_pdf_ppl(jj) = local_pdf_ppl(jj) + bin_ice_con(i,j,k,ibin)*1000.0E0
                                elseif (habit == 5) then
                                   local_pdf_pcl(jj) = local_pdf_pcl(jj) + bin_ice_con(i,j,k,ibin)*1000.0E0
                                elseif (habit == 6) then
                                   local_pdf_pir(jj) = local_pdf_pir(jj) + bin_ice_con(i,j,k,ibin)*1000.0E0
                                endif
                             else
                                call Iice_shape_mod(habit,bin_ice_alen(i,j,k,ibin),bin_ice_clen(i,j,k,ibin),bin_ice_dlen(i,j,k,ibin))
                                if (habit == 1) then
                                   local_pdf_pla(jj) = local_pdf_pla(jj) + bin_ice_con(i,j,k,ibin)*1000.0E0
                                else if (habit == 2) then
                                   local_pdf_den(jj) = local_pdf_den(jj) + bin_ice_con(i,j,k,ibin)*1000.0E0
                                else if (habit == 3) then
                                   local_pdf_col(jj) = local_pdf_col(jj) + bin_ice_con(i,j,k,ibin)*1000.0E0
                                endif
                             endif


                             if (bin_ice_alen(i,j,k,ibin) < 1.e-15 .or. bin_ice_clen(i,j,k,ibin) < 1.e-15 .or. alpha_ic < 1.e-7 .or. alpha_ic > 100.0E0) then
                                hex_cap = 0.0E0
                             else
                                cry_clen = bin_ice_clen(i,j,k,ibin)
                                cry_alen = bin_ice_alen(i,j,k,ibin)
                                if (cry_clen == cry_alen .or. is_mod == 2) then
                                   hex_cap = cry_alen
                                else if (cry_alen > cry_clen) then
                                   d = sqrt(cry_alen**2 - cry_clen**2)
                                   eps = sqrt(1.0E0 - (cry_clen/cry_alen)**2)
                                   hex_cap = d/asin(eps)
                                else
                                   d = sqrt(cry_clen**2 - cry_alen**2)
                                   eps = sqrt(1.0E0 - 1.0E0/(cry_clen/cry_alen)**2)
                                   hex_cap = d/log((1.0E0 + eps)*(cry_clen/cry_alen))
                                endif
                             endif
                              
                             if (ships_ice_type <= 2) then
                                if (habit <= 3) then
                                   capacitance = hex_cap
                                else if (habit == 4) then
                                   eps = 0.968245837E0
                                   d = pla_e*eps
                                   capacitance = d/asin(eps)
                                else if (habit == 5) then
                                   CAP_ros = 0.619753727E0
                                   capacitance = CAP_ros*ros_r
                                else if (habit == 6) then
                                   eps = 0.968245837E0
                                   d = max(pla_e,ros_r)*eps
                                   capacitance = d/asin(eps)
                                endif
                             else if (ships_ice_type == 3) then
                                capacitance = 0.25E0*(bin_ice_Dmax(i,j,k,ibin)*dens(i,j,k)/bin_ice_con(i,j,k,ibin)/pi*6.0E0)**(1.0E0/3.0E0)
                             else
                                if (alpha_im == 1.0E0) then
                                   capacitance = asm
                                else if (alpha_im < 1.0E0) then
                                   d = sqrt(asm*asm - csm*csm)
                                   eps = sqrt(1.0E0 - alpha_im**2)
                                   capacitance = d/asin(eps)
                                else
                                   d = sqrt(csm*csm - asm*asm)
                                   eps = sqrt(1.0E0 - 1.0E0/alpha_im**2)
                                   capacitance = d/log((1.0E0 + eps)*alpha_im)
                                endif
                             endif
                                 
                             N_sc = d_vis/dens(i,j,k)/1000.0E0/D_v
                             Xv = N_sc**(1.0E0/3.0E0)*sqrt(Nre)
                             if (Xv < 1.4E0) then
                                fv = 1.0E0 + 0.108E0*(Xv**2)
                             else if (1.4E0 <= Xv .and. Xv <= 51.4E0) then
                                fv = 0.78E0 + 0.308E0*Xv
                             else
                                fv = 0.78E0 + 0.308E0*51.4E0
                             endif

                             local_pdf_split_cap(jj) = local_pdf_split_cap(jj) + bin_ice_con(i,j,k,ibin)*capacitance
                             local_pdf_split_vent(jj) = local_pdf_split_vent(jj) + bin_ice_con(i,j,k,ibin)*fv
                             local_pdf_split_kinetic(jj) = local_pdf_split_kinetic(jj) + bin_ice_con(i,j,k,ibin)*fkn
                                 
                                 
                             if (asm > csm) then
                                local_pdf_sweptVc(jj) = local_pdf_sweptVc(jj) + bin_ice_ctv(i,j,k,ibin)*100.0E0*pi*asm*asm*bin_ice_con(i,j,k,ibin)
                                local_pdf_sweptVm(jj) = local_pdf_sweptVm(jj) + bin_ice_mtv(i,j,k,ibin)*100.0E0*pi*asm*asm*bin_ice_con(i,j,k,ibin)!*bin_ice_mass(i,j,k,ibin)*dens(i,j,k)
                                local_pdf_sweptVcb(jj) = local_pdf_sweptVcb(jj) + vtm*100.0*pi*asm*asm*bin_ice_con(i,j,k,ibin)
                                local_mpdf_sweptVcb(jj,counter1) = local_mpdf_sweptVcb(jj,counter1) + vtm*100.0*pi*asm*asm*bin_ice_con(i,j,k,ibin)
                                local_pdf_sweptVcb_real(jj) = local_pdf_sweptVcb_real(jj) + abs(vtm - liquid_cbtv)*100.0E0*pi*(asm+liquid_eff_rad_cm)*(asm+liquid_eff_rad_cm)*bin_ice_con(i,j,k,ibin)*liquid_number*liquid_mass
                                local_pdf_split_area(jj) = local_pdf_split_area(jj) + pi*(asm+liquid_eff_rad_cm)*(asm+liquid_eff_rad_cm)*bin_ice_con(i,j,k,ibin)
                                local_pdf_split_areaiceonly(jj) = local_pdf_split_areaiceonly(jj) + pi*(asm)*(asm)*bin_ice_con(i,j,k,ibin)
                                if (ice_type == 2) then
                                   local_pdf_split_areacry(jj) = local_pdf_split_areacry(jj) + pi*(asm+liquid_eff_rad_cm)*(asm+liquid_eff_rad_cm)*bin_ice_con(i,j,k,ibin)
                                   local_pdf_split_areaiceonlycry(jj) = local_pdf_split_areaiceonlycry(jj) + pi*(asm)*(asm)*bin_ice_con(i,j,k,ibin)
                                endif
                                if (ice_type == 2 .or. ice_type == 5) then
                                   local_pdf_split_areacryrcr(jj) = local_pdf_split_areacryrcr(jj) + pi*(asm+liquid_eff_rad_cm)*(asm+liquid_eff_rad_cm)*bin_ice_con(i,j,k,ibin)
                                   local_pdf_split_areaiceonlycryrcr(jj) = local_pdf_split_areaiceonlycryrcr(jj) + pi*(asm)*(asm)*bin_ice_con(i,j,k,ibin)
                                endif
                                total_sweptVc = total_sweptVc + bin_ice_ctv(i,j,k,ibin)*100.0E0*pi*asm*asm*bin_ice_con(i,j,k,ibin)
                                total_sweptVcb = total_sweptVcb + vtm*100.0*pi*asm*asm*bin_ice_con(i,j,k,ibin)
                                total_sweptVcb_real = total_sweptVcb_real + abs(vtm - liquid_cbtv)*100.0E0*pi*(asm+liquid_eff_rad_cm)*(asm+liquid_eff_rad_cm)*bin_ice_con(i,j,k,ibin)*liquid_number*liquid_mass
                             else
                                local_pdf_sweptVc(jj) = local_pdf_sweptVc(jj) + bin_ice_ctv(i,j,k,ibin)*100.0E0*4.0E0*asm*csm*bin_ice_con(i,j,k,ibin)
                                local_pdf_sweptVm(jj) = local_pdf_sweptVm(jj) + bin_ice_mtv(i,j,k,ibin)*100.0E0*4.0E0*asm*csm*bin_ice_con(i,j,k,ibin)!*bin_ice_mass(i,j,k,ibin)*dens(i,j,k)
                                local_pdf_sweptVcb(jj) = local_pdf_sweptVcb(jj) + vtm*100.0E0*4.0E0*asm*csm*bin_ice_con(i,j,k,ibin)
                                local_mpdf_sweptVcb(jj,counter1) = local_mpdf_sweptVcb(jj,counter1) +  vtm*100.0E0*4.0E0*asm*csm*bin_ice_con(i,j,k,ibin)
                                local_pdf_sweptVcb_real(jj) = local_pdf_sweptVcb_real(jj) + abs(vtm-liquid_cbtv)*100.0E0*4.0E0*(asm+liquid_eff_rad_cm)*(csm+liquid_eff_rad_cm)*bin_ice_con(i,j,k,ibin)*liquid_number*liquid_mass
                                local_pdf_split_area(jj) = local_pdf_split_area(jj) + 4.0E0*(asm+liquid_eff_rad_cm)*(csm+liquid_eff_rad_cm)*bin_ice_con(i,j,k,ibin)
                                local_pdf_split_areaiceonly(jj) = local_pdf_split_areaiceonly(jj) + 4.0E0*(asm)*(csm)*bin_ice_con(i,j,k,ibin)
                                if (ice_type == 2) then
                                local_pdf_split_areacry(jj) = local_pdf_split_areacry(jj) + 4.0E0*(asm+liquid_eff_rad_cm)*(csm+liquid_eff_rad_cm)*bin_ice_con(i,j,k,ibin)
                                local_pdf_split_areaiceonlycry(jj) = local_pdf_split_areaiceonlycry(jj) + 4.0E0*(asm)*(csm)*bin_ice_con(i,j,k,ibin)
                                endif
                                if (ice_type == 2 .or. ice_type == 5) then
                                local_pdf_split_areacryrcr(jj) = local_pdf_split_areacryrcr(jj) + 4.0E0*(asm+liquid_eff_rad_cm)*(csm+liquid_eff_rad_cm)*bin_ice_con(i,j,k,ibin)
                                local_pdf_split_areaiceonlycryrcr(jj) = local_pdf_split_areaiceonlycryrcr(jj) + 4.0E0*(asm)*(csm)*bin_ice_con(i,j,k,ibin)
                                endif
                                
                                total_sweptVc = total_sweptVc + bin_ice_ctv(i,j,k,ibin)*100.0E0*4.0E0*asm*csm*bin_ice_con(i,j,k,ibin)
                                total_sweptVcb = total_sweptVcb + vtm*100.0E0*4.0E0*asm*csm*bin_ice_con(i,j,k,ibin)
                                total_sweptVcb_real = total_sweptVcb_real + abs(vtm-liquid_cbtv)*100.0E0*4.0E0*(asm+liquid_eff_rad_cm)*(csm+liquid_eff_rad_cm)*bin_ice_con(i,j,k,ibin)*liquid_number*liquid_mass
                             endif

                             local_pdf_split_Nwmass(jj) = local_pdf_split_Nwmass(jj) + liquid_number*liquid_mass*bin_ice_con(i,j,k,ibin)
                             local_pdf_split_deltaVcb(jj) = local_pdf_split_deltaVcb(jj) + abs(vtm-liquid_cbtv)*100.0E0*bin_ice_con(i,j,k,ibin)
                                    
                             local_pdf_mtv(jj) = local_pdf_mtv(jj) + bin_ice_mtv(i,j,k,ibin)*bin_ice_mass(i,j,k,ibin)*dens(i,j,k)*1000.0E0
                             local_pdf_mass(jj) = local_pdf_mass(jj) + bin_ice_mass(i,j,k,ibin)*dens(i,j,k)*1000.0E0 ! gm-3
                             if (habit == 1) then
                                local_pdf_mass_pla(jj) = local_pdf_mass_pla(jj) + bin_ice_mass(i,j,k,ibin)*dens(i,j,k)*1000.0E0
                             else if (habit == 2) then
                                local_pdf_mass_den(jj) = local_pdf_mass_den(jj) + bin_ice_mass(i,j,k,ibin)*dens(i,j,k)*1000.0E0
                             else if (habit == 3) then
                                local_pdf_mass_col(jj) = local_pdf_mass_col(jj) + bin_ice_mass(i,j,k,ibin)*dens(i,j,k)*1000.0E0
                             else if (habit == 4) then
                                local_pdf_mass_ppl(jj) = local_pdf_mass_ppl(jj) + bin_ice_mass(i,j,k,ibin)*dens(i,j,k)*1000.0E0
                             elseif (habit == 5) then
                                local_pdf_mass_pcl(jj) = local_pdf_mass_pcl(jj) + bin_ice_mass(i,j,k,ibin)*dens(i,j,k)*1000.0E0
                             elseif (habit == 6) then
                                local_pdf_mass_pir(jj) = local_pdf_mass_pir(jj) + bin_ice_mass(i,j,k,ibin)*dens(i,j,k)*1000.0E0
                             endif
                             if (ice_type == 1) then
                                local_pdf_mass_agg(jj) = local_pdf_mass_agg(jj) + bin_ice_mass(i,j,k,ibin)*dens(i,j,k)*1000.0E0
                             else if (ice_type == 2) then
                                local_pdf_mass_cry(jj) = local_pdf_mass_cry(jj) + bin_ice_mass(i,j,k,ibin)*dens(i,j,k)*1000.0E0
                             else if (ice_type == 3) then
                                local_pdf_mass_rag(jj) = local_pdf_mass_rag(jj) + bin_ice_mass(i,j,k,ibin)*dens(i,j,k)*1000.0E0
                             else if (ice_type == 4) then
                                local_pdf_mass_gra(jj) = local_pdf_mass_gra(jj) + bin_ice_mass(i,j,k,ibin)*dens(i,j,k)*1000.0E0
                             else if (ice_type == 5) then
                                local_pdf_mass_rcr(jj) = local_pdf_mass_rcr(jj) + bin_ice_mass(i,j,k,ibin)*dens(i,j,k)*1000.0E0
                             endif
                             local_pdf_mfrac_cry(jj) = local_pdf_mfrac_cry(jj) + bin_ice_con(i,j,k,ibin)*(bin_ice_mcry(i,j,k,ibin)/bin_ice_mass(i,j,k,ibin))
                             local_pdf_mfrac_rim(jj) = local_pdf_mfrac_rim(jj) + bin_ice_con(i,j,k,ibin)*(bin_ice_mrim(i,j,k,ibin)/bin_ice_mass(i,j,k,ibin))
                             vcs = bin_ice_Dmax(i,j,k,ibin)*dens(i,j,k)/bin_ice_con(i,j,k,ibin)
                             if (vcs /= 0.0E0) then
                                local_pdf_bulk(jj) = local_pdf_bulk(jj) + bin_ice_mass(i,j,k,ibin)*dens(i,j,k)/1000.0E0/vcs ! gcm-3 / cm3
                             endif
                             local_pdf_ctv(jj) = local_pdf_ctv(jj) + bin_ice_ctv(i,j,k,ibin)*bin_ice_con(i,j,k,ibin)*1000.0E0

                              
                             local_pdf_cbtv(jj) = local_pdf_cbtv(jj) + vtm*bin_ice_con(i,j,k,ibin)*1000.0E0
                             local_pdf_citv(jj) = local_pdf_citv(jj) + vti*bin_ice_con(i,j,k,ibin)*1000.0E0
                             local_pdf_RE(jj) = local_pdf_RE(jj) + Nre*bin_ice_con(i,j,k,ibin)*1000.0E0
                             local_pdf_conc(jj) = local_pdf_conc(jj) + bin_ice_con(i,j,k,ibin)*1000.0E0 !L-1
                             local_pdf_rhi(jj) = local_pdf_rhi(jj) + rhi(i,j,k)*bin_ice_con(i,j,k,ibin)*1000.0E0
                             local_pdf_T(jj) = local_pdf_T(jj) + temperature(i,j,k)*bin_ice_con(i,j,k,ibin)*1000.0E0
                             local_pdf_count(jj) = local_pdf_count(jj) + 1.0E0
                             exit

                          endif ! diameter inside jj bin
                           
                       enddo ! jj

                    endif ! ice bin con limit
                 enddo ! ibin

                 local_N = 0.0E0
                 local_N(1) = local_NP(1)
                 do jj = 2, max_bini
                    local_N(jj) = local_N(jj-1) + local_NP(jj)
                 enddo
                 do jj = 1, max_bini
                    temp1 = temp1 + local_NP(jj)
                    if (jj == 1) then
                       local_dNidlogD(jj,counter1) = (local_N(jj+1) - local_N(jj))/(log10(diameteribin(jj+1)) - log10(diameteribin(jj)))
                    else if (jj == max_bini) then
                       local_dNidlogD(jj,counter1) = (local_N(jj) - local_N(jj-1))/(log10(diameteribin(jj)) - log10(diameteribin(jj-1)))
                    else
                       local_dNidlogD(jj,counter1) = (local_N(jj+1) - local_N(jj-1))/(log10(diameteribin(jj+1)) - log10(diameteribin(jj-1)))
                    endif
                 enddo

                 a2d = 0.0E0
                 error_number = 0.0E0
                 WVT = 0.0E0
                 call cal_lincubprms_vec(a2d, error_number, max_bini, con_ice_psd, mass_ice_psd, massbini_bothbnd)
                 massbini_leftbnd = 0.0E0
                 massbini_rightbnd = 0.0E0
                 do jj = 1, max_bini
                    if (a2d(jj,4) < -9.98e+34) then
                       !linear distribution
                       if(a2d(jj,1) > 0.0E0) then
                          massbini_leftbnd(jj) = massbini_bnd(jj)
                          massbini_rightbnd(jj) = massbini_bnd(jj+1)
                       else if(a2d(jj,1) == -1.0E0) then
                          massbini_leftbnd(jj) = a2d(jj,2)
                          massbini_rightbnd(jj) = massbini_bnd(jj+1)
                       else if(a2d(jj,1) == -2.0E0) then
                          massbini_leftbnd(jj) = massbini_bnd(jj)
                          massbini_rightbnd(jj) = a2d(jj,2)
                       endif
                       ! re-write the parameters as n(m)=a0+a1*m
                       a2d(jj,1) = max(0.0E0,a2d(jj,1))-a2d(jj,2)*a2d(jj,3)
                       a2d(jj,2) = a2d(jj,3)
                       a2d(jj,3) = 0.0E0
                       a2d(jj,4) = 0.0E0
                    else
                       massbini_leftbnd(jj) = massbini_bnd(jj)
                       massbini_rightbnd(jj) = massbini_bnd(jj+1)
                    endif
                 enddo
                 d_vis = get_dynamic_viscosity(temperature(i,j,k))
                 call cal_wterm_vel_v3_vec(WVT, max_bini, a2d, massbini_leftbnd, massbini_rightbnd, X_ice_psd, Q_ice_psd, den_ice_psd, alpha_ice_psd, mean_mass_ice_psd, char_len_ice_psd, vtm_ice_psd, dens(i,j,k)/1000.0E0, d_vis, error_number)
                 
                 do ibin = 1, max_bini
                    local_Dmax = bin_ice_aDmax(i,j,k,ibin)
                    do jj = 1, max_bini
                       if (local_Dmax >= diameteribin_bnd(jj) .and. local_Dmax < diameteribin_bnd(jj+1)) then
                          local_pdf_wctv(jj) = local_pdf_wctv(jj) + WVT(ibin,2)*0.01E0*bin_ice_con(i,j,k,ibin)*1000.0E0 ! cm to m
                          exit
                       endif
                    enddo
                 enddo

                 local_pdf_temp(counter1) = temperature(i,j,k)
                 local_pdf_temp_rim(counter1) = ice_mass_rim(i,j,k)*(fz(k+1)-fz(k))*DX*DY
                 local_pdf_temp_rimNORMliq(counter1) = ice_mass_rim(i,j,k)*1000000.0E0/(qliq(i,j,k)*dens(i,j,k))
                 local_pdf_temp_vap(counter1) = (ice_mass_vap(i,j,k) + ice_mass_eva(i,j,k))*(fz(k+1)-fz(k))*DX*DY
                 local_pdf_temp_imm(counter1) = ice_mass_imm(i,j,k)*(fz(k+1)-fz(k))*DX*DY
                 local_pdf_temp_aggcon(counter1) = aggcon
                 local_pdf_temp_rcrcon(counter1) = rcrcon
                 local_pdf_temp_ragcon(counter1) = ragcon
                 local_pdf_temp_gracon(counter1) = gracon
                 local_pdf_temp_crycon(counter1) = crycon
                 local_pdf_rad(counter1) = liquid_eff_rad
                 local_pdf_nliq(counter1) = liquid_number
                 local_pdf_lwc(counter1) = dens(i,j,k)*qliq(i,j,k)*1000.0E0
                 local_pdf_wvel(counter1) = w(i,j,k)
                 local_pdf_temp_sweptVc(counter1) = total_sweptVc
                 local_pdf_temp_sweptVcb(counter1) = total_sweptVcb
                 local_pdf_temp_sweptVcb_real(counter1) = total_sweptVcb_real
                 local_pdf_cbtv_liq(counter1) = liquid_cbtv

                 do ibin = 1, max_bini

                    if (local_mpdf_riming_ice(ibin,counter1) == 0.0E0) then
                       local_mpdf_riming_ice(ibin,counter1) = -10.0E0
                    endif
                    if (local_mpdf_vapor_ice(ibin,counter1) == 0.0E0) then
                       local_mpdf_vapor_ice(ibin,counter1) = -10.0E0
                    endif
                    if (local_mpdf_sweptVcb(ibin,counter1) == 0.0E0) then
                       local_mpdf_sweptVcb(ibin,counter1) = -10.0E0
                    else
                       local_mpdf_sweptVcb(ibin,counter1) = local_mpdf_sweptVcb(ibin,counter1)/local_pdf_conc(ibin)*1000.0E0
                    endif

                 enddo
                
              else

                 do ibin = 1, max_binr
                    local_dNdlogD(ibin,counter1) = -10.0E0
                 enddo
                 do ibin = 1, max_bini
                    local_dNidlogD(ibin,counter1) = -10.0E0
                 enddo

                 local_pdf_temp(counter1) = -10.0E0
                 local_pdf_temp_rim(counter1) = -10.0E0
                 local_pdf_temp_rimNORMliq(counter1) = -10.0E0
                 local_pdf_temp_vap(counter1) = -10.0E0
                 local_pdf_temp_imm(counter1) = -10.0E0
                 local_pdf_temp_aggcon(counter1) = -10.0E0
                 local_pdf_temp_rcrcon(counter1) = -10.0E0
                 local_pdf_temp_ragcon(counter1) = -10.0E0
                 local_pdf_temp_gracon(counter1) = -10.0E0
                 local_pdf_temp_crycon(counter1) = -10.0E0
                 local_pdf_rad(counter1) = -10.0E0
                 local_pdf_nliq(counter1) = -10.0E0
                 local_pdf_lwc(counter1) = -10.0E0
                 local_pdf_wvel(counter1) = -10.0E0
                 local_pdf_temp_sweptVc(counter1) = -10.0E0
                 local_pdf_temp_sweptVcb(counter1) = -10.0E0
                 local_pdf_temp_sweptVcb_real(counter1) = -10.0E0
                 local_pdf_cbtv_liq(counter1) = -10.0E0

                 local_mpdf_riming_ice(:,counter1) = -10.0E0
                 local_mpdf_vapor_ice(:,counter1) = -10.0E0
                 local_mpdf_sweptVcb(:,counter1) = -10.0E0


              endif ! qliq limit
              counter1 = counter1 + 1
           enddo ! j
        enddo ! i
     enddo ! k
     
     
     call MPI_REDUCE( local_pdf_allvapor_liq, pdf_allvapor_liq, max_bini, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( local_pdf_alleva_liq, pdf_alleva_liq, max_bini, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( local_pdf_conc_liq, pdf_conc_liq, max_bini, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( local_pdf_mass_liq, pdf_mass_liq, max_bini, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( local_pdf_riming_liq, pdf_riming_liq, max_bini, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( local_pdf_vapor_liq, pdf_vapor_liq, max_bini, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( local_pdf_eva_liq, pdf_eva_liq, max_bini, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )

     call MPI_REDUCE( local_pdf_riming_ice, pdf_riming_ice, max_bini, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( local_pdf_vapor_ice, pdf_vapor_ice, max_bini, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( local_pdf_eva_ice, pdf_eva_ice, max_bini, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( local_pdf_1riming_ice, pdf_1riming_ice, max_bini, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( local_pdf_1vapor_ice, pdf_1vapor_ice, max_bini, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( local_pdf_1eva_ice, pdf_1eva_ice, max_bini, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )

     call MPI_REDUCE( local_pdf_aggmass, pdf_aggmass, max_bini, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( local_pdf_crymass, pdf_crymass, max_bini, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( local_pdf_rimmass, pdf_rimmass, max_bini, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )

     call MPI_REDUCE( local_pdf_acratio, pdf_acratio, max_bini, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( local_pdf_adratio, pdf_adratio, max_bini, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( local_pdf_imacratio, pdf_imacratio, max_bini, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )

     call MPI_REDUCE( local_pdf_arearatio, pdf_arearatio, max_bini, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( local_pdf_arearatio_sidelook, pdf_arearatio_sidelook, max_bini, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( local_pdf_arearatio_total, pdf_arearatio_total, max_bini, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )

     call MPI_REDUCE( local_pdf_agg, pdf_agg, max_bini, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( local_pdf_cry, pdf_cry, max_bini, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( local_pdf_gra, pdf_gra, max_bini, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( local_pdf_rag, pdf_rag, max_bini, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( local_pdf_rcr, pdf_rcr, max_bini, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )

     call MPI_REDUCE( local_pdf_crymd, pdf_crymd, max_bini, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( local_pdf_gramd, pdf_gramd, max_bini, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( local_pdf_rcrmd, pdf_rcrmd, max_bini, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( local_pdf_cryrcrmd, pdf_cryrcrmd, max_bini, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )

     call MPI_REDUCE( local_pdf_gimacratio, pdf_gimacratio, max_bini, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( local_pdf_gctv, pdf_gctv, max_bini, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( local_pdf_gconc, pdf_gconc, max_bini, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( local_pdf_gsweptVc, pdf_gsweptVc, max_bini, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( local_pdf_gsweptVm, pdf_gsweptVm, max_bini, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )

     call MPI_REDUCE( local_pdf_pla, pdf_pla, max_bini, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( local_pdf_col, pdf_col, max_bini, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( local_pdf_den, pdf_den, max_bini, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     if (poly_exist) then
        call MPI_REDUCE( local_pdf_ppl, pdf_ppl, max_bini, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
        call MPI_REDUCE( local_pdf_pcl, pdf_pcl, max_bini, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
        call MPI_REDUCE( local_pdf_pir, pdf_pir, max_bini, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     endif

     call MPI_REDUCE( local_pdf_split_cap, pdf_split_cap, max_bini, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( local_pdf_split_vent, pdf_split_vent, max_bini, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( local_pdf_split_kinetic, pdf_split_kinetic, max_bini, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( local_pdf_split_area, pdf_split_area, max_bini, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( local_pdf_split_areaiceonly, pdf_split_areaiceonly, max_bini, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( local_pdf_split_areacry, pdf_split_areacry, max_bini, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( local_pdf_split_areaiceonlycry, pdf_split_areaiceonlycry, max_bini, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( local_pdf_split_areacryrcr, pdf_split_areacryrcr, max_bini, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( local_pdf_split_areaiceonlycryrcr, pdf_split_areaiceonlycryrcr, max_bini, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( local_pdf_split_Nwmass, pdf_split_Nwmass, max_bini, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( local_pdf_split_deltaVcb, pdf_split_deltaVcb, max_bini, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )

     call MPI_REDUCE( local_pdf_sweptVc, pdf_sweptVc, max_bini, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( local_pdf_sweptVm, pdf_sweptVm, max_bini, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( local_pdf_sweptVcb, pdf_sweptVcb, max_bini, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( local_pdf_sweptVcb_real, pdf_sweptVcb_real, max_bini, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )

     call MPI_REDUCE( local_pdf_mfrac_cry, pdf_mfrac_cry, max_bini, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( local_pdf_mfrac_rim, pdf_mfrac_rim, max_bini, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )

     call MPI_REDUCE( local_pdf_mass, pdf_mass, max_bini, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( local_pdf_mass_pla, pdf_mass_pla, max_bini, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( local_pdf_mass_den, pdf_mass_den, max_bini, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( local_pdf_mass_col, pdf_mass_col, max_bini, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( local_pdf_mass_ppl, pdf_mass_ppl, max_bini, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( local_pdf_mass_pcl, pdf_mass_pcl, max_bini, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( local_pdf_mass_pir, pdf_mass_pir, max_bini, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( local_pdf_mass_cry, pdf_mass_cry, max_bini, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( local_pdf_mass_rcr, pdf_mass_rcr, max_bini, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( local_pdf_mass_agg, pdf_mass_agg, max_bini, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( local_pdf_mass_rag, pdf_mass_rag, max_bini, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( local_pdf_mass_gra, pdf_mass_gra, max_bini, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( local_pdf_mtv, pdf_mtv, max_bini, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( local_pdf_bulk, pdf_bulk, max_bini, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )

     call MPI_REDUCE( local_pdf_conc, pdf_conc, max_bini, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( local_pdf_ctv, pdf_ctv, max_bini, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( local_pdf_cbtv, pdf_cbtv, max_bini, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( local_pdf_citv, pdf_citv, max_bini, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( local_pdf_wctv, pdf_wctv, max_bini, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( local_pdf_RE, pdf_RE, max_bini, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )

     call MPI_REDUCE( local_pdf_rhi, pdf_rhi, max_bini, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )
     call MPI_REDUCE( local_pdf_T, pdf_T, max_bini, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )

     call MPI_REDUCE( local_pdf_count, pdf_count, max_bini, MPI_REAL4, MPI_SUM, master, MPI_COMM_WORLD, ierr )



     call MPI_GATHER(local_pdf_temp, sendcount_median, MPI_REAL4, pdf_temp, sendcount_median, MPI_REAL4, master, MPI_COMM_WORLD, ierr )
     call MPI_GATHER(local_pdf_temp_rim, sendcount_median, MPI_REAL4, pdf_temp_rim, sendcount_median, MPI_REAL4, master, MPI_COMM_WORLD, ierr )
     call MPI_GATHER(local_pdf_temp_rimNORMliq, sendcount_median, MPI_REAL4, pdf_temp_rimNORMliq, sendcount_median, MPI_REAL4, master, MPI_COMM_WORLD, ierr )
     call MPI_GATHER(local_pdf_temp_vap, sendcount_median, MPI_REAL4, pdf_temp_vap, sendcount_median, MPI_REAL4, master, MPI_COMM_WORLD, ierr )
     call MPI_GATHER(local_pdf_temp_imm, sendcount_median, MPI_REAL4, pdf_temp_imm, sendcount_median, MPI_REAL4, master, MPI_COMM_WORLD, ierr )

     call MPI_GATHER(local_pdf_temp_aggcon, sendcount_median, MPI_REAL4, pdf_temp_aggcon, sendcount_median, MPI_REAL4, master, MPI_COMM_WORLD, ierr )
     call MPI_GATHER(local_pdf_temp_rcrcon, sendcount_median, MPI_REAL4, pdf_temp_rcrcon, sendcount_median, MPI_REAL4, master, MPI_COMM_WORLD, ierr )
     call MPI_GATHER(local_pdf_temp_ragcon, sendcount_median, MPI_REAL4, pdf_temp_ragcon, sendcount_median, MPI_REAL4, master, MPI_COMM_WORLD, ierr )
     call MPI_GATHER(local_pdf_temp_crycon, sendcount_median, MPI_REAL4, pdf_temp_crycon, sendcount_median, MPI_REAL4, master, MPI_COMM_WORLD, ierr )
     call MPI_GATHER(local_pdf_temp_gracon, sendcount_median, MPI_REAL4, pdf_temp_gracon, sendcount_median, MPI_REAL4, master, MPI_COMM_WORLD, ierr )

     call MPI_GATHER(local_pdf_rad, sendcount_median, MPI_REAL4, pdf_rad, sendcount_median, MPI_REAL4, master, MPI_COMM_WORLD, ierr )
     call MPI_GATHER(local_pdf_nliq, sendcount_median, MPI_REAL4, pdf_nliq, sendcount_median, MPI_REAL4, master, MPI_COMM_WORLD, ierr )
     call MPI_GATHER(local_pdf_lwc, sendcount_median, MPI_REAL4, pdf_lwc, sendcount_median, MPI_REAL4, master, MPI_COMM_WORLD, ierr )
     call MPI_GATHER(local_pdf_wvel, sendcount_median, MPI_REAL4, pdf_wvel, sendcount_median, MPI_REAL4, master, MPI_COMM_WORLD, ierr )

     call MPI_GATHER(local_pdf_temp_sweptVc, sendcount_median, MPI_REAL4, pdf_temp_sweptVc, sendcount_median, MPI_REAL4, master, MPI_COMM_WORLD, ierr )
     call MPI_GATHER(local_pdf_temp_sweptVcb, sendcount_median, MPI_REAL4, pdf_temp_sweptVcb, sendcount_median, MPI_REAL4, master, MPI_COMM_WORLD, ierr )
     call MPI_GATHER(local_pdf_temp_sweptVcb_real, sendcount_median, MPI_REAL4, pdf_temp_sweptVcb_real, sendcount_median, MPI_REAL4, master, MPI_COMM_WORLD, ierr )
     call MPI_GATHER(local_pdf_cbtv_liq, sendcount_median, MPI_REAL4, pdf_cbtv_liq, sendcount_median, MPI_REAL4, master, MPI_COMM_WORLD, ierr )

     do ibin = 1, max_binr
        call MPI_GATHER(local_dNdlogD(ibin,:), sendcount_median, MPI_REAL4, dNdlogD(ibin,:), sendcount_median, MPI_REAL4, master, MPI_COMM_WORLD, ierr )
        !if (irank == master) then
        !   dNdlogD(ibin,:) = global_var(:)
        !endif
     enddo
     do ibin = 1, max_bini
        call MPI_GATHER(local_dNidlogD(ibin,:), sendcount_median, MPI_REAL4, dNidlogD(ibin,:), sendcount_median, MPI_REAL4, master, MPI_COMM_WORLD, ierr )
        call MPI_GATHER(local_mpdf_vapor_ice(ibin,:), sendcount_median, MPI_REAL4, global_mpdf_vapor_ice(ibin,:), sendcount_median, MPI_REAL4, master, MPI_COMM_WORLD, ierr )
        call MPI_GATHER(local_mpdf_riming_ice(ibin,:), sendcount_median, MPI_REAL4, global_mpdf_riming_ice(ibin,:), sendcount_median, MPI_REAL4, master, MPI_COMM_WORLD, ierr )
        call MPI_GATHER(local_mpdf_sweptVcb(ibin,:), sendcount_median, MPI_REAL4, global_mpdf_sweptVcb(ibin,:), sendcount_median, MPI_REAL4, master, MPI_COMM_WORLD, ierr )

        !if (irank == master) then
        !   dNdlogD(ibin,:) = global_var(:)
        !endif
     enddo


     if (irank == master) then

        do ibin = 1, max_binr
           if (pdf_conc_liq(ibin) /= 0.0E0) then
              pdf_cbtv_liq(ibin) = pdf_cbtv_liq(ibin)/pdf_conc_liq(ibin)
           endif
        enddo

        do ibin = 1, max_bini
           if (pdf_mass(ibin) /= 0.0E0) then
              pdf_mtv(ibin) = pdf_mtv(ibin)/pdf_mass(ibin)
              pdf_crymass(ibin) = pdf_crymass(ibin)/pdf_mass(ibin)
              pdf_aggmass(ibin) = pdf_aggmass(ibin)/pdf_mass(ibin)
              pdf_rimmass(ibin) = pdf_rimmass(ibin)/pdf_mass(ibin)
              !pdf_sweptVm(ibin) = pdf_sweptVm(ibin)/pdf_mass(ibin)*1000.0
           endif
            
           if (pdf_conc(ibin) /= 0.0E0) then
              pdf_mfrac_cry(ibin) = pdf_mfrac_cry(ibin)/pdf_conc(ibin)*1000.0E0
              pdf_mfrac_rim(ibin) = pdf_mfrac_rim(ibin)/pdf_conc(ibin)*1000.0E0
              
              pdf_wctv(ibin) = pdf_wctv(ibin)/pdf_conc(ibin) !/pdf_count(ibin)
              pdf_ctv(ibin) = pdf_ctv(ibin)/pdf_conc(ibin)
              pdf_cbtv(ibin) = pdf_cbtv(ibin)/pdf_conc(ibin)
              pdf_citv(ibin) = pdf_citv(ibin)/pdf_conc(ibin)
              pdf_RE(ibin) = pdf_RE(ibin)/pdf_conc(ibin)
              pdf_cry(ibin) = pdf_cry(ibin)/pdf_conc(ibin)
              pdf_agg(ibin) = pdf_agg(ibin)/pdf_conc(ibin)
              pdf_rcr(ibin) = pdf_rcr(ibin)/pdf_conc(ibin)
              pdf_gra(ibin) = pdf_gra(ibin)/pdf_conc(ibin)
              pdf_rag(ibin) = pdf_rag(ibin)/pdf_conc(ibin)
              if (poly_exist) then
                 pdf_ppl(ibin) = pdf_ppl(ibin)/pdf_conc(ibin)
                 pdf_pcl(ibin) = pdf_pcl(ibin)/pdf_conc(ibin)
                 pdf_pir(ibin) = pdf_pir(ibin)/pdf_conc(ibin)
              endif
              pdf_pla(ibin) = pdf_pla(ibin)/pdf_conc(ibin)
              pdf_den(ibin) = pdf_den(ibin)/pdf_conc(ibin)
              pdf_col(ibin) = pdf_col(ibin)/pdf_conc(ibin)
              pdf_acratio(ibin) = pdf_acratio(ibin)/pdf_conc(ibin)
              pdf_adratio(ibin) = pdf_adratio(ibin)/pdf_conc(ibin)
              pdf_imacratio(ibin) = pdf_imacratio(ibin)/pdf_conc(ibin)
              pdf_arearatio(ibin) = pdf_arearatio(ibin)/pdf_conc(ibin)
              pdf_arearatio_sidelook(ibin) = pdf_arearatio_sidelook(ibin)/pdf_conc(ibin)
              pdf_arearatio_total(ibin) = pdf_arearatio_total(ibin)/pdf_conc(ibin)
              pdf_sweptVc(ibin) = pdf_sweptVc(ibin)/pdf_conc(ibin)*1000.0E0
              pdf_sweptVcb(ibin) = pdf_sweptVcb(ibin)/pdf_conc(ibin)*1000.0E0
              pdf_sweptVm(ibin) = pdf_sweptVm(ibin)/pdf_conc(ibin)*1000.0E0
              pdf_1riming_ice(ibin) = pdf_1riming_ice(ibin)/pdf_conc(ibin)
              pdf_1vapor_ice(ibin) = pdf_1vapor_ice(ibin)/pdf_conc(ibin)
              pdf_1eva_ice(ibin) = pdf_1eva_ice(ibin)/pdf_conc(ibin)
              pdf_bulk(ibin) = pdf_bulk(ibin)/pdf_conc(ibin)*1000.0E0
              pdf_split_cap(ibin) = pdf_split_cap(ibin)/pdf_conc(ibin)*1000.0E0
              pdf_split_vent(ibin) = pdf_split_vent(ibin)/pdf_conc(ibin)*1000.0E0
              pdf_split_kinetic(ibin) = pdf_split_kinetic(ibin)/pdf_conc(ibin)*1000.0E0
              pdf_split_area(ibin) = pdf_split_area(ibin)/pdf_conc(ibin)*1000.0E0
              pdf_split_areaiceonly(ibin) = pdf_split_areaiceonly(ibin)/pdf_conc(ibin)*1000.0E0
              pdf_split_deltaVcb(ibin) = pdf_split_deltaVcb(ibin)/pdf_conc(ibin)*1000.0E0
              pdf_split_Nwmass(ibin) = pdf_split_Nwmass(ibin)/pdf_conc(ibin)*1000.0E0
              pdf_rhi(ibin) = pdf_rhi(ibin)/pdf_conc(ibin)
              pdf_T(ibin) = pdf_T(ibin)/pdf_conc(ibin)
           endif
           if (pdf_gconc(ibin) /= 0.0E0) then
              pdf_gctv(ibin) = pdf_gctv(ibin)/pdf_gconc(ibin)
              pdf_gimacratio(ibin) = pdf_gimacratio(ibin)/pdf_gconc(ibin)
              pdf_gsweptVc(ibin) = pdf_gsweptVc(ibin)/pdf_gconc(ibin)*1000.0E0
              pdf_gsweptVm(ibin) = pdf_gsweptVm(ibin)/pdf_gconc(ibin)*1000.0E0
           endif
        enddo
        if (pdf_cry(ibin) /= 0.0E0) then
           pdf_split_areacry(ibin) = pdf_split_areacry(ibin)/pdf_cry(ibin)*1000.0E0
           pdf_split_areaiceonlycry(ibin) = pdf_split_areaiceonlycry(ibin)/pdf_cry(ibin)*1000.0E0
        endif
        if (pdf_cry(ibin) + pdf_rcr(ibin) /= 0.0E0) then
           pdf_split_areacryrcr(ibin) = pdf_split_areacryrcr(ibin)/(pdf_cry(ibin) + pdf_rcr(ibin))*1000.0E0
           pdf_split_areaiceonlycryrcr(ibin) = pdf_split_areaiceonlycryrcr(ibin)/(pdf_cry(ibin) + pdf_rcr(ibin))*1000.0E0
        endif


        ! COMPUTE LIQUID AND ICE SIZE DISTRIBUTIONS
        if (plot_liquid_psd) then

           mean_dNdlogD = 0.0E0
           median_dNdlogD = 0.0E0

           ! liquid size distribution
           do ibin = 1, max_binr

              median_array_size1 = 0

              do i = 1, count_global_median
                 if (dNdlogD(ibin,i) >= -1.0E0) then
                    !if (t == 1 .and. global_var_nliq(i) > 0.0) then
                    !   print *, i, "  global  ", global_var_nliq(i)
                    !endif
                    median_array_size1 = median_array_size1 + 1
                 endif
              enddo

              write(*,*) "liquid", ibin, median_array_size1

              if (median_array_size1 >= 1) then
                 allocate(median_array1(median_array_size1))
                 median_array1 = 0.0E0

                 counter1 = 0
                 do i = 1, count_global_median
                    if (dNdlogD(ibin,i) >= -1.0E0) then
                       counter1 = counter1 + 1
                       median_array1(counter1) = dNdlogD(ibin,i)
                       mean_dNdlogD(ibin) = mean_dNdlogD(ibin) + dNdlogD(ibin,i)
                    endif
                 enddo

                 mean_dNdlogD(ibin) = mean_dNdlogD(ibin)/real(median_array_size1,SP)

                 if (median_array_size1 >= 2) then
                    call quickSorti(median_array1,median_array_size1,1,median_array_size1)
                 endif
              
                 median_dNdlogD(ibin) = median_array1(int(median_array_size1/2))

                 deallocate(median_array1)
              endif

           enddo

        endif

        if (plot_ice_psd) then

           mean_dNidlogD = 0.0E0
           median_dNidlogD = 0.0E0

           ! ice size distribution
           do ibin = 1, max_bini

              median_array_size1 = 0

              do i = 1, count_global_median
                 if (dNidlogD(ibin,i) >= -1.0E0) then
                    !if (t == 1 .and. global_var_nliq(i) > 0.0) then
                    !   print *, i, "  global  ", global_var_nliq(i)
                    !endif
                    median_array_size1 = median_array_size1 + 1
                 endif
              enddo

              write(*,*) "ice", ibin, median_array_size1

              if (median_array_size1 >= 1) then
                 allocate(median_array1(median_array_size1))
                 median_array1 = 0.0E0

                 counter1 = 0
                 do i = 1, count_global_median
                    if (dNidlogD(ibin,i) >= -1.0E0) then
                       counter1 = counter1 + 1
                       median_array1(counter1) = dNidlogD(ibin,i)
                       mean_dNidlogD(ibin) = mean_dNidlogD(ibin) + dNidlogD(ibin,i)
                    endif
                 enddo

                 mean_dNidlogD(ibin) = mean_dNidlogD(ibin)/real(median_array_size1,SP)

                 if (median_array_size1 >= 2) then
                    call quickSorti(median_array1,median_array_size1,1,median_array_size1)
                 endif
              
                 median_dNidlogD(ibin) = median_array1(int(median_array_size1/2))

                 deallocate(median_array1)
              endif

           enddo


        endif ! if plot_psd

        do ibin = 1, max_bini

           median_array_size1 = 0
           
           do i = 1, count_global_median
              if (global_mpdf_vapor_ice(ibin,i) >= -1.0E0) then
                 median_array_size1 = median_array_size1 + 1
              endif
           enddo

           write(*,*) "ice vapor", ibin, median_array_size1

           if (median_array_size1 >= 1) then
              allocate(median_array1(median_array_size1))
              median_array1 = 0.0E0

              counter1 = 0
              do i = 1, count_global_median
                 if (global_mpdf_vapor_ice(ibin,i) >= -1.0E0) then
                    counter1 = counter1 + 1
                    median_array1(counter1) = global_mpdf_vapor_ice(ibin,i)
                 endif
              enddo

              if (median_array_size1 >= 2) then
                 call quickSorti(median_array1,median_array_size1,1,median_array_size1)
              endif
              
              mpdf_vapor_ice(ibin,1) = median_array1(int(median_array_size1/2))
              mpdf_vapor_ice(ibin,2) = median_array1(int(median_array_size1*0.05))
              mpdf_vapor_ice(ibin,3) = median_array1(int(median_array_size1*0.95))

              deallocate(median_array1)

           endif

           median_array_size1 = 0
           
           do i = 1, count_global_median
              if (global_mpdf_riming_ice(ibin,i) >= -1.0E0) then
                 median_array_size1 = median_array_size1 + 1
              endif
           enddo

           write(*,*) "ice riming", ibin, median_array_size1

           if (median_array_size1 >= 1) then
              allocate(median_array1(median_array_size1))
              median_array1 = 0.0E0

              counter1 = 0
              do i = 1, count_global_median
                 if (global_mpdf_riming_ice(ibin,i) >= -1.0E0) then
                    counter1 = counter1 + 1
                    median_array1(counter1) = global_mpdf_riming_ice(ibin,i)
                 endif
              enddo

              if (median_array_size1 >= 2) then
                 call quickSorti(median_array1,median_array_size1,1,median_array_size1)
              endif
              
              mpdf_riming_ice(ibin,1) = median_array1(int(median_array_size1/2))
              mpdf_riming_ice(ibin,2) = median_array1(int(median_array_size1*0.05))
              mpdf_riming_ice(ibin,3) = median_array1(int(median_array_size1*0.95))

              deallocate(median_array1)

           endif

           median_array_size1 = 0
           
           do i = 1, count_global_median
              if (global_mpdf_sweptVcb(ibin,i) >= -1.0E0) then
                 median_array_size1 = median_array_size1 + 1
              endif
           enddo

           write(*,*) "ice sweptVcb", ibin, median_array_size1

           if (median_array_size1 >= 1) then
              allocate(median_array1(median_array_size1))
              median_array1 = 0.0E0

              counter1 = 0
              do i = 1, count_global_median
                 if (global_mpdf_sweptVcb(ibin,i) >= -1.0E0) then
                    counter1 = counter1 + 1
                    median_array1(counter1) = global_mpdf_sweptVcb(ibin,i)
                 endif
              enddo

              if (median_array_size1 >= 2) then
                 call quickSorti(median_array1,median_array_size1,1,median_array_size1)
              endif
              
              mpdf_sweptVcb(ibin,1) = median_array1(int(median_array_size1/2))
              mpdf_sweptVcb(ibin,2) = median_array1(int(median_array_size1*0.05))
              mpdf_sweptVcb(ibin,3) = median_array1(int(median_array_size1*0.95))

              deallocate(median_array1)

           endif

        enddo


        ! output
        if (ct(t) < 10.0E0) then
           write(word,'(I1)') int(ct(t))
        else if (ct(t) < 100.0E0) then
           write(word,'(I2)') int(ct(t))
        else if (ct(t) < 1000.0E0) then
           write(word,'(I3)') int(ct(t))
        else if (ct(t) < 10000.0E0) then
           write(word,'(I4)') int(ct(t))
        else if (ct(t) < 100000.0E0) then
           write(word,'(I5)') int(ct(t))
        endif

        if (plot_liquid_psd) then
           OUTPUT_CONFIG_FNAME = trim(OUTPUT_DIR)//"data_ldistr_t"//trim(word)//".0.dat"
           call register_io(IO_OUTPUT_CONFIG,OUTPUT_CONFIG_FNAME)
           do ibin = 1, max_binr
              write(IO_OUTPUT_CONFIG,'(3ES15.5)') &
                   diameterbin(ibin), &
                   mean_dNdlogD(ibin), &
                   median_dNdlogD(ibin)
           enddo
           call close_io(IO_OUTPUT_CONFIG)
        endif

        if (plot_ice_psd) then
           OUTPUT_CONFIG_FNAME = trim(OUTPUT_DIR)//"data_idistr_t"//trim(word)//".0.dat"
           call register_io(IO_OUTPUT_CONFIG,OUTPUT_CONFIG_FNAME)
           do ibin = 1, max_bini
              write(IO_OUTPUT_CONFIG,'(3ES15.5)') &
                   diameteribin(ibin), &
                   mean_dNidlogD(ibin), &
                   median_dNidlogD(ibin)
           enddo
           call close_io(IO_OUTPUT_CONFIG)
        endif

        OUTPUT_CONFIG_FNAME = trim(OUTPUT_DIR)//"data_mpdf_vapor_ice_t"//trim(word)//".0.dat"
        call register_io(IO_OUTPUT_CONFIG,OUTPUT_CONFIG_FNAME)
        do ibin = 1, max_bini
           write(IO_OUTPUT_CONFIG,'(4ES15.5)') &
                diameteribin(ibin), &
                mpdf_vapor_ice(ibin,1), &
                mpdf_vapor_ice(ibin,2), &
                mpdf_vapor_ice(ibin,3)
        enddo
        call close_io(IO_OUTPUT_CONFIG)

        OUTPUT_CONFIG_FNAME = trim(OUTPUT_DIR)//"data_mpdf_sweptVcb_t"//trim(word)//".0.dat"
        call register_io(IO_OUTPUT_CONFIG,OUTPUT_CONFIG_FNAME)
        do ibin = 1, max_bini
           write(IO_OUTPUT_CONFIG,'(4ES15.5)') &
                diameteribin(ibin), &
                mpdf_sweptVcb(ibin,1), &
                mpdf_sweptVcb(ibin,2), &
                mpdf_sweptVcb(ibin,3)
        enddo
        call close_io(IO_OUTPUT_CONFIG)

        OUTPUT_CONFIG_FNAME = trim(OUTPUT_DIR)//"data_mpdf_riming_ice_t"//trim(word)//".0.dat"
        call register_io(IO_OUTPUT_CONFIG,OUTPUT_CONFIG_FNAME)
        do ibin = 1, max_bini
           write(IO_OUTPUT_CONFIG,'(4ES15.5)') &
                diameteribin(ibin), &
                mpdf_riming_ice(ibin,1), &
                mpdf_riming_ice(ibin,2), &
                mpdf_riming_ice(ibin,3)
        enddo
        call close_io(IO_OUTPUT_CONFIG)

        OUTPUT_CONFIG_FNAME = trim(OUTPUT_DIR)//"data_pdf_mass_t"//trim(word)//".0.dat"
        call register_io(IO_OUTPUT_CONFIG,OUTPUT_CONFIG_FNAME)
        write(IO_OUTPUT_CONFIG,'(20ES15.5)') diameteribin(:)
        do ibin = 1, max_bini
           write(IO_OUTPUT_CONFIG,'(10ES15.5)') pdf_mass(ibin),pdf_crymass(ibin),pdf_rimmass(ibin),pdf_aggmass(ibin),pdf_mtv(ibin),pdf_sweptVm(ibin), pdf_crymd(ibin), pdf_gramd(ibin), pdf_rcrmd(ibin), pdf_cryrcrmd(ibin)
        enddo
        call close_io(IO_OUTPUT_CONFIG)

        OUTPUT_CONFIG_FNAME = trim(OUTPUT_DIR)//"data_pdf_habit_mass_t"//trim(word)//".0.dat"
        call register_io(IO_OUTPUT_CONFIG,OUTPUT_CONFIG_FNAME)
        write(IO_OUTPUT_CONFIG,'(20ES15.5)') diameteribin(:)
        do ibin = 1, max_bini
           write(IO_OUTPUT_CONFIG,'(7ES15.5)') pdf_mass(ibin),pdf_mass_pla(ibin),pdf_mass_den(ibin),pdf_mass_col(ibin),pdf_mass_ppl(ibin),pdf_mass_pcl(ibin),pdf_mass_pir(ibin)
        enddo
        call close_io(IO_OUTPUT_CONFIG)

        OUTPUT_CONFIG_FNAME = trim(OUTPUT_DIR)//"data_pdf_type_mass_t"//trim(word)//".0.dat"
        call register_io(IO_OUTPUT_CONFIG,OUTPUT_CONFIG_FNAME)
        write(IO_OUTPUT_CONFIG,'(20ES15.5)') diameteribin(:)
        do ibin = 1, max_bini
           write(IO_OUTPUT_CONFIG,'(8ES15.5)') pdf_mass(ibin),pdf_mass_cry(ibin),pdf_mass_rcr(ibin),pdf_mass_agg(ibin),pdf_mass_rag(ibin),pdf_mass_gra(ibin), pdf_mfrac_cry(ibin), pdf_mfrac_rim(ibin)
        enddo
        call close_io(IO_OUTPUT_CONFIG)

        OUTPUT_CONFIG_FNAME = trim(OUTPUT_DIR)//"data_pdf_split_t"//trim(word)//".0.dat"
        call register_io(IO_OUTPUT_CONFIG,OUTPUT_CONFIG_FNAME)
        write(IO_OUTPUT_CONFIG,'(20ES15.5)') diameteribin(:)
        do ibin = 1, max_bini
           write(IO_OUTPUT_CONFIG,'(13ES15.5)') pdf_split_cap(ibin),pdf_split_vent(ibin),pdf_split_kinetic(ibin),pdf_split_area(ibin),pdf_split_deltaVcb(ibin),pdf_split_Nwmass(ibin),pdf_rhi(ibin),pdf_T(ibin),pdf_split_areaiceonly(ibin),pdf_split_areacry(ibin),pdf_split_areaiceonlycry(ibin),pdf_split_areacryrcr(ibin),pdf_split_areaiceonlycryrcr(ibin)
        enddo
        call close_io(IO_OUTPUT_CONFIG)

        OUTPUT_CONFIG_FNAME = trim(OUTPUT_DIR)//"data_pdf_conc_t"//trim(word)//".0.dat"
        call register_io(IO_OUTPUT_CONFIG,OUTPUT_CONFIG_FNAME)
        write(IO_OUTPUT_CONFIG,'(20ES15.5)') diameteribin(:)
        if (poly_exist) then
           do ibin = 1, max_bini
              write(IO_OUTPUT_CONFIG,'(23ES15.5)') pdf_conc(ibin), & ! 1
                   pdf_cry(ibin), & ! 2
                   pdf_rcr(ibin), & ! 3
                   pdf_agg(ibin), & ! 4
                   pdf_gra(ibin), & ! 5
                   pdf_rag(ibin), & ! 6
                   pdf_pla(ibin), & ! 7
                   pdf_den(ibin), & ! 8
                   pdf_col(ibin), & ! 9
                   pdf_ppl(ibin), & ! 10
                   pdf_pcl(ibin), & ! 11
                   pdf_pir(ibin), & ! 12
                   pdf_ctv(ibin), & ! 13
                   pdf_acratio(ibin), & ! 14
                   pdf_imacratio(ibin), & ! 15
                   pdf_sweptVc(ibin), & ! 16
                   pdf_count(ibin),pdf_bulk(ibin),pdf_wctv(ibin),pdf_adratio(ibin),pdf_arearatio(ibin),pdf_arearatio_sidelook(ibin),pdf_arearatio_total(ibin)
           enddo
        else
           do ibin = 1, max_bini
              write(IO_OUTPUT_CONFIG,'(20ES15.5)') pdf_conc(ibin), & ! 1
                   pdf_cry(ibin), & ! 2
                   pdf_rcr(ibin), & ! 3
                   pdf_agg(ibin), & ! 4
                   pdf_gra(ibin), & ! 5
                   pdf_rag(ibin), & ! 6
                   pdf_pla(ibin), & ! 7
                   pdf_den(ibin), & ! 8
                   pdf_col(ibin), & ! 9
                   pdf_ctv(ibin), & ! 10
                   pdf_acratio(ibin), & ! 11
                   pdf_imacratio(ibin), & ! 12
                   pdf_sweptVc(ibin), & ! 13
                   pdf_count(ibin),pdf_bulk(ibin),pdf_wctv(ibin),pdf_adratio(ibin),pdf_arearatio(ibin),pdf_arearatio_sidelook(ibin),pdf_arearatio_total(ibin)
           enddo
        endif
        call close_io(IO_OUTPUT_CONFIG)

        OUTPUT_CONFIG_FNAME = trim(OUTPUT_DIR)//"data_pdf_bohm_t"//trim(word)//".0.dat"
        call register_io(IO_OUTPUT_CONFIG,OUTPUT_CONFIG_FNAME)
        write(IO_OUTPUT_CONFIG,'(20ES15.5)') diameteribin(:)
        do ibin = 1, max_bini
           write(IO_OUTPUT_CONFIG,'(8ES15.5)') pdf_conc(ibin),pdf_ctv(ibin),pdf_cbtv(ibin),pdf_RE(ibin),pdf_sweptVc(ibin),pdf_sweptVcb(ibin),pdf_citv(ibin),pdf_sweptVcb_real(ibin)
        enddo
        call close_io(IO_OUTPUT_CONFIG)

        OUTPUT_CONFIG_FNAME = trim(OUTPUT_DIR)//"data_rate_mass_t"//trim(word)//".0.dat"
        call register_io(IO_OUTPUT_CONFIG,OUTPUT_CONFIG_FNAME)
        write(IO_OUTPUT_CONFIG,'(20ES15.5)') diameteribin(:)
        do ibin = 1, max_bini
           write(IO_OUTPUT_CONFIG,'(8ES15.5)') diameteribin(ibin), pdf_conc(ibin), pdf_riming_ice(ibin),pdf_vapor_ice(ibin),pdf_eva_ice(ibin), pdf_1riming_ice(ibin),pdf_1vapor_ice(ibin),pdf_1eva_ice(ibin)
        enddo
        call close_io(IO_OUTPUT_CONFIG)

        OUTPUT_CONFIG_FNAME = trim(OUTPUT_DIR)//"data_rate_liq_mass_t"//trim(word)//".0.dat"
        call register_io(IO_OUTPUT_CONFIG,OUTPUT_CONFIG_FNAME)
        write(IO_OUTPUT_CONFIG,'(40ES15.5)') diameterbin(:)
        do ibin = 1, max_binr
           write(IO_OUTPUT_CONFIG,'(9ES15.5)') diameterbin(ibin),pdf_riming_liq(ibin),pdf_vapor_liq(ibin),pdf_eva_liq(ibin),pdf_conc_liq(ibin),pdf_mass_liq(ibin),pdf_cbtv_liq(ibin),pdf_allvapor_liq(ibin),pdf_alleva_liq(ibin)
        enddo
        call close_io(IO_OUTPUT_CONFIG)

        OUTPUT_CONFIG_FNAME = trim(OUTPUT_DIR)//"data_pdf_gconc_t"//trim(word)//".0.dat"
        call register_io(IO_OUTPUT_CONFIG,OUTPUT_CONFIG_FNAME)
        write(IO_OUTPUT_CONFIG,'(20ES15.5)') diameteribin(:)
        do ibin = 1, max_bini
           write(IO_OUTPUT_CONFIG,'(4ES15.5)') pdf_gconc(ibin),pdf_gctv(ibin),pdf_gimacratio(ibin),pdf_gsweptVc(ibin)
        enddo
        call close_io(IO_OUTPUT_CONFIG)

        OUTPUT_CONFIG_FNAME = trim(OUTPUT_DIR)//"data_pdf_temp_t"//trim(word)//".0.dat"
        call register_io(IO_OUTPUT_CONFIG,OUTPUT_CONFIG_FNAME)
        do i = 1, count_global_median
           if (pdf_temp(i) >= -1.0E0) then
              write(IO_OUTPUT_CONFIG,'(18ES15.5)') pdf_temp(i),pdf_rad(i),pdf_lwc(i),pdf_nliq(i),pdf_wvel(i),pdf_temp_rim(i),pdf_temp_vap(i),pdf_temp_crycon(i),pdf_temp_rcrcon(i),pdf_temp_aggcon(i),pdf_temp_gracon(i),pdf_temp_ragcon(i),pdf_temp_sweptVc(i),pdf_temp_sweptVcb(i),pdf_temp_sweptVcb_real(i),pdf_cbtv_liq(i),pdf_temp_rimNORMliq(i),pdf_temp_imm(i)
           endif
        enddo
        call close_io(IO_OUTPUT_CONFIG)


     endif ! if rank = master


  enddo ! t = 1, nt



  deallocate(FILE_NAME,jobs_index)
  deallocate(fz,fx,cx,cy,cz,ct)
  deallocate(dens,temperature,w,qliq,qhyd,qice,bin_liq_con,bin_liq_ctv,bin_liq_mtv,bin_ice_con)
  deallocate(bin_ice_Dmax,bin_ice_aDmax,bin_ice_mass,bin_ice_magg,bin_ice_mcry,bin_ice_mrim,bin_ice_mfrz,bin_ice_ctv,bin_ice_mtv)
  deallocate(bin_ice_alen,bin_ice_clen,bin_ice_dlen)
  deallocate(ice_mass_rim,ice_mass_imm,ice_mass_vap,ice_mass_eva)
  if (poly_exist) then
     deallocate(bin_ice_nex,bin_ice_aglen,bin_ice_cglen)
  endif


  call MPI_FINALIZE( ierr )

contains

  subroutine check(status)
    integer, intent ( in) :: status
    
    if(status /= nf90_noerr) then 
      print *, trim(nf90_strerror(status))
      call PRC_abort
      stop "Stopped"
    end if
  end subroutine check  

  subroutine register_io(input,file_name,namelist_file)
    implicit none

    integer, intent(inout) :: input
    character(len=*), intent(in) :: file_name
    logical, optional, intent(in) :: namelist_file
    integer :: i
    logical :: i_opened


    if (current_io_number > MAX_IO_NUMBER) then
       write(*,*) irank, "IO number exceeded max number", current_io_number, MAX_IO_NUMBER
       call PRC_abort
    endif
    do i = 1, MAX_IO_NUMBER
       inquire(i,OPENED=i_opened)
       if (.not. i_opened) then
          input = i
          if (present(namelist_file)) then
             if (namelist_file) then
                open(input,file=trim(file_name),status='old', &
                     form="formatted", delim='apostrophe', iostat=ierr )
             endif
          else
             open(input,file=trim(file_name),status='unknown',iostat=ierr)
          endif
          if ( ierr /= 0 ) then
             write (*, *) "ERROR: Fail to open the configuration file :", trim(INPUT_CONFIG_FNAME)
             call PRC_abort
          endif

          current_io_number = current_io_number + 1
          exit
       endif
    enddo

  end subroutine register_io


  subroutine close_io(input)
    implicit none

    integer, intent(inout) :: input
    integer :: i
    logical :: i_opened


    do i = 1, MAX_IO_NUMBER
       if (i == input) then
          inquire(i,OPENED=i_opened)
          if (i_opened) then
             close(input)
             current_io_number = current_io_number - 1
             return
          endif
       endif
    enddo

    write(*,*) "WARNING IO: ", i, current_io_number
    call PRC_abort

  end subroutine close_io

  subroutine PRC_abort
    implicit none
    if ( PRC_mpi_alive ) then
       ! tentative approach; input "PRC_UNIVERSAL_COMM_WORLD".
       !call MPI_COMM_CALL_ERRHANDLER(MPI_COMM_WORLD,PRC_ABORT_code,ierr)
       write(*,*) "abort operation in rank", irank
       call MPI_FINALIZE( ierr )
    endif

    stop
  end subroutine PRC_abort


  subroutine quickSorti(inputArray,size,start,end)
    implicit none
    integer, intent(in) :: start, end
    integer, intent(in) :: size
    real(SP), intent(inout) :: inputArray(size)

    integer :: temp, temp1, outer_loop, k, kk, kk_pivot, segmentNumber
    integer :: i, pivot_array_size, pivot_index
    integer :: pivots(size), pivot_size, pivot_array(size)
    logical :: loop_hdl

    if (start >= end) then
       return
    endif
    
    loop_hdl = .true.

    temp = end - start + 1
    outer_loop = int(temp/2) + 1
    
    pivot_size = 1
    pivots(pivot_size) = start
    pivot_size = pivot_size + 1
    pivots(pivot_size) = end
    k = 1
    segmentNumber = 1
    do while (k <= outer_loop)
       kk = 1
       temp1 = 0
       kk_pivot = 2
       pivot_array_size = 1
       do while (kk <= segmentNumber)
          call quickPartitioni(inputArray,size,pivot_index,pivots(kk_pivot-1),pivots(kk_pivot))
          if (pivot_index-1 > pivots(kk_pivot-1)) then
             pivot_array(pivot_array_size) = pivots(kk_pivot-1)
             pivot_array_size = pivot_array_size + 1
             pivot_array(pivot_array_size) = pivot_index-1
             pivot_array_size = pivot_array_size + 1
             temp1 = temp1 + 1
          endif
          if (pivot_index+1 < pivots(kk_pivot)) then
             pivot_array(pivot_array_size) = pivot_index+1
             pivot_array_size = pivot_array_size + 1
             pivot_array(pivot_array_size) = pivots(kk_pivot)
             pivot_array_size = pivot_array_size + 1
             temp1 = temp1 + 1
          endif
          kk = kk + 1
          kk_pivot = kk_pivot + 2
       enddo
        
       segmentNumber = temp1
       
       if (temp1 == 0) then
          exit
       endif
       
       i = 1
       do while (i <= pivot_array_size)
          pivots(i) = pivot_array(i)
          i = i + 1
       enddo
       k = k + 1
    enddo
  end subroutine quickSorti
    
  subroutine quickPartitioni(inputArray,size,output,start,end)
    implicit none
    integer, intent(in) :: start, end
    integer, intent(in) :: size
    real(SP), intent(inout) :: inputArray(size)
    integer, intent(out) :: output
    
    real(SP) :: pivot, temp
    integer :: k, i
    
    if (start >= end) then
       output = start
       return
    endif
        
    k = start
    pivot = inputArray(end)
    i = start
    do while (k < end)
       if (inputArray(k) < pivot) then
          temp = inputArray(k)
          inputArray(k) = inputArray(i)
          inputArray(i) = temp
          i = i + 1
       endif
       k = k + 1
    enddo
    inputArray(end) = inputArray(i)
    inputArray(i) = pivot
    output = i
    return

  end subroutine quickPartitioni


  subroutine Iice_habit(ice_type,icemass,crymass,aggmass,rimmass)
    implicit none
    real(SP), intent(in) :: icemass, crymass, aggmass, rimmass
    integer, intent(out) :: ice_type

    ice_type = -1
    if (rimmass > 0.1 * icemass) then
       if (aggmass < 0.1 * icemass) then
          if (rimmass < crymass) then
             ice_type = 5 ! rimed crystals
          else
             ice_type = 4 ! graupel
          endif
       else
          if (rimmass > aggmass) then
             ice_type = 4 ! graupel
          else
             ice_type = 3 ! rimed aggregates
          endif
       endif
    else
       if (aggmass < 0.1 * icemass) then
          ice_type = 2 ! pristine crystals
       else
          ice_type = 1 ! aggregates
       endif
    endif
    return

  end subroutine Iice_habit

  subroutine Iice_shape(habit,a_axis,c_axis,d_axis)
    implicit none
    real(SP), intent(in) :: a_axis, c_axis, d_axis
    integer, intent(out) :: habit

    habit = -1
    if (c_axis > a_axis) then
       habit = 3 ! columnar crystals
    else
       if (d_axis > 2.0/3.0 * a_axis) then
          habit = 2 ! dendrites
       else
          habit = 1 ! hexagonal plates
       endif
    endif
  end subroutine Iice_shape

  subroutine Iice_shapePoly(habit,a_axis,c_axis,d_axis,ag_axis,cg_axis,ex_cry)
    implicit none
    real(SP), intent(in) :: a_axis, c_axis, d_axis, ag_axis, cg_axis, ex_cry
    integer, intent(out) :: habit

    real(SP) :: temp

    habit = -1
    if (ex_cry < 0.5) then
       if (c_axis > a_axis) then
          habit = 3 ! columnar crystals
       else
          if (d_axis > 2.0/3.0 * a_axis) then
             habit = 2 ! dendrites
          else
             habit = 1 ! hexagonal plates
          endif
       endif
    else
       if (a_axis == 0.0 .and. c_axis == 0.0) then
          habit = -1
       else if (a_axis == 0.0) then
          temp = 1.0
       else if (c_axis == 0.0) then
          temp = -1.0
       else
          temp = ag_axis/a_axis - cg_axis/c_axis
       endif
       if (temp >= 0.5) then
          habit = 4 ! planar polycrystals
       else if (temp <= -0.5) then
          habit = 5 ! columnar polycrystals
       else
          habit = 6 ! irregular polycrystals
       endif
    endif

    return
  end subroutine Iice_shapePoly

  subroutine Iice_shape_mod(habit,a_axis,c_axis,d_axis)
    implicit none
    real(SP), intent(in) :: a_axis, c_axis, d_axis
    integer, intent(out) :: habit

    habit = -1
    if (c_axis > a_axis*1.01E0) then
       habit = 3 ! columnar crystals
    else
       if (d_axis > 2.0/3.0 * a_axis) then
          habit = 2 ! dendrites
       else
          habit = 1 ! hexagonal plates
       endif
    endif
  end subroutine Iice_shape_mod

  subroutine Iice_shapePoly_mod(habit,a_axis,c_axis,d_axis,ag_axis,cg_axis,ex_cry)
    implicit none
    real(SP), intent(in) :: a_axis, c_axis, d_axis, ag_axis, cg_axis, ex_cry
    integer, intent(out) :: habit

    real(SP) :: temp

    habit = -1
    if (ex_cry < 0.5) then
       if (c_axis > a_axis*1.01E0) then
          habit = 3 ! columnar crystals
       else
          if (d_axis > 2.0/3.0 * a_axis) then
             habit = 2 ! dendrites
          else
             habit = 1 ! hexagonal plates
          endif
       endif
    else
       if (a_axis == 0.0 .and. c_axis == 0.0) then
          habit = -1
       else if (a_axis == 0.0) then
          temp = 1.0
       else if (c_axis == 0.0) then
          temp = -1.0
       else
          temp = ag_axis/a_axis - cg_axis/c_axis
       endif
       if (temp >= 0.5) then
          habit = 4 ! planar polycrystals
       else if (temp <= -0.5) then
          habit = 5 ! columnar polycrystals
       else
          habit = 6 ! irregular polycrystals
       endif
    endif

    return
  end subroutine Iice_shapePoly_mod

  subroutine convert_shtype(output,habit)
    implicit none
    integer, intent(in) :: habit
    integer, intent(out) :: output
    
    if (habit == 2) then
       output = 1
    else if (habit == 5) then
       output = 2
    else if (habit == 1) then
       output = 3
    else if (habit == 3) then
       output = 4
    else if (habit == 4) then
       output = 5
    endif
    return
  end subroutine convert_shtype

  subroutine convert_habit(output,habit)
    implicit none
    integer, intent(in) :: habit
    integer, intent(out) :: output

    if (habit == 1) then
       output = 1
    else if (habit == 2) then
       output = 2
    else if (habit == 3) then
       output = 3
    else if (habit == 4) then
       output = 5
    else if (habit == 5) then
       output = 4
    else
       output = 6
    endif
    return
  end subroutine convert_habit

  function isign(integer_number) result(output)
    implicit none
    integer, intent(in) :: integer_number
    integer :: output

    if (integer_number >= 0) then
       output = 1
    else
       output = -1
    endif
  end function isign

  function dsign(float_number) result (output)
    implicit none
    real(SP), intent(in) :: float_number
    real(SP) :: output

    if (float_number >= 0.0E0) then
       output = 1.0E0
    else
       output = -1.0E0
    endif
  end function dsign

  function ifloor(float_number) result(output)
    implicit none
    real(SP), intent(in) :: float_number
    integer :: output

    if (float_number >= 0.0) then
       output = int(float_number)
    else
       output = int(float_number-1.0E0)
    endif
  end function ifloor

  subroutine get_total_sfc_area( output, sh_type, habit, is_mod, alen, clen, psi_ic, r, mode )
    ! sh 1 = pristine crystal
    ! sh 2 = rimed crystal
    ! sh 3 = aggregate
    ! sh 4 = rimed aggregate
    ! sh 5 = graupel
    ! ha 1, 2, 3 = plate, dendrite, column
    ! ha 4, 5, 6 = columnar, planar, irregular
    implicit none
    integer, intent(in) :: sh_type, habit, is_mod, mode
    real(SP), intent(in) :: alen, clen, psi_ic, r
    real(SP), intent(out) :: output

    real(SP) :: C_ros
    integer :: i_mode_eq1, i_sh_le2, i_habit_eq4, i_ismod2_eq1, i_agtc, i_habit_le3, &
               i_aeqc, i_em1_le0
    real(SP) :: e1, e2, A_a, v_ros, v_cir, v_obl_1, v_obl_2, v_prl, v_hex, xx


    ! +++ gives total surface area +++
    ! use Chiruta and Wang (2003)

    C_ros = 4.532390052

    i_mode_eq1 = max(0.0E0,(-isign(mode-1)*max(0,mode-1)+1.0E0))* &
                 max(0.0E0,(-isign(mode-1)*min(0,mode-1)+1.0E0))

    i_sh_le2 = 0.5E0*(1.0E0-real(isign(sh_type-3),SP))

    i_habit_eq4 = max(0.0E0,(-isign(habit-4)*max(0,habit-4)+1.0E0))* &
                  max(0.0E0,(-isign(habit-4)*min(0,habit-4)+1.0E0))

    i_ismod2_eq1 = max(0.0E0,(-isign(is_mod-1)*max(0,is_mod-1)+1.0E0))* &
                   max(0.0E0,(-isign(is_mod-1)*min(0,is_mod-1)+1.0E0))

    i_agtc = 0.5E0*(1.0E0 - dsign(clen-alen))
    i_aeqc = max(0,ifloor(-dsign(alen-clen)*max(0.0,alen-clen)+1.0E0))* &
             max(0,ifloor(-dsign(alen-clen)*min(0.0,alen-clen)+1.0E0))

    i_habit_le3 = 0.5E0*(1.0E0-real(isign(habit-4),SP))

    e1 = sqrt(1.0E0 - min(1.0E0,(clen*clen)/(alen*alen)))
    e2 = sqrt(1.0E0 - min(1.0E0,(alen*alen)/(clen*clen)))
    i_em1_le0 = 0.5E0*(1.0E0-dsign(e1 - 1.0E0 - 1.0E-30))

    A_a=1.5*sq_three*(1.0-psi_ic)*alen*alen
    xx = 0.5*alen*sqrt(1.0+3.0*psi_ic*psi_ic)

    v_ros=C_ros*r*r
    v_cir=4.0E0*pi*alen*alen
    v_obl_1=2.0E0*pi*alen*alen
    v_obl_2=2.0E0*pi*alen*alen + pi*clen*clen * log((1.0E0+e1)/max(1.0E-30,(1.0E0-e1)))/max(1.0E-30,e1)
    v_prl=2.0E0*pi*alen*alen+2.0E0*pi*alen*clen*asin(e2)/max(1.0E-30,e2)
    v_hex=2.0E0*A_a + 24.0E0*clen*xx

    !if (abs(i_sh_le2*i_habit_eq4*i_ismod2_eq1 - 1.0) < 1.e-6):
    !    print("error, v_ros selected")
    !    print(i_sh_le2, i_habit_eq4, i_ismod2_eq1)
    !    sys.exit(0)

    output = real(i_mode_eq1,SP) * &
            (real(i_sh_le2*i_habit_eq4*i_ismod2_eq1,SP) * &
             v_ros + &
             (1.0E0-real(i_sh_le2*i_habit_eq4*i_ismod2_eq1,SP)) * &
             ( real(i_aeqc,SP)*v_cir + &
               real(i_agtc,SP) * &
               (real(i_em1_le0,SP)*v_obl_1+(1.0E0-real(i_em1_le0,SP))*v_obl_2) + &
               (1.0E0-real(i_aeqc,SP))*(1.0E0-real(i_agtc,SP)) * &
               v_prl ) &
         ) + &
            (1.0E0-real(i_mode_eq1,SP)) * &
            ( real(i_sh_le2,SP) * &
              (  real(i_habit_le3*i_ismod2_eq1,SP)*v_hex + &
                 real(i_habit_eq4*i_ismod2_eq1,SP)*v_ros + &
                 (1.0E0-real(i_habit_le3*i_ismod2_eq1,SP))*(1.0E0-real(i_habit_eq4*i_ismod2_eq1,SP)) * &
                 ( real(i_aeqc,SP)*v_cir + real(i_agtc,SP)* &
                   (real(i_em1_le0,SP)*v_obl_1+(1.0E0-real(i_em1_le0,SP))*v_obl_2) + &
                   (1.0E0-real(i_aeqc,SP))*(1.0E0-real(i_agtc,SP))* &
                   v_prl &
               )) + &
              (1.0E0-real(i_sh_le2,SP))* &
              ( real(i_aeqc,SP)*v_cir + &
                real(i_agtc,SP)* &
                (real(i_em1_le0,SP)*v_obl_1+(1.0E0-real(i_em1_le0,SP))*v_obl_2) + &
                (1.0E0-real(i_aeqc,SP))*(1.0E0-real(i_agtc,SP))* &
                v_prl))

  end subroutine get_total_sfc_area


  subroutine get_effect_area( output, sh_type2, habit, is_mod, alen, clen, psi_ic, r, mode)
    ! sh 1 = pristine crystal
    ! sh 2 = rimed crystal
    ! sh 3 = aggregate
    ! sh 4 = rimed aggregate
    ! sh 5 = graupel
    ! ha 1, 2, 3 = plate, dendrite, column
    ! ha 4, 5, 6 = columnar, planar, irregular
    implicit none
    integer, intent(in) :: sh_type2, habit, is_mod, mode
    real(SP), intent(in) :: alen, clen, psi_ic, r
    real(SP), intent(out) :: output

    real(SP) :: C_ros
    integer :: i_mode_eq1, i_sh_le2, i_habit_eq4, i_ismod2_eq1, i_agtc, i_habit_le3
    real(SP) :: v_ros, v_obl, v_prl, v_hex, v_col

    ! +++ area projected from the top of the basal plane +++
    ! use Chiruta and Wang (2003) for 4 lobes
    C_ros=0.899

    i_mode_eq1=max(0.0E0,(-isign(mode-1)*max(0,mode-1)+1.0E0))* &
                max(0.0E0,(-isign(mode-1)*min(0,mode-1)+1.0E0))

    i_sh_le2=0.5E0*(1.0E0-real(isign(sh_type2-3),SP))

    i_habit_eq4=max(0.0E0,(-isign(habit-4)*max(0,habit-4)+1.0E0))* &
                 max(0.0E0,(-isign(habit-4)*min(0,habit-4)+1.0E0))

    i_ismod2_eq1=max(0.0E0,(-isign(is_mod-1)*max(0,is_mod-1)+1.0E0))* &
                  max(0.0E0,(-isign(is_mod-1)*min(0,is_mod-1)+1.0E0))

    i_agtc=0.5E0*(1.0E0-dsign(clen-alen))
    !i_agtc=1.0E0
    !if (i_sh_le2 == 1) then
    !   write(*,*) i_agtc
    !end if

    i_habit_le3=0.5E0*(1.0E0-real(isign(habit-4)))

    v_ros=C_ros*r*r
    v_obl=pi*alen*alen
    v_prl=4.0E0*clen*alen
    v_hex=1.5E0*sq_three*(1.0E0-psi_ic)*alen*alen
    v_col=4.0E0*clen*alen

    !if (abs(i_sh_le2*i_habit_eq4*i_ismod2_eq1 - 1.0E0) < 1.e-6):
    !    print("error, v_ros selected")
    !    print(i_sh_le2, i_habit_eq4, i_ismod2_eq1)
    !    sys.exit(0)

    output = real(i_mode_eq1,SP)* &
          ( &
            real(i_sh_le2*i_habit_eq4*i_ismod2_eq1,SP)*v_ros + &
            (1.0E0-real(i_sh_le2*i_habit_eq4*i_ismod2_eq1,SP))* &
            (real(i_agtc,SP)*v_obl+(1.0E0-real(i_agtc,SP))*v_prl)  &
        )+ &
          (1.0E0-real(i_mode_eq1,SP))* &
          ( &
            real(i_sh_le2,SP)* &
            ( &
              real(i_habit_le3*i_ismod2_eq1,SP)* &
              (real(i_agtc,SP)*v_hex+ &
               (1.0E0-real(i_agtc,SP))*v_col)+ &
              real(i_habit_eq4*i_ismod2_eq1,SP)*v_ros+ &
              (1.0E0-real(i_habit_le3*i_ismod2_eq1,SP))*(1.0E0-real(i_habit_eq4*i_ismod2_eq1,SP))* &
              (real(i_agtc,SP)*v_obl+(1.0E0-real(i_agtc,SP))*v_prl) &
          ) + &
            (1.0E0-real(i_sh_le2,SP))* &
            (real(i_agtc,SP)*v_obl+(1.0E0-real(i_agtc,SP))*v_prl) &
        )

  !if (output < 0.0E0) then
  !   write(*,'(A,8ES15.5)') "minus", v_ros, v_obl, v_prl, v_hex, v_col, clen, alen, psi_ic
  !endif
    
  end subroutine get_effect_area

  subroutine get_effect_area_sph( output, sh_type2, habit, is_mod, alen, clen, psi_ic, r, mode)
    ! sh 1 = pristine crystal
    ! sh 2 = rimed crystal
    ! sh 3 = aggregate
    ! sh 4 = rimed aggregate
    ! sh 5 = graupel
    ! ha 1, 2, 3 = plate, dendrite, column
    ! ha 4, 5, 6 = columnar, planar, irregular
    implicit none
    integer, intent(in) :: sh_type2, habit, is_mod, mode
    real(SP), intent(in) :: alen, clen, psi_ic, r
    real(SP), intent(out) :: output

    real(SP) :: C_ros
    integer :: i_mode_eq1, i_sh_le2, i_habit_eq4, i_ismod2_eq1, i_agtc, i_habit_le3
    real(SP) :: v_ros, v_obl, v_prl, v_hex, v_col

    ! +++ area projected from the top of the basal plane +++
    ! use Chiruta and Wang (2003) for 4 lobes
    C_ros=0.899

    i_mode_eq1=max(0.0E0,(-isign(mode-1)*max(0,mode-1)+1.0E0))* &
                max(0.0E0,(-isign(mode-1)*min(0,mode-1)+1.0E0))

    i_sh_le2=0.5E0*(1.0E0-real(isign(sh_type2-3),SP))

    i_habit_eq4=max(0.0E0,(-isign(habit-4)*max(0,habit-4)+1.0E0))* &
                 max(0.0E0,(-isign(habit-4)*min(0,habit-4)+1.0E0))

    i_ismod2_eq1=max(0.0E0,(-isign(is_mod-1)*max(0,is_mod-1)+1.0E0))* &
                  max(0.0E0,(-isign(is_mod-1)*min(0,is_mod-1)+1.0E0))

    i_agtc=0.5E0*(1.0E0-dsign(clen-alen))
    !i_agtc=1.0E0
    !if (i_sh_le2 == 1) then
    !   write(*,*) i_agtc
    !end if

    i_habit_le3=0.5E0*(1.0E0-real(isign(habit-4)))

    v_ros=C_ros*r*r
    v_obl=pi*alen*alen
    v_prl=pi*alen*alen
    v_hex=1.5E0*sq_three*(1.0E0-psi_ic)*alen*alen
    v_col=1.5E0*sq_three*(1.0E0-psi_ic)*alen*alen

    !if (abs(i_sh_le2*i_habit_eq4*i_ismod2_eq1 - 1.0E0) < 1.e-6):
    !    print("error, v_ros selected")
    !    print(i_sh_le2, i_habit_eq4, i_ismod2_eq1)
    !    sys.exit(0)

    output = real(i_mode_eq1,SP)* &
          ( &
            real(i_sh_le2*i_habit_eq4*i_ismod2_eq1,SP)*v_ros + &
            (1.0E0-real(i_sh_le2*i_habit_eq4*i_ismod2_eq1,SP))* &
            (real(i_agtc,SP)*v_obl+(1.0E0-real(i_agtc,SP))*v_prl)  &
        )+ &
          (1.0E0-real(i_mode_eq1,SP))* &
          ( &
            real(i_sh_le2,SP)* &
            ( &
              real(i_habit_le3*i_ismod2_eq1,SP)* &
              (real(i_agtc,SP)*v_hex+ &
               (1.0E0-real(i_agtc,SP))*v_col)+ &
              real(i_habit_eq4*i_ismod2_eq1,SP)*v_ros+ &
              (1.0E0-real(i_habit_le3*i_ismod2_eq1,SP))*(1.0E0-real(i_habit_eq4*i_ismod2_eq1,SP))* &
              (real(i_agtc,SP)*v_obl+(1.0E0-real(i_agtc,SP))*v_prl) &
          ) + &
            (1.0E0-real(i_sh_le2,SP))* &
            (real(i_agtc,SP)*v_obl+(1.0E0-real(i_agtc,SP))*v_prl) &
        )

  !if (output < 0.0E0) then
  !   write(*,'(A,8ES15.5)') "minus", v_ros, v_obl, v_prl, v_hex, v_col, clen, alen, psi_ic
  !endif
    
  end subroutine get_effect_area_sph

  subroutine get_effect_area_sidelook( output, sh_type2, habit, is_mod, alen, clen, psi_ic, r, mode)
    ! sh 1 = pristine crystal
    ! sh 2 = rimed crystal
    ! sh 3 = aggregate
    ! sh 4 = rimed aggregate
    ! sh 5 = graupel
    ! ha 1, 2, 3 = plate, dendrite, column
    ! ha 4, 5, 6 = columnar, planar, irregular
    implicit none
    integer, intent(in) :: sh_type2, habit, is_mod, mode
    real(SP), intent(in) :: alen, clen, psi_ic, r
    real(SP), intent(out) :: output

    real(SP) :: C_ros
    integer :: i_mode_eq1, i_sh_le2, i_habit_eq4, i_ismod2_eq1, i_agtc, i_habit_le3
    real(SP) :: v_ros, v_obl, v_prl, v_hex, v_col

    ! +++ area projected from the top of the basal plane +++
    ! use Chiruta and Wang (2003) for 4 lobes
    C_ros=0.899

    i_mode_eq1=max(0.0E0,(-isign(mode-1)*max(0,mode-1)+1.0E0))* &
                max(0.0E0,(-isign(mode-1)*min(0,mode-1)+1.0E0))

    i_sh_le2=0.5E0*(1.0E0-real(isign(sh_type2-3),SP))

    i_habit_eq4=max(0.0E0,(-isign(habit-4)*max(0,habit-4)+1.0E0))* &
                 max(0.0E0,(-isign(habit-4)*min(0,habit-4)+1.0E0))

    i_ismod2_eq1=max(0.0E0,(-isign(is_mod-1)*max(0,is_mod-1)+1.0E0))* &
                  max(0.0E0,(-isign(is_mod-1)*min(0,is_mod-1)+1.0E0))

    i_agtc=0.5E0*(1.0E0-dsign(clen-alen))
    !i_agtc=1.0E0
    !if (i_sh_le2 == 1) then
    !   write(*,*) i_agtc
    !end if

    i_habit_le3=0.5E0*(1.0E0-real(isign(habit-4)))

    v_ros=C_ros*r*r
    v_obl=pi*clen*alen!pi*alen*alen
    v_prl=pi*clen*alen
    v_hex=4.0E0*clen*alen!1.5E0*sq_three*(1.0E0-psi_ic)*alen*alen
    v_col=4.0E0*clen*alen

    !if (abs(i_sh_le2*i_habit_eq4*i_ismod2_eq1 - 1.0E0) < 1.e-6):
    !    print("error, v_ros selected")
    !    print(i_sh_le2, i_habit_eq4, i_ismod2_eq1)
    !    sys.exit(0)

    output = real(i_mode_eq1,SP)* &
          ( &
            real(i_sh_le2*i_habit_eq4*i_ismod2_eq1,SP)*v_ros + &
            (1.0E0-real(i_sh_le2*i_habit_eq4*i_ismod2_eq1,SP))* &
            (real(i_agtc,SP)*v_obl+(1.0E0-real(i_agtc,SP))*v_prl)  &
        )+ &
          (1.0E0-real(i_mode_eq1,SP))* &
          ( &
            real(i_sh_le2,SP)* &
            ( &
              real(i_habit_le3*i_ismod2_eq1,SP)* &
              (real(i_agtc,SP)*v_hex+ &
               (1.0E0-real(i_agtc,SP))*v_col)+ &
              real(i_habit_eq4*i_ismod2_eq1,SP)*v_ros+ &
              (1.0E0-real(i_habit_le3*i_ismod2_eq1,SP))*(1.0E0-real(i_habit_eq4*i_ismod2_eq1,SP))* &
              (real(i_agtc,SP)*v_obl+(1.0E0-real(i_agtc,SP))*v_prl) &
          ) + &
            (1.0E0-real(i_sh_le2,SP))* &
            (real(i_agtc,SP)*v_obl+(1.0E0-real(i_agtc,SP))*v_prl) &
        )

  !if (output < 0.0E0) then
  !   write(*,'(A,8ES15.5)') "minus", v_ros, v_obl, v_prl, v_hex, v_col, clen, alen, psi_ic
  !endif
    
  end subroutine get_effect_area_sidelook

  subroutine get_circum_area( output, sh_type2, habit, is_mod, alen, clen, psi_ic, r, mode )
    ! sh 1 = pristine crystal
    ! sh 2 = rimed crystal
    ! sh 3 = aggregate
    ! sh 4 = rimed aggregate
    ! sh 5 = graupel
    ! ha 1, 2, 3 = plate, dendrite, column
    ! ha 4, 5, 6 = columnar, planar, irregular
    implicit none
    integer, intent(in) :: sh_type2, habit, is_mod, mode
    real(SP), intent(in) :: alen, clen, psi_ic, r
    real(SP), intent(out) :: output

    integer :: i_mode_eq1, i_sh_le2, i_habit_eq4, i_ismod2_eq1, i_agtc
    real(SP) :: v_ros, v_obl, v_prl

    ! +++ area projected from the top of the basal plane +++

    i_mode_eq1=max(0.0E0,(-isign(mode-1)*max(0,mode-1)+1.0E0))* &
                max(0.0E0,(-isign(mode-1)*min(0,mode-1)+1.0E0))
    i_sh_le2=0.5E0*(1.0E0-real(isign(sh_type2-3),SP))
    i_habit_eq4=max(0.0E0,(-isign(habit-4)*max(0,habit-4)+1.0E0))* &
                 max(0.0E0,(-isign(habit-4)*min(0,habit-4)+1.0E0))
    i_ismod2_eq1=max(0.0E0,(-isign(is_mod-1)*max(0,is_mod-1)+1.0E0))* &
                  max(0.0E0,(-isign(is_mod-1)*min(0,is_mod-1)+1.0E0))
    i_agtc=0.5E0*(1.0E0-dsign(clen-alen))

    v_ros=pi*r*r
    v_obl=pi*alen*alen
    v_prl=pi*clen*alen

    !if (abs(i_sh_le2*i_habit_eq4*i_ismod2_eq1 - 1.0E0) < 1.e-6):
    !    print("error, v_ros selected")
    !    print(i_sh_le2, i_habit_eq4, i_ismod2_eq1)
    !    sys.exit(0)

    output=real(i_sh_le2*i_habit_eq4*i_ismod2_eq1,SP)*v_ros+ &
         (1.0E0-real(i_sh_le2*i_habit_eq4*i_ismod2_eq1,SP))* &
         (real(i_agtc,SP)*v_obl+(1.0E0-real(i_agtc,SP))*v_prl)

  end subroutine get_circum_area

  subroutine get_perimeter( output, sh_type, habit, is_mod, alen, clen, a_ic, d_ic, r, mode )
    ! sh 1 = pristine crystal
    ! sh 2 = rimed crystal
    ! sh 3 = aggregate
    ! sh 4 = rimed aggregate
    ! sh 5 = graupel
    ! ha 1, 2, 3 = plate, dendrite, column
    ! ha 4, 5, 6 = columnar, planar, irregular
    implicit none
    integer, intent(in) :: sh_type, habit, is_mod, mode
    real(SP), intent(in) :: alen, clen, a_ic, d_ic, r
    real(SP), intent(out) :: output

    real(SP) :: C_ros!=1.06655585 ! sens test ros original 2.1331117, changed to 1.06655585
    integer :: i_mode_eq1, i_sh_le2, i_habit_eq4, i_ismod2_eq1, i_agtc, i_habit_le3
    real(SP) :: v_ros, v_obl, v_prl, v_hex, v_col, xx

    ! +++ perimeter normal to the flow +++
    ! use Chiruta and Wang (2003)
    C_ros=2.1331117

    i_mode_eq1=max(0.0E0,(-isign(mode-1)*max(0,mode-1)+1.0E0))* &
                max(0.0E0,(-isign(mode-1)*min(0,mode-1)+1.0E0))

    i_sh_le2=0.5E0*(1.0E0-real(isign(sh_type-3),SP))

    i_habit_eq4=max(0.0E0,(-isign(habit-4)*max(0,habit-4)+1.0E0))* &
                 max(0.0E0,(-isign(habit-4)*min(0,habit-4)+1.0E0))

    i_ismod2_eq1=max(0.0E0,(-isign(is_mod-1)*max(0,is_mod-1)+1.0E0))* &
                  max(0.0E0,(-isign(is_mod-1)*min(0,is_mod-1)+1.0E0))

    i_agtc=0.5E0*(1.0E0-dsign(clen-alen))

    i_habit_le3=0.5E0*(1.0E0-real(isign(habit-4),SP))

    xx = sqrt( 0.25E0*a_ic*a_ic + d_ic*d_ic - a_ic*d_ic/2.0E0 )

    v_ros=C_ros*r
    v_obl=2.0E0*pi*alen
    v_prl=pi*(3.0E0*(alen+clen)- &
         sqrt((3.0E0*alen + clen)*(alen + 3.0E0*clen)))
    v_hex=12.0E0*xx
    v_col=4.0E0*(clen+alen)

    !if (abs(i_sh_le2*i_habit_eq4*i_ismod2_eq1 - 1.0E0) < 1.e-6):
    !    print("error, v_ros selected")
    !    print(i_sh_le2, i_habit_eq4, i_ismod2_eq1)
    !    sys.exit(0)

    output = real(i_mode_eq1,SP)* &
        (  real(i_sh_le2*i_habit_eq4*i_ismod2_eq1,SP)*v_ros+ &
           (1.0E0-real(i_sh_le2*i_habit_eq4*i_ismod2_eq1,SP))* &
           ( real(i_agtc,SP)*v_obl+(1.0E0-real(i_agtc,SP))*v_prl) ) &
        + &
        (1.0E0-real(i_mode_eq1,SP))* &
        ( real(i_sh_le2,SP)* &
          ( real(i_habit_le3*i_ismod2_eq1,SP)* &
            ( real(i_agtc,SP)*v_hex+(1.0E0-real(i_agtc,SP))*v_col) + &
            real(i_habit_eq4*i_ismod2_eq1,SP)*v_ros + &
            (1.0E0-real(i_habit_le3*i_ismod2_eq1,SP))*(1.0E0-real(i_habit_eq4*i_ismod2_eq1,SP))* &
            ( real(i_agtc,SP)*v_obl+(1.0E0-real(i_agtc,SP))*v_prl) ) + &
          (1.0E0-real(i_sh_le2,SP))* &
          ( real(i_agtc,SP)*v_obl+(1.0E0-real(i_agtc,SP))*v_prl) )

  end subroutine


  subroutine get_vip(output,is_mod,phi_cs,semi_aip)
    implicit none
    integer, intent(in) :: is_mod
    real(SP), intent(in) :: phi_cs, semi_aip
    real(SP), intent(out) :: output

    integer :: i_ismod_eq1

    i_ismod_eq1=max(0.0E0,(-isign(is_mod-1)*max(0,is_mod-1)+1.0E0))* &
                 max(0.0E0,(-isign(is_mod-1)*min(0,is_mod-1)+1.0E0))

    output = real(i_ismod_eq1,SP)*2.0*pi*phi_cs*semi_aip*semi_aip*semi_aip + &
           (1.0E0-real(i_ismod_eq1,SP))*4.0/3.0*pi*phi_cs*semi_aip*semi_aip*semi_aip
 
  end subroutine get_vip

  subroutine get_area_ratio_sol( output, sh_type, habit, is_mod, ap_den, mode, A_e, A_c, A_e_ic, A_c_ic, q_cs, mcry, magg, mrim, con, V_ic )
    ! sh 1 = pristine crystal
    ! sh 2 = rimed crystal
    ! sh 3 = aggregate
    ! sh 4 = rimed aggregate
    ! sh 5 = graupel
    ! ha 1, 2, 3 = plate, dendrite, column
    ! ha 4, 5, 6 = columnar, planar, irregular
    implicit none
    integer, intent(in) :: sh_type, habit, is_mod, mode
    real(SP), intent(in) :: ap_den, A_e, A_c, A_e_ic, A_c_ic, q_cs, mcry, magg, mrim, con, V_ic
    real(SP), intent(out) :: output

    real(SP) :: mrat_agg, mrat_grp, q_sidepl, q_ic_sidepl, q_0, q_ic, q_ic_0
    real(SP) :: mass_q_agg, mass_q_grp, mass1, mass2, dum1, dum2, q_agg, q_grp
    real(SP) :: i_sh_le2, i_sh_le4, i_habit_ge5, i_ismod2_eq1, i_mode_eq1, i_qcs_gt0, i_mass1_lt_massqagg, i_mass2_lt_massqgrp

    ! +++ return area ratio necesarry for calculating 
    !     terminal velocity +++

    ! this should correspond with aspect ratio diagnosis
    mrat_agg=10.0E0
    mrat_grp=10.0E0

    i_sh_le2=0.5E0*(1.0E0-real(isign(sh_type-3),SP))
    i_sh_le4=0.5E0*(1.0E0-real(isign(sh_type-5),SP))
    i_habit_ge5=0.5E0*(1.0E0+real(isign(habit-5),SP))

    i_ismod2_eq1=max(0.0E0,(-isign(is_mod-1)*max(0,is_mod-1)+1.0E0))* &
                  max(0.0E0,(-isign(is_mod-1)*min(0,is_mod-1)+1.0E0))

    i_mode_eq1=max(0.0E0,(-isign(mode-1)*max(0,mode-1)+1.0E0))* &
                max(0.0E0,(-isign(mode-1)*min(0,mode-1)+1.0E0))

    ! use Heymsfield et al (2002)'s side planes
    q_sidepl=max(0.18E0,min(0.88E0,(ap_den/0.35E0)**(1.0E0/2.34E0)))
    q_0=A_e/A_c

    ! use Heymsfield et al (2002)'s side planes
    q_ic_sidepl=max(0.18E0,min(0.88E0,(mcry/con/V_ic/0.35E0)**(1.0E0/2.34E0)))
    q_ic_0=A_e_ic/A_c_ic
    q_ic=real(i_habit_ge5,SP)*q_ic_sidepl+ &
          (1.0E0-real(i_habit_ge5,SP))*q_ic_0

    mass_q_agg=mcry/con*mrat_agg
    mass1=(mcry+magg)/con
    dum1=log10(q_ic/max(1.0e-30,q_cs))/log10(mcry/con/mass_q_agg) 
    dum2=mass1/mass_q_agg
    dum1=dum2**dum1
    q_agg=max(0.1E0,min(1.0E0,q_cs*dum1))

    mass_q_grp=(mcry+magg)/con*mrat_grp
    mass2=(mcry+magg+mrim)/con
    dum1=log10(q_ic/max(1.0e-30,q_cs))/log10(mcry/con/mass_q_grp)
    dum2=mass1/mass_q_grp
    dum1=dum2**dum1
    q_grp=max(0.1E0,min(1.0E0,q_cs*dum1))

    i_qcs_gt0=0.5E0*(1.0E0+dsign(q_cs-1.0e-30))
    i_mass1_lt_massqagg=0.5E0*(1.0E0-dsign(mass1-mass_q_agg))
    i_mass2_lt_massqgrp=0.5E0*(1.0E0-dsign(mass2-mass_q_grp))

    output = real(i_sh_le2,SP)* &
        (real(i_habit_ge5*i_ismod2_eq1,SP)*q_sidepl+ &
         (1.0E0-real(i_habit_ge5*i_ismod2_eq1,SP))*q_0 ) + &
        (1.0E0-real(i_sh_le2,SP))*real(i_sh_le4,SP)* &
        ( real(i_mode_eq1,SP)*q_0+ &
          (1.0E0-real(i_mode_eq1,SP))* &
          ( &
            real(i_qcs_gt0,SP)* &
            ( &
              real(i_mass1_lt_massqagg,SP)*( &
                                           q_agg &
                                       )+ &
              (1.0E0-real(i_mass1_lt_massqagg,SP))*( &
                                                 q_cs &
                                             ) &
          )+ &
            (1.0E0-real(i_qcs_gt0,SP))*q_ic &
        ) &
      ) + &
        (1.0E0-real(i_sh_le4,SP))* &
        ( real(i_mode_eq1,SP)*q_0+ &
          (1.0E0-real(i_mode_eq1,SP))* &
          (real(i_qcs_gt0,SP)* &
           ( &
             real(i_mass2_lt_massqgrp,SP)*( &
                                          q_grp &
                                      )+ &
             (1.0E0-real(i_mass2_lt_massqgrp,SP))*( &
                                                q_cs &
                                            ) &
         )+ &
           (1.0E0-real(i_qcs_gt0,SP))*q_ic &
       ) &
      )

  end subroutine get_area_ratio_sol


  subroutine get_arearatio( output, output_sidelook, output_total, sh_type, habit, is_mod, ap_den, mode, A_e, A_c, A_e_ic, A_c_ic, A_e_sidelook, A_e_ic_sidelook, A_e_total, q_cs, mcry, magg, mrim, con, V_ic, V_cs, alen, clen, dlen, semi_alen, semi_clen )
    ! sh 1 = pristine crystal
    ! sh 2 = rimed crystal
    ! sh 3 = aggregate
    ! sh 4 = rimed aggregate
    ! sh 5 = graupel
    ! ha 1, 2, 3 = plate, dendrite, column
    ! ha 4, 5, 6 = columnar, planar, irregular
    implicit none
    integer, intent(in) :: sh_type, habit, is_mod, mode
    real(SP), intent(in) :: ap_den, A_e, A_c, A_e_ic, A_c_ic, q_cs, mcry, magg, mrim, con, V_ic, V_cs, alen, clen, dlen, semi_alen, semi_clen, A_e_sidelook, A_e_ic_sidelook, A_e_total
    real(SP), intent(out) :: output, output_sidelook, output_total

    real(SP) :: mrat_agg, mrat_grp, q_sidepl, q_ic_sidepl, q_0, q_ic, q_ic_0, crystal_cirb
    real(SP) :: mass_q_agg, mass_q_grp, mass1, mass2, dum1, dum2, q_agg, q_grp
    real(SP) :: i_sh_le2, i_sh_le4, i_habit_ge5, i_ismod2_eq1, i_mode_eq1, i_qcs_gt0, i_mass1_lt_massqagg, i_mass2_lt_massqgrp, i_agtc

    ! +++ return area ratio necesarry for calculating 
    !     terminal velocity +++

    ! this should correspond with aspect ratio diagnosis
    mrat_agg=10.0E0
    mrat_grp=10.0E0

    i_sh_le2=0.5E0*(1.0E0-real(isign(sh_type-3),SP))
    i_sh_le4=0.5E0*(1.0E0-real(isign(sh_type-5),SP))
    i_habit_ge5=0.5E0*(1.0E0+real(isign(habit-5),SP))

    i_ismod2_eq1=max(0.0E0,(-isign(is_mod-1)*max(0,is_mod-1)+1.0E0))* &
                  max(0.0E0,(-isign(is_mod-1)*min(0,is_mod-1)+1.0E0))

    i_mode_eq1=max(0.0E0,(-isign(mode-1)*max(0,mode-1)+1.0E0))* &
                max(0.0E0,(-isign(mode-1)*min(0,mode-1)+1.0E0))

    ! use Heymsfield et al (2002)'s side planes
    q_sidepl=max(0.18E0,min(0.88E0,(ap_den/0.35E0)**(1.0E0/2.34E0)))
    i_agtc=0.5E0*(1.0E0-dsign(semi_clen-semi_alen))
    !i_agtc = 1.0E0
    !crystal_cirb = real(i_agtc,SP)*pi*semi_alen*semi_alen + (1.0E0 - real(i_agtc,SP))*pi*(semi_clen*semi_clen+semi_alen*semi_alen)
    crystal_cirb = pi*semi_alen*semi_alen ! only works for forcing look down
    !q_0=A_e/A_c
    !q_0=A_e / (pi*((V_cs*3.0/4.0/pi)**(1.0/3.0))**2.0)
    q_0=A_e / crystal_cirb
    if (q_0 > 1.0E0) then
       !write(*,*), i_agtc, q_0, A_e, crystal_cirb, clen, alen, semi_clen, semi_alen, mrim/mcry
       !call PRC_abort
    endif

    ! use Heymsfield et al (2002)'s side planes
    q_ic_sidepl=max(0.18E0,min(0.88E0,(mcry/con/V_ic/0.35E0)**(1.0E0/2.34E0)))
    i_agtc=0.5E0*(1.0E0-dsign(clen-alen))
    !i_agtc = 1.0E0
    !crystal_cirb = real(i_agtc,SP)*pi*alen*alen + (1.0E0 - real(i_agtc,SP))*pi*(clen*clen+alen*alen)
    crystal_cirb = pi*alen*alen ! only works for forcing look down
    !q_ic_0=A_e_ic/A_c_ic
    !q_ic_0 = A_e_ic / (pi*((V_ic*3.0/4.0/pi)**(1.0/3.0))**2.0)
    q_ic_0 = A_e_ic / crystal_cirb
    q_ic=real(i_habit_ge5,SP)*q_ic_sidepl+ &
          (1.0E0-real(i_habit_ge5,SP))*q_ic_0

    mass_q_agg=mcry/con*mrat_agg
    mass1=(mcry+magg)/con
    dum1=log10(q_ic/max(1.0e-30,q_cs))/log10(mcry/con/mass_q_agg) 
    dum2=mass1/mass_q_agg
    dum1=dum2**dum1
    q_agg=max(0.1E0,min(1.0E0,q_cs*dum1))

    mass_q_grp=(mcry+magg)/con*mrat_grp
    mass2=(mcry+magg+mrim)/con
    !dum1=log10(q_ic/max(1.0e-30,q_cs))/log10(mcry/con/mass_q_grp)
    dum1=log10(q_ic/max(1.0e-30,0.8D0))/log10(mcry/con/mass_q_grp)
    dum2=mass2/mass_q_grp
    dum1=dum2**dum1
    !q_grp=max(0.1E0,min(1.0E0,q_cs*dum1))
    q_grp=max(0.1E0,min(1.0E0,0.8D0*dum1))

    i_qcs_gt0=0.5E0*(1.0E0+dsign(q_cs-1.0e-30))
    i_mass1_lt_massqagg=0.5E0*(1.0E0-dsign(mass1-mass_q_agg))
    i_mass2_lt_massqgrp=0.5E0*(1.0E0-dsign(mass2-mass_q_grp))

    output = real(i_sh_le2,SP)* &
               (real(i_habit_ge5*i_ismod2_eq1,SP)             *q_sidepl+ &
                (1.0E0-real(i_habit_ge5*i_ismod2_eq1,SP))     *q_0 ) + &
             (1.0E0-real(i_sh_le2,SP))* &
               real(i_sh_le4,SP)* &
                 (real(i_mode_eq1,SP)                         *q_0+ &
                  (1.0E0-real(i_mode_eq1,SP))* &
          ( &
            real(i_qcs_gt0,SP)* &
            ( &
              real(i_mass1_lt_massqagg,SP)*( &
                                           q_agg &
                                       )+ &
              (1.0E0-real(i_mass1_lt_massqagg,SP))*( &
                                                 q_cs &
                                             ) &
          )+ &
            (1.0E0-real(i_qcs_gt0,SP))*q_ic &
        ) &
      ) + &
        (1.0E0-real(i_sh_le4,SP))* &
        ( real(i_mode_eq1,SP)*q_0+ &
          (1.0E0-real(i_mode_eq1,SP))* &
          (real(i_qcs_gt0,SP)* &
           ( &
             real(i_mass2_lt_massqgrp,SP)*( &
                                          q_grp &
                                      )+ &
             (1.0E0-real(i_mass2_lt_massqgrp,SP))*( &
                                                q_cs &
                                            ) &
         )+ &
           (1.0E0-real(i_qcs_gt0,SP))*q_ic &
       ) &
      )


    !!! side look calculation
    ! use Heymsfield et al (2002)'s side planes
    q_sidepl=max(0.18E0,min(0.88E0,(ap_den/0.35E0)**(1.0E0/2.34E0)))
    i_agtc=0.5E0*(1.0E0-dsign(semi_clen-semi_alen))
    !i_agtc = 1.0E0
    crystal_cirb = pi*(semi_clen*semi_clen+semi_alen*semi_alen)
    !crystal_cirb = real(i_agtc,SP)*pi*semi_alen*semi_alen + (1.0E0 - real(i_agtc,SP))*pi*semi_clen*semi_clen ! only works for forcing look down
    !q_0=A_e/A_c
    !q_0=A_e / (pi*((V_cs*3.0/4.0/pi)**(1.0/3.0))**2.0)
    q_0=A_e_sidelook / crystal_cirb
    if (q_0 > 1.0E0) then
       !write(*,*), "side", i_agtc, A_e_sidelook, crystal_cirb, semi_clen, semi_alen
    endif

    ! use Heymsfield et al (2002)'s side planes
    q_ic_sidepl=max(0.18E0,min(0.88E0,(mcry/con/V_ic/0.35E0)**(1.0E0/2.34E0)))
    i_agtc=0.5E0*(1.0E0-dsign(clen-alen))
    !i_agtc = 1.0E0
    crystal_cirb = pi*(clen*clen+alen*alen)
    !crystal_cirb = real(i_agtc,SP)*pi*alen*alen + (1.0E0 - real(i_agtc,SP))*pi*clen*clen ! only works for forcing look down
    !q_ic_0=A_e_ic/A_c_ic
    !q_ic_0 = A_e_ic / (pi*((V_ic*3.0/4.0/pi)**(1.0/3.0))**2.0)
    q_ic_0 = A_e_ic_sidelook / crystal_cirb
    if (q_ic_0 > 1.0E0) then
       !write(*,*), "side ic", i_agtc, A_e_ic_sidelook, crystal_cirb, clen, alen
    endif
    q_ic=real(i_habit_ge5,SP)*q_ic_sidepl+ &
          (1.0E0-real(i_habit_ge5,SP))*q_ic_0

    mass_q_agg=mcry/con*mrat_agg
    mass1=(mcry+magg)/con
    dum1=log10(q_ic/max(1.0e-30,q_cs))/log10(mcry/con/mass_q_agg) 
    dum2=mass1/mass_q_agg
    dum1=dum2**dum1
    q_agg=max(0.1E0,min(1.0E0,q_cs*dum1))

    mass_q_grp=(mcry+magg)/con*mrat_grp
    mass2=(mcry+magg+mrim)/con
    !dum1=log10(q_ic/max(1.0e-30,q_cs))/log10(mcry/con/mass_q_grp)
    dum1=log10(q_ic/max(1.0e-30,0.8D0))/log10(mcry/con/mass_q_grp)
    dum2=mass2/mass_q_grp
    dum1=dum2**dum1
    !q_grp=max(0.1E0,min(1.0E0,q_cs*dum1))
    q_grp=max(0.1E0,min(1.0E0,0.8D0*dum1))

    i_qcs_gt0=0.5E0*(1.0E0+dsign(q_cs-1.0e-30))
    i_mass1_lt_massqagg=0.5E0*(1.0E0-dsign(mass1-mass_q_agg))
    i_mass2_lt_massqgrp=0.5E0*(1.0E0-dsign(mass2-mass_q_grp))

    output_sidelook = real(i_sh_le2,SP)* &
               (real(i_habit_ge5*i_ismod2_eq1,SP)             *q_sidepl+ &
                (1.0E0-real(i_habit_ge5*i_ismod2_eq1,SP))     *q_0 ) + &
             (1.0E0-real(i_sh_le2,SP))* &
               real(i_sh_le4,SP)* &
                 (real(i_mode_eq1,SP)                         *q_0+ &
                  (1.0E0-real(i_mode_eq1,SP))* &
          ( &
            real(i_qcs_gt0,SP)* &
            ( &
              real(i_mass1_lt_massqagg,SP)*( &
                                           q_agg &
                                       )+ &
              (1.0E0-real(i_mass1_lt_massqagg,SP))*( &
                                                 q_cs &
                                             ) &
          )+ &
            (1.0E0-real(i_qcs_gt0,SP))*q_ic &
        ) &
      ) + &
        (1.0E0-real(i_sh_le4,SP))* &
        ( real(i_mode_eq1,SP)*q_0+ &
          (1.0E0-real(i_mode_eq1,SP))* &
          (real(i_qcs_gt0,SP)* &
           ( &
             real(i_mass2_lt_massqgrp,SP)*( &
                                          q_grp &
                                      )+ &
             (1.0E0-real(i_mass2_lt_massqgrp,SP))*( &
                                                q_cs &
                                            ) &
         )+ &
           (1.0E0-real(i_qcs_gt0,SP))*q_ic &
       ) &
      )



    !!! total look calculation
    ! use Heymsfield et al (2002)'s side planes
    q_sidepl=max(0.18E0,min(0.88E0,(ap_den/0.35E0)**(1.0E0/2.34E0)))
    i_agtc=0.5E0*(1.0E0-dsign(semi_clen-semi_alen))
    !i_agtc = 1.0E0
    !crystal_cirb = pi*(semi_clen*semi_clen+semi_alen*semi_alen)
    crystal_cirb = real(i_agtc,SP)*pi*semi_alen*semi_alen + (1.0E0 - real(i_agtc,SP))*pi*semi_clen*semi_clen ! only works for forcing look down
    !q_0=A_e/A_c
    !q_0=A_e / (pi*((V_cs*3.0/4.0/pi)**(1.0/3.0))**2.0)
    q_0=A_e_total / crystal_cirb / 4.0E0
    if (q_0 > 1.0E0) then
       !write(*,*), "side", i_agtc, A_e_sidelook, crystal_cirb, semi_clen, semi_alen
    endif

    ! use Heymsfield et al (2002)'s side planes
    q_ic_sidepl=max(0.18E0,min(0.88E0,(mcry/con/V_ic/0.35E0)**(1.0E0/2.34E0)))
    i_agtc=0.5E0*(1.0E0-dsign(clen-alen))
    !i_agtc = 1.0E0
    !crystal_cirb = pi*(clen*clen+alen*alen)
    crystal_cirb = real(i_agtc,SP)*pi*alen*alen + (1.0E0 - real(i_agtc,SP))*pi*clen*clen ! only works for forcing look down
    !q_ic_0=A_e_ic/A_c_ic
    !q_ic_0 = A_e_ic / (pi*((V_ic*3.0/4.0/pi)**(1.0/3.0))**2.0)
    q_ic_0 = A_e_total / crystal_cirb / 4.0E0
    if (q_ic_0 > 1.0E0) then
       !write(*,*), "side ic", i_agtc, A_e_ic_sidelook, crystal_cirb, clen, alen
    endif
    q_ic=real(i_habit_ge5,SP)*q_ic_sidepl+ &
          (1.0E0-real(i_habit_ge5,SP))*q_ic_0

    mass_q_agg=mcry/con*mrat_agg
    mass1=(mcry+magg)/con
    dum1=log10(q_ic/max(1.0e-30,q_cs))/log10(mcry/con/mass_q_agg) 
    dum2=mass1/mass_q_agg
    dum1=dum2**dum1
    q_agg=max(0.1E0,min(1.0E0,q_cs*dum1))

    mass_q_grp=(mcry+magg)/con*mrat_grp
    mass2=(mcry+magg+mrim)/con
    !dum1=log10(q_ic/max(1.0e-30,q_cs))/log10(mcry/con/mass_q_grp)
    dum1=log10(q_ic/max(1.0e-30,0.8D0))/log10(mcry/con/mass_q_grp)
    dum2=mass2/mass_q_grp
    dum1=dum2**dum1
    !q_grp=max(0.1E0,min(1.0E0,q_cs*dum1))
    q_grp=max(0.1E0,min(1.0E0,0.8D0*dum1))

    i_qcs_gt0=0.5E0*(1.0E0+dsign(q_cs-1.0e-30))
    i_mass1_lt_massqagg=0.5E0*(1.0E0-dsign(mass1-mass_q_agg))
    i_mass2_lt_massqgrp=0.5E0*(1.0E0-dsign(mass2-mass_q_grp))

    output_total = real(i_sh_le2,SP)* &
               (real(i_habit_ge5*i_ismod2_eq1,SP)             *q_sidepl+ &
                (1.0E0-real(i_habit_ge5*i_ismod2_eq1,SP))     *q_0 ) + &
             (1.0E0-real(i_sh_le2,SP))* &
               real(i_sh_le4,SP)* &
                 (real(i_mode_eq1,SP)                         *q_0+ &
                  (1.0E0-real(i_mode_eq1,SP))* &
          ( &
            real(i_qcs_gt0,SP)* &
            ( &
              real(i_mass1_lt_massqagg,SP)*( &
                                           q_agg &
                                       )+ &
              (1.0E0-real(i_mass1_lt_massqagg,SP))*( &
                                                 q_cs &
                                             ) &
          )+ &
            (1.0E0-real(i_qcs_gt0,SP))*q_ic &
        ) &
      ) + &
        (1.0E0-real(i_sh_le4,SP))* &
        ( real(i_mode_eq1,SP)*q_0+ &
          (1.0E0-real(i_mode_eq1,SP))* &
          (real(i_qcs_gt0,SP)* &
           ( &
             real(i_mass2_lt_massqgrp,SP)*( &
                                          q_grp &
                                      )+ &
             (1.0E0-real(i_mass2_lt_massqgrp,SP))*( &
                                                q_cs &
                                            ) &
         )+ &
           (1.0E0-real(i_qcs_gt0,SP))*q_ic &
       ) &
      )



  end subroutine get_arearatio


  function get_dynamic_viscosity(d_T) result (output)
    implicit none
    real(SP), intent(in) :: d_T
    real(SP) :: output

    real(SP) :: Tc

    ! calculate the dynamic viscosity, g/cm/s
    ! ambient temperature (K)
    
    Tc = d_T - 273.15E0
    ! from Pruppacker's book 
    if (Tc >= 0.0E0) then
       output=(1.718E0 + 0.0049E0*Tc)*1.0e-4
    else
       output=(1.718E0 + 0.0049E0*Tc - 1.2e-5*Tc*Tc)*1.0e-4
    endif

  end function get_dynamic_viscosity

  function get_sfc_tension(d_T) result(output)
    implicit none
    real(SP), intent(in) :: d_T
    real(SP) :: output

    real(SP) :: an(7), Tc

    ! calculate the surface tension of water: erg/cm^2
    an = (/75.93E0, 0.115E0, 6.818e-2, 6.511e-3, 2.933e-4, 6.283e-6, 5.285e-8/)
    ! valid between 40 and -40 Celisus
    Tc = max(-45.0E0,min(40.0E0,d_T-273.15E0))
    output = an(1)+an(2)*Tc+an(3)*Tc*Tc+an(4)*Tc*Tc*Tc+an(5)*Tc*Tc*Tc*Tc+ &
             an(6)*Tc*Tc*Tc*Tc*Tc+an(7)*Tc*Tc*Tc*Tc*Tc*Tc

  end function get_sfc_tension

  function get_diffusivity(P,T) result (output)
    implicit none
    real(SP), intent(in) :: P, T
    real(SP) :: output

    real(SP) :: T_0, P_0

    ! **********************************************************************
    ! Calculate the diffusivity of water vapor in air for temperatures
    ! between -40 to 40 C according to Hall and Pruppacher (1976)
    ! **********************************************************************
    ! ambient pressure and temperature in g/s^2/cm and Kelvin
    T_0 = 273.15E0    ! K
    P_0 = 1013250.0E0 ! g/s^2/cm
    output = 0.211E0 * ((T/T_0)**1.94E0) * (P_0/P/10.0E0) ! convert from Pascal to g/s^2/cm

  end function get_diffusivity

  function get_fkn(diffusivity,T,phase,r0) result (output)
    implicit none
    real(SP), intent(in) :: diffusivity, T, r0
    integer, intent(in) :: phase
    real(SP) :: output

    real(SP) :: beta, beta_w, beta_i1, beta_i2, delta, R_v

    ! NOTE:
    ! this should be the same as get_mod_diffusivity.
    !
    ! deposition coefficient, thickness of a bc
    beta_w=0.036E0
    beta_i1=0.5E0
    beta_i2=0.006E0
    delta=1.0e-5
    R_v = 4.615e+06

    if (phase == 1) then
       beta=beta_w
    else
       beta=beta_i1
    endif
    
    ! +++ calculate kinetic effect +++
    output = r0/(r0+delta)+(diffusivity/beta/r0)*sqrt(2.0E0*pi/R_v/T)
    output = 1.0E0/output

  end function get_fkn

  function get_len_c2a(amass) result (output)
    implicit none
    real(SP), intent(in) :: amass
    real(SP) :: output

    real(SP) :: a_ros, b_ros, N_ros, den_i

    ! mass-dimension relationship of bullet roseete
    ! parameters for mass-dimension relationship from Chiruta and Wang (2003)
    a_ros = 0.3257E0
    b_ros = 0.5206E0
    N_ros = 4.0E0
    den_i = 0.91668E0
    
    output = (amass/(den_i*a_ros*N_ros**b_ros))**(1.0E0/3.0E0)
    
  end function get_len_c2a

  function get_len_s1(amass) result (output)
    implicit none
    real(SP), intent(in) :: amass
    real(SP) :: output

    real(SP) :: a, b, c1

    ! mass-dimension relationship of S1
    ! side plane crystal mass-length relation from Mitchel et al (1990)
    a = 0.0041900509E0
    b = 2.3E0
    ! assume that the shape is cylinder that has the aspect ratio of 0.25 for all sizes, so
    ! when it exceed max density, radiuas has to be increased.
    c1 = 0.885565704066706E0
    output = max(0.5E0*(amass/a)**(1.0E0/b),c1*amass**(1.0E0/3.0E0))

  end function get_len_s1

  function get_len_s3(amass) result (output)
    implicit none
    real(SP), intent(in) :: amass
    real(SP) :: output

    real(SP) :: a, b, c1

    ! mass-dimension relationship of S3
    ! S3 crystal mass-length relation from Mitchel et al (1990)
    a = 0.0027696359E0
    b = 2.1E0
    ! assume that the shape is cylinder that has the aspect ratio of 0.25 for all sizes, so
    ! when it exceed max density, radiuas has to be increased.
    c1 = 0.885565704066706E0
    output = max(0.5E0*(amass/a)**(1.0E0/b),c1*amass**(1.0E0/3.0E0))

  end function get_len_s3

  subroutine get_elen_rlen( elen, rlen, habit, mcry, alen, clen, aglen, cglen )
    implicit none
    real(SP), intent(in) :: mcry, alen, clen, aglen, cglen
    integer, intent(in) :: habit
    real(SP), intent(out) :: elen, rlen

    real(SP) :: xlen_s1, xlen_c2a, xlen_s3, i_ag_ge_cgclen, i_cg_ge_agalen


    xlen_s1 = get_len_s1(mcry)
    xlen_c2a = get_len_c2a(mcry)
    xlen_s3 = get_len_s3(mcry)

    if (habit <= 3) then !  .or. alen < 1.e-15 .or. clen < 1.e-15
       elen = 0.0E0
       rlen = 0.0E0
       return
    else
       i_ag_ge_cgclen = 0.5E0*(1.0E0 + dsign(aglen-(cglen/clen+0.5E0)*alen))
       i_cg_ge_agalen = 0.5E0*(1.0E0 + dsign(cglen-(aglen/alen+0.5E0)*clen))
    endif

    ! side planes or radiating assemblage of plates: S1 or P7a
    ! bullet rosette
    ! combination of side planes, bullets, and columns: S3
    elen=&
          i_ag_ge_cgclen*xlen_s1 + &
          (1.0E0 - i_ag_ge_cgclen)*i_cg_ge_agalen*0.0E0 + &
          (1.0E0 - i_ag_ge_cgclen)*(1.0E0 - i_cg_ge_agalen)* &
          xlen_s3

    rlen=&
          i_ag_ge_cgclen*0.0E0 + &
          (1.0E0 - i_ag_ge_cgclen)*i_cg_ge_agalen* &
          xlen_c2a + &
          (1.0E0 - i_ag_ge_cgclen)*(1.0E0 - i_cg_ge_agalen)* &
          xlen_s3 

    if (elen <= 0.0E0 .or. rlen <= 0.0E0) then
       write(*,'(A,5ES15.6)') "elen error: ", elen, rlen, xlen_s1, xlen_c2a, xlen_s3
       call PRC_abort
    endif

    return

  end subroutine get_elen_rlen

  

  subroutine cal_lincubprms_vec(a, error_number, nbin, con, amass, binb3d)
    implicit none
    real(SP), intent(in) :: con(nbin), amass(nbin), binb3d(nbin+1,2)
    integer, intent(in) :: nbin
    real(SP), intent(out) :: a(nbin,4)
    integer, intent(out) :: error_number(nbin)

    real(SP) :: a_cub(nbin,4), bd1, bd2, x_0, delta_m, am0, b0, b1, b2, b3
    real(SP) :: amb_L, anb_L, xib_L, amb_R, anb_R, xib_R
    real(SP) :: A_L, B_L, C_L, A_R, B_R, C_R, q, a0_c, a1_c, a2_c, a3_c, delta
    real(SP) :: am_e, an_me, an_ml, an_mr
    integer :: i, i_con_gt0_left, i_con_gt0_right, i_delta_ge0, i_ame_lt0, i_anml_lt0, i_anmr_lt0, i_large, i_cubic

    !     **********************************************************************
    !     Calculate the parameters of the linear distribution and cubic
    !     distributions:
    !     n(x) = n_0 + k ( x - x_0 )
    !          = max(0,a(1))   + a(3) ( x - a(2))
    !          = max(0,a(1))-a(2)*a(3) + a(3)*x
    !
    !    Reference
    !      Dinh and Durran (2012), Atm. Chem. Phys.
    !     **********************************************************************
    !

    !  
    !  first, fit a piece-wise linear
    !
    do i = 1, nbin
       if( (binb3d(i,1)<1.0e-30 .and. binb3d(i,2)<1.0e-30) .or. (binb3d(i,1)==binb3d(i,2))) then
          a(i,1)=0.0E0
          a(i,2)=0.0E0
          a(i,3)=0.0E0
          a(i,4)=0.0E0
          error_number(i)=4
       else if(con(i)>0.0 .and. amass(i)>0.0) then
          bd1=binb3d(i,1)
          bd2=binb3d(i,2)
          x_0=0.5E0*(bd1+bd2)
          a(i,2)=x_0
          a(i,1)=con(i)/(bd2-bd1)
          a(i,3)=12.0E0*(amass(i)-x_0*con(i))/(bd2-bd1)**3
          
          a(i,4)=-9.99e+35!100 ! indication of linear
               
       else
          a(i,1)=0.0E0
          a(i,2)=0.0E0
          a(i,3)=0.0E0
          a(i,4)=0.0E0
          error_number(i)=10
       endif

    enddo

    do i = 1, nbin
       if( a(i,1) + a(i,3)*( binb3d(i,1) - a(i,2)) < 0.0E0 ) then
          x_0=3.0*amass(i)/con(i) - 2.0*binb3d(i,2)
          a(i,2) = x_0
          a(i,3) = 2.0E0*con(i)/max((binb3d(i,2) - x_0)**2,1.0e-35)
          a(i,1) = -1.0E0

          error_number(i)=ifloor(0.5*(1.0-dsign(binb3d(i,2)-x_0))+1.0e-5)  ! 1

       else if( a(i,1) +a(i,3)*( binb3d(i,2) - a(i,2)) < 0.0E0 ) then
          x_0=3.0*amass(i)/con(i) - 2.0*binb3d(i,1)
          a(i,2) = x_0
          a(i,3) = - 2.0E0*con(i)/max((binb3d(i,1) - x_0)**2,1.0e-35)
          a(i,1) = -2.0E0

          error_number(i)=ifloor(1.5E0*(1.0E0-dsign(x_0-binb3d(i,1)))+1.0e-5)   ! 3
          error_number(i)=ifloor(1.0E0*(1.0E0-dsign(x_0))+1.0e-5)   ! 2
       endif

       if( abs(a(i,1))>1.0e+35 .or. abs(a(i,2))>1.0e+35 .or. abs(a(i,3))>1.0e+35) then
          error_number(i) = 4
       endif

    enddo

    !
    ! second, calculate the cubic 
    !

    do i = 1, nbin
       if( ( error_number(i)==0 .and. error_number(max(1,i-1))==0 .and. error_number(min(nbin,i+1))==0  ) .and. (i > 1 .and. i < nbin) ) then

          bd1 = binb3d(i,1)
          bd2 = binb3d(i,2)
          delta_m = bd2 - bd1
          am0 = 0.5E0*(bd1 + bd2)

          b0 = con(i)/delta_m
          b1 = 6.0E0*(amass(i)-am0*con(i))/(delta_m*delta_m)

          ! negative flags
          i_con_gt0_left  = int(0.5E0*(1.0E0-dsign(1.0e-30-con(i-1))))
          i_con_gt0_right = int(0.5E0*(1.0E0-dsign(1.0e-30-con(i+1))))

          ! left mean mass 
          amb_L = amass(i-1)/con(i-1)
          ! number concentration at the mean mass
          anb_L = max(0.0,a(i-1,1))+a(i-1,3)*(amb_L-a(i-1,2))
          ! mean Kai value
          xib_L = 2.0E0*(amb_L-am0)/delta_m

          ! right mean mass 
          amb_R = amass(i+1)/con(i+1)
          ! number concentration at the mean mass
          anb_R = max(0.0E0,a(i+1,1))+a(i+1,3)*(amb_R-a(i+1,2))
          ! mean Kai value
          xib_R = 2.0E0*(amb_R-am0)/delta_m

          A_L = 0.5E0*(3.0E0*xib_L*xib_L - 1.0E0)
          B_L = 0.5E0*(5.0E0*xib_L*xib_L*xib_L - 3.0E0*xib_L)
          C_L = anb_L - b0 - b1*xib_L

          A_R = 0.5E0*(3.0E0*xib_R*xib_R - 1.0E0)
          B_R = 0.5E0*(5.0E0*xib_R*xib_R*xib_R - 3.0E0*xib_R)
          C_R = anb_R - b0 - b1*xib_R

          b3 = (C_L*A_R - A_L*C_R)/B_L*A_R - A_L*B_R
          b2 = (C_L - b3*B_L)/A_L

          q = am0/delta_m
          a0_c = b0 - 2.0E0*q*b1 + (6.0E0*q*q - 0.5E0)*b2 - (20.0E0*q*q*q - 3.0E0*q)*b3
          a1_c = (2.0E0*b1 - 12.0E0*q*b2 + (60.0E0*q*q - 3.0E0)*b3)/delta_m
          a2_c = (6.0E0*b2 - 60.0E0*q*b3)/(delta_m*delta_m)
          a3_c = 20.0E0*b3/(delta_m*delta_m*delta_m)

          ! criteria for negative valune in the bin
          delta = a2_c*a2_c - 3.0E0*a1_c*a3_c
          am_e = (-a2_c + sqrt(max(0.0E0,delta)))/(3.0E0*a3_c)
          an_me = a0_c + a1_c*am_e + a2_c*am_e*am_e + a3_c*am_e*am_e*am_e
          i_delta_ge0 = int(0.5E0*(1.0E0 + dsign(delta)))
          i_ame_lt0 = int(0.5E0*(1.0E0 - dsign(an_me)))
      
          an_ml = a0_c + a1_c*bd1 + a2_c*bd1*bd1 + a3_c*bd1*bd1*bd1
          an_mr = a0_c + a1_c*bd2 + a2_c*bd2*bd2 + a3_c*bd2*bd2*bd2
          i_anml_lt0 = int(0.5E0*(1.0E0 - dsign(an_ml)))
          i_anmr_lt0 = int(0.5E0*(1.0E0 - dsign(an_mr)))

          ! too large coefficients
          !  this seems to cause infinity in shift bin
          i_large = int(0.5E0*(1.0E0 + dsign(abs(a0_c)-1.0e+30)))

          i_cubic = i_con_gt0_left*i_con_gt0_right * (1 - i_delta_ge0*i_ame_lt0)*(1 - i_anml_lt0)*(1 - i_anmr_lt0) * (1 - i_large)

      
          a_cub(i,1)=real(i_cubic,SP)*a0_c + real(1-i_cubic,SP)*a(i,1)
      
          a_cub(i,2)=real(i_cubic,SP)*a1_c + real(1-i_cubic,SP)*a(i,2)
      
          a_cub(i,3)=real(i_cubic,SP)*a2_c + real(1-i_cubic,SP)*a(i,3)
      
          a_cub(i,4)=real(i_cubic,SP)*a3_c + real(1-i_cubic,SP)*(-9.99e+35)
          

       endif

    enddo

    ! 
    ! Finally, update a parameters
    !
                
    do i = 1, nbin

       if( ( error_number(i)==0 .and. error_number(max(1,i-1))==0 .and. error_number(min(nbin,i+1))==0  ) .and. (i > 1 .and. i < nbin) ) then

          a(i,1) = a_cub(i,1)
          a(i,2) = a_cub(i,2)
          a(i,3) = a_cub(i,3)
          a(i,4) = a_cub(i,4)
      
       endif

    enddo

    return

  end subroutine cal_lincubprms_vec

  subroutine cal_wterm_vel_v3_vec(WVT, nbin, a2d, m1, m2, D1, Q1, particle_den, particle_alpha, particle_mean_mass, particle_char_len, particle_vtm, den_a, d_vis, error_number)
    implicit none
    integer, intent(in) :: nbin
    real(SP), intent(in) :: a2d(nbin,4)
    real(SP), intent(in) :: m1(nbin), m2(nbin), D1(nbin), Q1(nbin)
    real(SP), intent(in) :: particle_den(nbin), particle_alpha(nbin), particle_mean_mass(nbin), particle_char_len(nbin), particle_vtm(nbin), den_a, d_vis
    integer, intent(in) :: error_number(nbin)
    real(SP), intent(out) :: WVT(nbin,2)

    ! +++ mode of shape to calculate the ventilation coefficient 
    !     and terminal velocity +++
    ! 1 : spheroid with a_len and c_len
    ! 2 : ice crystal model
    !integer,intent(in)   :: mode


    real(SP), parameter         :: den_w = 1.0e+0
    real(SP), dimension(4) :: a_rex, b_rex

    real(SP) :: X1, X2
    real(SP) :: A1, B1, aM, aN
    real(SP), dimension(nbin) :: aMT, aNT
    real(SP), dimension(4)     :: a
    real(SP) :: xA, xB, xC, xD

    ! parameters to define NRe=a*X**b
    ! for liquid
    ! does not include diameter > 5.0mm
    real(SP),parameter :: &
         la1=4.049863E-02,lb1=9.939921E-01,&
         la2=7.395895E-02,lb2=8.073323E-01,&
         la3=2.557850E-01,lb3=6.412501E-01,&
         la4=1.033221E+00,lb4=5.235578E-01,&
         LXlim1=2.517541E+01,LXlim2=1.700411E+03,LXlim3=1.356160E+05

    real(SP),parameter :: D_max=0.45,m_max=0.047712938426395
    real(SP) :: v_max

    ! for solid
    ! for ylim   1.000000E+00   2.000000E+01   2.000000E+02
    real(SP),parameter :: &
         sa1=4.742587E-02,sb1=9.910232E-01,&
         sa2=9.447403E-02,sb2=7.722376E-01,&
         sa3=2.549053E-01,sb3=6.299768E-01,&
         sa4=7.880099E-01,sb4=5.236878E-01,&
         SXlim1=2.167592E+01,SXlim2=1.027213E+03,SXlim3=3.934172E+04

    ! define terminal velocity regimes by masses
    real(SP),dimension(5) :: X_b
    ! masses and diameter devided by terminal velocity regimes.
    real(SP) :: m_sub1,m_sub2,D_1,D_2

    integer :: i_den_gt0p5
    integer  :: i,j,n,in

    real(SP),parameter :: chiarui_1=0.0E0, chiarui_2=1.0e+30

    ! ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    ! calculate the terminal velocity weighted by concentration and mass.
    ! use the approximation of NRe= a*X**b fitted to results by Bohm equation.
    ! ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

    ! --- in case of solid phase ---
    a_rex = (/ sa1,sa2,sa3,sa4 /)
    b_rex = (/ sb1,sb2,sb3,sb4 /)
    X_b = (/ chiarui_1,SXlim1,SXlim2,SXlim3,chiarui_2 /)

    WVT(:,:) = 0.0E0
    aNT(:) = 0.0E0
    aMT(:) = 0.0E0

    do j = 1, 4
       do i = 1, nbin

          a(:) = 0.0E0
          if (error_number(i) == 0) then

             a(1) = a2d(i,1)
             a(2) = a2d(i,2)
             a(3) = a2d(i,3)
             a(4) = a2d(i,4)

             X1 = D1(i)*m1(i)
             X2 = D1(i)*m2(i)

             if(X_b(j+1) < X1 .or. X_b(j) > X2) then
                ! do nothing
             else
                m_sub1 = max(m1(i),X_b(j)/D1(i))
                m_sub2 = min(m2(i),X_b(j+1)/D1(i))

                xA = m_sub2+m_sub1
                xB = m_sub2-m_sub1
                xC = m_sub2*m_sub1
                xD = m_sub2*m_sub2+m_sub1*m_sub1

                aN = a(1)*xB + 0.5E0*a(2)*xA*xB + a(3)*xB*(xD+xC)/3.0E0 + 0.25E0*a(4)*xD*xA*xB

                aM = 0.5E0*a(1)*xA*xB + a(2)*xB*(xD+xC)/3.0E0 + 0.25E0*a(3)*xD*xA*xB + 0.2E0*a(4)*xB*(xA*xA*(xD-xC)+xC*xC)

                A1 = d_vis**(1.0E0 - 2.0E0*b_rex(j))*den_a**(b_rex(j)-1.0E0)*(8.0E0*gg/pi)**b_rex(j)*(Q1(i)*max(particle_alpha(i),1.0E0))**(-b_rex(j))*particle_mean_mass(i)**(1.0E0/3.0E0)/particle_char_len(i)*a_rex(j)

                B1 = b_rex(j) - 1.0E0/3.0E0

                ! 1. mass-weighted terminal velocity
                WVT(i,1) = WVT(i,1) + max(0.0E0, &
                     A1*( a(1)/(B1 + 2.0E0)*(m_sub2**(B1 + 2.0E0) - m_sub1**(B1 + 2.0E0)) &
                     + a(2)/(B1 + 3.0E0)*(m_sub2**(B1 + 3.0E0) - m_sub1**(B1 + 3.0E0)) &
                     + a(3)/(B1 + 4.0E0)*(m_sub2**(B1 + 4.0E0) - m_sub1**(B1 + 4.0E0)) &
                     + a(4)/(B1 + 5.0E0)*(m_sub2**(B1 + 5.0E0) - m_sub1**(B1 + 5.0E0)) &
                ))
          
                ! 2. con-weighted terminal velocity
                WVT(i,2) = WVT(i,2) + max(0.0E0, &
                     A1*( a(1)/(B1 + 1.0E0)*(m_sub2**(B1 + 1.0E0) - m_sub1**(B1 + 1.0E0)) &
                     + a(2)/(B1 + 2.0E0)*(m_sub2**(B1 + 2.0E0) - m_sub1**(B1 + 2.0E0)) &
                     + a(3)/(B1 + 3.0E0)*(m_sub2**(B1 + 3.0E0) - m_sub1**(B1 + 3.0E0)) &
                     + a(4)/(B1 + 4.0E0)*(m_sub2**(B1 + 4.0E0) - m_sub1**(B1 + 4.0E0)) &
                     ))
                    
                aNT(i) = aNT(i) + aN
                aMT(i) = aMT(i) + aM
             endif
          endif
       enddo
    enddo


    do i = 1, nbin
       if (error_number(i) == 0) then
          i_den_gt0p5 = 0.5E0*(1.0E0 - dsign(0.5E0 - particle_den(i)))
          v_max = i_den_gt0p5*5.0e+3 + &
               (1.0E0 - i_den_gt0p5)*2.0e+3 
             
          WVT(i,1) = min(max(0.0E0,WVT(i,1)/aMT(i)),v_max)
          WVT(i,2) = min(max(0.0E0,WVT(i,2)/aNT(i)),v_max)
       else if(error_number(i) < 10) then
          WVT(i,1) = min(max(0.0E0,particle_vtm(i)),1.5e+3)
          WVT(i,2) = min(max(0.0E0,particle_vtm(i)),1.5e+3)
       endif
    enddo


  end subroutine cal_wterm_vel_v3_vec


end program mpipaper5analysis_time 


